@@ -2,22 +2,24 @@
 #define __APPEARANCE_H__
 
 #include "appearancebase.h"
+#include <qfont.h>
 
 class MusE;
 class Arranger;
 class QColor;
 
 //---------------------------------------------------------
-//   transpose widget
+//   Appearance Dialog
 //---------------------------------------------------------
 
 class Appearance : public AppearanceDialogBase {
-      MusE *seq;
-      Arranger *arr;
+      Arranger* arr;
       QString currentBg;
       QColor color[16];
+      QFont lfont[6];
 
       Q_OBJECT
+      void updateFonts();
 
    private slots:
       void apply();
@@ -26,9 +28,16 @@ class Appearance : public AppearanceDialogBase {
       void configBackground();
       void configColor();
       void colorItemSelectionChanged();
+      void browseFont(int);
+      void browseFont0();
+      void browseFont1();
+      void browseFont2();
+      void browseFont3();
+      void browseFont4();
+      void browseFont5();
 
    public:
-      Appearance(MusE*, Arranger*, QWidget* parent=0, const char* name=0);
+      Appearance(Arranger*, QWidget* parent=0, const char* name=0);
       };
 
 #endif
