@@ -1,12 +1,11 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: pcanvas.cpp,v 1.1 2002/01/30 14:54:03 muse Exp $
+//    $Id: pcanvas.cpp,v 1.5 2004/01/05 20:22:04 spamatica Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
 #include <stdio.h>
-#include <math.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <errno.h>
@@ -20,24 +19,57 @@
 #include <qdragobject.h>
 #include <qpopupmenu.h>
 #include <qurl.h>
+#include <qmenudata.h>
 
+#include "widgets/tools.h"
 #include "midithread.h"
 #include "pcanvas.h"
 #include "midieditor.h"
 #include "globals.h"
 #include "icons.h"
 #include "event.h"
-#include "../sf/sndfile.h"
 #include "xml.h"
+#include "wave.h"
+#include "audio.h"
+
+//---------------------------------------------------------
+//   ColorListItem
+//---------------------------------------------------------
+
+class ColorListItem : public QCustomMenuItem {
+      int r, g, b;
+      int h;
+      int fontheight;
+      QString label;
+      virtual QSize sizeHint() { return QSize(80, h); }
+      virtual void paint(QPainter* p, const QColorGroup&, bool /*act*/, bool /*enabled*/, int x, int y, int /*w*/, int h)
+            {
+            p->fillRect(x+5, y+2, h-4, h-4, QBrush(QColor(r,g,b)));
+            p->drawText(x+5 + h - 4 + 3, y+(fontheight * 3) / 4, label);
+            }
+
+   public:
+      ColorListItem(int _r, int _g, int _b, int _h,  int _fh, const char* txt) {
+            r = _r;
+            g = _g;
+            b = _b;
+            h = _h;
+            label = txt;
+	      fontheight = _fh;
+            }
+      QString text() const { return QString("PartColor"); }
+      };
 
 //---------------------------------------------------------
 //   NPart
 //---------------------------------------------------------
 
-NPart::NPart(Part* e, int index) : CItem((Event*)0, e)
+NPart::NPart(Part* e) : CItem((Event*)0, e)
       {
-      setPos(QPoint(e->posTick(), index*TH+1));
-      setBBox(QRect(e->posTick(), index*TH+1, e->lenTick(), TH));
+      int th = track()->height();
+      int y  = track()->y();
+      setPos(QPoint(e->posTick(), y + 1));
+      setBBox(QRect(e->posTick(), y + 1, e->lenTick(), th));
       }
 
 //---------------------------------------------------------
@@ -47,11 +79,12 @@ NPart::NPart(Part* e, int index) : CItem((Event*)0, e)
 PartCanvas::PartCanvas(int* r, QWidget* parent, int sx, int sy)
    : Canvas(parent, sx, sy)
       {
-      setBg(white);
+//      setBg(QColor(192, 208, 255));
+	setBg(QColor(98,124,168));
 
       setAcceptDrops(true);
       _raster = r;
-      _showPartType  = 1;       // 1 - names, 2 events, 3 cakewalk style
+      _showPartType  = 1;      // 1 - names + (2 events or 4 cakewalk style)
       _showPartEvent = 5;      // noteOn + controller
 
       setFocusPolicy(StrongFocus);
@@ -66,6 +99,41 @@ PartCanvas::PartCanvas(int* r, QWidget* parent, int sx, int sy)
       partsChanged();
       }
 
+//---------------------------------------------------------
+//   y2pitch
+//---------------------------------------------------------
+
+int PartCanvas::y2pitch(int y) const
+      {
+      TrackList* tl = song->tracks();
+      int yy  = 0;
+      int idx = 0;
+      for (iTrack it = tl->begin(); it != tl->end(); ++it, ++idx) {
+            int h = (*it)->height();
+            if ((y >= yy) && (y < yy+h))
+                  break;
+            yy += h;
+            }
+      return idx;
+      }
+
+//---------------------------------------------------------
+//   pitch2y
+//---------------------------------------------------------
+
+int PartCanvas::pitch2y(int p) const
+      {
+      TrackList* tl = song->tracks();
+      int yy = 0;
+      int idx = 0;
+      for (iTrack it = tl->begin(); it != tl->end(); ++it, ++idx) {
+            if (idx == p)
+                  break;
+            yy += (*it)->height();
+            }
+      return yy;
+      }
+
 //---------------------------------------------------------
 //   leaveEvent
 //---------------------------------------------------------
@@ -108,7 +176,7 @@ void PartCanvas::viewMouseDoubleClickEvent(QMouseEvent* event)
                   if (lineEditor == 0) {
                         lineEditor = new QLineEdit(this);
                         connect(lineEditor, SIGNAL(returnPressed()),
-                        SLOT(returnPressed()));
+                           SLOT(returnPressed()));
                         lineEditor->setFrame(true);
                         }
                   lineEditor->setText(editPart->name());
@@ -123,13 +191,22 @@ void PartCanvas::viewMouseDoubleClickEvent(QMouseEvent* event)
                   }
             }
       //
-      // doppelclick erzeugt part zwischen linker
-      // und rechter marke
+      // double click creates new part between left and
+      // right mark
 
       else {
-            unsigned trackNo = event->y()/TH;
-            if (pos[2] - pos[1] > 0 && trackNo < tracks->size()) {
-                  Track* track = tracks->findIdx(trackNo);
+            TrackList* tl = song->tracks();
+            iTrack it;
+            int yy = 0;
+            int y = event->y();
+            for (it = tl->begin(); it != tl->end(); ++it) {
+                  int h = (*it)->height();
+                  if (y >= yy && y < (yy + h))
+                        break;
+                  yy += h;
+                  }
+            if (pos[2] - pos[1] > 0 && it != tl->end()) {
+                  Track* track = *it;
                   switch(track->type()) {
                         case Track::MIDI:
                         case Track::DRUM:
@@ -137,8 +214,8 @@ void PartCanvas::viewMouseDoubleClickEvent(QMouseEvent* event)
                               MidiPart* part = new MidiPart((MidiTrack*)track);
                               part->setPosTick(pos[1]);
                               part->setLenTick(pos[2]-pos[1]);
-                              part->setName(track->tname());
-                              NPart* np = new NPart(part, trackNo);
+                              part->setName(track->name());
+                              NPart* np = new NPart(part);
                               items.add(np);
                               deselectAll();
                               part->setSelected(true);
@@ -156,7 +233,7 @@ void PartCanvas::viewMouseDoubleClickEvent(QMouseEvent* event)
 //   startUndo
 //---------------------------------------------------------
 
-void PartCanvas::startUndo(bool)
+void PartCanvas::startUndo(DragType)
       {
       song->startUndo();
       }
@@ -165,9 +242,10 @@ void PartCanvas::startUndo(bool)
 //   endUndo
 //---------------------------------------------------------
 
-void PartCanvas::endUndo(bool flag)
+void PartCanvas::endUndo(DragType t)
       {
-      song->endUndo(flag ? SC_PART_INSERTED : SC_PART_MODIFIED);
+      song->endUndo((t == MOVE_COPY || t == MOVE_CLONE)
+         ? SC_PART_INSERTED : SC_PART_MODIFIED);
       }
 
 //---------------------------------------------------------
@@ -175,13 +253,13 @@ void PartCanvas::endUndo(bool flag)
 //    return false, if copy/move not allowed
 //---------------------------------------------------------
 
-bool PartCanvas::moveItem(CItem* item, const QPoint& newpos, bool copy)
+bool PartCanvas::moveItem(CItem* item, const QPoint& newpos, DragType t)
       {
       NPart* npart    = (NPart*) item;
       Part* spart     = npart->part();
       Track* track    = npart->track();
       int dtick       = newpos.x();
-      unsigned ntrack = (item->mp().y() + item->height()/2) / TH;
+      unsigned ntrack = y2pitch(item->mp().y());
       Track::TrackType type = track->type();
 
       if (tracks->index(track) == ntrack && (dtick == spart->posTick())) {
@@ -195,32 +273,40 @@ bool PartCanvas::moveItem(CItem* item, const QPoint& newpos, bool copy)
       Track* dtrack = tracks->findIdx(ntrack);
 
       if (dtrack->type() != type) {
-            QMessageBox::critical(this, "MusE",
-               tr("Cannot copy/move to different Track-Type"));
+            QMessageBox::critical(this, QString("MusE"),
+               tr("Cannot copy/move/clone to different Track-Type"));
             return false;
             }
 
-      int delta   = dtick - spart->posTick();
-      Part* dpart = dtrack->newPart(spart);
-
+      Part* dpart;
+      bool clone = (t == MOVE_CLONE) || (spart->events()->arefCount() > 1);
+      dpart = dtrack->newPart(spart, clone);
       dpart->setPosTick(dtick);
 
-      EventList* se = spart->events();
-      EventList* de = dpart->events();
-      for (iEvent i = se->begin(); i != se->end(); ++i) {
-            Event* oldEvent = i->second;
-            Event* ev = oldEvent->clone();
-            ev->setPort(dtrack->outPort());
-            ev->setChannel(dtrack->outChannel());
-            ev->setPosTick(ev->posTick() + delta);
-            de->add(ev);
+      if (!clone) {
+            //
+            // Copy Events
+            //
+            EventList* se = spart->events();
+            EventList* de = dpart->events();
+            for (iEvent i = se->begin(); i != se->end(); ++i) {
+                  Event* oldEvent = i->second;
+                  Event* ev = oldEvent->clone();
+                  de->add(ev);
+                  }
             }
-      if (copy) {
-            midiThread->msgAddPart(dpart, false);
+      if (t == MOVE_COPY || t == MOVE_CLONE) {
+            if (dtrack->type() == Track::WAVE)
+                  audio->msgAddPart((WavePart*)dpart);
+            else
+                  midiThread->msgAddPart(dpart);
             }
-      else {
+      else if (t == MOVE_MOVE) {
             dpart->setSelected(spart->selected());
-            midiThread->msgChangePart(spart, dpart, false);
+            if (dtrack->type() == Track::WAVE)
+                  audio->msgChangePart((WavePart*)spart, (WavePart*)dpart);
+            else
+                  midiThread->msgChangePart(spart, dpart, false);
             }
       if (song->len() < (dpart->lenTick() + dpart->posTick()))
             song->setLen(dpart->lenTick() + dpart->posTick());
@@ -233,11 +319,11 @@ bool PartCanvas::moveItem(CItem* item, const QPoint& newpos, bool copy)
 
 QPoint PartCanvas::raster(const QPoint& p) const
       {
-      int track = ((p.y()+TH/2)/TH)*TH;
+      int y = pitch2y(y2pitch(p.y()));
       int x = sigmap.raster(p.x(), *_raster);
       if (x < 0)
             x = 0;
-      return QPoint(x, track);
+      return QPoint(x, y);
       }
 
 //---------------------------------------------------------
@@ -251,7 +337,7 @@ void PartCanvas::partsChanged()
       for (iTrack t = tracks->begin(); t != tracks->end(); ++t) {
             PartList* pl = (*t)->parts();
             for (iPart i = pl->begin(); i != pl->end(); ++i) {
-                  NPart* np = new NPart(i->second, idx);
+                  NPart* np = new NPart(i->second);
                   items.add(np);
                   if (i->second->selected()) {
                         selectItem(np, true);
@@ -300,11 +386,11 @@ void PartCanvas::resizeItem(CItem* i, bool noSnap)
 CItem* PartCanvas::newItem(const QPoint& pos, int)
       {
       int x = sigmap.raster(pos.x(), *_raster);
-      unsigned trackIndex = pos.y()/TH;
+      unsigned trackIndex = y2pitch(pos.y());
       if (trackIndex >= tracks->size())
             return 0;
       Track* track = tracks->findIdx(trackIndex);
-      Part* pa;
+      Part* pa  = 0;
       NPart* np = 0;
       switch(track->type()) {
             case Track::MIDI:
@@ -319,9 +405,9 @@ CItem* PartCanvas::newItem(const QPoint& pos, int)
                   pa->setLenTick(0);
                   break;
             }
-      pa->setName(track->tname());
+      pa->setName(track->name());
       pa->setColorIndex(curColorIndex);
-      np = new NPart(pa, trackIndex);
+      np = new NPart(pa);
       return np;
       }
 
@@ -354,15 +440,6 @@ bool PartCanvas::deleteItem(CItem* i)
       return true;
       }
 
-//---------------------------------------------------------
-//   setCurColorIndex
-//---------------------------------------------------------
-
-void PartCanvas::setCurColorIndex(int index)
-      {
-      curColorIndex = index;
-      }
-
 //---------------------------------------------------------
 //   splitItem
 //---------------------------------------------------------
@@ -406,9 +483,23 @@ QPopupMenu* PartCanvas::genItemPopup(CItem* item)
 
       partPopup->insertSeparator();
       partPopup->insertItem(tr("rename"), 0);
+      QPopupMenu* colorPopup = new QPopupMenu(this);
+      partPopup->insertItem(tr("color"), colorPopup);
+
+      // part color selection
+      const QFontMetrics& fm = colorPopup->fontMetrics();
+      int h = fm.lineSpacing();
+
+      PartColor* p = partColors;
+      for (int i = 0; i < NUM_PARTCOLORS; ++i, ++p) {
+            ColorListItem* item = new ColorListItem(p->r, p->g, p->b, h, fontMetrics().height(), p->name);
+            colorPopup->insertItem(item, 20+i);
+            }
+
       partPopup->insertItem(*deleteIcon, tr("delete"), 1);
       partPopup->insertItem(*cutIcon, tr("split"),  2);
       partPopup->insertItem(*glueIcon, tr("glue"),   3);
+      partPopup->insertItem(tr("de-clone"), 15);
 
       partPopup->insertSeparator();
       switch(trackType) {
@@ -428,6 +519,8 @@ QPopupMenu* PartCanvas::genItemPopup(CItem* item)
 
       partPopup->setItemEnabled(1, true);
       partPopup->setItemEnabled(4, true);
+      bool isClone = npart->part()->events()->arefCount() > 1;
+      partPopup->setItemEnabled(15, isClone);
       return partPopup;
       }
 
@@ -442,7 +535,20 @@ void PartCanvas::itemPopup(CItem* item, int n, const QPoint& pt)
       pl->add(npart->part());
       switch(n) {
             case 0:     // rename
-                  printf("not implemented\n");
+                  {
+                  editPart = npart;
+                  QRect r = map(curItem->bbox());
+                  if (lineEditor == 0) {
+                        lineEditor = new QLineEdit(this);
+                        connect(lineEditor, SIGNAL(returnPressed()),
+                        SLOT(returnPressed()));
+                        lineEditor->setFrame(true);
+                        }
+                  lineEditor->setText(editPart->name());
+                  lineEditor->setFocus();
+                  lineEditor->show();
+                  lineEditor->setGeometry(r);
+                  }
                   break;
             case 1:     // delete
                   deleteItem(item);
@@ -483,6 +589,32 @@ void PartCanvas::itemPopup(CItem* item, int n, const QPoint& pt)
             case 14:
                   emit startEditor(pl, 4);
                   return;
+            case 15:    // declone
+                  {
+                  Part* spart  = npart->part();
+                  Track* track = npart->track();
+                  Part* dpart  = track->newPart(spart, false);
+
+                  EventList* se = spart->events();
+                  EventList* de = dpart->events();
+                  for (iEvent i = se->begin(); i != se->end(); ++i) {
+                        Event* oldEvent = i->second;
+                        Event* ev = oldEvent->clone();
+                        de->add(ev);
+                        }
+                  song->startUndo();
+                  midiThread->msgChangePart(spart, dpart, false);
+                  song->endUndo(SC_PART_MODIFIED);
+                  }
+            case 20 ... NUM_PARTCOLORS+20:
+                  curColorIndex = n - 20;
+                  npart->part()->setColorIndex(curColorIndex);
+                  redraw();
+                  break;
+
+            default:
+                  printf("unknown action %d\n", n);
+                  break;
             }
       delete pl;
       }
@@ -493,6 +625,8 @@ void PartCanvas::itemPopup(CItem* item, int n, const QPoint& pt)
 
 void PartCanvas::mousePress(QMouseEvent* event)
       {
+      if (event->state() & ShiftButton)
+            return;
       QPoint pt = event->pos();
       CItem* item = items.find(pt);
       if (item == 0)
@@ -505,6 +639,14 @@ void PartCanvas::mousePress(QMouseEvent* event)
             case GlueTool:
                   glueItem(item);
                   break;
+            case MuteTool:
+                  {
+                  NPart* np = (NPart*) item;
+                  Part*  p = np->part();
+                  p->setMute(!p->mute());
+                  redraw();
+                  break;
+                  }
             }
       }
 
@@ -526,6 +668,23 @@ void PartCanvas::mouseMove(const QPoint& pos)
       emit timeChanged(sigmap.raster(pos.x(), *_raster));
       }
 
+//---------------------------------------------------------
+//   y2Track
+//---------------------------------------------------------
+
+Track* PartCanvas::y2Track(int y) const
+      {
+      TrackList* l = song->tracks();
+      int ty = 0;
+      for (iTrack it = l->begin(); it != l->end(); ++it) {
+            int h = (*it)->height();
+            if (y >= ty && y < ty + h)
+                  return *it;
+            ty += h;
+            }
+      return 0;
+      }
+
 //---------------------------------------------------------
 //   keyPress
 //---------------------------------------------------------
@@ -533,6 +692,12 @@ void PartCanvas::mouseMove(const QPoint& pos)
 void PartCanvas::keyPress(QKeyEvent* event)
       {
       int key = event->key();
+      if (key == Key_Delete) {
+            song->startUndo();
+            song->msgRemoveParts();
+            song->endUndo(SC_PART_REMOVED);
+            return;
+            }
       if (!isSingleSelection()) {
             event->ignore();  // give global accelerators a chance
             return;
@@ -544,14 +709,41 @@ void PartCanvas::keyPress(QKeyEvent* event)
                   break;
                   }
             }
-      Part* part = ((NPart*)item)->part();
-      int tick = part->posTick();
+      Part* part   = ((NPart*)item)->part();
+      int tick     = part->posTick();
       Track* track = part->track();
 
       CItem* newItem = 0;
       Part* newPart = 0;
 
+      bool alt = event->state() & AltButton;
+
       switch(key) {
+            case Key_Return:
+                  if (!item)
+                        break;
+                  else {
+                        PartList* pl = new PartList;
+                        NPart* npart = (NPart*)(item);
+                        pl->add(npart->part());
+                        int type = 0;
+                        switch (track->type()) { //Check if track is wave or drum, else track is midi
+                              case Track::DRUM:
+                              type = 3;
+                              break;
+
+                              case Track::WAVE:
+                              type = 4;
+                              break;
+
+                              default:
+                              break;
+                        }
+
+                        emit startEditor(pl, type);
+                        }
+                  break;
+
             case Key_Right:
                   for (iCItem i = items.begin(); i != items.end(); ++i) {
                         NPart* npart = (NPart*)(i->second);
@@ -597,6 +789,103 @@ void PartCanvas::keyPress(QKeyEvent* event)
                               }
                         }
                   break;
+
+            case Key_P:
+                  if (alt) {
+                        song->setPos(1,part->posTick());
+                        song->setPos(2,part->posTick() + part->lenTick());
+                        }
+                  break;
+
+            case Key_Up:
+                  if(!item)
+                        break;
+                  else {
+                        //To get an idea of which track is above us:
+                        int stepsize = rmapxDev(1);
+                        track = y2Track(track->y() - 1);
+                        if (track == 0)
+                              break;
+
+                        CItem *aboveL = 0, *aboveR = 0;
+                        //Upper limit: song end , lower limit: song start
+                        int ulimit  = song->len();
+                        int llimit = 0;
+                        int middle = (item->x() + item->width()/2);
+                        while (newItem == 0) {
+                              int y = track->y() + 1;
+                              int xoffset = 0;
+                              int xleft   = middle - xoffset;
+                              int xright  = middle + xoffset;
+                              while ((xleft > llimit || xright < ulimit)  && (aboveL == 0) && (aboveR == 0)) {
+                                    xoffset += stepsize;
+                                    xleft  = middle - xoffset;
+                                    xright = middle + xoffset;
+                                    if (xleft >= 0)
+                                          aboveL = items.find(QPoint(xleft,y));
+                                    if (xright <= ulimit)
+                                          aboveR = items.find(QPoint(xright,y));
+                                    }
+
+                              if ((aboveL || aboveR) != 0) { //We've hit something
+                                    CItem* above  = 0;
+                                    above = (aboveL !=0) ? aboveL : aboveR;
+                                    newItem = above;
+                                    }
+                              else { //We didn't hit anything. Move to track above, if there is one
+                                    track = y2Track(track->y() - 1);
+                                    if (track == 0)
+                                          break;
+                                    }
+                              }
+                        }
+                  break;
+
+            case Key_Down:
+                  if(!item)
+                        break;
+                  else {
+                        //To get an idea of which track is below us:
+                        int stepsize = rmapxDev(1);
+                        track = y2Track(track->y() + track->height() + 1 );
+                        if (track == 0)
+                              break;
+
+                        CItem *belowL = 0, *belowR = 0;
+                        //Upper limit: song end , lower limit: song start
+                        int ulimit = song->len();
+                        int llimit = 0;
+                        int middle = (item->x() + item->width()/2);
+                        while (newItem == 0) {
+                              int y = track->y() + 1;
+                              int xoffset = 0;
+                              int xleft   = middle - xoffset;
+                              int xright  = middle + xoffset;
+                              while ((xleft > llimit || xright < ulimit)  && (belowL == 0) && (belowR == 0)) {
+                                    xoffset += stepsize;
+                                    xleft  = middle - xoffset;
+                                    xright = middle + xoffset;
+                                    if (xleft >= 0)
+                                          belowL = items.find(QPoint(xleft,y));
+                                    if (xright <= ulimit)
+                                          belowR = items.find(QPoint(xright,y));
+                                    }
+
+                              if ((belowL || belowR) != 0) { //We've hit something
+                                    CItem* below = 0;
+                                    below = (belowL !=0) ? belowL : belowR;
+                                    newItem = below;
+                                    }
+                              else {
+                                    //Get next track below, or abort if this is the lowest
+                                    track = y2Track(track->y() + track->height() + 1 );
+                                    if (track == 0)
+                                          break;
+                                    }
+                              } // end while
+                        } // end else
+                  break;
+
             default:
                   event->ignore();  // give global accelerators a chance
                   break;
@@ -604,6 +893,18 @@ void PartCanvas::keyPress(QKeyEvent* event)
       if (newItem) {
             selectItem(item, false);
             selectItem(newItem, true);
+            //Check if we've hit the upper or lower boundaries of the window. If so, set a new position
+            if (newItem->x() < mapxDev(0)) {
+                  int curpos = pos[0];
+                  setPos(0,newItem->x(),true);
+                  setPos(0,curpos,false); //Dummy to put the current position back once we've scrolled
+                  }
+            else if (newItem->x() > mapxDev(width())) {
+                  int curpos = pos[0];
+                  setPos(0,newItem->x(),true);
+                  setPos(0,curpos,false); //Dummy to put the current position back once we've scrolled
+                  }
+            redraw();
             }
       }
 
@@ -614,26 +915,38 @@ void PartCanvas::keyPress(QKeyEvent* event)
 
 void PartCanvas::drawItem(QPainter& p, const CItem* item, const QRect& rect) const
       {
-      int from = rect.x();
-      int to   = from + rect.width();
+      int from   = rect.x();
+      int to     = from + rect.width();
 
       Part* part = ((NPart*)item)->part();
+      int pTick  = part->posTick();
+      from      -= pTick;
+      to        -= pTick;
+
       QRect r    = item->bbox();
       int i      = part->colorIndex();
 
       p.setPen(black);
+      if (part->mute()) {
+            p.setBrush(gray);
+            p.drawRect(r);
+            }
       if (item->isMoving()) {
             p.setBrush(gray);
             p.drawRect(r);
             p.setBrush(NoBrush);
             p.drawRect(item->mp().x(), item->mp().y()+1, item->width(), item->height());
             }
+      else if (part->mute())
+            return;
       else if (part->selected()) {
             p.setBrush(black);
             p.setPen(QColor(partColors[i].r, partColors[i].g, partColors[i].b));
             p.drawRect(r);
             }
       else {
+            bool clone = part->events()->arefCount() > 1;
+            p.setPen(QPen(black, 2, clone ? DashLine : SolidLine));
             p.setBrush(QColor(partColors[i].r, partColors[i].g, partColors[i].b));
             p.drawRect(r);
             }
@@ -651,17 +964,7 @@ void PartCanvas::drawItem(QPainter& p, const CItem* item, const QRect& rect) con
             waveType = false;
             }
 
-      if (_showPartType == 1) {     // show names
-            // draw name
-            QRect rr = map(r);
-            rr.setX(rr.x() + 3);
-            p.save();
-            p.setFont(font1);
-            p.setWorldXForm(false);
-            p.drawText(rr, AlignVCenter|AlignLeft, part->name());
-            p.restore();
-            }
-      else if (_showPartType == 2) {      // show events
+      if (_showPartType & 2) {      // show events
             if (mp) {
                   p.setPen(darkGray);
                   EventList* events = mp->events();
@@ -680,38 +983,63 @@ void PartCanvas::drawItem(QPainter& p, const CItem* item, const QRect& rect) con
                            || ((_showPartEvent &32) && (type == MidiEvent::Pitch))
                            || ((_showPartEvent &64) && (type == MidiEvent::Sysex || type == MidiEvent::Meta))
                            ) {
-                              int t = i->first;
-                              p.drawLine(t, r.y()+2, t, r.y()+TH-4);
+                              int t = i->first + pTick;
+                              int th = part->track()->height();
+                              p.drawLine(t, r.y()+2, t, r.y()+th-4);
                               }
                         }
-                  return;
                   }
-            if (wp)
+            else if (wp)
                   drawWavePart(p, rect, wp, r);
             }
 
-      else if (_showPartType == 3) {      // show Cakewalk Style
+      else {      // show Cakewalk Style
             if (mp) {
                   p.setPen(darkGray);
                   EventList* events = mp->events();
                   iEvent ito(events->lower_bound(to));
                   for (iEvent i = events->begin(); i != ito; ++i) {
-                        int t = i->first;
-                        if ((t + i->second->lenTick()) < from)
+                        int t  = i->first + pTick;
+                        int te = t + i->second->lenTick();
+
+                        if (t > (to + pTick))
+                              break;
+
+                        if (te < (from + pTick))
                               continue;
-                         MidiEvent::EventType type = ((MidiEvent*)i->second)->type();
-                         if (type == MidiEvent::Note) {
-                              int note=(127-((MidiEvent*)i->second)->pitch())*(TH-6)/127;
-                              int length=((MidiEvent*)i->second)->lenTick();
-                              p.drawLine(t, r.y()+note, t+length, r.y()+note);
+
+                        if (te > (to + pTick))
+                              te = to + pTick;
+
+                        MidiEvent::EventType type = ((MidiEvent*)i->second)->type();
+                        if (type == MidiEvent::Note) {
+                              int pitch = ((MidiEvent*)i->second)->pitch();
+                              int th = part->track()->height();
+                              int y     = r.y() + th - (pitch * (th-6) / 127);
+                              p.drawLine(t, y, te, y);
                               }
                         }
-                  return;
                   }
-
-            if (wp)
+            else if (wp)
                   drawWavePart(p, rect, wp, r);
             }
+      if (_showPartType & 1) {     // show names
+            // draw name
+	      // FN: Set text color depending on part color (black / white)
+		int brightness;
+		brightness =  partColors[i].r*29 + partColors[i].g*59 + partColors[i].b*12;
+		if (brightness < 12000 || part->selected())
+		      p.setPen(white);   /* too dark: use white for text color */
+		else
+			p.setPen(black);  /* otherwise use black */
+            QRect rr = map(r);
+            rr.setX(rr.x() + 3);
+            p.save();
+            p.setFont(font1);
+            p.setWorldXForm(false);
+            p.drawText(rr, AlignTop|AlignLeft, part->name());
+            p.restore();
+            }
       }
 
 //---------------------------------------------------------
@@ -740,7 +1068,6 @@ void PartCanvas::drawWavePart(QPainter& p,
       int hh = pr.height();
       int h  = hh/2;
       int y  = pr.y() + h;
-//      int w  = pr.width();
 
       EventList* el = wp->events();
       for (iEvent e = el->begin(); e != el->end(); ++e) {
@@ -749,7 +1076,7 @@ void PartCanvas::drawWavePart(QPainter& p,
             Clip* clip        = event->clip();
             if (!clip)
                   continue;
-            const SndFile* f  = clip->file();
+            const SndFile* f  = clip->file1();
             if (!f)
                   continue;
             int samples = event->lenSample();
@@ -786,9 +1113,9 @@ void PartCanvas::drawWavePart(QPainter& p,
                   //  multi channel display
                   //
                   h = hh / (f->channels() * 2);
-                  int cc = hh%(f->channels() * 2) ? 0 : 1;
+                  int cc = hh % (f->channels() * 2) ? 0 : 1;
                   for (int i = x1; i < x2; i++) {
-                        y  = rr.y() + h;
+                        y  = pr.y() + h;
                         SampleV sa[f->channels()];
                         const_cast<SndFile*>(f)->read(sa, xScale, pos);
                         pos += xScale;
@@ -851,6 +1178,8 @@ void PartCanvas::copy(PartList* pl)
       {
       if (pl->empty())
             return;
+      bool isWave = pl->begin()->second->track()->type() == Track::WAVE;
+
       //---------------------------------------------------
       //    write parts as XML into tmp file
       //---------------------------------------------------
@@ -864,10 +1193,14 @@ void PartCanvas::copy(PartList* pl)
       Xml xml(tmp);
 
       int level = 0;
-      int offset = -(pl->begin()->second->posTick());
+      int tick = 0;
       for (ciPart p = pl->begin(); p != pl->end(); ++p) {
-            p->second->write(level, xml, offset);
+            p->second->write(level, xml);
+            int endTick = p->second->endTick();
+            if (endTick > tick)
+                  tick = endTick;
             }
+      song->setPos(0, tick);
 
       //---------------------------------------------------
       //    read tmp file into QTextDrag Object
@@ -885,9 +1218,9 @@ void PartCanvas::copy(PartList* pl)
       char* fbuf  = (char*)mmap(0, n+1, PROT_READ|PROT_WRITE,
          MAP_PRIVATE, fileno(tmp), 0);
       fbuf[n] = 0;
-      QTextDrag* drag = new QTextDrag(fbuf);
-      drag->setSubtype("partlist");
-      QApplication::clipboard()->setData(drag);
+      QTextDrag* drag = new QTextDrag(QString(fbuf));
+      drag->setSubtype(QCString(isWave ? "wavepartlist" : "midipartlist"));
+      QApplication::clipboard()->setData(drag, QClipboard::Clipboard);
       munmap(fbuf, n);
       fclose(tmp);
       }
@@ -917,10 +1250,11 @@ int PartCanvas::pasteAt(const QString& pt, Track* track, int pos)
                                     p = new WavePart((WaveTrack*)track);
                               else
                                     break;
-                              p->read(xml, pos);
+                              p->read(xml);
                               p->setPosTick(pos);
                               midiThread->msgAddPart(p);
                               pos += p->lenTick();
+                              return pos;
                               }
                         else
                               xml.unknown("Arranger::paste");
@@ -944,7 +1278,8 @@ void PartCanvas::paste()
       for (iTrack i = tl->begin(); i != tl->end(); ++i) {
             if ((*i)->selected()) {
                   if (track) {
-                        printf("cannot paste: multiple tracks selected\n");
+                        QMessageBox::critical(this, QString("MusE"),
+                          tr("Cannot paste: multiple tracks selected"));
                         return;
                         }
                   else
@@ -952,25 +1287,55 @@ void PartCanvas::paste()
                   }
             }
       if (track == 0) {
-            printf("cannot paste: no track selected?\n");
+            QMessageBox::critical(this, QString("MusE"),
+              tr("Cannot paste: no track selected"));
             return;
             }
-      QCString subtype("partlist");
-      QMimeSource* ms = QApplication::clipboard()->data();
-      QString pt;
-      if (!QTextDrag::decode(ms, pt, subtype)) {
-            printf("cannot paste: bad data type\n");
+
+      QClipboard* cb  = QApplication::clipboard();
+      QMimeSource* ms = cb->data(QClipboard::Clipboard);
+
+      bool midiPart = false;
+      bool wavePart = false;
+
+      for (int i = 0; const char* format = ms->format(i); ++i) {
+            format = ms->format(i);
+            if (strcmp(format, "text/midipartlist") == 0) {
+                  if (track->type() == Track::WAVE) {
+                        QMessageBox::critical(this, QString("MusE"),
+                              tr("Cannot paste midi parts to wave track"));
+                        return;
+                        }
+                  midiPart = true;
+                  }
+            else if (strcmp(format, "text/wavepartlist") == 0) {
+                  if (track->type() != Track::WAVE) {
+                        QMessageBox::critical(this, QString("MusE"),
+                        tr("Cannot paste wave parts to midi track"));
+                        return;
+                        }
+                  wavePart = true;
+                  }
+            }
+
+      if (!(midiPart || wavePart)) {
+            QMessageBox::critical(this, QString("MusE"),
+               tr("Cannot paste: wrong data type"));
             return;
             }
-      int tick = pasteAt(pt, track, song->cpos());
-      song->setPos(0, tick);
+      QCString subtype = 0;
+      QString txt = cb->text(subtype);
+      if (!txt.isEmpty()) {
+            int tick = pasteAt(txt, track, song->cpos());
+            song->setPos(0, tick);
+            }
       }
 
 //---------------------------------------------------------
 //   startDrag
 //---------------------------------------------------------
 
-void PartCanvas::startDrag(CItem* item, bool copymode)
+void PartCanvas::startDrag(CItem* item, DragType t)
       {
       NPart* p = (NPart*)(item);
       Part* part = p->part();
@@ -987,8 +1352,7 @@ void PartCanvas::startDrag(CItem* item, bool copymode)
             }
       Xml xml(tmp);
       int level = 0;
-      int offset = -(part->posTick());
-      part->write(level, xml, offset);
+      part->write(level, xml);
 
       //---------------------------------------------------
       //    read tmp file into QTextDrag Object
@@ -1006,9 +1370,9 @@ void PartCanvas::startDrag(CItem* item, bool copymode)
       char* fbuf  = (char*)mmap(0, n, PROT_READ|PROT_WRITE,
          MAP_PRIVATE, fileno(tmp), 0);
       fbuf[n] = 0;
-      QTextDrag* drag = new QTextDrag(fbuf, this);
+      QTextDrag* drag = new QTextDrag(QString(fbuf), this);
       drag->setSubtype("partlist");
-      if (copymode)
+      if (t == MOVE_COPY || t == MOVE_CLONE)
             drag->dragCopy();
       else
             drag->dragMove();
@@ -1078,7 +1442,7 @@ void PartCanvas::viewDropEvent(QDropEvent* event)
                   int x = sigmap.raster(event->pos().x(), *_raster);
                   if (x < 0)
                         x = 0;
-                  unsigned trackNo = event->pos().y()/TH;
+                  unsigned trackNo = y2pitch(event->pos().y());
                   Track* track = 0;
                   if (trackNo < tracks->size())
                         track = tracks->findIdx(trackNo);
@@ -1145,8 +1509,14 @@ void PartCanvas::drawCanvas(QPainter& p, const QRect& rect)
       // horizontal lines
       //--------------------------------
 
-      for (int n = (y/TH)*TH; n < (y + h); n += TH)
-	      p.drawLine(x, n, x + w, n);
+      TrackList* tl = song->tracks();
+      int yy = 0;
+      for (iTrack it = tl->begin(); it != tl->end(); ++it) {
+            if (yy > y+h)
+                  break;
+	      p.drawLine(x, yy, x + w, yy);
+            yy += (*it)->height();
+            }
 
       //--------------------------------
       // vertical lines
