@@ -1,27 +1,35 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: tlist.cpp,v 1.2 2002/02/13 11:42:56 muse Exp $
+//    $Id: tlist.cpp,v 1.3 2004/01/05 20:03:57 spamatica Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
+#include <qfileinfo.h>
+#include <qpainter.h>
 #include <qlineedit.h>
 #include <qpopupmenu.h>
 #include <qmessagebox.h>
+#include <qscrollbar.h>
+#include <cmath>
 
+#include "globals.h"
 #include "icons.h"
-#include "seq.h"
 #include "scrollscale.h"
 #include "tlist.h"
 #include "xml.h"
-#include "device.h"
+#include "driver/mididev.h"
 #include "midiport.h"
-#include "audioport.h"
 #include "comment.h"
 #include "track.h"
 #include "song.h"
 #include "header.h"
 #include "midithread.h"
+#include "node.h"
+#include "audio.h"
+#include "app.h"
+
+static const int MIN_TRACKHEIGHT = 20;
 
 //---------------------------------------------------------
 //   THeaderTip::maybeTip
@@ -36,15 +44,16 @@ void THeaderTip::maybeTip(const QPoint &pos)
       QRect r(w->sectionPos(section), 0, w->sectionSize(section),
          w->height());
       QString p;
-      switch(section) {
-            case COL_RECORD:   p = w->tr("Enable Recording"); break;
-            case COL_ACTIVITY: p = w->tr("Track Activity"); break;
-            case COL_MUTE:     p = w->tr("Mute Indicator"); break;
-            case COL_CLASS:    p = w->tr("Track Type"); break;
-            case COL_NAME:     p = w->tr("Track Name"); break;
-            case COL_OCHANNEL: p = w->tr("Output Channel Number"); break;
-            case COL_OPORT:    p = w->tr("Output Port"); break;
-            case COL_TIMELOCK: p = w->tr("Time Lock"); break;
+      switch (section) {
+            case COL_RECORD:   p = QHeader::tr("Enable Recording"); break;
+            case COL_ACTIVITY: p = QHeader::tr("Track Activity"); break;
+            case COL_MUTE:     p = QHeader::tr("Mute Indicator"); break;
+            case COL_SOLO:     p = QHeader::tr("Solo/Pre Fader Listening"); break;
+            case COL_CLASS:    p = QHeader::tr("Track Type"); break;
+            case COL_NAME:     p = QHeader::tr("Track Name"); break;
+            case COL_OCHANNEL: p = QHeader::tr("Output Channel Number"); break;
+            case COL_OPORT:    p = QHeader::tr("Output Port"); break;
+            case COL_TIMELOCK: p = QHeader::tr("Time Lock"); break;
             default: return;
             }
       tip(r, p);
@@ -54,20 +63,42 @@ void THeaderTip::maybeTip(const QPoint &pos)
 //   TList
 //---------------------------------------------------------
 
-TList::TList(Header* hdr, QWidget* parent, int ymag)
-   : View(parent, 1, ymag)
+TList::TList(Header* hdr, QWidget* parent, const char* name)
+   : QWidget(parent, name, WRepaintNoErase | WResizeNoErase)
       {
-      header    = hdr;
-      setBg(white);
+      ypos = 0;
       setFocusPolicy(QWidget::StrongFocus);
+      setMouseTracking(true);
+      header    = hdr;
+
       scroll    = 0;
       editTrack = 0;
       editor    = 0;
-      drag = NORMAL;
-      selectedTrackColor = yellow;
+      mode      = NORMAL;
       activityColor = black;
 
-      connect(song, SIGNAL(heartBeat()), SLOT(updateActivity()));
+      setBackgroundMode(NoBackground);
+      setBackgroundColor(QColor(198,224,168));
+
+      brush.setStyle(SolidPattern);
+      brush.setColor(palette().active().background());
+
+      connect(song, SIGNAL(heartBeat()),               SLOT(updateActivity()));
+      connect(song, SIGNAL(recordFlagChanged(Track*)), SLOT(redraw()));
+      connect(song, SIGNAL(songChanged(int)),          SLOT(songChanged(int)));
+      }
+
+//---------------------------------------------------------
+//   songChanged
+//---------------------------------------------------------
+
+void TList::songChanged(int flags)
+      {
+      if (flags & (SC_MUTE | SC_SOLO | SC_RECFLAG | SC_TRACK_INSERTED
+         | SC_TRACK_REMOVED | SC_TRACK_MODIFIED | SC_ROUTE))
+            redraw();
+      if (flags & (SC_TRACK_INSERTED | SC_TRACK_REMOVED | SC_TRACK_MODIFIED))
+            adjustScrollbar();
       }
 
 //---------------------------------------------------------
@@ -77,69 +108,103 @@ TList::TList(Header* hdr, QWidget* parent, int ymag)
 
 static void drawCenteredPixmap(QPainter& p, const QPixmap* pm, const QRect& r)
       {
-      p.save();
-      p.setWorldXForm(false);
       p.drawPixmap(r.x() + (r.width() - pm->width())/2, r.y() + (r.height() - pm->height())/2, *pm);
-      p.restore();
       }
 
 //---------------------------------------------------------
-//   drawCenteredText
+//   paintEvent
 //---------------------------------------------------------
 
-static void drawCenteredText(QPainter& p, const QString& s, const QRect& r)
+void TList::paintEvent(QPaintEvent* ev)
       {
-      p.save();
-      p.setWorldXForm(false);
-      p.drawText(r, Qt::AlignVCenter|Qt::AlignLeft, s);
-      p.restore();
+      if (!pmValid)
+            paint(ev->rect());
+      bitBlt(this, ev->rect().topLeft(), &pm, ev->rect(), CopyROP, true);
       }
 
 //---------------------------------------------------------
-//   draw
+//   redraw
 //---------------------------------------------------------
 
-void TList::draw(QPainter& p, const QRect&rect)
+void TList::redraw()
       {
-      int x = rect.x();
-      int y = rect.y();
-      int w = rect.width();
-      int h = rect.height();
+      paint(QRect(0, 0, pm.width(), pm.height()));
+      update();
+      }
+
+//---------------------------------------------------------
+//   redraw
+//---------------------------------------------------------
+
+void TList::redraw(const QRect& r)
+      {
+      paint(r);
+      update(r);
+      }
+
+//---------------------------------------------------------
+//   paint
+//---------------------------------------------------------
+
+void TList::paint(const QRect& r)
+      {
+      if (!isVisible())
+            return;
+      QRect rect(r);
+      if (!pmValid) {
+            pmValid = true;
+            rect = QRect(0, 0, pm.width(), pm.height());
+            }
+      QPainter p(&pm);
+
+      if (bgPixmap.isNull())
+            p.fillRect(rect, brush);
+      else
+            p.drawTiledPixmap(rect, bgPixmap, QPoint(rect.x(), ypos + rect.y()));
+      p.setClipRegion(rect);
+
+      int y  = rect.y();
+      int w  = rect.width();
+      int h  = rect.height();
+      int x1 = rect.x();
+      int x2 = rect.x() + w;
 
       //---------------------------------------------------
       //    Tracks
       //---------------------------------------------------
 
-      p.setPen(black);
       TrackList* l = song->tracks();
       int idx = 0;
-      for (iTrack i = l->begin(); i != l->end(); ++i, ++idx) {
-            int yy = idx * TH;
-            if (yy >= y + h)
+      int yy  = -ypos;
+      for (iTrack i = l->begin(); i != l->end(); ++idx, yy += (*i)->height(), ++i) {
+            Track* track = *i;
+            int trackHeight = track->height();
+            if (yy >= (y + h))
                   break;
-            if (yy + TH < y)
+            if ((yy + trackHeight) < y)
                   continue;
             //
             // clear one row
             //
             if ((*i)->selected()) {
-                  p.fillRect(x, yy, w, TH, selectedTrackColor);
-//                  p.fillRect(header->sectionPos(COL_ACTIVITY) + header->sectionSize(COL_ACTIVITY) + 1, yy, w, TH, selectedTrackColor);
+                  p.fillRect(x1, yy, w, trackHeight, palette().active().highlight());
+                  p.setPen(palette().active().highlightedText());
+                  }
+            else {
+                  p.setPen(palette().active().text());
                   }
-            else
-                  p.eraseRect(x, yy, w, TH);
-            Track::TrackType type = (*i)->type();
 
+            Track::TrackType type = (*i)->type();
             int x = 0;
             for (int index = 0; index < header->count(); ++index) {
                   int section = header->mapToSection(index);
                   int w   = header->sectionSize(section);
-                  QRect r = p.xForm(QRect(x+2, yy, w-4, TH));
+                  QRect r = p.xForm(QRect(x+2, yy, w-4, trackHeight));
 
                   switch (section) {
                         case COL_RECORD:
-                              if ((*i)->recordFlag())
-                                    drawCenteredPixmap(p, record1Icon, r);
+                              drawCenteredPixmap(p,
+                                 (*i)->recordFlag() ? reddotIcon : darkreddotIcon, r);
                               break;
                         case COL_CLASS:
                               {
@@ -153,9 +218,14 @@ void TList::draw(QPainter& p, const QRect&rect)
                               }
                               break;
                         case COL_MUTE:
-                              if (song->mute(dynamic_cast<SoundSource*>(*i))) {
-                                    drawCenteredPixmap(p, muteIcon, r);
-                                    }
+                              if ((*i)->node()->off())
+                                    drawCenteredPixmap(p, offIcon, r);
+                              else if ((*i)->node()->mute())
+                                    drawCenteredPixmap(p, editmuteSIcon, r);
+                              break;
+                        case COL_SOLO:
+                              if ((*i)->node()->solo())
+                                    drawCenteredPixmap(p, bluedotIcon, r);
                               break;
                         case COL_TIMELOCK:
                               if ((type == Track::MIDI || type == Track::DRUM)
@@ -164,13 +234,21 @@ void TList::draw(QPainter& p, const QRect&rect)
                                     }
                               break;
                         case COL_NAME:
-                              drawCenteredText(p, (*i)->tname(), r);
+                              p.drawText(r, Qt::AlignVCenter|Qt::AlignLeft, (*i)->name());
                               break;
                         case COL_OCHANNEL:
                               {
                               QString s;
-                              s.setNum((*i)->outChannel() + 1);
-                              drawCenteredText(p, s, r);
+                              int n;
+                              if (type == Track::WAVE) {
+                                    // show number of ports
+                                    n = ((WaveTrack*)(*i))->ports();
+                                    }
+                              else {
+                                    n = (*i)->outChannel() + 1;
+                                    }
+                              s.setNum(n);
+                              p.drawText(r, Qt::AlignVCenter|Qt::AlignLeft, s);
                               }
                               break;
                         case COL_OPORT:
@@ -181,30 +259,21 @@ void TList::draw(QPainter& p, const QRect&rect)
                               switch((*i)->type()) {
                                     case Track::WAVE:
                                           {
-                                          AudioNode* node = ((WaveTrack*)(*i))->route();
-                                          if (node == song->master())
-                                                s.sprintf("master");
-                                          else if (node == song->group(0))
-                                                s.sprintf("group0");
-                                          else if (node == song->group(1))
-                                                s.sprintf("group1");
-                                          else if (node == song->group(2))
-                                                s.sprintf("group2");
-                                          else if (node == song->group(3))
-                                                s.sprintf("group3");
+                                          WaveTrack* wt = (WaveTrack*)(*i);
+                                          s = node2Name(wt->route());
                                           }
                                           break;
                                     case Track::MIDI:
                                     case Track::DRUM:
                                           {
-                                          Device* dev = (*i)->outDevice();
+                                          MidiDevice* dev = ((MidiTrack*)(*i))->outDevice();
                                           if (dev)
                                                 s.sprintf("%d(%s)", outport,
                                                     dev->name().latin1());
                                           }
                                           break;
                                     }
-                              drawCenteredText(p, s, r);
+                              p.drawText(r, Qt::AlignVCenter|Qt::AlignLeft, s);
                               }
                               break;
                         default:
@@ -212,39 +281,30 @@ void TList::draw(QPainter& p, const QRect&rect)
                         }
                   x += header->sectionSize(section);
                   }
-            }
-      //---------------------------------------------------
-      //    horizontal lines
-      //---------------------------------------------------
-
-      p.setPen(gray);
-      int yy  = (y / TH) * TH;
-      for (; yy < y + h; yy += TH) {
-            p.drawLine(x, yy, x + w, yy);
-            }
-      if (drag == DRAG) {
-            int y  = (startY/TH) * TH;
-            int dy = startY - y;
-            int yy = curY - dy;
-            p.setPen(green);
-            p.drawLine(x, yy, x + w, yy);
-            p.drawLine(x, yy+TH, x + w, yy+TH);
             p.setPen(gray);
+            p.drawLine(x1, yy, x2, yy);
+            }
+      p.drawLine(x1, yy, x2, yy);
+
+      if (mode == DRAG) {
+            int yy = curY - dragYoff;
+            p.setPen(green);
+            p.drawLine(x1, yy, x2, yy);
+            p.drawLine(x1, yy + dragHeight, x2, yy+dragHeight);
             }
 
       //---------------------------------------------------
-      //    vertical Lines
+      //    draw vertical lines
       //---------------------------------------------------
 
-      p.setWorldXForm(false);
       int n = header->count();
       int xpos = 0;
+      p.setPen(gray);
       for (int index = 0; index < n-1; index++) {
             int section = header->mapToSection(index);
             xpos += header->sectionSize(section);
             p.drawLine(xpos, 0, xpos, height());
             }
-      p.setWorldXForm(true);
       }
 
 //---------------------------------------------------------
@@ -254,50 +314,75 @@ void TList::draw(QPainter& p, const QRect&rect)
 void TList::returnPressed()
       {
       editor->hide();
-      if (editor->text() != editTrack->tname()) {
+      if (editor->text() != editTrack->name()) {
             Track* track = editTrack->clone();
-            track->setTName(editor->text());
+            track->setName(editor->text());
             midiThread->msgChangeTrack(editTrack, track);
             }
       editTrack = 0;
       setFocus();
       }
 
+//---------------------------------------------------------
+//   adjustScrollbar
+//---------------------------------------------------------
+
+void TList::adjustScrollbar()
+      {
+      int h = 0;
+      TrackList* l = song->tracks();
+      for (iTrack it = l->begin(); it != l->end(); ++it) {
+            h += (*it)->height();
+            (*it)->setY(h);
+            }
+      scroll->setMaxValue(h + 30);
+      redraw();
+      }
+
+//---------------------------------------------------------
+//   y2Track
+//---------------------------------------------------------
+
+Track* TList::y2Track(int y) const
+      {
+      TrackList* l = song->tracks();
+      int ty = 0;
+      for (iTrack it = l->begin(); it != l->end(); ++it) {
+            int h = (*it)->height();
+            if (y >= ty && y < ty + h)
+                  return *it;
+            ty += h;
+            }
+      return 0;
+      }
+
 //---------------------------------------------------------
 //   viewMouseDoubleClickEvent
 //---------------------------------------------------------
 
-void TList::viewMouseDoubleClickEvent(QMouseEvent* ev)
+void TList::mouseDoubleClickEvent(QMouseEvent* ev)
       {
-      int x = ev->x();
-      bool shift = ev->state() & ShiftButton;
-
+      int x       = ev->x();
+      bool shift  = ev->state() & ShiftButton;
       int section = header->sectionAt(x);
       if (section == -1)
             return;
 
-      unsigned track = ev->y() / TH;
-      if (track >= song->tracks()->size()) {
+      Track* t = y2Track(ev->y() + ypos);
+
+      if (t == 0) {
             Track* t = song->addTrack();
             if (!shift)
                   song->tracks()->deselect();
             t->setSelected(true);
             emit selectionChanged();
-            //
-            //   evtl Scrollbar justieren
-            //
-            unsigned ntracks = mapyDev(height()) / TH;
-            if (song->tracks()->size() + 2 > ntracks) {
-                  scroll->setRange(0, (song->tracks()->size() + 2)*TH);
-                  }
+            // adjustScrollbar();
             }
       else {
-            Track* t = song->tracks()->findIdx(track);
-
-            int colx = mapx(header->sectionPos(section));
-            int colw = rmapx(header->sectionSize(section));
-            int coly = mapy(track * TH);
-            int colh = rmapy(TH);
+            int colx = header->sectionPos(section);
+            int colw = header->sectionSize(section);
+            int coly = t->y() - ypos;
+            int colh = t->height();
 
             if (section == COL_NAME) {
                   editTrack = t;
@@ -307,13 +392,13 @@ void TList::viewMouseDoubleClickEvent(QMouseEvent* ev)
                            SLOT(returnPressed()));
                         editor->setFrame(true);
                         }
-                  editor->setText(editTrack->tname());
+                  editor->setText(editTrack->name());
                   editor->end(false);
                   editor->setGeometry(colx, coly, colw, colh);
                   editor->show();
                   }
             else
-                  viewMousePressEvent(ev);
+                  mousePressEvent(ev);
             }
       }
 
@@ -352,11 +437,10 @@ void TList::oportPropertyPopupMenu(Track* t, int x, int y)
             return;
       int oPort      = t->outPort();
       MidiPort* port = &midiPorts[oPort];
-// printf("Channel %d %d %d\n", oChannel, port->hasGui(), port->guiVisible());
 
       QPopupMenu* p = new QPopupMenu(this);
       p->setCheckable(true);
-      p->insertItem("Show Gui", 0);
+      p->insertItem(tr("Show Gui"), 0);
 
       p->setItemEnabled(0, port->hasGui());
       p->setItemChecked(0, port->guiVisible());
@@ -368,7 +452,6 @@ void TList::oportPropertyPopupMenu(Track* t, int x, int y)
       delete p;
       }
 
-
 //---------------------------------------------------------
 //   tracklistChanged
 //---------------------------------------------------------
@@ -444,49 +527,103 @@ void TList::moveSelection(int n)
       }
 
 //---------------------------------------------------------
-//   viewMousePressEvent
+//   mousePressEvent
 //---------------------------------------------------------
 
-void TList::viewMousePressEvent(QMouseEvent* ev)
+void TList::mousePressEvent(QMouseEvent* ev)
       {
-      int x             = ev->x();
-      int y             = ev->y();
-      int button        = ev->button();
-      bool shift        = ev->state() & ShiftButton;
-      unsigned track    = y / TH;
-      TrackColumn col   = TrackColumn(header->sectionAt(x));
-      TrackList* tracks = song->tracks();
-      if (track >= tracks->size())
-            return;
+      int x       = ev->x();
+      int y       = ev->y();
+      int button  = ev->button();
+      bool shift  = ev->state() & ShiftButton;
+      Track* t    = y2Track(y + ypos);
 
+      TrackColumn col = TrackColumn(header->sectionAt(x));
+      if (t == 0) {
+            if (button == QMouseEvent::RightButton) {
+                  QPopupMenu* p = new QPopupMenu(this);
+                  p->clear();
+                  p->insertItem(*noteIcon, tr("Add Midi Track"), 0);
+                  p->insertItem(*stickIcon, tr("Add Drum Track"), 1);
+                  p->insertItem(*waveIcon, tr("Add Wave Track"), 2);
+                  int n = p->exec(ev->globalPos(), 0);
+                  switch (n) {
+                        case -1:
+                              break;
+                        case 0:     // add midi track
+                              t = song->addTrack(Track::MIDI);
+                              break;
+                        case 1:     // add drum track
+                              t = song->addTrack(Track::DRUM);
+                              break;
+                        case 2:     // add wave track
+                              t = song->addTrack(Track::WAVE);
+                              break;
+                        }
+                  if (t) {
+                        song->tracks()->deselect();
+                        t->setSelected(true);
+                        emit selectionChanged();
+                        adjustScrollbar();
+                        }
+                 delete p;
+                 }
+            return;
+            }
+
+      TrackList* tracks = song->tracks();
+      dragYoff = y - (t->y() - ypos);
       startY   = y;
-      sTrack   = track;
-      drag     = START_DRAG;
-      Track* t = tracks->findIdx(track);
+
+      if (resizeFlag) {
+            mode   = RESIZE;
+            int y  = ev->y();
+            int ty = -ypos;
+            sTrack = 0;
+            for (iTrack it = tracks->begin(); it != tracks->end(); ++it, ++sTrack) {
+                  int h = (*it)->height();
+                  ty += h;
+                  if (y >= (ty-2) && y < (ty+2))
+                        break;
+                  }
+            return;
+            }
+      mode = START_DRAG;
 
       switch (col) {
             case COL_RECORD:
-                  if (button == QMouseEvent::LeftButton)
-                        t->setRecordFlag(!t->recordFlag());
+                  if (t->type() == Track::WAVE)
+                        song->setRecordFlag(t, !t->recordFlag() && ((WaveTrack*)t)->canRecord());
+                  else
+                        song->setRecordFlag(t, !t->recordFlag());
                   break;
             case COL_NONE:
             case COL_ACTIVITY:
                   break;
             case COL_CLASS:
-                  if (button == QMouseEvent::LeftButton)
-                        classesPopupMenu(track, mapx(x), mapy(track * TH));
+                  classesPopupMenu(tracks->index(t), x, t->y() - ypos);
                   break;
             case COL_OPORT:
                   if (button == QMouseEvent::LeftButton)
-                        portsPopupMenu(t, mapx(x), mapy(track * TH));
+                        portsPopupMenu(t, x, t->y() - ypos);
                   else if (button == QMouseEvent::RightButton)
-                        oportPropertyPopupMenu(t, mapx(x), mapy(track*TH));
+                        oportPropertyPopupMenu(t, x, t->y() - ypos);
                   break;
             case COL_MUTE:
-                  if (button == QMouseEvent::LeftButton) {
-                        SoundSource* ss = dynamic_cast<SoundSource*>(t);
-                        song->setMute(ss, !song->mute(ss));
-                        }
+                  {
+                  SNode* node = t->node();
+                  if (node->off())
+                        node->setOff(false);
+                  else
+                        node->setMute(!node->mute());
+                  }
+                  break;
+
+            case COL_SOLO:
+                  {
+                  SNode* ss = t->node();
+                  ss->setSolo(!ss->solo());
+                  }
                   break;
 
             case COL_NAME:
@@ -513,7 +650,9 @@ void TList::viewMousePressEvent(QMouseEvent* ev)
                                           for (iTrack it = tracks->begin(); it != tracks->end(); ++it)
                                                 (*it)->setSelected(false);
                                           t->setSelected(true);
+                                          song->startUndo();
                                           midiThread->msgRemoveTracks();
+                                          song->endUndo(SC_TRACK_REMOVED);
                                           break;
 
                                     case 1:     // show track comment
@@ -534,8 +673,7 @@ void TList::viewMousePressEvent(QMouseEvent* ev)
                   break;
 
             case COL_TIMELOCK:
-                  if (button == QMouseEvent::LeftButton)
-                        t->setLocked(!t->locked());
+                  t->setLocked(!t->locked());
                   break;
 
             case COL_OCHANNEL:
@@ -562,20 +700,47 @@ void TList::viewMousePressEvent(QMouseEvent* ev)
       }
 
 //---------------------------------------------------------
-//   viewMouseMoveEvent
+//   mouseMoveEvent
 //---------------------------------------------------------
 
-void TList::viewMouseMoveEvent(QMouseEvent* ev)
+void TList::mouseMoveEvent(QMouseEvent* ev)
       {
-      curY = ev->y();
+      if (ev->state() == 0) {
+            int y = ev->y();
+            int ty = -ypos;
+            TrackList* tracks = song->tracks();
+            iTrack it;
+            for (it = tracks->begin(); it != tracks->end(); ++it) {
+                  int h = (*it)->height();
+                  ty += h;
+                  if (y >= (ty-2) && y < (ty+2)) {
+                        if (!resizeFlag) {
+                              resizeFlag = true;
+                              setCursor(QCursor(splitVCursor));
+                              }
+                        break;
+                        }
+                  }
+            if (it == tracks->end() && resizeFlag) {
+                  setCursor(QCursor(arrowCursor));
+                  resizeFlag = false;
+                  }
+            return;
+            }
+      curY      = ev->y();
       int delta = curY - startY;
-      switch (drag) {
+      switch (mode) {
             case START_DRAG:
                   if (delta < 0)
                         delta = -delta;
                   if (delta <= 2)
-                        return;
-                  drag = DRAG;
+                        break;
+                  mode = DRAG;
+                  {
+                  Track* t   = y2Track(startY + ypos);
+                  dragHeight = t->height();
+                  sTrack     = song->tracks()->index(t);
+                  }
                   setCursor(QCursor(sizeVerCursor));
                   redraw();
                   break;
@@ -584,25 +749,41 @@ void TList::viewMouseMoveEvent(QMouseEvent* ev)
             case DRAG:
                   redraw();
                   break;
+            case RESIZE:
+                  {
+                  Track* t = song->tracks()->findIdx(sTrack);
+                  int h  = t->height() + delta;
+                  startY = curY;
+                  if (h < MIN_TRACKHEIGHT)
+                        h = MIN_TRACKHEIGHT;
+                  t->setHeight(h);
+                  song->update(SC_TRACK_MODIFIED);
+                  }
+                  break;
             }
       }
 
 //---------------------------------------------------------
-//   viewMouseReleaseEvent
+//   mouseReleaseEvent
 //---------------------------------------------------------
 
-void TList::viewMouseReleaseEvent(QMouseEvent* ev)
+void TList::mouseReleaseEvent(QMouseEvent* ev)
       {
-      if (drag == DRAG) {
-            int y = ev->y();
-            unsigned dTrack = y / TH;
-            midiThread->msgMoveTrack(sTrack, dTrack);
+      if (mode == DRAG) {
+            Track* t = y2Track(ev->y() + ypos);
+            if (t) {
+                  int dTrack = song->tracks()->index(t);
+                  midiThread->msgMoveTrack(sTrack, dTrack);
+                  }
+            }
+      if (mode != NORMAL) {
+            mode = NORMAL;
             setCursor(QCursor(arrowCursor));
+            redraw();
             }
-      drag = NORMAL;
-      redraw();
       if (editTrack)
             editor->setFocus();
+      adjustScrollbar();
       }
 
 //---------------------------------------------------------
@@ -616,33 +797,37 @@ void TList::updateActivity()
       TrackList* tl = song->tracks();
       int idx = 0;
       QPainter p(this);
-      setPainter(p);
-      for (iTrack i = tl->begin(); i != tl->end(); ++i, ++idx) {
+      int y = 0;
+      for (iTrack i = tl->begin(); i != tl->end();
+         ++idx, y += (*i)->height(), ++i) {
             // normal cast does not work!
-            SoundSource* ss = dynamic_cast<SoundSource*>(*i);
-            int y = idx * TH;
+            SNode* ss = (*i)->node();
+            if (ss->activity() == ss->lastActivity())
+                  continue;
             int w = header->sectionSize(COL_ACTIVITY);
             int x = header->sectionPos(COL_ACTIVITY);
 
             int val  = ss->activity();
-            int cval = ss->curActivity();
-
-            int diff = val - cval;
-            if (diff == 0)
-                  continue;
-            diff /= 4;
-            cval = diff ? cval + diff : val;
-            ss->setCurActivity(cval);
 
             double ko = double(w) / 10.0;
-            int nval  = int(log2(double(cval)) * ko);
+            int nval  = lrint(log2(double(val)) * ko);
             if (nval < 0)
                   nval = 0;
             if (nval > w)
                   nval = w;
 
+            ss->setLastActivity(val);
+            int mval = val/10;
+            if (mval == 0)
+                  mval = 1;
+            val -= mval;              // ramp down
+            if (val < 0)
+                  val = 0;
+            ss->setActivity(val);
+
+            int th = (*i)->height();
             if (activityMode == 0) {
-                  p.fillRect(x+1, y+1, nval, TH-1, activityColor);
+                  p.fillRect(x+1, y+1, nval, th-1, activityColor);
                   }
             else {
 	            for (int count = 1; count < nval; count++) {
@@ -652,10 +837,10 @@ void TList::updateActivity()
 	                  int b = 0;
 	                  if (!(count % 2))
 	                        r = g = b = 0xff;
-	                  p.fillRect(count+x, y+1, 1, TH-1, QBrush(QColor(r, g, b)));
+	                  p.fillRect(count+x, y+1, 1, -1, QBrush(QColor(r, g, b)));
 	                  }
 	            }
-            p.fillRect(x+nval+1, y+1, w-nval-1, TH-1, white);
+            p.fillRect(x+nval+1, y+1, w-nval-1, th-1, white);
             }
       }
 
@@ -666,8 +851,8 @@ void TList::updateActivity()
 static bool checkEmpty(Track* t, QWidget* parent)
       {
       if (t->parts()->size()) {
-            QMessageBox::information(parent, "MusE",
-               parent->tr("Cannot transform non empty track"));
+            QMessageBox::information(parent, QString("MusE"),
+               QWidget::tr("Cannot transform non empty track"));
             return true;
             }
       return false;
@@ -691,7 +876,7 @@ void TList::classesPopupMenu(int track, int x, int y)
       if (n == -1)
             return;
 
-      QString name  = t->tname();
+      QString name  = t->name();
       bool selected = t->selected();
 
       switch (Track::TrackType(n)) {
@@ -702,10 +887,11 @@ void TList::classesPopupMenu(int track, int x, int y)
                         if (checkEmpty(t, this))
                               break;
                         MidiTrack* mt = new MidiTrack(name);
+                        WaveTrack* wt = (WaveTrack*) t;
                         mt->setSn(t->sn());
                         mt->setSelected(selected);
-                        mt->setMute(((WaveTrack*)t)->mute());
-                        mt->setSolo(((WaveTrack*)t)->soloMute());
+                        mt->setMute(wt->mute());
+                        mt->setSolo(wt->solo());
                         midiThread->msgChangeTrack(t, mt);
                         }
                   break;
@@ -716,11 +902,12 @@ void TList::classesPopupMenu(int track, int x, int y)
                         if (checkEmpty(t, this))
                               break;
                         MidiTrack* mt = new MidiTrack(name);
+                        WaveTrack* wt = (WaveTrack*) t;
                         mt->setSn(t->sn());
                         mt->setType(Track::DRUM);
                         mt->setSelected(selected);
-                        mt->setMute(((WaveTrack*)t)->mute());
-                        mt->setSolo(((WaveTrack*)t)->soloMute());
+                        mt->setMute(wt->mute());
+                        mt->setSolo(wt->solo());
                         midiThread->msgChangeTrack(t, mt);
                         }
                   break;
@@ -730,15 +917,27 @@ void TList::classesPopupMenu(int track, int x, int y)
                               break;
                         MidiTrack* mt = (MidiTrack*) t;
                         // MUTABOR!
-                        bool mute     = mt->mute();
-                        bool soloMute = mt->soloMute();
+                        bool mute = mt->mute();
+                        bool solo = mt->mute();
+
+                  if (WaveTrack::firstWaveTrack == true && QFileInfo(museProject).isRelative()) {
+                        if(QMessageBox::information( this, "MusE",
+                                 tr("This is your first wave track and the project\n"
+                                    "doesn't appear to be saved yet.\n\n"
+                                    "It is recommended that you save a project file now!\n\n"
+                                    "By doing that you are implicitly selecting a folder\n"
+                                    "where the audio files will end up."), tr("Save dialog"), tr("Cancel"),
+				    QString::null ,0,1 ) == 0) {
+				           muse->saveAs();
+                                 }
+                        }
 
                         WaveTrack* wt = new WaveTrack(name);
 
                         wt->setSn(t->sn());
                         wt->setSelected(selected);
                         wt->setMute(mute);
-                        wt->setSolo(soloMute);
+                        wt->setSolo(solo);
                         midiThread->msgChangeTrack(t, wt);
 
                         if (selected)
@@ -806,17 +1005,43 @@ void TList::setActivityColor(QColor c)
       }
 
 //---------------------------------------------------------
-//   setSelectedTrackColor
+//   setYPos
 //---------------------------------------------------------
 
-void TList::setSelectedTrackColor(QColor c)
+void TList::setYPos(int y)
       {
-      selectedTrackColor = c;
-      redraw();
+      int delta  = ypos - y;         // -  -> shift up
+      ypos  = y;
+      if (pm.isNull())
+            return;
+      if (!pmValid) {
+            redraw();
+            return;
+            }
+      int w = width();
+      int h = height();
+      QRect r;
+      if (delta >= h || delta <= -h)
+            r = QRect(0, 0, w, h);
+      else if (delta < 0) {   // shift up
+            bitBlt(&pm,  0, 0, &pm, 0, -delta, w, h + delta, CopyROP, true);
+            r = QRect(0, h + delta, w, -delta);
+            }
+      else {                  // shift down
+            bitBlt(&pm,  0, delta, &pm, 0, 0, w, h-delta, CopyROP, true);
+            r = QRect(0, 0, w, delta);
+            }
+      paint(r);
+      update();
       }
 
-QColor TList::getSelectedTrackColor() const
+//---------------------------------------------------------
+//   resizeEvent
+//---------------------------------------------------------
+
+void TList::resizeEvent(QResizeEvent* ev)
       {
-      return selectedTrackColor;
+      pm.resize(ev->size());
+      pmValid = false;
       }
 
