@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: audioprefetch.h,v 1.2 2001/11/20 15:19:31 muse Exp $
+//  $Id: audioprefetch.h,v 1.1.1.1 2003/10/29 10:05:06 wschweer Exp $
 //
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -12,30 +12,34 @@
 #include "thread.h"
 
 
-enum { PREFETCH_TICK
+enum { PREFETCH_TICK, PREFETCH_SEEK
       };
 
 struct PrefetchMsg : public ThreadMsg {
       int pos;
-      int bufferNo;
       };
 
 //---------------------------------------------------------
-//   AudioThread
+//   AudioPrefetch
 //---------------------------------------------------------
 
 class AudioPrefetch : public Thread {
-      int currentFrameSize;
+      int readSamplePos;
 
       virtual void processMsg1(const void*);
-      void prefetch(int pos, int bufferNo);
+      void prefetch();
+      void seek(int pos);
+      int seekpos;
 
    public:
-      AudioPrefetch(bool rt=true, int prio=50, bool ml = true);
+      AudioPrefetch(int prio, const char* name);
       ~AudioPrefetch();
       virtual void start();
 
-      void msgTick(int samplePos, int bufferNo);
+      void msgTick();
+      void msgSeek(int samplePos);
       };
 
+extern AudioPrefetch* audioPrefetch;
+
 #endif
