@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: conf.cpp,v 1.2 2002/02/27 08:48:09 muse Exp $
+//  $Id: conf.cpp,v 1.1.1.1 2003/10/29 10:05:08 wschweer Exp $
 //
 //  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -35,9 +35,7 @@
 #include "seq.h"
 #include "conf.h"
 #include "pitchedit.h"
-#include "device.h"
 #include "midiport.h"
-#include "audioport.h"
 #include "driver/mididev.h"
 #include "driver/midiserial.h"
 #include "driver/audiodev.h"
@@ -50,6 +48,8 @@
 #include "midiitransform.h"
 #include "synth.h"
 #include "audioconf.h"
+#include "audio.h"
+#include "sync.h"
 
 #include <errno.h>
 
@@ -151,8 +151,12 @@ void MusE::readConfigTransport(Xml& xml)
             QString tag = xml.s1();
             switch (token) {
                   case Xml::TagStart:
-                        if (tag == "geometry")
+                        if (tag == "geometry") {
                               configGeometryTransport = readGeometry(xml);
+                              // use only position
+                              configGeometryTransport.setWidth(0);
+                              configGeometryTransport.setHeight(0);
+                              }
                         else if (tag == "handlecolor")
                               configTransportHandleColor = readColor(xml);
                         else
@@ -280,14 +284,26 @@ void MusE::readConfiguration(Xml& xml)
                               configGeometryMain = readGeometry(xml);
                         else if (tag == "theme")
                               configTheme = xml.parse1();
-                        else if (tag == "fontSize")
-                              configFontSize = xml.parseInt();
+                        else if (tag == "fontSize")  // obsolete
+                              ;
+                        else if (tag == "font0")
+                              font0.fromString(xml.parse1());
+                        else if (tag == "font1")
+                              font1.fromString(xml.parse1());
+                        else if (tag == "font2")
+                              font2.fromString(xml.parse1());
+                        else if (tag == "font3")
+                              font3.fromString(xml.parse1());
+                        else if (tag == "font4")
+                              font4.fromString(xml.parse1());
+                        else if (tag == "font5")
+                              font5.fromString(xml.parse1());
                         else if (tag == "activityMode")
                               configActivityMode = xml.parseInt();
                         else if (tag == "activityColor")
                               configActivityColor = readColor(xml);
                         else if (tag == "selectedTrackColor")
-                              configSelectedTrackColor = readColor(xml);
+                              /*configSelectedTrackColor =*/ readColor(xml);
                         else if (tag == "extendedMidi")
                               extendedMidi = xml.parseInt();
                         else if (tag == "midiExportDivision")
@@ -312,10 +328,18 @@ void MusE::readConfiguration(Xml& xml)
                               readConfigTransport(xml);
                         else if (tag == "bigtime")
                               readConfigBigTime(xml);
-                        else if (tag == "deviceid")
-                              deviceId = xml.parseInt();
-                        else if (tag == "syncport")
-                              extSyncPort= xml.parseInt();
+                        else if (tag == "deviceid")    // obsolete
+                              xml.parseInt();
+                        else if (tag == "txDeviceId")
+                              txDeviceId = xml.parseInt();
+                        else if (tag == "rxDeviceId")
+                              rxDeviceId = xml.parseInt();
+                        else if (tag == "syncport")   // obsolete
+                              xml.parseInt();
+                        else if (tag == "txSyncPort")
+                              txSyncPort= xml.parseInt();
+                        else if (tag == "rxSyncPort")
+                              rxSyncPort= xml.parseInt();
                         else if (tag == "mtctype")
                               mtcType= xml.parseInt();
                         else if (tag == "extSync")
@@ -367,6 +391,14 @@ void MusE::readConfiguration(Xml& xml)
                               division = xml.parseInt();
                         else if (tag == "samplerate")
                               sampleRate = xml.parseInt();
+                        else if (tag == "useAlsa") {
+                              useAlsaAudio = xml.parseInt();
+                              useJackAudio = !useAlsaAudio;
+                              }
+                        else if (tag == "useJack") {
+                              useJackAudio = xml.parseInt();
+                              useAlsaAudio = ! useJackAudio;
+                              }
                         else if (tag == "segmentsize")
                               segmentSize = xml.parseInt();
                         else if (tag == "rtcTicks")
@@ -374,9 +406,21 @@ void MusE::readConfiguration(Xml& xml)
                         else if (tag == "minMeter")
                               minMeter = xml.parseInt();
                         else if (tag == "minSlider")
-                              minSlider = xml.parseInt();
+                              minSlider = xml.parseDouble();
                         else if (tag == "guiRefresh")
                               guiRefresh = xml.parseInt();
+                        else if (tag == "helpBrowser")
+                              helpBrowser = xml.parse1();
+                        else if (tag == "audioInput") {
+                              QString s = xml.parse1();
+                              if (audioDevice)
+                                    audioDevice->setInputPort(s);
+                              }
+                        else if (tag == "audioOutput") {
+                              QString s = xml.parse1();
+                              if (audioDevice)
+                                    audioDevice->setOutputPort(s);
+                              }
                         else if (tag == "serialPort")
                               readSerialPort(xml);
                         else if (tag == "midiTransform")
@@ -445,13 +489,22 @@ void MusE::writeConfiguration(int level, Xml& xml) const
             xml.intTag(level, "segmentsize", segmentSize);
       if (rtcTicks != 4096)
             xml.intTag(level, "rtcTicks", rtcTicks);
+      if (useAlsaAudio)
+            xml.intTag(level, "useAlsa", useAlsaAudio);
+      if (useJackAudio)
+            xml.intTag(level, "useJack", useJackAudio);
       if (minMeter != -60)
             xml.intTag(level, "minMeter", minMeter);
-      if (minSlider != -60)
-            xml.intTag(level, "minSlider", minSlider);
+      if (minSlider != -60.0)
+            xml.doubleTag(level, "minSlider", minSlider);
       if (guiRefresh != 20)
             xml.intTag(level, "guiRefresh", guiRefresh);
-
+      if (helpBrowser != QString("konqueror"))
+            xml.strTag(level, "helpBrowser", helpBrowser);
+      if (audioDevice) {
+            xml.strTag(level, "audioInput", audioDevice->inputPort());
+            xml.strTag(level, "audioOutput", audioDevice->outputPort());
+            }
       xml.intTag(level, "extendedMidi", extendedMidi);
       xml.intTag(level, "midiExportDivision", midiDivision);
       xml.intTag(level, "midiInputDevice", midiInputPorts);
@@ -462,20 +515,30 @@ void MusE::writeConfiguration(int level, Xml& xml) const
       xml.intTag(level, "midiFilterCtrl2", midiFilterCtrl2);
       xml.intTag(level, "midiFilterCtrl3", midiFilterCtrl3);
       xml.intTag(level, "midiFilterCtrl4", midiFilterCtrl4);
-      xml.intTag(level, "deviceid", deviceId);
-      xml.nput(level, "<theme>%s</theme>\n", _currentTheme.data());
-      xml.nput(level, "<fontSize>%d</fontSize>\n", configFontSize);
+      xml.intTag(level, "txDeviceId", txDeviceId);
+      xml.intTag(level, "rxDeviceId", rxDeviceId);
+      xml.nput(level, "<theme>%s</theme>\n", _currentTheme.ascii());
+
+      xml.strTag(level, "font0", font0.toString());
+      xml.strTag(level, "font1", font1.toString());
+      xml.strTag(level, "font2", font2.toString());
+      xml.strTag(level, "font3", font3.toString());
+      xml.strTag(level, "font4", font4.toString());
+      xml.strTag(level, "font5", font5.toString());
+
       xml.nput(level, "<activityColor r=\"%d\" g=\"%d\" b=\"%d\"></activityColor>\n",
       		arranger->getActivityColor().red(),
       		arranger->getActivityColor().green(),
       		arranger->getActivityColor().blue());
-      xml.nput(level, "<selectedTrackColor r=\"%d\" g=\"%d\" b=\"%d\"></selectedTrackColor>\n",
+/*      xml.nput(level, "<selectedTrackColor r=\"%d\" g=\"%d\" b=\"%d\"></selectedTrackColor>\n",
       		arranger->getSelectedTrackColor().red(),
       		arranger->getSelectedTrackColor().green(),
       		arranger->getSelectedTrackColor().blue());
+ */
       xml.intTag(level, "activityMode", arranger->getActivityMode());
 
-      xml.intTag(level, "syncport", extSyncPort);
+      xml.intTag(level, "txSyncPort", txSyncPort);
+      xml.intTag(level, "rxSyncPort", rxSyncPort);
       xml.intTag(level, "mtctype", mtcType);
       xml.nput(level, "<mtcoffset>%02d:%02d:%02d:%02d:%02d</mtcoffset>\n",
         mtcOffset.h(), mtcOffset.m(), mtcOffset.s(),
@@ -583,9 +646,9 @@ void MusE::configMidiFile()
 //---------------------------------------------------------
 
 MidiFileConfig::MidiFileConfig()
-   : QGroupBox(2, Horizontal, "Config exported Midi Files", 0)
+   : QGroupBox(2, Horizontal, tr("Config exported Midi Files"), 0)
       {
-      setCaption("MusE");
+      setCaption(tr("MusE: Config exported Midi Files"));
       QWidget* w = new QWidget(this);
       QGridLayout* grid = new QGridLayout(w);
       grid->setSpacing(5);
@@ -595,9 +658,9 @@ MidiFileConfig::MidiFileConfig()
 
       QLabel* l1 = new QLabel(tr("Division"), w);
       QComboBox* cb1 = new QComboBox(w);
-      cb1->insertItem("96");
-      cb1->insertItem("192");
-      cb1->insertItem("384");
+      cb1->insertItem(QString("96"));
+      cb1->insertItem(QString("192"));
+      cb1->insertItem(QString("384"));
       int curItem = -1;
       switch(midiDivision) {
             case 96:    curItem = 0; break;
