@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: confmport.cpp,v 1.2 2002/02/28 09:02:25 muse Exp $
+//  $Id: confmport.cpp,v 1.3 2003/10/30 12:11:50 wschweer Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -34,7 +34,6 @@
 #include "transport.h"
 #include "arranger.h"
 #include "seq.h"
-#include "device.h"
 #include "midiport.h"
 #include "driver/mididev.h"
 #include "driver/midiserial.h"
@@ -45,6 +44,8 @@
 #include "minstrument.h"
 #include "synth.h"
 #include "midithread.h"
+#include "mess.h"
+#include "audio.h"
 
 enum { DEVCOL_NO = 0, DEVCOL_GUI, DEVCOL_REC, DEVCOL_INSTR, DEVCOL_NAME,
        DEVCOL_STATE };
@@ -86,11 +87,11 @@ static int speed2idx(const QString& s)
 static QString speed2str(int s)
       {
       switch(s) {
-            case B9600:   return "9600";
-            case B19200:   return "19200";
-            case B38400:   return "38400";
+            case B9600:   return QString("9600");
+            case B19200:  return QString("19200");
+            case B38400:  return QString("38400");
             }
-      return "?";
+      return QString("?");
       }
 
 
@@ -116,11 +117,11 @@ static int handshaking2int(const QString& s)
 static QString handshaking2str(int v)
       {
       switch(v) {
-            case 0:  return "None";
-            case 1:  return "RTS/CTS";
-            case 2:  return "Xon/Xoff";
+            case 0:  return QString("None");
+            case 1:  return QString("RTS/CTS");
+            case 2:  return QString("Xon/Xoff");
             }
-      return "?";
+      return QString("?");
       }
 
 //---------------------------------------------------------
@@ -145,19 +146,19 @@ static int type2int(const QString& s)
 static QString type2str(int t)
       {
       switch(t) {
-            case 0:  return "Raw Device";
-            case 1:  return "Serial Port";
-            case 2:  return "Pipe";
+            case 0:  return QString("Raw Device");
+            case 1:  return QString("Serial Port");
+            case 2:  return QString("Pipe");
             }
-      return "?";
+      return QString("?");
       }
 
 //---------------------------------------------------------
 //   MidiRawDeviceDialog
 //---------------------------------------------------------
 
-MidiRawDeviceDialog::MidiRawDeviceDialog(QWidget* parent=0, const char* name = 0,
-   bool modal = false, WFlags fl = 0)
+MidiRawDeviceDialog::MidiRawDeviceDialog(QWidget* parent, const char* name,
+   bool modal, WFlags fl)
    : MidiRawDeviceDialogBase(parent, name, modal, fl)
       {
       connect(buttonDeviceNew, SIGNAL(clicked()), SLOT(deviceNew()));
@@ -221,8 +222,8 @@ void MidiRawDeviceDialog::devicesSelectionChanged(QListViewItem* item)
             type->setCurrentItem(type2int(item->text(2)));
             }
       else {
-            editName->setText("");
-            editPath->setText("");
+            editName->setText(QString(""));
+            editPath->setText(QString(""));
             virtPorts->setValue(0);
             speed->setCurrentItem(2);
             handshaking->setCurrentItem(0);
@@ -279,9 +280,9 @@ void MidiRawDeviceDialog::typeChanged(const QString& text)
             }
       else {
             speed->setEnabled(false);
-            item->setText(4, "---");
+            item->setText(4, QString("---"));
             handshaking->setEnabled(false);
-            item->setText(5, "---");
+            item->setText(5, QString("---"));
             }
       item->setText(2, text);
       }
@@ -330,10 +331,12 @@ void MidiRawDeviceDialog::handshakingChanged(const QString& text)
 
 void MidiRawDeviceDialog::deviceNew()
       {
-      QListViewItem* item = new QListViewItem(listDevices, "", "", "Serial Port", "1", "38400", "None");
+      QListViewItem* item = new QListViewItem(listDevices,
+         QString(""), QString(""), QString("Serial Port"), QString("1"),
+         QString("38400"), QString("None"));
       listDevices->setCurrentItem(item);
-      editName->setText("");
-      editPath->setText("");
+      editName->setText(QString(""));
+      editPath->setText(QString(""));
       virtPorts->setValue(0);
       speed->setCurrentItem(2);
       handshaking->setCurrentItem(0);
@@ -430,13 +433,13 @@ void MidiRawDeviceDialog::accept()
 void MidiRawDeviceDialog::devicePath()
       {
       QString s;
-      if (editName->text().isEmpty())
-            s = "/dev";
+      if (editPath->text().isEmpty())
+            s = QString("/dev");
       else
-            s = editName->text();
-      s = QFileDialog::getOpenFileName(s, 0, this, "midiRawDevicePathDialog");
+            s = editPath->text();
+      s = QFileDialog::getOpenFileName(s, QString::null, this, "midiRawDevicePathDialog");
       if (!s.isEmpty())
-            editName->setText(s);
+            editPath->setText(s);
       }
 
 //---------------------------------------------------------
@@ -529,7 +532,6 @@ void MPConfig::rbClicked(QListViewItem* item, const QPoint&, int col)
 //                                    midiInstruments.push_back(*ii);
                                     port->setInstrument(*ii);
                                     (*ii)->setDevice(dev);
-printf("set Device midi %p\n", dev);
                                     break;
                                     }
                               }
@@ -579,25 +581,25 @@ QString MPWhatsThis::text(const QPoint& pos)
       {
       int n = header->cellAt(pos.x());
       if (n == -1)
-            return 0;
+            return QString::null;
       switch (header->mapToLogical(n)) {
             case DEVCOL_NO:
-                  return header->tr("Port Number");
+                  return QHeader::tr("Port Number");
             case DEVCOL_GUI:
-                  return header->tr("enable gui for device");
+                  return QHeader::tr("enable gui for device");
             case DEVCOL_REC:
-                  return header->tr("enables recording from the device");
+                  return QHeader::tr("enables recording from the device");
             case DEVCOL_NAME:
-                  return header->tr("Name of the midi device associated with"
+                  return QHeader::tr("Name of the midi device associated with"
                        " this port number");
             case DEVCOL_INSTR:
-                  return header->tr("Instrument connected to port");
+                  return QHeader::tr("Instrument connected to port");
             case DEVCOL_STATE:
-                  return header->tr("State: result of opening the device");
+                  return QHeader::tr("State: result of opening the device");
             default:
                   break;
             }
-      return 0;
+      return QString::null;
       }
 
 //---------------------------------------------------------
@@ -605,25 +607,15 @@ QString MPWhatsThis::text(const QPoint& pos)
 //    Midi Port Config
 //---------------------------------------------------------
 
-MPConfig::MPConfig()
-   : QWidget(0, "midiPortTable",
-   WStyle_Customize | WStyle_NormalBorder | WStyle_Title | WStyle_ContextHelp)
+MPConfig::MPConfig(QWidget* parent, char* name)
+   : SynthConfigBase(parent, name)
       {
       popup      = 0;
       instrPopup = 0;
       rdDialog   = 0;
 
-      setCaption(tr("MusE: Midi Port Table"));
-      QVBoxLayout* box = new QVBoxLayout(this);
-      mdevView  = new QListView(this);
       mdevView->setSorting(-1);
       mdevView->setAllColumnsShowFocus(true);
-      mdevView->addColumn("Port",   40);
-      mdevView->addColumn("GUI",    40);
-      mdevView->addColumn("Rec",    40);
-      mdevView->addColumn("Instrument", 110);
-      mdevView->addColumn("Device Name", 110);
-      mdevView->addColumn("State");
       mdevView->setColumnWidthMode(3, QListView::Maximum);
       mdevView->setColumnAlignment(0, AlignCenter);
       mdevView->setFocusPolicy(NoFocus);
@@ -638,12 +630,13 @@ MPConfig::MPConfig()
 
       new MPWhatsThis(mdevView, mdevView->header());
 
-      songChanged(0);
-
-      box->addWidget(mdevView, 10);
       connect(mdevView, SIGNAL(pressed(QListViewItem*,const QPoint&,int)),
          this, SLOT(rbClicked(QListViewItem*,const QPoint&,int)));
       connect(song, SIGNAL(songChanged(int)), SLOT(songChanged(int)));
+      connect(song, SIGNAL(songChanged(int)), SLOT(songChanged(int)));
+      connect(addInstance, SIGNAL(clicked()), SLOT(addInstanceClicked()));
+      connect(removeInstance, SIGNAL(clicked()), SLOT(removeInstanceClicked()));
+      songChanged(0);
       }
 
 //---------------------------------------------------------
@@ -664,14 +657,14 @@ void MPConfig::songChanged(int)
             if (port->instrument())
                   item->setText(DEVCOL_INSTR, port->instrument()->iname());
             else
-                  item->setText(DEVCOL_INSTR, "unknown");
+                  item->setText(DEVCOL_INSTR, tr("unknown"));
             if (dev) {
                   item->setText(DEVCOL_NAME, dev->name());
                   if (port->rwFlags() & 0x2)
                         item->setPixmap(DEVCOL_REC, *dotIcon);
                   }
             else {
-                  item->setText(DEVCOL_NAME, "none");
+                  item->setText(DEVCOL_NAME, tr("none"));
                   item->setPixmap(DEVCOL_GUI, *dothIcon);
                   }
             if (port->hasGui()) {
@@ -684,6 +677,87 @@ void MPConfig::songChanged(int)
             item->setPixmap(DEVCOL_NAME, *buttondownIcon);
             mdevView->insertItem(item);
             }
+      synthList->clear();
+      for (int i = 0; i < nsynthis; ++i) {
+            if (synthis[i] == 0)    //DEBUG
+                  continue;
+            QString s = synthis[i]->baseName();
+            QListViewItem* item = new QListViewItem(synthList);
+            item->setText(0, s);
+            s.setNum(synthis[i]->instances());
+            item->setText(1, s);
+            item->setText(2, QString(synthis[i]->label()));
+            }
+      instanceList->clear();
+      for (iSynthI si = synthiInstances.begin(); si != synthiInstances.end(); ++si) {
+            QListViewItem* iitem = new QListViewItem(instanceList);
+            iitem->setText(0, (*si)->iname());
+            QString port;
+            port.sprintf("%d:%d",
+               (*si)->mess()->alsaClient(),
+               (*si)->mess()->alsaPort());
+            iitem->setText(1, port);
+            }
+      }
+
+//---------------------------------------------------------
+//   addInstanceClicked
+//---------------------------------------------------------
+
+void MPConfig::addInstanceClicked()
+      {
+      QListViewItem* item = synthList->selectedItem();
+      if (item == 0)
+            return;
+      SynthI* si = song->createSynthI(item->text(2));
+      audio->msgAddSynthI(si);      // add to instance list & connect audio
+      midiThread->msgAddSynthI(si); // add to midi poll
+
+      QString name(si->iname());
+      QListViewItem* iitem = new QListViewItem(instanceList);
+      iitem->setText(0, name);
+      QString port;
+      port.sprintf("%d:%d", si->mess()->alsaClient(), si->mess()->alsaPort());
+
+      iitem->setText(1, port);
+
+      song->updateAudioMixer();
+
+      QString inst;
+      inst.setNum(si->synth()->instances());
+      item->setText(1, inst);
+      }
+
+//---------------------------------------------------------
+//   removeInstanceClicked
+//---------------------------------------------------------
+
+void MPConfig::removeInstanceClicked()
+      {
+      QListViewItem* item = instanceList->selectedItem();
+      if (item == 0)
+            return;
+      iSynthI ii;
+      for (ii = synthiInstances.begin(); ii != synthiInstances.end(); ++ii) {
+            if ((*ii)->iname() == item->text(0))
+                  break;
+            }
+      if (ii == synthiInstances.end()) {
+            printf("synthesizerConfig::removeInstanceClicked(): synthi not found\n");
+            return;
+            }
+      int port;
+      for (port = 0; port < MIDI_PORTS; ++port) {
+            if (midiPorts[port].instrument() == *ii)
+                  break;
+            }
+      if (port != MIDI_PORTS) {
+            // synthi is attached
+            midiThread->setMidiDevice(&midiPorts[port], 0);
+            midiPorts[port].setInstrument(genericMidiInstrument);
+            }
+      audio->msgRemoveSynthI(*ii);
+      song->update();
       }
 
 //---------------------------------------------------------
@@ -693,7 +767,7 @@ void MPConfig::songChanged(int)
 void MusE::configMidiPorts()
       {
       if (!midiPortConfig)
-            midiPortConfig = new MPConfig();
+            midiPortConfig = new MPConfig(0, "midiPortConfig");
       midiPortConfig->show();
       }
 
