@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: ctrlcanvas.cpp,v 1.3 2002/02/07 10:02:48 muse Exp $
+//    $Id: ctrlcanvas.cpp,v 1.2 2003/11/19 21:28:40 lunar_shuttle Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -14,6 +14,7 @@
 #include <qlabel.h>
 #include <qcursor.h>
 
+#include "globals.h"
 #include "ctrledit.h"
 #include "midieditor.h"
 #include "icons.h"
@@ -42,10 +43,11 @@ CEvent::CEvent(MidiEvent* e, MidiPart* pt, int v)
 
 bool CEvent::contains(int x1, int x2) const
       {
-      int tick = _event ? _event->posTick() : 0;
-      return ((tick >= x1 && tick < x2)
-         || (ex >= x1 && ex < x2)
-         || (tick < x1 && ex >= x2));
+      int tick1 = _event ? _event->posTick() + _part->posTick() : 0;
+      int tick2 = ex + _part->posTick();
+      return ((tick1 >= x1 && tick1 < x2)
+         || (tick2 >= x1 && tick2 < x2)
+         || (tick1 < x1 && tick2 >= x2));
       }
 
 //---------------------------------------------------------
@@ -53,7 +55,7 @@ bool CEvent::contains(int x1, int x2) const
 //---------------------------------------------------------
 
 CtrlCanvas::CtrlCanvas(MidiEditor* e, QWidget* parent, int xmag,
-   const char* name = 0) : View(parent, xmag, 1, name)
+   const char* name) : View(parent, xmag, 1, name)
       {
       setBg(white);
       editor = e;
@@ -62,6 +64,7 @@ CtrlCanvas::CtrlCanvas(MidiEditor* e, QWidget* parent, int xmag,
       pos[0] = 0;
       pos[1] = 0;
       pos[2] = 0;
+      keyFilter = -1; //Start with no key filter
       connect(song, SIGNAL(posChanged(int, int, bool)), this, SLOT(setPos(int, int, bool)));
 
       setMouseTracking(true);
@@ -206,6 +209,15 @@ void CtrlCanvas::setController(const MidiController& c)
       songChanged(-1);
       }
 
+//---------------------------------------------------------
+//   setKeyfilter
+//---------------------------------------------------------
+void CtrlCanvas::setKeyfilter(char in_keyfilter)
+   {
+   keyFilter = in_keyfilter;
+   songChanged(-1); //Correct?
+   }
+
 //---------------------------------------------------------
 //   leaveEvent
 //---------------------------------------------------------
@@ -327,7 +339,13 @@ void CtrlCanvas::songChanged(int)
                         case MidiController::Velo:
                               if (e->type() != MidiEvent::Note)
                                     break;
-                              items.add(new CEvent(e, part, e->velo()));
+                              //If it's velocity and keyFilter is activated, only add items with the same key
+                              if (keyFilter == -1) { //If no keyFilter, just add it:
+                                 items.add(new CEvent(e, part, e->velo()));
+                                 }
+                              else if (e->dataA() == keyFilter) //If keyFilter matches, add it
+                                 items.add(new CEvent(e, part, e->velo()));
+
                               break;
                         case MidiController::Controller7:
                               if (e->type() != MidiEvent::Ctrl7)
@@ -484,35 +502,31 @@ void CtrlCanvas::newValRamp(int x1, int y1, int x2, int y2)
             if (nval > max)
                   nval = max;
 
+            int tick = x - curPart->posTick();
             MidiEvent* event = 0;
             switch(ctrl.type()) {
                   case MidiController::Controller7:
                         event = new MidiEvent(
-                           curTrack->outPort(), curTrack->outChannel(),
-                           x, MidiEvent::Ctrl7, ctrl.hnum(), nval, 0, 0);
+                           tick, MidiEvent::Ctrl7, ctrl.hnum(), nval, 0, 0);
                         break;
                   case MidiController::Controller14:
                         event = new MidiEvent(
-                           curTrack->outPort(), curTrack->outChannel(),
-                           x, MidiEvent::Ctrl14, ctrl.hnum(), nval, ctrl.lnum(), 0);
+                           tick, MidiEvent::Ctrl14, ctrl.hnum(), nval, ctrl.lnum(), 0);
                         break;
                   case MidiController::RPN:
                         event = new MidiEvent(
-                           curTrack->outPort(), curTrack->outChannel(),
-                           x, MidiEvent::RPN, ctrl.hnum()*128+ctrl.lnum(), nval, 0, 0);
+                           tick, MidiEvent::RPN, ctrl.hnum()*128+ctrl.lnum(), nval, 0, 0);
                         break;
                   case MidiController::NRPN:
                         event = new MidiEvent(
-                           curTrack->outPort(), curTrack->outChannel(),
-                           x, MidiEvent::RPN, ctrl.hnum()*128+ctrl.lnum(), nval, 0, 0);
+                           tick, MidiEvent::RPN, ctrl.hnum()*128+ctrl.lnum(), nval, 0, 0);
                         break;
                   case MidiController::XgSysex:
                   case MidiController::Sysex:
                         break;
                   case MidiController::Pitch:
                         event = new MidiEvent(
-                           curTrack->outPort(), curTrack->outChannel(),
-                           x, MidiEvent::Pitch, nval, 0, 0, 0);
+                           tick, MidiEvent::Pitch, nval, 0, 0, 0);
                         break;
                   case MidiController::Velo:
                         break;
@@ -547,7 +561,7 @@ void CtrlCanvas::changeValRamp(int x1, int y1, int x2, int y2)
 
                   MidiPart* part   = ev->part();
 
-                  int x = event->posTick();
+                  int x = event->posTick() + ev->part()->posTick();
                   int y = (x2==x1) ? y1 : (((y2-y1)*(x-x1))/(x2-x1))+y1;
                   int nval = (((h - y) *  (max-min)) / h) + min;
                   if (nval > max)
@@ -699,39 +713,61 @@ void CtrlCanvas::changeVal(int x1, int x2, int y)
       int nval = (((h - y) *  (max-min)) / h) + min;
       if (nval > max)
             nval = max;
+
       for (ciCEvent i = items.begin(); i != items.end(); ++i) {
-            if ((*i)->contains(x1, x2)) {
-                  CEvent* ev       = *i;
-                  MidiEvent* event = ev->event();
-                  MidiPart* part   = ev->part();
-                  ev->setVal(nval);
-                  if (ctrl.type() == MidiController::Velo) {
-                        if ((event->velo() != nval)) {
+            if (!(*i)->contains(x1, x2))
+                  continue;
+
+            CEvent* ev       = *i;
+            MidiEvent* event = ev->event();
+            MidiPart* part   = ev->part();
+            ev->setVal(nval);
+            if (ctrl.type() == MidiController::Velo) {
+                  if ((event->velo() != nval)) {
+                        MidiEvent* newEvent = new MidiEvent(*event);
+                        newEvent->setVelo(nval);
+                        midiThread->msgChangeEvent(event, newEvent, part, false);
+                        ev->setEvent(newEvent);
+                        changed = true;
+                        }
+                  }
+            else if (ctrl.type() == MidiController::Controller7) {
+                  if (event) {
+                        if ((event->cntrlVal() != nval)) {
                               MidiEvent* newEvent = new MidiEvent(*event);
-                              newEvent->setVelo(nval);
+                              newEvent->setB(nval);
                               midiThread->msgChangeEvent(event, newEvent, part, false);
                               ev->setEvent(newEvent);
                               changed = true;
                               }
                         }
-                  else if (ctrl.type() == MidiController::Controller7) {
-                        if (event) {
-                              if ((event->cntrlVal() != nval)) {
-                                    MidiEvent* newEvent = new MidiEvent(*event);
-                                    newEvent->setB(nval);
-                                    midiThread->msgChangeEvent(event, newEvent, part, false);
-                                    ev->setEvent(newEvent);
-                                    changed = true;
-                                    }
+                  else {
+                        MidiPort* port = &midiPorts[curTrack->outPort()];
+                        int channel = curTrack->outChannel();
+                        int oval = port->iState(channel)->controller[ctrl.hnum()];
+                        if (oval != nval) {
+                              port->iState(channel)->controller[ctrl.hnum()] = nval;
+                              changed = true;
                               }
-                        else {
-                              MidiPort* port = &midiPorts[curTrack->outPort()];
-                              int channel = curTrack->outChannel();
-                              int oval = port->iState(channel)->controller[ctrl.hnum()];
-                              if (oval != nval) {
-                                    port->iState(channel)->controller[ctrl.hnum()] = nval;
-                                    changed = true;
-                                    }
+                        }
+                  }
+            else if (ctrl.type() == MidiController::Pitch) {
+                  if (event) {
+                        if ((event->pitch() != nval)) {
+                              MidiEvent* newEvent = new MidiEvent(
+                                 event->posTick(), MidiEvent::Pitch, nval, 0, 0, 0);
+                              midiThread->msgChangeEvent(event, newEvent, part, false);
+                              ev->setEvent(newEvent);
+                              changed = true;
+                              }
+                        }
+                  else {
+                        MidiPort* port = &midiPorts[curTrack->outPort()];
+                        int channel = curTrack->outChannel();
+                        int oval = port->iState(channel)->pitch;
+                        if (oval != nval) {
+                              port->iState(channel)->pitch = nval;
+                              changed = true;
                               }
                         }
                   }
@@ -760,15 +796,16 @@ void CtrlCanvas::newVal(int x1, int x2, int y)
       bool song_changed = false;
       for (ciCEvent i = items.begin(); i != items.end(); ++i) {
             CEvent* ev = *i;
+            int partTick = ev->part()->posTick();
             MidiEvent* event = ev->event();
             if (event == 0)
                   continue;
-            int x = event->posTick();
-            if (x < xx1)
+            int ax = event->posTick() + partTick;
+            if (ax < xx1)
                   continue;
-            if (x >= xx2)
+            if (ax >= xx2)
                   break;
-            if (x == xx1) {
+            if (ax == xx1) {
                   // change event
                   found = true;
                   ev->setVal(nval);
@@ -784,13 +821,13 @@ void CtrlCanvas::newVal(int x1, int x2, int y)
                               break;
                         case MidiController::Pitch:
                               {
-                              //TODO: check for change
-                              MidiEvent* newEvent = new MidiEvent(
-                                 curTrack->outPort(), curTrack->outChannel(),
-                                 xx1, MidiEvent::Pitch, nval, 0, 0, 0);
-                              midiThread->msgChangeEvent(event, newEvent, ev->part(), false);
-                              ev->setEvent(newEvent);
-                              song_changed = true;
+                              if (event->pitch() != nval) {
+                                    MidiEvent* newEvent = new MidiEvent(
+                                       xx1-partTick, MidiEvent::Pitch, nval, 0, 0, 0);
+                                    midiThread->msgChangeEvent(event, newEvent, ev->part(), false);
+                                    ev->setEvent(newEvent);
+                                    song_changed = true;
+                                    }
                               }
                               break;
                         default:
@@ -798,47 +835,40 @@ void CtrlCanvas::newVal(int x1, int x2, int y)
                               break;
                         }
                   }
-            else if (x < xx2) {
+            else if (ax < xx2) {
                   // delete event
-                  MidiEvent* event = ev->event();
-                  if (event) {
-                        midiThread->msgDeleteEvent(event, ev->part(), false);
-                        song_changed = true;
-                        }
+                  midiThread->msgDeleteEvent(event, ev->part(), false);
+                  song_changed = true;
                   }
             }
       if (!found) {
             // new event
             MidiEvent* event = 0;
+            int tick = xx1 - curPart->posTick();
 
             switch(ctrl.type()) {
                   case MidiController::Controller7:
                         event = new MidiEvent(
-                           curTrack->outPort(), curTrack->outChannel(),
-                           xx1, MidiEvent::Ctrl7, ctrl.hnum(), nval, 0, 0);
+                           tick, MidiEvent::Ctrl7, ctrl.hnum(), nval, 0, 0);
                         break;
                   case MidiController::Controller14:
                         event = new MidiEvent(
-                           curTrack->outPort(), curTrack->outChannel(),
-                           xx1, MidiEvent::Ctrl14, ctrl.hnum(), nval, ctrl.lnum(), 0);
+                           tick, MidiEvent::Ctrl14, ctrl.hnum(), nval, ctrl.lnum(), 0);
                         break;
                   case MidiController::RPN:
                         event = new MidiEvent(
-                           curTrack->outPort(), curTrack->outChannel(),
-                           xx1, MidiEvent::RPN, ctrl.hnum()*128+ctrl.lnum(), nval, 0, 0);
+                           tick, MidiEvent::RPN, ctrl.hnum()*128+ctrl.lnum(), nval, 0, 0);
                         break;
                   case MidiController::NRPN:
                         event = new MidiEvent(
-                           curTrack->outPort(), curTrack->outChannel(),
-                           xx1, MidiEvent::RPN, ctrl.hnum()*128+ctrl.lnum(), nval, 0, 0);
+                           tick, MidiEvent::RPN, ctrl.hnum()*128+ctrl.lnum(), nval, 0, 0);
                         break;
                   case MidiController::XgSysex:
                   case MidiController::Sysex:
                         break;
                   case MidiController::Pitch:
                         event = new MidiEvent(
-                           curTrack->outPort(), curTrack->outChannel(),
-                           xx1, MidiEvent::Pitch, nval, 0, 0, 0);
+                           tick, MidiEvent::Pitch, nval, 0, 0, 0);
                         break;
                   case MidiController::Velo:
                         break;
@@ -864,6 +894,10 @@ void CtrlCanvas::deleteVal(int x1, int x2, int)
       int xx1 = editor->rasterVal1(x1);
       int xx2 = editor->rasterVal2(x2);
 
+      int partTick = curPart->posTick();
+      xx1 -= partTick;
+      xx2 -= partTick;
+
       bool song_changed = false;
       for (ciCEvent i = items.begin(); i != items.end(); ++i) {
             CEvent* ev = *i;
@@ -929,7 +963,7 @@ void CtrlCanvas::pdraw(QPainter& p, const QRect& rect)
             p.setPen(blue);
             for (iCEvent i = items.begin(); i != items.end(); ++i) {
                   CEvent* e = *i;
-                  int tick = mapx(e->event()->posTick());
+                  int tick = mapx(e->event()->posTick() + e->part()->posTick());
                   if (tick <= x)
                         continue;
                   if (tick > x+w)
@@ -947,7 +981,7 @@ void CtrlCanvas::pdraw(QPainter& p, const QRect& rect)
             int lval = -1;
             for (iCEvent i = items.begin(); i != items.end(); ++i) {
                   CEvent* e = *i;
-                  int tick = mapx(e->event() ? e->event()->posTick() : 0);
+                  int tick = mapx(e->event() ? e->event()->posTick() + e->part()->posTick() : 0);
                   if (tick <= x) {
                         if (e->val() == -1)
                               lval = -1;
