@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: device.h,v 1.2 2001/11/20 15:19:31 muse Exp $
+//  $Id: device.h,v 1.1.1.1 2003/10/29 10:05:12 wschweer Exp $
 //
 //  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -16,13 +16,10 @@
 //---------------------------------------------------------
 
 class Device {
-//      int useCount;
 
    public:
       enum DeviceType { MidiDevice, WaveDevice };
 
-   private:
-
    protected:
       QString _name;
       DeviceType _type;
@@ -41,8 +38,6 @@ class Device {
       void setName(const QString& s)   { _name = s; }
       const DeviceType type() const    { return _type; }
       void setDeviceType(DeviceType t) { _type = t; }
-//      bool used() const                { return useCount > 0; }
-//      void use(bool flag)              { useCount += flag ? 1 : -1; }
       int port() const                 { return _port; }
       void setPort(int p)              { _port = p; }
       };
