@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: mess.cpp,v 1.6 2002/02/27 08:48:09 muse Exp $
+//    $Id: mess.cpp,v 1.1.1.1 2003/10/29 10:06:11 wschweer Exp $
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -14,73 +14,11 @@
 #include <sys/time.h>
 #include <errno.h>
 #include <unistd.h>
+#include <cmath>
 
 #include "ladspa.h"
 
-extern void __init();
-
-
-//---------------------------------------------------------
-//   doSetuid
-//    Restore the effective UID to its original value.
-//---------------------------------------------------------
-
-static void doSetuid(uid_t euid, uid_t ruid)
-      {
-      int status;
-
-#ifdef _POSIX_SAVED_IDS
-      status = seteuid (euid);
-#else
-      status = setreuid (ruid, euid);
-#endif
-      if (status < 0) {
-            fprintf(stderr, "Couldn't set uid.\n");
-            }
-      }
-
-//---------------------------------------------------------
-//   undoSetuid
-//    Set the effective UID to the real UID.
-//---------------------------------------------------------
-
-static void undoSetuid(uid_t euid, uid_t ruid)
-      {
-      int status;
-
-#ifdef _POSIX_SAVED_IDS
-      status = seteuid (ruid);
-#else
-      status = setreuid (euid, ruid);
-#endif
-      if (status < 0) {
-            fprintf (stderr, "Couldn't set uid.\n");
-            exit (status);
-            }
-      }
-
-//---------------------------------------------------------
-//   curTime
-//---------------------------------------------------------
-
-static double curTime()
-      {
-      struct timeval t;
-      struct timezone tz;
-      gettimeofday(&t, &tz);
-      return (double)((double)t.tv_sec + (t.tv_usec / 1000000.0));
-      }
-
-//---------------------------------------------------------
-//   midi_run
-//---------------------------------------------------------
-
-static void* midi_run(void* d)
-      {
-      Mess* s = (Mess*) d;
-      s->midiRun();
-      pthread_exit(0);
-      }
+extern void initMess();
 
 #ifdef RTCAP
 //---------------------------------------------------------
@@ -97,41 +35,117 @@ void getCapabilities()
 #endif
 
 //---------------------------------------------------------
-//   midiRun
+//   doSetuid
+//    Restore the effective UID to its original value.
 //---------------------------------------------------------
 
-void Mess::midiRun()
+#ifdef _POSIX_SAVED_IDS
+static void doSetuid(uid_t euid, uid_t)
+#else
+static void doSetuid(uid_t euid, uid_t ruid)
+#endif
       {
 #ifdef RTCAP
       extern void getCapabilities();
       getCapabilities();
 #else
-      doSetuid(euid, ruid);
+      int status;
+
+#ifdef _POSIX_SAVED_IDS
+      status = seteuid (euid);
+#else
+      status = setreuid (ruid, euid);
 #endif
-      struct sched_param sp;
+      if (status < 0) {
+            perror("doSetuid: Couldn't set uid.\n");
+            }
+#endif
+      }
+
+//---------------------------------------------------------
+//   undoSetuid
+//    Set the effective UID to the real UID.
+//---------------------------------------------------------
+
+static void undoSetuid(uid_t euid, uid_t ruid)
+      {
+#ifndef RTCAP
+      int status;
+
+#ifdef _POSIX_SAVED_IDS
+      status = seteuid (ruid);
+#else
+      status = setreuid (euid, ruid);
+#endif
+      if (status < 0) {
+            fprintf(stderr, "undoSetuid: Couldn't set uid (eff:%d,real:%d): %s\n",
+               euid, ruid, strerror(errno));
+            exit (status);
+            }
+#endif
+      }
+
+//---------------------------------------------------------
+//   curTime
+//---------------------------------------------------------
+
+static double curTime()
+      {
+      struct timeval t;
+      gettimeofday(&t, 0);
+      return (double)((double)t.tv_sec + (t.tv_usec / 1000000.0));
+      }
+
+//---------------------------------------------------------
+//   midi_run
+//---------------------------------------------------------
+
+static void* midi_run(void* d)
+      {
+      Mess* s = (Mess*) d;
+      s->midiRun();
+      pthread_exit(0);
+      }
+
+//---------------------------------------------------------
+//   midiRun
+//---------------------------------------------------------
+
+void Mess::midiRun()
+      {
+#define BIG_ENOUGH_STACK (1024*1024*1)
+      char buf[BIG_ENOUGH_STACK];
+      for (int i = 0; i < BIG_ENOUGH_STACK; i++)
+            buf[i] = i;
+#undef BIG_ENOUGH_STACK
+
       pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, 0);
       pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, 0);
 
-      sp.sched_priority = 20;
-	if (sched_setscheduler(0, SCHED_RR, &sp) == 0) {
-            fprintf(stderr, "mess: %s: running as realtime process now (priority %d)\n",
-               _name, 20);
+      int policy;
+      if ((policy = sched_getscheduler (0)) < 0) {
+            printf("Cannot get current client scheduler: %s\n", strerror(errno));
             }
-	else {
-            fprintf(stderr, "mess: %s: Can't get realtime priority: %s\n",
-               _name, strerror(errno));
-            }
-      if (mlockall(MCL_FUTURE))
-            perror("mess: Cannot lock memory");
-#ifndef RTCAP
-      undoSetuid(euid, ruid);
-#endif
+
+      printf("Mess: MidiThread set to %s priority 60\n",
+         policy == SCHED_FIFO ? "SCHED_FIFO" : "SCHED_OTHER");
+
+      pthread_mutex_lock(&messLock);
+      pthread_cond_signal(&messReady);
+      pthread_mutex_unlock(&messLock);
 
       MEvent* ev;
       for (;;) {
             int n = poll(pfd, npfd, -1);  // wait infinite for event
-            if (n <= 0)
+            if (n < 0) {
+                  perror("MusE Mess: poll failed");
+                  break;
+                  }
+            if (n == 0) {
+                  fprintf(stderr, "MusE: Mess: poll return zero\n");
+                  sleep(1);
                   continue;
+                  }
             for(;;) {
                   snd_seq_event_t* event;
                   int rv = snd_seq_event_input(alsaSeq, &event);
@@ -145,21 +159,24 @@ void Mess::midiRun()
                   if (event->type == SND_SEQ_EVENT_PORT_START)
                         continue;
 
-                  unsigned pos = int ((curTime() - frame) * _sampleRate);
+                  unsigned pos = lrint((curTime() - frame) * _sampleRate);
 
                   ev = 0;
                   switch(event->type) {
 //                        case SND_SEQ_EVENT_PORT_START:
                         case SND_SEQ_EVENT_PORT_EXIT:
+                        case SND_SEQ_EVENT_PORT_SUBSCRIBED:
                               break;
 
                         case SND_SEQ_EVENT_NOTEON:
                         case SND_SEQ_EVENT_KEYPRESS:
                         case SND_SEQ_EVENT_NOTEOFF:
-                              ev = new MEvent(pos, event->type,
-                                      event->data.note.channel,
-                                      event->data.note.note,
-                                      event->data.note.velocity);
+                              ev = allocMEvent();
+                              ev->setSamplePos(pos);
+                              ev->setType(event->type);
+                              ev->setChan(event->data.note.channel);
+                              ev->setDataA(event->data.note.note);
+                              ev->setDataB(event->data.note.velocity);
                               break;
 
                         case SND_SEQ_EVENT_CONTROL14:
@@ -168,18 +185,22 @@ void Mess::midiRun()
                         case SND_SEQ_EVENT_CONTROLLER:
                         case SND_SEQ_EVENT_PGMCHANGE:
                         case SND_SEQ_EVENT_CHANPRESS:
-                              ev = new MEvent(pos, event->type,
-                                      event->data.control.channel,
-                                      event->data.control.param,
-                                      event->data.control.value);
+                              ev = allocMEvent();
+                              ev->setSamplePos(pos);
+                              ev->setType(event->type);
+                              ev->setChan(event->data.control.channel);
+                              ev->setDataA(event->data.control.param);
+                              ev->setDataB(event->data.control.value);
                               break;
 
 
                         case SND_SEQ_EVENT_PITCHBEND:
-                              ev = new MEvent(pos, event->type,
-                                      event->data.control.channel,
-                                      (event->data.control.value >> 7) & 0x7f,
-                                      event->data.control.value & 0x7f);
+                              ev = allocMEvent();
+                              ev->setSamplePos(pos);
+                              ev->setType(event->type);
+                              ev->setChan(event->data.control.channel);
+                              ev->setDataA((event->data.control.value >> 7) & 0x7f);
+                              ev->setDataB(event->data.control.value & 0x7f);
                               break;
 
                         case SND_SEQ_EVENT_SYSEX:
@@ -187,7 +208,11 @@ void Mess::midiRun()
                               int len = event->data.ext.len;
                               unsigned char* data = new unsigned char[len];
                               memcpy(data, event->data.ext.ptr, len);
-                              ev = new MEvent(pos, event->type, 0, 0, 0, data, len);
+                              ev = allocMEvent();
+                              ev->setSamplePos(pos);
+                              ev->setType(event->type);
+                              ev->setData(data);
+                              ev->setDataLen(len);
                               }
                               break;
 
@@ -219,89 +244,87 @@ void Mess::midiRun()
 Mess::Mess(const char* cn, int chn)
    : _className(strdup(cn))
       {
-      _channels   = chn;
-      outputPorts = new float*[_channels];
-      _name       = 0;
-      alsaSeq     = 0;
-      pfd         = 0;
-      npfd        = 0;
-      startFrame  = 0.0;
-      _sampleRate = 44100;
+      realTimePriority = 60;
+      head             = 0;
+      chunks           = 0;
+      _channels        = chn;
+      outputPorts      = new float*[_channels];
+      _name            = 0;
+      alsaSeq          = 0;
+      pfd              = 0;
+      npfd             = 0;
+      startFrame       = 0.0;
+      _sampleRate      = 44100;          // default
       pthread_mutex_init(&lock, 0);
+      pthread_mutex_init(&messLock, 0);
+      pthread_cond_init(&messReady, 0);
       }
 
 //---------------------------------------------------------
 //   Mess
 //---------------------------------------------------------
 
-#if (SND_LIB_MAJOR==0) && (SND_LIB_MINOR==9)
 Mess::~Mess()
       {
+      pthread_cancel(midiThread);
+      pthread_join(midiThread, 0);
+      pthread_mutex_destroy(&lock);
+      pthread_mutex_destroy(&messLock);
+
+      Chunk* n = chunks;
+      while (n) {
+            Chunk* p = n;
+            n = n->next;
+            delete p;
+            }
+
       delete[] outputPorts;
       if (_name)
             delete _name;
       if (_className)
             delete _className;
-      if (alsaSeq) {
-#ifdef snd_seq_port_info_alloca
-            int error = snd_seq_delete_port(alsaSeq, _alsaPort.port);
-#else
-            snd_seq_port_info_t *pinfo;
-            pinfo = (snd_seq_port_info_t*)alloca(sizeof(*pinfo));
-	      memset(pinfo, 0, sizeof(*pinfo));
-	      pinfo->client = _alsaPort.client;
-	      pinfo->port   = _alsaPort.port;
-            int error     = snd_seq_delete_port(alsaSeq, pinfo);
-#endif
-            if (error < 0) {
-                  fprintf(stderr, "ALSA: cannot delete port: %s\n",
-                     snd_strerror(error));
-                  }
+      if (!alsaSeq) {
+            printf("~Mess(): no seq!\n");
+            return;
+            }
+      int error = snd_seq_delete_port(alsaSeq, _alsaPort.port);
+      if (error) {
+            fprintf(stderr, "ALSA: cannot delete port: %s\n",
+               snd_strerror(error));
+            }
+      // why does this not remove client from client table?:
+      error = snd_seq_close(alsaSeq);
+      if (error) {
+            fprintf(stderr, "ALSA: cannot close seq: %s\n",
+               snd_strerror(error));
             }
       }
-#endif
 
 //---------------------------------------------------------
-//   Mess
+//   grow
 //---------------------------------------------------------
 
-#if (SND_LIB_MAJOR==0) && (SND_LIB_MINOR==5)
-Mess::~Mess()
+void Mess::grow()
       {
-      delete outputPorts;
-      if (_name)
-            delete _name;
-      if (_className)
-            delete _className;
-      if (alsaSeq) {
-            snd_seq_port_info_t* pinfo = new snd_seq_port_info_t;
-	      memset(pinfo, 0, sizeof(*pinfo));
-	      pinfo->client = _alsaPort.client;
-	      pinfo->port   = _alsaPort.port;
-            int error     = snd_seq_delete_port(alsaSeq, pinfo);
-            if (error < 0) {
-                  fprintf(stderr, "ALSA: cannot delete port: %s\n",
-                     snd_strerror(error));
-                  }
-            delete pinfo;
+      Chunk* n = new Chunk;
+      n->next  = chunks;
+      chunks   = n;
+
+      char* start = n->mem;
+      char* last  = &start[(nelem-1)*sizeof(MEvent)];
+
+      for (char* p = start; p < last; p += sizeof(MEvent)) {
+            reinterpret_cast<Verweis*>(p)->next
+              = reinterpret_cast<Verweis*>(p + sizeof(MEvent));
             }
-      }
-#endif
-
-//---------------------------------------------------------
-//   setName
-//---------------------------------------------------------
-
-void Mess::setName(const char* s)
-      {
-      _name = strdup(s);
+      reinterpret_cast<Verweis*>(last)->next = 0;
+      head = reinterpret_cast<Verweis*>(start);
       }
 
 //---------------------------------------------------------
 //   registerAlsa
 //---------------------------------------------------------
 
-#if (SND_LIB_MAJOR==0) && (SND_LIB_MINOR==9)
 void Mess::registerAlsa()
       {
       //-----------------------------------------
@@ -312,7 +335,7 @@ void Mess::registerAlsa()
       if (alsaSeq == 0) {
             int error = snd_seq_open(&alsaSeq, "hw", SND_SEQ_OPEN_DUPLEX|SND_SEQ_NONBLOCK, 0);
             if (error < 0) {
-                  fprintf(stderr, "Could not open ALSA sequencer: %s\n",
+                  fprintf(stderr, "Mess: Could not open ALSA sequencer: %s\n",
                      snd_strerror(error));
                   alsaSeq = 0;
                   return;
@@ -330,50 +353,36 @@ void Mess::registerAlsa()
 
       //-----------------------------------------
       //  find an unused alsa port name
+      //  find muse port
       //-----------------------------------------
 
+      bool museFound = false;
       char buffer[256];
       for (int i = 1;; ++i) {
             bool found = false;
             sprintf(buffer, "%s-%d", _className, i);
 
             snd_seq_client_info_t* cinfo;
-#ifdef snd_seq_client_info_alloca
             snd_seq_client_info_alloca(&cinfo);
             snd_seq_client_info_set_client(cinfo, -1);
-#else
-            cinfo = (snd_seq_client_info_t*)alloca(sizeof(*cinfo));
-            memset(cinfo, 0, sizeof(*cinfo));
-            cinfo->client = -1;
-#endif
             while (snd_seq_query_next_client(alsaSeq, cinfo) >= 0) {
-#ifdef SND_SEQ_GROUP_SYSTEM
-                  if (strcmp(cinfo->group, SND_SEQ_GROUP_SYSTEM) == 0)
-                        continue;
-#endif
                   snd_seq_port_info_t *pinfo;
-#ifdef snd_seq_port_info_alloca
                   snd_seq_port_info_alloca(&pinfo);
                   snd_seq_port_info_set_client(pinfo, snd_seq_client_info_get_client(cinfo));
                   snd_seq_port_info_set_port(pinfo, -1);
-#else
-                  pinfo = (snd_seq_port_info_t*)alloca(sizeof(*pinfo));
-                  memset(pinfo, 0, sizeof(*pinfo));
-                  pinfo->client = cinfo->client;
-                  pinfo->port = -1;
-#endif
                   while (snd_seq_query_next_port(alsaSeq, pinfo) >= 0) {
                         const char* portname;
-#ifdef snd_seq_port_info_alloca
                         portname = snd_seq_port_info_get_name(pinfo);
-#else
-                        portname = pinfo->name;
-#endif
-// printf("compare port <%s> <%s>\n", portname, buffer);
                         if (strcmp(portname, buffer) == 0) {
                               found = true;
                               break;
                               }
+                        if (strcmp(portname, "MusE Port 0") == 0) {
+                              museFound = true;
+                              snd_seq_port_info_set_client(pinfo, snd_seq_client_info_get_client(cinfo));
+                              _musePort.port   = snd_seq_port_info_get_port(pinfo);
+                              _musePort.client = snd_seq_client_info_get_client(cinfo);
+                              }
                         }
                   if (found)
                         break;
@@ -381,63 +390,19 @@ void Mess::registerAlsa()
             if (!found)
                   break;
             }
+      if (!museFound) {
+            printf("Mess: muse port not found!\n");
+            return;
+            }
       _name = strdup(buffer);
 
-      // search muse port
-      bool found = false;
-      snd_seq_client_info_t* cinfo;
-#ifdef snd_seq_client_info_alloca
-      snd_seq_client_info_alloca(&cinfo);
-      snd_seq_client_info_set_client(cinfo, -1);
-#else
-      cinfo = (snd_seq_client_info_t*)alloca(sizeof(*cinfo));
-      memset(cinfo, 0, sizeof(*cinfo));
-      cinfo->client = -1;
-#endif
-      while (snd_seq_query_next_client(alsaSeq, cinfo) >= 0) {
-            found = false;
-            snd_seq_port_info_t *pinfo;
-#ifdef snd_seq_port_info_alloca
-            snd_seq_port_info_alloca(&pinfo);
-            snd_seq_port_info_set_client(pinfo, snd_seq_client_info_get_client(cinfo));
-            snd_seq_port_info_set_port(pinfo, -1);
-#else
-            pinfo = (snd_seq_port_info_t*)alloca(sizeof(*pinfo));
-            memset(pinfo, 0, sizeof(*pinfo));
-            pinfo->client = cinfo->client;
-            pinfo->port = -1;
-#endif
-            while (snd_seq_query_next_port(alsaSeq, pinfo) >= 0) {
-                  const char* portname;
-#ifdef snd_seq_port_info_alloca
-                  portname = snd_seq_port_info_get_name(pinfo);
-#else
-                  portname = pinfo->name;
-#endif
-                  if (strcmp(portname, "MusE Port 0") == 0) {
-                        found = true;
-                  snd_seq_port_info_set_client(pinfo, snd_seq_client_info_get_client(cinfo));
-
-                        _musePort.port   = snd_seq_port_info_get_port(pinfo);
-                        _musePort.client = snd_seq_client_info_get_client(cinfo);
-                        break;
-                        }
-                  }
-            if (found)
-                  break;
-            }
-      if (!found) {
-            printf("muse port not found!\n");
-            }
-
       //-----------------------------------------
       //    create port to alsa sequencer
       //-----------------------------------------
 
       int port  = snd_seq_create_simple_port(alsaSeq, _name,
          SND_SEQ_PORT_CAP_WRITE | SND_SEQ_PORT_CAP_SUBS_WRITE,
-      SND_SEQ_PORT_TYPE_APPLICATION);
-// printf("create new alsa port <%s>\n", _name);
+         SND_SEQ_PORT_TYPE_APPLICATION);
       if (port < 0) {
             perror("create port");
             return;
@@ -445,134 +410,47 @@ void Mess::registerAlsa()
       _alsaPort.port   = port;
       _alsaPort.client = snd_seq_client_id(alsaSeq);
 
+      //----------------------------------------------
       // create the midi thread
-      int err = pthread_create(&midiThread, 0, midi_run, this);
+      // with SCHED_FIFO and priority 60
+
+      pthread_attr_t* attributes = 0;
+      doSetuid(euid, ruid);
+      if (realTimePriority) {
+            struct sched_param rt_param;
+            memset(&rt_param, 0, sizeof(rt_param));
+            rt_param.sched_priority = 60;
+
+            attributes = new pthread_attr_t;
+            pthread_attr_init(attributes);
+
+            if (pthread_attr_setschedpolicy(attributes, SCHED_FIFO)) {
+                  printf("Mess: cannot set FIFO scheduling class for RT thread\n");
+                  }
+            if (pthread_attr_setschedparam(attributes, &rt_param)) {
+                  printf("Mess: Cannot set scheduling priority for RT thread (%s)\n", strerror(errno));
+                  }
+            if (pthread_attr_setscope(attributes, PTHREAD_SCOPE_SYSTEM)) {
+                  printf("Mess: Cannot set scheduling scope for RT thread\n");
+                  }
+            }
+      pthread_mutex_lock(&messLock);
+
+      int err = pthread_create(&midiThread, attributes, midi_run, this);
+
       if (err) {
             printf("Mess: Couldn't create midi thread: %s\n",
                strerror(errno));
+      } else
+	      pthread_cond_wait(&messReady, &messLock);
+      pthread_mutex_unlock(&messLock);
+
+      if (attributes) {
+            pthread_attr_destroy(attributes);
+            delete attributes;
             }
+      undoSetuid(euid, ruid);
       }
-#endif
-
-#if (SND_LIB_MAJOR==0) && (SND_LIB_MINOR==5)
-//---------------------------------------------------------
-//   registerAlsa
-//    ALSA 5 version
-//---------------------------------------------------------
-
-void Mess::registerAlsa()
-      {
-      //-----------------------------------------
-      //  connect to ALSA and get poll
-      //  descriptors
-      //-----------------------------------------
-      if (alsaSeq == 0) {
-            int error = snd_seq_open(&alsaSeq, SND_SEQ_OPEN|SND_SEQ_NONBLOCK);
-            if (error < 0) {
-                  fprintf(stderr, "Could not open ALSA sequencer: %s\n",
-                     snd_strerror(error));
-                  alsaSeq = 0;
-                  return;
-                  }
-            }
-      int alsaSeqFd = snd_seq_file_descriptor(alsaSeq);
-      npfd         = 1;
-      pfd          = new pollfd;
-      pfd->fd      = alsaSeqFd;
-      pfd->events  = POLLIN;
-      pfd->revents = 0;
-
-      //-----------------------------------------
-      //  find an unused alsa port name
-      //-----------------------------------------
-
-      snd_seq_system_info_t sysinfo;
-      int error = snd_seq_system_info(alsaSeq, &sysinfo);
-      if (error < 0) {
-            fprintf(stderr, "Cound not get ALSA sequencer info: %s\n",
-               snd_strerror(error));
-            return;
-            }
-
-      char buffer[256];
-      buffer[0] = 0;
-      for (int i = 1;; ++i) {
-            bool found = false;
-            sprintf(buffer, "%s-%d", _className, i);
-            for (int client = 0; client < sysinfo.clients; ++client) {
-                  snd_seq_client_info_t cinfo;
-                  if (snd_seq_get_any_client_info(alsaSeq, client, &cinfo) < 0)
-                        continue;
-
-                  for (int port = 0; port < sysinfo.ports; port++) {
-                        snd_seq_port_info_t pinfo;
-
-                        if (snd_seq_get_any_port_info(alsaSeq, client, port, &pinfo) < 0)
-                              continue;
-                        const char* portname = pinfo.name;
-                        if (strcmp(portname, buffer) == 0) {
-                              found = true;
-                              break;
-                              }
-                        }
-                  if (found)
-                        break;
-                  }
-            if (!found)
-                  break;
-            }
-      _name = strdup(buffer);
-
-      // search muse port
-      bool found;
-      for (int client = 0; client < sysinfo.clients; ++client) {
-            found = false;
-            snd_seq_client_info_t cinfo;
-            if (snd_seq_get_any_client_info(alsaSeq, client, &cinfo) < 0)
-                  continue;
-            for (int port = 0; port < sysinfo.ports; port++) {
-                  snd_seq_port_info_t pinfo;
-
-                  if (snd_seq_get_any_port_info(alsaSeq, client, port, &pinfo) < 0)
-                        continue;
-                  const char* portname = pinfo.name;
-                  if (strcmp(portname, "MusE Port 0") == 0) {
-                        found = true;
-                        _musePort.port   = pinfo.port;
-                        _musePort.client = cinfo.client;
-                        break;
-                        }
-                  }
-            if (found)
-                  break;
-            }
-      if (!found) {
-            printf("muse port not found!\n");
-            }
-
-      //-----------------------------------------
-      //    create port to alsa sequencer
-      //-----------------------------------------
-
-      int port  = snd_seq_create_simple_port(alsaSeq, _name,
-         SND_SEQ_PORT_CAP_WRITE | SND_SEQ_PORT_CAP_SUBS_WRITE,
-      SND_SEQ_PORT_TYPE_APPLICATION);
-// printf("create new alsa port <%s>\n", _name);
-      if (port < 0) {
-            perror("create port");
-            return;
-            }
-      _alsaPort.port   = port;
-      _alsaPort.client = snd_seq_client_id(alsaSeq);
-
-      // create the midi thread
-      int err = pthread_create(&midiThread, 0, midi_run, this);
-      if (err) {
-            printf("Mess: Couldn't create midi thread: %s\n",
-               strerror(errno));
-            }
-      }
-#endif
 
 //---------------------------------------------------------
 //   getPatchName
@@ -610,19 +488,12 @@ void Mess::sendSysex(const unsigned char* p, int n)
       pp += n;
       *pp = 0xf7;
 
-#if (SND_LIB_MAJOR==0) && (SND_LIB_MINOR==5)
-      int alsaSeqFd = snd_seq_file_descriptor(alsaSeq);
-      int error = write(alsaSeqFd, buf, len);
-      if (error != len)
-            fprintf(stderr, "sysex: midi write error: %s\n", snd_strerror(error));
-#endif
-
-#if (SND_LIB_MAJOR==0) && (SND_LIB_MINOR==9)
       int error = snd_seq_event_output_direct(alsaSeq, &event);
       if (error < 0)
-            fprintf(stderr, "sysex: midi write error: %s\n", snd_strerror(error));
-#endif
-      delete buf;
+            fprintf(stderr, "sysex: src:%d:%d-dst:%d:%d midi write error (n=%d): %s\n",
+                _alsaPort.port, _alsaPort.client, _musePort.port, _musePort.client,
+                n, snd_strerror(error));
+      delete[] buf;
       }
 
 //---------------------------------------------------------
@@ -635,10 +506,11 @@ void Mess::processreplace(int len)
       startFrame = curTime();
       pthread_mutex_unlock(&lock);
 
-      int offset = 0;
       for (int channel = 0; channel < _channels; ++channel)
             memset(outputPorts[channel], 0, len * sizeof(float));
 
+      int offset = 0;
+
       //---------------------------------------------------
       //  process all midi events for this buffer
       //---------------------------------------------------
@@ -655,7 +527,10 @@ void Mess::processreplace(int len)
                   }
             MEvent* e = events.front();
             int samplePos = e->samplePos();
+            if (samplePos < 0)                  // ???
+                  samplePos = 0;
             if (samplePos >= len) {
+                  samplePos %= len;
                   for (iMEvent i = events.begin(); i != events.end(); ++i) {
                         (*i)->adjustSamplePos(len);
                         }
@@ -671,10 +546,11 @@ void Mess::processreplace(int len)
 
             int samplesToSynthesize = samplePos - offset;
 
+// printf("write %d %d  %d\n", samplesToSynthesize, offset, samplePos);
             write(samplesToSynthesize, outputPorts, offset);
             offset += samplesToSynthesize;
             processEvent(e);
-            delete e;
+            freeMEvent(e);
             }
 
       //---------------------------------------------------
@@ -682,7 +558,10 @@ void Mess::processreplace(int len)
       //---------------------------------------------------
 
       int samplesToSynthesize = len - offset;
-      write(samplesToSynthesize, outputPorts, offset);
+      if (samplesToSynthesize > 0) {
+// printf("write %d %d\n", samplesToSynthesize, offset);
+            write(samplesToSynthesize, outputPorts, offset);
+            }
       }
 
 //---------------------------------------------------------
@@ -833,7 +712,6 @@ static void connect(LADSPA_Handle Instance, unsigned long port, float* data)
 
 static void activate(LADSPA_Handle instance)
       {
-      ((Mess *)instance)->init();
       ((Mess *)instance)->registerAlsa();
       }
 
@@ -902,7 +780,7 @@ static LADSPA_Descriptor descriptor = {
 
 const LADSPA_Descriptor* ladspa_descriptor(unsigned long i)
       {
-      __init();         // create ladspa descritpor
+      initMess();         // create ladspa descriptor
       return (i == 0) ? &descriptor : 0;
       }
 
@@ -921,6 +799,6 @@ void Mess::initMess(unsigned long id, const char* label, const char* maker,
       descriptor.Maker     = maker;
       descriptor.Copyright = copyright;
       descriptor.PortCount = ports;
-      xinstantiate          = inst;
+      xinstantiate         = inst;
       }
 
