@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: event.cpp,v 1.3 2002/02/27 08:48:09 muse Exp $
+//  $Id: event.cpp,v 1.1.1.1 2003/10/29 10:05:08 wschweer Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -14,9 +14,11 @@
 #include "midiport.h"
 #include "globals.h"
 #include "event.h"
+#include "mpevent.h"
 #include "drummap.h"
 #include "marker/marker.h"
 #include "minstrument.h"
+#include "wave.h"
 
 #include <unistd.h>
 #include <stdio.h>
@@ -33,26 +35,20 @@
 Event::Event()
       {
       refs      = 0;
-      _port     = 0;
-      _channel  = 0;
       _selected = false;
       }
 
-Event::Event(int p, int ch, int t, int l)
+Event::Event(int t, int l)
       {
       refs     = 0;
       setPosTick(t);
       setLenTick(l);
-      _port     = p;
-      _channel  = ch;
       _selected = false;
       }
 Event::Event(const Event& ev)
   : PosLen(ev)
       {
       refs      = 0;
-      _port     = ev._port;
-      _channel  = ev._channel;
       _selected = ev._selected;
       }
 
@@ -68,39 +64,67 @@ void Event::move(int offset)
 MidiEvent::MidiEvent()
       {
       _type    = NoEvent;
-      _data    = 0;
+      edata    = 0;
       _enh     = 0;
       a        = 0;
       b        = 0;
       c        = 0;
       _voice   = 0;
       _stem    = 0;
-      _dataLen = 0;
       _trk     = 0;
       }
 
-MidiEvent::MidiEvent(int p, int ch, int tick, EventType t,
-   int _a, int _b, int _c, int len) : Event(p, ch, tick, len)
+MidiEvent::MidiEvent(int tick, EventType t,
+   int _a, int _b, int _c, int len) : Event(tick, len)
       {
       _type    = t;
-      _data    = 0;
+      edata    = 0;
       a        = _a;
       b        = _b;
       c        = _c;         // velo off
       _enh     = 0;
       _voice   = 0;
       _stem    = 0;
-      _dataLen = 0;
       _trk     = 0;
       }
 
-MidiEvent::MidiEvent(int p, int ch, int tick, EventType t,
-   int len, const unsigned char* ptr) : Event(p, ch, tick, 0)
+MidiEvent::MidiEvent(const MidiPlayEvent* ev)
+   : Event()
+      {
+      edata    = 0;
+      c        = 0;         // velo off
+      _enh     = 0;
+      _voice   = 0;
+      _stem    = 0;
+      _trk     = 0;
+      switch(ev->type()) {
+            case 0x80:  _type = NoteOff; break;
+            case 0x90:  _type = Note;    break;
+            case 0xa0:  _type = PAfter;  break;
+            case 0xb0:  _type = Ctrl7;   break;
+            case 0xc0:  _type = Program; break;
+            case 0xd0:  _type = CAfter;  break;
+            case 0xe0:
+                  _type = Pitch;
+                  a = ((ev->dataB() << 7) + ev->dataA()) - 8192;
+                  b = 0;
+                  return;
+            case 0xf0:
+                  edata = ev->eventData();
+                  ++(edata->refCount);
+                  _type = Sysex;
+                  break;
+            }
+      a = ev->dataA();
+      b = ev->dataB();
+      }
+
+MidiEvent::MidiEvent(int tick, EventType t,
+   int len, const unsigned char* ptr) : Event(tick, 0)
       {
       _type    = t;
-      _dataLen = len;
-      _data    = new unsigned char[len];
-      memcpy(_data, ptr, len);
+      edata    = new EvData(ptr, len);
+      edata->refCount = 1;
       _enh     = 0;
       a        = 0;
       b        = 0;
@@ -122,26 +146,24 @@ MidiEvent::MidiEvent(const MidiEvent& ev)
       _stem      = ev._stem;
       _trk       = ev._trk;
       attributes = ev.attributes;
-      _dataLen   = ev._dataLen;
-      _data      = ev._data;
-      if (ev._data) {
-            _data = new unsigned char[ev._dataLen];
-            memcpy(_data, ev._data, ev._dataLen);
-            }
+      edata      = ev.edata;
+      if (edata)
+            ++(edata->refCount);
       }
+
 MidiEvent::~MidiEvent()
       {
-      if (_data)
-            delete _data;
+      if (edata && (--(edata->refCount) == 0))
+            delete edata;
       }
 
 //---------------------------------------------------------
 //   MidiEvent::mid
 //---------------------------------------------------------
 
-Event* MidiEvent::mid(const Pos& b, const Pos& e)
+Event* MidiEvent::mid(int b, int e)
       {
-      if (posTick() < b.posTick() || posTick() >= e.posTick())
+      if (posTick() < b || posTick() >= e)
             return 0;
       return new MidiEvent(*this);
       }
@@ -156,8 +178,8 @@ WaveEvent::WaveEvent()
       _clip = 0;
       }
 
-WaveEvent::WaveEvent(int p, int c, int tick, Clip* cl)
-   : Event(p, c, tick, 0)
+WaveEvent::WaveEvent(int tick, Clip* cl)
+   : Event(tick, 0)
       {
       setType(SAMPLES);
       setLenSample(cl->samples());
@@ -168,22 +190,21 @@ WaveEvent::WaveEvent(int p, int c, int tick, Clip* cl)
 //   WaveEvent::mid
 //---------------------------------------------------------
 
-Event* WaveEvent::mid(const Pos& b, const Pos& e)
+Event* WaveEvent::mid(int b, int e)
       {
       WaveEvent* ev = new WaveEvent(*this);
-      ev->setPort(_port);
-      ev->setChannel(_channel);
 
-      int offset = b.posSample() - posSample();
-      int end = posSample() + lenSample();
-      if (e.posSample() < end)
-            end = e.posSample();
+      int offset = b - posSample();
+      int end    = endSample();
+      if (e < end)
+            end = e;
 
-      Clip* cl = new Clip(_clip->file(), _clip->spos()+offset, end-b.posSample());
+      int len = end - b;
+      Clip* cl = new Clip(_clip->file1(), _clip->spos()+offset, len);
       cl->incRef();
       ev->setClip(cl);
-      ev->setPos(b);
-      ev->setLenSample(e.posSample()-b.posSample());
+      ev->setPosSample(b);
+      ev->setLenSample(len);
       return ev;
       }
 
@@ -204,8 +225,7 @@ void AttributeList::addLyrics(const QString& s)
 
 bool MidiEvent::isNoteOff(MidiEvent* e)
       {
-      return (e->isNoteOff() && (e->pitch() == a)
-         && e->channel() == _channel && e->port() == _port);
+      return (e->isNoteOff() && (e->pitch() == a));
       }
 
 //---------------------------------------------------------
@@ -243,11 +263,11 @@ void MidiEvent::dump() const
             case RPN:       p = "RPN     "; break;
             default:        p = "??      "; break;
             }
-      printf("  <%s> ch:%d prt:%d a:0x%x(%d) b:0x%x(%d) c:0x%x(%d), len %3d trk:%p\n",
-         p, _channel+1, _port+1, a, a, b, b, c, c, _dataLen, _trk);
-      if (_data) {
-            for (int i = 0; i < _dataLen; ++i) {
-                  printf("%02x ", _data[i]);
+      printf("  <%s> a:0x%x(%d) b:0x%x(%d) c:0x%x(%d), trk:%p\n",
+         p, a, a, b, b, c, c, _trk);
+      if (edata) {
+            for (int i = 0; i < edata->dataLen; ++i) {
+                  printf("%02x ", edata->data[i]);
                   }
             printf("\n");
             }
@@ -256,6 +276,37 @@ void MidiEvent::dump() const
 //      unsigned char* _data;
       }
 
+//---------------------------------------------------------
+//   readEventList
+//---------------------------------------------------------
+
+void EventList::read(Xml& xml, const char* name)
+      {
+      for (;;) {
+            Xml::Token token = xml.parse();
+            const QString& tag = xml.s1();
+            switch (token) {
+                  case Xml::Error:
+                  case Xml::End:
+                        return;
+                  case Xml::TagStart:
+                        if (tag == "event") {
+                              Event* e = new MidiEvent();
+                              e->read(xml);
+                              add(e);
+                              }
+                        else
+                              xml.unknown("readEventList");
+                        break;
+                  case Xml::TagEnd:
+                        if (tag == name)
+                              return;
+                  default:
+                        break;
+                  }
+            }
+      }
+
 //---------------------------------------------------------
 //   EventList
 //---------------------------------------------------------
@@ -353,7 +404,7 @@ void EventList::dump() const
 //   MidiEvent::write
 //---------------------------------------------------------
 
-void MidiEvent::write(int level, Xml& xml, int offset=0) const
+void MidiEvent::write(int level, Xml& xml, int offset) const
       {
       xml.nput(level++, "<event tick=\"%d\"", posTick() + offset);
       switch(_type) {
@@ -364,11 +415,11 @@ void MidiEvent::write(int level, Xml& xml, int offset=0) const
                   xml.nput(" type=\"%d\"", _type);
                   break;
             }
-      if (_data) {
-            xml.nput(" datalen=\"%d\">\n", _dataLen);
+      if (edata) {
+            xml.nput(" datalen=\"%d\">\n", edata->dataLen);
             xml.nput(level, "");
-            for (int i = 0; i < _dataLen; ++i)
-                  xml.nput("%02x ", _data[i] & 0xff);
+            for (int i = 0; i < edata->dataLen; ++i)
+                  xml.nput("%02x ", edata->data[i] & 0xff);
             xml.nput("\n");
             xml.tag(level, "/event");
             }
@@ -465,14 +516,15 @@ void MidiEvent::readAttr(Xml& xml)
 void MidiEvent::read(Xml& xml)
       {
       _type  = Note;
-      _data  = 0;
+      edata  = 0;
       a      = 0;
       b      = 0;
       c      = 0;
       _voice = 0;
       _enh   = 0;
       _stem  = 0;
-      _dataLen = 0;
+
+      int dataLen = 0;
       for (;;) {
             Xml::Token token = xml.parse();
             const QString& tag = xml.s1();
@@ -490,11 +542,16 @@ void MidiEvent::read(Xml& xml)
                   case Xml::Text:
                         {
                         const char*s = tag.latin1();
-                        _data = new unsigned char[_dataLen];
-                        unsigned char*d = _data;
-                        for (int i = 0; i < _dataLen; ++i) {
-                              *d++ = strtol(s, 0, 16);
-                              s += 3;
+                        edata = new EvData;
+                        edata->data = new unsigned char[dataLen];
+                        edata->dataLen = dataLen;
+                        edata->refCount = 1;
+                        unsigned char* d = edata->data;
+// printf("read SYSEX <%s>\n", s);
+                        for (int i = 0; i < dataLen; ++i) {
+                              char* endp;
+                              *d++ = strtol(s, &endp, 16);
+                              s = endp;
                               }
                         }
                         break;
@@ -518,7 +575,7 @@ void MidiEvent::read(Xml& xml)
                         else if (tag == "stem")
                               _stem = xml.s2().toInt();
                         else if (tag == "datalen")
-                              _dataLen = xml.s2().toInt();
+                              dataLen = xml.s2().toInt();
                         break;
                   case Xml::TagEnd:
                         if (tag == "event")
@@ -529,6 +586,10 @@ void MidiEvent::read(Xml& xml)
             }
       }
 
+//---------------------------------------------------------
+//   WaveEvent::read
+//---------------------------------------------------------
+
 void WaveEvent::read(Xml& xml)
       {
       _clip = 0;
@@ -546,7 +607,8 @@ void WaveEvent::read(Xml& xml)
                         else if (tag == "clip") {
                               const QString& name = xml.parse1();
                               _clip = waveClips->search(name);
-                              _clip->incRef();
+                              if (_clip)
+                                    _clip->incRef();
                               }
                         else
                               xml.unknown("Event");
@@ -565,7 +627,7 @@ void WaveEvent::read(Xml& xml)
 //   write
 //---------------------------------------------------------
 
-void WaveEvent::write(int level, Xml& xml, int offset=0) const
+void WaveEvent::write(int level, Xml& xml, int offset) const
       {
       xml.tag(level++, "event");
       xml.strTag(level, "clip", _clip->name());
@@ -586,3 +648,19 @@ Event* WaveEvent::clone()
       return new WaveEvent(*this);
       }
 
+MidiEvent& MidiEvent::operator=(const MidiEvent& ev)
+      {
+      printf("OPERATOR = MidiEvent\n");
+      abort();
+#if 0
+      if (edata == ev.edata)
+            return *this;
+      if (edata && (--(edata->refCount) == 0))
+            delete edata;
+      memcpy(this, &ev, sizeof(MidiPlayEvent));
+      if (edata)
+            (edata->refCount)++;
+#endif
+      return *this;
+      }
+
