@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: font.h,v 1.2 2001/11/20 15:19:31 muse Exp $
+//  $Id: font.h,v 1.1.1.1 2003/10/29 10:05:08 wschweer Exp $
 //
 //  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -17,7 +17,7 @@
 
 class Font : public QFont {
    public:
-      Font() : QFont("Carter") { }
+      Font() : QFont(QString("Carter")) { }
       Font(const QFont& f) : QFont(f) {}
       Font(const QString& f, int pointSize=12, int weight = Normal,
         bool italic = false) : QFont(f, pointSize, weight, italic) {}
