@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: editevent.cpp,v 1.1 2002/01/30 12:08:38 muse Exp $
+//    $Id: editevent.cpp,v 1.1.1.1 2003/10/29 10:05:23 wschweer Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -60,16 +60,16 @@ char* hex2string(QWidget* parent, const char* src, int& len)
             long val =  strtol(src, &ep, 16);
             if (ep == src) {
                   QMessageBox::information(parent,
-                     "MusE",
-                     "Cannot convert sysex string");
+                     QString("MusE"),
+                     QWidget::tr("Cannot convert sysex string"));
                   return 0;
                   }
             src    = ep;
             *dst++ = val;
             if (dst - buffer >= 2048) {
                   QMessageBox::information(parent,
-                     "MusE",
-                     "Hex String too long (2048 bytes limit)");
+                     QString("MusE"),
+                     QWidget::tr("Hex String too long (2048 bytes limit)"));
                   return 0;
                   }
             }
@@ -85,7 +85,7 @@ char* hex2string(QWidget* parent, const char* src, int& len)
 //---------------------------------------------------------
 
 MidiEvent* EditNoteDialog::getEvent(int tick, const MidiEvent* event,
-   QWidget* parent = 0)
+   QWidget* parent)
       {
       EditNoteDialog* dlg = new EditNoteDialog(tick, event, parent);
       MidiEvent* nevent = 0;
@@ -97,7 +97,7 @@ MidiEvent* EditNoteDialog::getEvent(int tick, const MidiEvent* event,
       }
 
 MidiEvent* EditSysexDialog::getEvent(int tick, const MidiEvent* event,
-   QWidget* parent=0)
+   QWidget* parent)
       {
       EditSysexDialog* dlg = new EditSysexDialog(tick, event, parent);
       MidiEvent* nevent = 0;
@@ -108,7 +108,7 @@ MidiEvent* EditSysexDialog::getEvent(int tick, const MidiEvent* event,
       return nevent;
       }
 
-MidiEvent* EditCtrl7Dialog::getEvent(int tick, const MidiEvent* event, QWidget* parent=0)
+MidiEvent* EditCtrl7Dialog::getEvent(int tick, const MidiEvent* event, QWidget* parent)
       {
       EditEventDialog* dlg = new EditCtrl7Dialog(tick, event, parent);
       MidiEvent* nevent = 0;
@@ -119,7 +119,7 @@ MidiEvent* EditCtrl7Dialog::getEvent(int tick, const MidiEvent* event, QWidget*
       return nevent;
       }
 
-MidiEvent* EditCtrl14Dialog::getEvent(int tick, const MidiEvent* event, QWidget* parent=0)
+MidiEvent* EditCtrl14Dialog::getEvent(int tick, const MidiEvent* event, QWidget* parent)
       {
       EditEventDialog* dlg = new EditCtrl14Dialog(tick, event, parent);
       MidiEvent* nevent = 0;
@@ -130,7 +130,7 @@ MidiEvent* EditCtrl14Dialog::getEvent(int tick, const MidiEvent* event, QWidget*
       return nevent;
       }
 
-MidiEvent* EditMetaDialog::getEvent(int tick, const MidiEvent* event, QWidget* parent=0)
+MidiEvent* EditMetaDialog::getEvent(int tick, const MidiEvent* event, QWidget* parent)
       {
       EditEventDialog* dlg = new EditMetaDialog(tick, event, parent);
       MidiEvent* nevent = 0;
@@ -141,7 +141,7 @@ MidiEvent* EditMetaDialog::getEvent(int tick, const MidiEvent* event, QWidget* p
       return nevent;
       }
 
-MidiEvent* EditPitchDialog::getEvent(int tick, const MidiEvent* event, QWidget* parent=0)
+MidiEvent* EditPitchDialog::getEvent(int tick, const MidiEvent* event, QWidget* parent)
       {
       EditEventDialog* dlg = new EditPitchDialog(tick, event, parent);
       MidiEvent* nevent = 0;
@@ -152,7 +152,7 @@ MidiEvent* EditPitchDialog::getEvent(int tick, const MidiEvent* event, QWidget*
       return nevent;
       }
 
-MidiEvent* EditCAfterDialog::getEvent(int tick, const MidiEvent* event, QWidget* parent=0)
+MidiEvent* EditCAfterDialog::getEvent(int tick, const MidiEvent* event, QWidget* parent)
       {
       EditEventDialog* dlg = new EditCAfterDialog(tick, event, parent);
       MidiEvent* nevent = 0;
@@ -163,7 +163,7 @@ MidiEvent* EditCAfterDialog::getEvent(int tick, const MidiEvent* event, QWidget*
       return nevent;
       }
 
-MidiEvent* EditPAfterDialog::getEvent(int tick, const MidiEvent* event, QWidget* parent=0)
+MidiEvent* EditPAfterDialog::getEvent(int tick, const MidiEvent* event, QWidget* parent)
       {
       EditEventDialog* dlg = new EditPAfterDialog(tick, event, parent);
       MidiEvent* nevent = 0;
@@ -178,8 +178,8 @@ MidiEvent* EditPAfterDialog::getEvent(int tick, const MidiEvent* event, QWidget*
 //   EditEventDialog
 //---------------------------------------------------------
 
-EditEventDialog::EditEventDialog(QWidget* parent=0,
-   const char* name=0) : QDialog(parent, name, true)
+EditEventDialog::EditEventDialog(QWidget* parent,
+   const char* name) : QDialog(parent, name, true)
       {
       QVBoxLayout* xlayout = new QVBoxLayout(this);
       layout = new QGridLayout;
@@ -209,7 +209,7 @@ EditEventDialog::EditEventDialog(QWidget* parent=0,
 //---------------------------------------------------------
 
 EditNoteDialog::EditNoteDialog(int tick, const MidiEvent* event,
-   QWidget* parent=0, const char* name=0)
+   QWidget* parent, const char* name)
    : EditNoteDialogBase(parent, name)
       {
       if (event) {
@@ -234,7 +234,7 @@ EditNoteDialog::EditNoteDialog(int tick, const MidiEvent* event,
 
 MidiEvent* EditNoteDialog::event()
       {
-      MidiEvent* event = new MidiEvent(0, 0,
+      MidiEvent* event = new MidiEvent(
          pos->pos().posTick(), MidiEvent::Note,
          pl->value(), il2->value(), il3->value(), il1->value());
       return event;
@@ -245,7 +245,7 @@ MidiEvent* EditNoteDialog::event()
 //---------------------------------------------------------
 
 EditSysexDialog::EditSysexDialog(int tick, const MidiEvent* event,
-   QWidget* parent=0, const char* name=0)
+   QWidget* parent, const char* name)
    : EditSysexDialogBase(parent, name)
       {
       sysex = 0;
@@ -274,7 +274,7 @@ EditSysexDialog::~EditSysexDialog()
 
 MidiEvent* EditSysexDialog::event()
       {
-      MidiEvent* event = new MidiEvent(0, 0,
+      MidiEvent* event = new MidiEvent(
          pos->pos().posTick(), MidiEvent::Sysex, len, sysex);
       return event;
       }
@@ -298,10 +298,10 @@ void EditSysexDialog::accept()
 //---------------------------------------------------------
 
 EditCtrl7Dialog::EditCtrl7Dialog(int tick, const MidiEvent* event,
-   QWidget* parent=0, const char* name=0)
+   QWidget* parent, const char* name)
    : EditEventDialog(parent, name)
       {
-      setCaption("MusE: Enter Controller Event");
+      setCaption(tr("MusE: Enter Controller Event"));
 
       QLabel* l1 = new QLabel(tr("Time Position"), this);
       pos = new PosEdit(this);
@@ -345,7 +345,7 @@ EditCtrl7Dialog::EditCtrl7Dialog(int tick, const MidiEvent* event,
 
 MidiEvent* EditCtrl7Dialog::event()
       {
-      MidiEvent* event = new MidiEvent(0, 0,
+      MidiEvent* event = new MidiEvent(
          pos->pos().posTick(), MidiEvent::Ctrl7,
          il2->value(), il3->value(), 0, 0);
       return event;
@@ -356,10 +356,10 @@ MidiEvent* EditCtrl7Dialog::event()
 //---------------------------------------------------------
 
 EditCtrl14Dialog::EditCtrl14Dialog(int tick, const MidiEvent* event,
-   QWidget* parent=0, const char* name=0)
+   QWidget* parent, const char* name)
    : EditEventDialog(parent, name)
       {
-      setCaption("MusE: Enter Controller14 Event");
+      setCaption(tr("MusE: Enter Controller14 Event"));
 
       QLabel* l1 = new QLabel(tr("Time Position"), this);
       pos = new PosEdit(this);
@@ -411,7 +411,7 @@ EditCtrl14Dialog::EditCtrl14Dialog(int tick, const MidiEvent* event,
 
 MidiEvent* EditCtrl14Dialog::event()
       {
-      MidiEvent* event = new MidiEvent(0, 0,
+      MidiEvent* event = new MidiEvent(
          pos->pos().posTick(), MidiEvent::Ctrl14,
          il2h->value(), il3->value(), il2l->value(), 0);
       return event;
@@ -422,11 +422,11 @@ MidiEvent* EditCtrl14Dialog::event()
 //---------------------------------------------------------
 
 EditMetaDialog::EditMetaDialog(int tick, const MidiEvent* ev,
-   QWidget* parent=0, const char* name=0)
+   QWidget* parent, const char* name)
    : EditEventDialog(parent, name)
       {
       meta = 0;
-      setCaption("MusE: Enter Meta Event");
+      setCaption(tr("MusE: Enter Meta Event"));
 
       QLabel* l1 = new QLabel(tr("Time Position"), this);
       pos = new PosEdit(this);
@@ -499,7 +499,7 @@ EditMetaDialog::~EditMetaDialog()
 
 MidiEvent* EditMetaDialog::event()
       {
-      MidiEvent* event = new MidiEvent(0, 0,
+      MidiEvent* event = new MidiEvent(
          pos->pos().posTick(), MidiEvent::Meta, len,
          (const unsigned char*)meta);
       event->setA(il2->value());
@@ -531,10 +531,10 @@ void EditMetaDialog::accept()
 //---------------------------------------------------------
 
 EditPitchDialog::EditPitchDialog(int tick, const MidiEvent* event,
-   QWidget* parent=0, const char* name=0)
+   QWidget* parent, const char* name)
    : EditEventDialog(parent, name)
       {
-      setCaption("MusE: Enter Pitch Shift");
+      setCaption(tr("MusE: Enter Pitch Shift"));
 
       QLabel* l1 = new QLabel(tr("Time Position"), this);
       pos = new PosEdit(this);
@@ -547,8 +547,9 @@ EditPitchDialog::EditPitchDialog(int tick, const MidiEvent* event,
       connect(il2, SIGNAL(valueChanged(int)), slider, SLOT(setValue(int)));
 
       if (event) {
-            int pitch = (event->dataB() << 7) + event->dataA();
+            int pitch = event->dataA();
             il2->setValue(pitch);
+            slider->setValue(pitch);
             pos->setValue(tick);
             }
       else {
@@ -570,9 +571,8 @@ MidiEvent* EditPitchDialog::event()
       {
       int val = il2->value();
 
-      MidiEvent* event = new MidiEvent(0, 0,
-         pos->pos().posTick(), MidiEvent::Pitch,
-         (val >> 7) & 0x7f, val & 0x7f, 0, 0);
+      MidiEvent* event = new MidiEvent(
+         pos->pos().posTick(), MidiEvent::Pitch, val, 0, 0, 0);
       return event;
       }
 
@@ -581,10 +581,10 @@ MidiEvent* EditPitchDialog::event()
 //---------------------------------------------------------
 
 EditCAfterDialog::EditCAfterDialog(int tick, const MidiEvent* event,
-   QWidget* parent=0, const char* name=0)
+   QWidget* parent, const char* name)
    : EditEventDialog(parent, name)
       {
-      setCaption("MusE: Enter Channel Aftertouch");
+      setCaption(tr("MusE: Enter Channel Aftertouch"));
 
       QLabel* l1 = new QLabel(tr("Time Position"), this);
       pos = new PosEdit(this);
@@ -622,7 +622,7 @@ EditCAfterDialog::EditCAfterDialog(int tick, const MidiEvent* event,
 
 MidiEvent* EditCAfterDialog::event()
       {
-      MidiEvent* event = new MidiEvent(0, 0,
+      MidiEvent* event = new MidiEvent(
          pos->pos().posTick(), MidiEvent::CAfter,
          il2->value(), 0, 0, 0);
       return event;
@@ -633,10 +633,10 @@ MidiEvent* EditCAfterDialog::event()
 //---------------------------------------------------------
 
 EditPAfterDialog::EditPAfterDialog(int tick, const MidiEvent* event,
-   QWidget* parent=0, const char* name=0)
+   QWidget* parent, const char* name)
    : EditEventDialog(parent, name)
       {
-      setCaption("MusE: Enter Poly Aftertouch");
+      setCaption(tr("MusE: Enter Poly Aftertouch"));
 
       QLabel* l1 = new QLabel(tr("Time Position"), this);
       pos = new PosEdit(this);
@@ -681,7 +681,7 @@ EditPAfterDialog::EditPAfterDialog(int tick, const MidiEvent* event,
 
 MidiEvent* EditPAfterDialog::event()
       {
-      MidiEvent* event = new MidiEvent(0, 0,
+      MidiEvent* event = new MidiEvent(
          pos->pos().posTick(), MidiEvent::CAfter,
          pl->value(), il2->value(), 0, 0);
       return event;
