@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: listedit.cpp,v 1.1 2002/01/30 12:08:38 muse Exp $
+//    $Id: listedit.cpp,v 1.1.1.1 2003/10/29 10:05:22 wschweer Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -9,7 +9,6 @@
 #include "mtscale.h"
 #include "scrollscale.h"
 #include "intlabel.h"
-#include "seq.h"
 #include "globals.h"
 #include "icons.h"
 #include "editevent.h"
@@ -52,7 +51,10 @@ class EventListItem : public QListViewItem {
       MidiPart* part;
 
       EventListItem(QListView* parent, MidiEvent* ev, MidiPart* p)
-         : QListViewItem(parent), event(ev), part(p) {}
+         : QListViewItem(parent) {
+            event = ev;
+            part  = p;
+            }
       virtual QString text(int col) const;
       virtual int compare(QListViewItem*i, int, bool) const {
             return event->posTick() - ((EventListItem*)i)->event->posTick();
@@ -65,7 +67,7 @@ class EventListItem : public QListViewItem {
 
 static QString midiMetaComment(const MidiEvent* ev)
       {
-      int meta = ev->dataA();
+      int meta  = ev->dataA();
       QString s = midiMetaName(meta);
 
       switch (meta) {
@@ -94,7 +96,7 @@ static QString midiMetaComment(const MidiEvent* ev)
             case 0x0e:
             case 0x0f:
                   {
-                  s += ": ";
+                  s += QString(": ");
                   const char* txt = (char*)(ev->data());
                   int len = ev->dataLen();
                   char buffer[len+1];
@@ -105,27 +107,27 @@ static QString midiMetaComment(const MidiEvent* ev)
                         if (buffer[i] == '\n' || buffer[i] == '\r')
                         buffer[i] = ' ';
                         }
-                  return s + buffer;
+                  return s + QString(buffer);
                   }
 
             case 0x20:
             case 0x21:
             default:
                   {
-                  s += ": ";
+                  s += QString(": ");
                   int i;
                   int len = ev->lenTick();
                   int n = len > 10 ? 10 : len;
                   for (i = 0; i < n; ++i) {
                         if (i >= ev->dataLen())
                               break;
-                        s += " 0x";
+                        s += QString(" 0x");
                         QString k;
                         k.setNum(ev->data()[i] & 0xff, 16);
                         s += k;
                         }
                   if (i == 10)
-                        s += "...";
+                        s += QString("...");
                   return s;
                   }
             }
@@ -211,7 +213,7 @@ QString EventListItem::text(int col) const
                   break;
             case 1:
                   {
-                  int t = event->posTick();
+                  int t = event->posTick() + part->posTick();
                   int bar, beat, tick;
                   sigmap.tickValues(t, &bar, &beat, &tick);
                   s.sprintf("%04d.%02d.%03d", bar+1, beat+1, tick);
@@ -220,74 +222,74 @@ QString EventListItem::text(int col) const
             case 2:
                   switch(event->type()) {
                         case MidiEvent::Note:
-                              s = "Note";
+                              s = QString("Note");
                               break;
                         case MidiEvent::NoteOff:
-                              s = "Noff";
+                              s = QString("Noff");
                               break;
                         case MidiEvent::Program:
-                              s = "Pgrm";
+                              s = QString("Pgrm");
                               break;
                         case MidiEvent::Ctrl7:
                               commentLabel = midiCtrl7Name(event->dataA());
-                              s = "Ctrl7";
+                              s = QString("Ctrl7");
                               break;
                         case MidiEvent::Ctrl14:
                               commentLabel = midiCtrl14Name(event->dataA(), event->dataC());
-                              s = "Ctrl14";
+                              s = QString("Ctrl14");
                               break;
                         case MidiEvent::Sysex:
                               {
-                              commentLabel = "len ";
+                              commentLabel = QString("len ");
                               QString k;
                               k.setNum(event->dataLen());
                               commentLabel += k;
-                              commentLabel += " ";
+                              commentLabel += QString(" ");
 
                               commentLabel += nameSysex(event->dataLen(), event->data());
                               int i;
                               for (i = 0; i < 10; ++i) {
                                     if (i >= event->dataLen())
                                           break;
-                                    commentLabel += " 0x";
+                                    commentLabel += QString(" 0x");
                                     QString k;
                                     k.setNum(event->data()[i] & 0xff, 16);
                                     commentLabel += k;
                                     }
                               if (i == 10)
-                                    commentLabel += "...";
+                                    commentLabel += QString("...");
                               }
-                              s = "SysEx";
+                              s = QString("SysEx");
                               break;
                         case MidiEvent::PAfter:
-                              s = "PoAT";
+                              s = QString("PoAT");
                               break;
                         case MidiEvent::CAfter:
-                              s = "ChAT";
+                              s = QString("ChAT");
                               break;
                         case MidiEvent::Pitch:
-                              s = "Pitch";
+                              s = QString("Pitch");
                               break;
                         case MidiEvent::Meta:
                               commentLabel = midiMetaComment(event);
-                              s = "Meta";
+                              s = QString("Meta");
                               break;
                         case MidiEvent::NRPN:
-                              s = "NRPN";
+                              s = QString("NRPN");
                               break;
                         case MidiEvent::RPN:
-                              s = "RPN";
+                              s = QString("RPN");
                               break;
                         case MidiEvent::NoEvent:
                         case MidiEvent::Symbol:
                         case MidiEvent::Quantize:
                         case MidiEvent::Clock:
-                              s = "???";
+                              s = QString("???");
                               break;
                         }
                   break;
             case 3:
-                  s.setNum(event->channel() + 1);
+                  s.setNum(part->track()->outChannel() + 1);
                   break;
             case 4:
                   if (event->type() == MidiEvent::Note || event->type() == MidiEvent::NoteOff) {
@@ -316,24 +318,24 @@ QString EventListItem::text(int col) const
                               break;
                         case MidiEvent::Sysex:
                               {
-                              s = "len ";
+                              s = QString("len ");
                               QString k;
                               k.setNum(event->dataLen());
                               s += k;
-                              s += " ";
+                              s += QString(" ");
 
                               commentLabel += nameSysex(event->dataLen(), event->data());
                               int i;
                               for (i = 0; i < 10; ++i) {
                                     if (i >= event->dataLen())
                                           break;
-                                    s += " 0x";
+                                    s += QString(" 0x");
                                     QString k;
                                     k.setNum(event->data()[i] & 0xff, 16);
                                     s += k;
                                     }
                               if (i == 10)
-                                    s += "...";
+                                    s += QString("...");
                               }
                               break;
                         case MidiEvent::Meta:
@@ -356,19 +358,19 @@ ListEdit::ListEdit(PartList* pl)
    : MidiEditor(0, 0, pl)
       {
       insertItems = new QActionGroup(this, "InsertItems", false);
-      QAction* insertNote = new QAction("Insert Note", QIconSet(*note1Icon), "insert Note",
+      QAction* insertNote = new QAction(tr("Insert Note"), QIconSet(*note1Icon), tr("insert Note"),
         CTRL+Key_N, insertItems);
-      QAction* insertSysEx = new QAction("Insert SysEx", QIconSet(*sysexIcon), "insert SysEx",
+      QAction* insertSysEx = new QAction(tr("Insert SysEx"), QIconSet(*sysexIcon), tr("insert SysEx"),
         CTRL+Key_S, insertItems);
-      QAction* insertCtrl = new QAction("Insert Ctrl", QIconSet(*ctrlIcon), "insert Ctrl",
+      QAction* insertCtrl = new QAction(tr("Insert Ctrl"), QIconSet(*ctrlIcon), tr("insert Ctrl"),
         CTRL+Key_T, insertItems);
-      QAction* insertMeta = new QAction("Insert Meta", QIconSet(*metaIcon), "insert Meta",
+      QAction* insertMeta = new QAction(tr("Insert Meta"), QIconSet(*metaIcon), tr("insert Meta"),
         0, insertItems);
-      QAction* insertPitch = new QAction("Insert Pitch", QIconSet(*pitchIcon), "insert Pitch",
+      QAction* insertPitch = new QAction(tr("Insert Pitch"), QIconSet(*pitchIcon), tr("insert Pitch"),
         CTRL+Key_E, insertItems);
-      QAction* insertCAfter = new QAction("Insert Channel Aftertouch", QIconSet(*cafterIcon), "insert Channel Aftertouch",
+      QAction* insertCAfter = new QAction(tr("Insert Channel Aftertouch"), QIconSet(*cafterIcon), tr("insert Channel Aftertouch"),
         CTRL+Key_A, insertItems);
-      QAction* insertPAfter = new QAction("Insert Key Aftertouch", QIconSet(*pafterIcon), "insert Poly Aftertouch",
+      QAction* insertPAfter = new QAction(tr("Insert Key Aftertouch"), QIconSet(*pafterIcon), tr("insert Poly Aftertouch"),
         CTRL+Key_P, insertItems);
 
       connect(insertNote,    SIGNAL(activated()), SLOT(editInsertNote()));
@@ -419,20 +421,20 @@ ListEdit::ListEdit(PartList* pl)
       QFontMetrics fm(liste->font());
       int n = fm.width('9');
       int b = 8;
-      int c = fm.width("Val B");
+      int c = fm.width(QString("Val B"));
       liste->setAllColumnsShowFocus(true);
       liste->setSorting(0);
       liste->setSelectionMode(QListView::Extended);
 
-      liste->addColumn("Tick",  n * 6 + b);
-      liste->addColumn("Bar",   fm.width("9999.99.999") + b);
-      liste->addColumn("Typ",   fm.width("SysEx") + b);
-      liste->addColumn("Ch",    n * 2 + b);
-      liste->addColumn("Val A", c + b);
-      liste->addColumn("Val B", c + b);
-      liste->addColumn("Val C", c + b);
-      liste->addColumn("Len",   n * 4 + b);
-      liste->addColumn("Comment");
+      liste->addColumn(tr("Tick"),  n * 6 + b);
+      liste->addColumn(tr("Bar"),   fm.width(QString("9999.99.999")) + b);
+      liste->addColumn(tr("Typ"),   fm.width(QString("SysEx")) + b);
+      liste->addColumn(tr("Ch"),    n * 2 + b);
+      liste->addColumn(tr("Val A"), c + b);
+      liste->addColumn(tr("Val B"), c + b);
+      liste->addColumn(tr("Val C"), c + b);
+      liste->addColumn(tr("Len"),   n * 4 + b);
+      liste->addColumn(tr("Comment"));
       liste->setResizeMode(QListView::LastColumn);
       connect(liste, SIGNAL(selectionChanged()), SLOT(selectionChanged()));
       connect(liste, SIGNAL(doubleClicked(QListViewItem*)), SLOT(doubleClicked(QListViewItem*)));
@@ -468,8 +470,6 @@ void ListEdit::editInsertNote()
       {
       MidiEvent* event = EditNoteDialog::getEvent(0, 0, this);
       if (event) {
-            event->setPort(curTrack->outPort());
-            event->setChannel(curTrack->outChannel());
             midiThread->msgAddEvent(event, curPart);
             }
       }
@@ -482,8 +482,6 @@ void ListEdit::editInsertSysEx()
       {
       MidiEvent* event = EditSysexDialog::getEvent(0, 0, this);
       if (event) {
-            event->setPort(curTrack->outPort());
-            event->setChannel(curTrack->outChannel());
             midiThread->msgAddEvent(event, curPart);
             }
       }
@@ -496,8 +494,6 @@ void ListEdit::editInsertCtrl7()
       {
       MidiEvent* event = EditCtrl7Dialog::getEvent(0, 0, this);
       if (event) {
-            event->setPort(curTrack->outPort());
-            event->setChannel(curTrack->outChannel());
             midiThread->msgAddEvent(event, curPart);
             }
       }
@@ -510,8 +506,6 @@ void ListEdit::editInsertCtrl14()
       {
       MidiEvent* event = EditCtrl14Dialog::getEvent(0, 0, this);
       if (event) {
-            event->setPort(curTrack->outPort());
-            event->setChannel(curTrack->outChannel());
             midiThread->msgAddEvent(event, curPart);
             }
       }
@@ -524,8 +518,6 @@ void ListEdit::editInsertMeta()
       {
       MidiEvent* event = EditMetaDialog::getEvent(0, 0, this);
       if (event) {
-            event->setPort(curTrack->outPort());
-            event->setChannel(curTrack->outChannel());
             midiThread->msgAddEvent(event, curPart);
             }
       }
@@ -538,8 +530,6 @@ void ListEdit::editInsertPitch()
       {
       MidiEvent* event = EditPitchDialog::getEvent(0, 0, this);
       if (event) {
-            event->setPort(curTrack->outPort());
-            event->setChannel(curTrack->outChannel());
             midiThread->msgAddEvent(event, curPart);
             }
       }
@@ -552,8 +542,6 @@ void ListEdit::editInsertCAfter()
       {
       MidiEvent* event = EditCAfterDialog::getEvent(0, 0, this);
       if (event) {
-            event->setPort(curTrack->outPort());
-            event->setChannel(curTrack->outChannel());
             midiThread->msgAddEvent(event, curPart);
             }
       }
@@ -566,8 +554,6 @@ void ListEdit::editInsertPAfter()
       {
       MidiEvent* event = EditPAfterDialog::getEvent(0, 0, this);
       if (event) {
-            event->setPort(curTrack->outPort());
-            event->setChannel(curTrack->outChannel());
             midiThread->msgAddEvent(event, curPart);
             }
       }
@@ -611,8 +597,6 @@ void ListEdit::editEvent(MidiEvent* event, MidiPart* part)
             }
       if (nevent) {
             // TODO: check for event != nevent
-            nevent->setPort(event->port());
-            nevent->setChannel(event->channel());
             midiThread->msgChangeEvent(event, nevent, part);
             }
       }
