@@ -1,13 +1,14 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: midi.cpp,v 1.2 2002/02/05 14:15:56 muse Exp $
+//  $Id: midi.cpp,v 1.3 2003/11/19 17:59:21 wschweer Exp $
 //
 //  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
 #include <errno.h>
 #include <values.h>
+#include <assert.h>
 
 #include "song.h"
 #include "midi.h"
@@ -290,9 +291,9 @@ void MidiFile::putvl(unsigned val)
 //   writeEvent
 //---------------------------------------------------------
 
-void MidiFile::writeEvent(MidiEvent* event)
+void MidiFile::writeEvent(int channel, MidiEvent* event)
       {
-      int c     = event->channel();
+      int c = channel;
       int nstat = 0;
 
       switch (event->type()) {
@@ -448,7 +449,7 @@ bool MidiFile::checkSysex(MidiTrack* track, unsigned int len, unsigned char* buf
                               // 3 - DRUM 2
                               // 4 - DRUM 3
                               // 5 - DRUM 4
-                              printf("xg part mode channel %d to %d\n", buf[4], buf[6]);
+                              printf("xg set part mode channel %d to %d\n", buf[4]+1, buf[6]);
                               if (buf[6] != 0)
                                     track->setType(Track::DRUM);
                               }
@@ -527,8 +528,6 @@ bool MidiFile::readTrack(bool mergeChannels)
                   return true;
             if (lastchan == -1) {
                   el->add(event);         // kann kein channel event sein
-                  event->setPort(track->outPort());
-                  event->setChannel(track->outChannel());
                   continue;
                   }
             if (port == -1) {
@@ -560,8 +559,6 @@ bool MidiFile::readTrack(bool mergeChannels)
                   if (port != track->outPort())
                         printf("2 HALOOOO %d %d\n", port, track->outPort());
                   el->add(event);
-                  event->setPort(track->outPort());
-                  event->setChannel(track->outChannel());
                   continue;
                   }
             if (lastchan != channel) {
@@ -579,8 +576,6 @@ bool MidiFile::readTrack(bool mergeChannels)
                                     continue;
                               if (mtrack->outChannel() == lastchan) {
                                     mtrack->events()->add(event);
-                                    event->setPort(mtrack->outPort());
-                                    event->setChannel(mtrack->outChannel());
                                     break;
                                     }
                               }
@@ -614,8 +609,6 @@ bool MidiFile::readTrack(bool mergeChannels)
                         el = track->events();
                         }
                   }
-            event->setPort(track->outPort());
-            event->setChannel(track->outChannel());
             el->add(event);
             }
       int end = curPos;
@@ -717,7 +710,7 @@ MidiEvent* MidiFile::readEvent(MidiTrack* track)
                   if (read(buffer, len)) {
                         printf("readEvent: error 4\n");
                         delete event;
-                        delete buffer;
+                        delete[] buffer;
                         return (MidiEvent*)-2;
                         }
                   if (buffer[len-1] != 0xf7) {
@@ -728,11 +721,12 @@ MidiEvent* MidiFile::readEvent(MidiTrack* track)
                         --len;      // don't count 0xf7
                   if (checkSysex(track, len, buffer)) {
                         event->setType(MidiEvent::Sysex);
-                        event->setDataLen(len);
-                        event->setData(buffer);
+                        EvData* ed = new EvData(buffer, len);
+                        ed->refCount = 1;
+                        event->setEvData(ed);
                         }
                   else {
-                        delete buffer;
+                        delete[] buffer;
                         delete event;
                         return (MidiEvent*)-1;
                         }
@@ -755,22 +749,22 @@ MidiEvent* MidiFile::readEvent(MidiTrack* track)
                         if (read(buffer, len)) {
                               printf("readEvent: error 7\n");
                               delete event;
-                              delete buffer;
+                              delete[] buffer;
                               return (MidiEvent*)-2;
                               }
                         }
                   buffer[len] = 0;
                   if (type == 0x2f) {           // End of Track
                         delete event;
-                        delete buffer;
+                        delete[] buffer;
                         return 0;
                         }
                   switch (type) {
                         case 0x03:        // Sequence-/TrackName
                               {
                               QString s((char*)buffer);
-                              track->setTName(s);
-                              delete buffer;
+                              track->setName(s);
+                              delete[] buffer;
                               delete event;
                               }
                               return (MidiEvent*)-1;  // DEBUG: eigentlich nchsten Event lesen
@@ -778,14 +772,14 @@ MidiEvent* MidiFile::readEvent(MidiTrack* track)
                               {
                               QString s((char*)buffer);
                               track->setComment(s);
-                              delete buffer;
+                              delete[] buffer;
                               delete event;
                               }
                               return (MidiEvent*)-1;  // DEBUG: eigentlich nchsten Event lesen
                         case 0x22:        // Channel Prefix
                               if (len == 1) {
                                     channelprefix = buffer[0];
-                                    delete buffer;
+                                    delete[] buffer;
                                     delete event;
                                     return (MidiEvent*)-1;
                                     }
@@ -795,7 +789,7 @@ MidiEvent* MidiFile::readEvent(MidiTrack* track)
                                     lastport = buffer[0];
                                     if (lastport == 10)
                                           lastport = 1;
-                                    delete buffer;
+                                    delete[] buffer;
                                     delete event;
                                     return (MidiEvent*)-1;
                                     }
@@ -804,7 +798,7 @@ MidiEvent* MidiFile::readEvent(MidiTrack* track)
                               {
                               int tempo = buffer[2] + (buffer[1] << 8) + (buffer[0] <<16);
                               tempomap.addTempo(t, tempo);
-                              delete buffer;
+                              delete[] buffer;
                               delete event;
                               }
                               return (MidiEvent*)-1;  // DEBUG: eigentlich nchsten Event lesen
@@ -817,7 +811,7 @@ MidiEvent* MidiFile::readEvent(MidiTrack* track)
                                     timesig_n *= 2;
                               sigmap.add(t, timesig_z, timesig_n);
                               delete event;
-                              delete buffer;
+                              delete[] buffer;
                               }
                               return (MidiEvent*)-1;  // DEBUG: eigentlich nchsten Event lesen
                         case 0x59:  // Key Signature
@@ -825,13 +819,13 @@ MidiEvent* MidiFile::readEvent(MidiTrack* track)
                               track->scale.set(buffer[0]);
                               track->scale.setMajorMinor(buffer[1]);
                               delete event;
-                              delete buffer;
+                              delete[] buffer;
                               }
                               return (MidiEvent*)-1;  // DEBUG: eigentlich nchsten Event lesen
                         case 0x6:   // Marker
                               song->addMarker(QString((const char*)(buffer)), event->posTick(), false);
                               delete event;
-                              delete buffer;
+                              delete[] buffer;
                               return (MidiEvent*)-1;  // DEBUG: eigentlich nchsten Event lesen
 
                         case 0:     // Sequence Number
@@ -847,8 +841,11 @@ MidiEvent* MidiFile::readEvent(MidiTrack* track)
 rest:
                         default:
                               event->setType(MidiEvent::Meta);
-                              event->setDataLen(len+1);
-                              event->setData(buffer);
+                              {
+                              EvData* ed = new EvData(buffer, len+1);
+                              ed->refCount = 1;
+                              event->setEvData(ed);
+                              }
                               event->setA(type);
                               break;
                         }
@@ -878,6 +875,7 @@ rest:
                         a = me;
                         }
                   b = 0;
+                  event->setA(a);
                   switch (status & 0xf0) {
                         case 0x80:        // Taste loslassen
                         case 0x90:        // Taste anschlagen
@@ -902,7 +900,6 @@ rest:
                               delete event;
                               return (MidiEvent*)-2;
                         }
-                  event->setA(a);
                   switch (status & 0xf0) {
                         case 0x80:        // Taste loslassen
                               event->setType(MidiEvent::NoteOff);
@@ -967,17 +964,17 @@ bool MidiFile::writeTrack(MidiTrack* track)
       //    Setup Measure:
       //    Trackname, Comment, Instrument, Volume, Pan
       //
-      if (!track->tname().isEmpty()) {
-            const char* name = track->tname().latin1();
+      if (!track->name().isEmpty()) {
+            const char* name = track->name().latin1();
             int len = strlen(name);
-            MidiEvent* ev = new MidiEvent(port, channel, 0, MidiEvent::Meta, len, (unsigned char*)name);
+            MidiEvent* ev = new MidiEvent(0, MidiEvent::Meta, len, (unsigned char*)name);
             ev->setA(0x3);    // Meta Sequence/Track Name
             events->add(ev);
             }
       if (!track->comment().isEmpty()) {
             const char* comment = track->comment().latin1();
             int len = strlen(comment);
-            MidiEvent* ev = new MidiEvent(port, channel, 0, MidiEvent::Meta, len, (unsigned char*)comment);
+            MidiEvent* ev = new MidiEvent(0, MidiEvent::Meta, len, (unsigned char*)comment);
             ev->setA(0x0f);   // Meta Text
             events->add(ev);
             }
@@ -986,27 +983,27 @@ bool MidiFile::writeTrack(MidiTrack* track)
 
       int tick = 300;
       if (state->controller[CTRL_HBANK] != -1) {
-            events->add(new MidiEvent(port, channel, tick, MidiEvent::Ctrl7, CTRL_HBANK,
+            events->add(new MidiEvent(tick, MidiEvent::Ctrl7, CTRL_HBANK,
                state->controller[CTRL_HBANK], 0, 0));
             tick += 10;
             }
       if (state->controller[CTRL_LBANK] != -1) {
-            events->add(new MidiEvent(port, channel, tick, MidiEvent::Ctrl7, CTRL_LBANK,
+            events->add(new MidiEvent(tick, MidiEvent::Ctrl7, CTRL_LBANK,
                state->controller[CTRL_LBANK], 0, 0));
             tick += 10;
             }
       if (state->program != -1) {
-            events->add(new MidiEvent(port, channel, tick, MidiEvent::Program, state->program,
+            events->add(new MidiEvent(tick, MidiEvent::Program, state->program,
                0, 0, 0));
             tick += 10;
             }
       if (state->controller[CTRL_VOLUME] != -1) {
-            events->add(new MidiEvent(port, channel, tick, MidiEvent::Ctrl7, CTRL_VOLUME,
+            events->add(new MidiEvent(tick, MidiEvent::Ctrl7, CTRL_VOLUME,
                state->controller[CTRL_VOLUME], 0, 0));
             tick += 10;
             }
       if (state->controller[CTRL_PANPOT] != -1) {
-            events->add(new MidiEvent(port, channel, tick, MidiEvent::Ctrl7, CTRL_PANPOT,
+            events->add(new MidiEvent(tick, MidiEvent::Ctrl7, CTRL_PANPOT,
                state->controller[CTRL_PANPOT], 0, 0));
             tick += 10;
             }
@@ -1020,14 +1017,17 @@ bool MidiFile::writeTrack(MidiTrack* track)
             MidiPart* part    = (MidiPart*) (p->second);
             EventList* evlist = part->events();
             for (iEvent i = evlist->begin(); i != evlist->end(); ++i) {
-                  MidiEvent* ev = (MidiEvent*)i->second;
-                  if (ev->isNote()) {
-                        if (ev->velo() == 0) {
+                  MidiEvent* ev    = (MidiEvent*)i->second;
+                  int tick         = ev->posTick() + part->posTick();
+                  MidiEvent* event = new MidiEvent(*ev);
+                  event->setPosTick(tick);
+
+                  if (event->isNote()) {
+                        if (event->velo() == 0) {
                               printf("Warning: midi note has velocity 0, (ignored)\n");
                               continue;
                               }
-                        MidiEvent* event = new MidiEvent(*ev);
-                        int len   = ev->lenTick();
+                        int len = event->lenTick();
 
                         //---------------------------------------
                         //   apply trackinfo values
@@ -1037,8 +1037,8 @@ bool MidiFile::writeTrack(MidiTrack* track)
                            || track->velocity
                            || track->compression != 100
                            || track->len != 100) {
-                              int pitch = ev->pitch();
-                              int velo  = ev->velo();
+                              int pitch = event->pitch();
+                              int velo  = event->velo();
                               pitch += track->transposition;
                               if (pitch > 127)
                                     pitch = 127;
@@ -1058,7 +1058,6 @@ bool MidiFile::writeTrack(MidiTrack* track)
                               }
                         events->add(event);
                         MidiEvent* off = new MidiEvent(*event);
-                        int tick = off->posTick();
                         if (len <= 0)
                               len = 1;
                         off->setPosTick(tick + len);
@@ -1071,7 +1070,7 @@ bool MidiFile::writeTrack(MidiTrack* track)
                         events->add(off);
                         }
                   else
-                        events->add(ev);
+                        events->add(event);
                   }
             }
 
@@ -1097,7 +1096,7 @@ bool MidiFile::writeTrack(MidiTrack* track)
             for (ciMarker m = ml->begin(); m != ml->end(); ++m) {
                   const char* name = m->second.name().latin1();
                   int len = strlen(name);
-                  MidiEvent* ev = new MidiEvent(port, channel,
+                  MidiEvent* ev = new MidiEvent(
                      m->first, MidiEvent::Meta,
                      len, (unsigned char*)name);
                   ev->setA(0x6);
@@ -1110,7 +1109,7 @@ bool MidiFile::writeTrack(MidiTrack* track)
             const char* copyright = song->copyright().latin1();
             if (copyright && *copyright) {
                   int len = strlen(copyright);
-                  MidiEvent* ev = new MidiEvent(port, channel, 0, MidiEvent::Meta,
+                  MidiEvent* ev = new MidiEvent(0, MidiEvent::Meta,
                      len, (unsigned char*)copyright);
                   ev->setA(0x2);
                   events->add(ev);
@@ -1120,15 +1119,15 @@ bool MidiFile::writeTrack(MidiTrack* track)
             //
             switch(song->mtype()) {
                   case MT_GM:
-                        events->add(new MidiEvent(port, channel, 0, MidiEvent::Sysex, gmOnMsgLen, gmOnMsg));
+                        events->add(new MidiEvent(0, MidiEvent::Sysex, gmOnMsgLen, gmOnMsg));
                         break;
                   case MT_GS:
-                        events->add(new MidiEvent(port, channel, 0, MidiEvent::Sysex, gmOnMsgLen, gmOnMsg));
-                        events->add(new MidiEvent(port, channel, 250, MidiEvent::Sysex, gsOnMsgLen, gsOnMsg));
+                        events->add(new MidiEvent(0, MidiEvent::Sysex, gmOnMsgLen, gmOnMsg));
+                        events->add(new MidiEvent(250, MidiEvent::Sysex, gsOnMsgLen, gsOnMsg));
                         break;
                   case MT_XG:
-                        events->add(new MidiEvent(port, channel, 0, MidiEvent::Sysex, gmOnMsgLen, gmOnMsg));
-                        events->add(new MidiEvent(port, channel, 250, MidiEvent::Sysex, xgOnMsgLen, xgOnMsg));
+                        events->add(new MidiEvent(0, MidiEvent::Sysex, gmOnMsgLen, gmOnMsg));
+                        events->add(new MidiEvent(250, MidiEvent::Sysex, xgOnMsgLen, xgOnMsg));
                         break;
                   case MT_UNKNOWN:
                         break;
@@ -1145,7 +1144,6 @@ bool MidiFile::writeTrack(MidiTrack* track)
                   data[1] = (tempo >> 8) & 0xff;
                   data[0] = (tempo >> 16) & 0xff;
                   MidiEvent* ev = new MidiEvent(
-                     port, channel,
                      event->tick, MidiEvent::Meta, 3, data);
                   ev->setA(0x51);
                   events->add(ev);
@@ -1171,7 +1169,6 @@ bool MidiFile::writeTrack(MidiTrack* track)
                               break;
                         }
                   MidiEvent* ev = new MidiEvent(
-                     port, channel,
                      event->tick, MidiEvent::Meta, 2, data);
                   ev->setA(0x58);
                   events->add(ev);
@@ -1188,7 +1185,7 @@ bool MidiFile::writeTrack(MidiTrack* track)
       for (iEvent i = events->begin(); i != events->end(); ++i) {
             putvl(((i->first-tick) * midiDivision + ::division/2)/::division);
             tick = i->first;
-            writeEvent((MidiEvent*)i->second);
+            writeEvent(channel, (MidiEvent*)i->second);
             }
       delete events;
 
@@ -1241,11 +1238,9 @@ void MidiFile::processTrack1(MidiTrack* track)
             track->setType(Track::DRUM);
 
       MidiPort* port = &midiPorts[track->outPort()];
-      int channel = track->outChannel();
-      port->resetIstate(channel, false);
-
-      int trackno  = song->tracks()->index(track);
-      bool karaoke = song->karaokeFlag();
+      int channel    = track->outChannel();
+      int trackno    = song->tracks()->index(track);
+      bool karaoke   = song->karaokeFlag();
 
       //---------------------------------------------------
       //    get initial controller state
@@ -1292,7 +1287,7 @@ void MidiFile::processTrack1(MidiTrack* track)
                                     if (trackno == 1 && memcmp(ev->data(), "@KMID", 5) == 0) {
                                           karaoke = true;
                                           song->setKaraokeFlag(true);
-                                          song->setKaraokeChannel(ev->channel());
+                                          song->setKaraokeChannel(channel);
                                           tevents->erase(i);
                                           }
                                     if ((trackno == 2) && karaoke) {
@@ -1305,11 +1300,11 @@ void MidiFile::processTrack1(MidiTrack* track)
                                                             break;
                                                       case 1:     // expect titel
                                                             ++karaokeState;  // language
-                                                            song->setName(p+2);
+                                                            song->setName(QString(p+2));
                                                             break;
                                                       case 2:     // expect interpret
                                                             ++karaokeState;
-                                                            song->setKomponist1(p+2);
+                                                            song->setKomponist1(QString(p+2));
                                                             break;
                                                       default:
                                                             printf("unexpected @ in karaoke\n");
@@ -1339,7 +1334,7 @@ void MidiFile::processTrack1(MidiTrack* track)
             }
 
       //---------------------------------------------------
-      //    NoteOff Events auflsen
+      //    resolve NoteOff events
       //---------------------------------------------------
 
       for (iEvent i = tevents->begin(); i != tevents->end(); ++i) {
@@ -1374,8 +1369,8 @@ void MidiFile::processTrack1(MidiTrack* track)
                               }
                         }
                   if (k == tevents->end()) {
-                        printf("-kein note-off! %d channel %d pitch %d velo %d\n",
-                           i->first, ev->channel(), ev->pitch(), ev->velo());
+                        printf("-kein note-off! %d pitch %d velo %d\n",
+                           i->first, ev->pitch(), ev->velo());
                         //
                         // ton am Taktende abschalten
                         //
@@ -1388,13 +1383,26 @@ void MidiFile::processTrack1(MidiTrack* track)
                   }
             }
 
+      //
+      // remap drum pitch with drumInmap
+      //
+      if (track->type() == Track::DRUM) {
+            for (iEvent i = tevents->begin(); i != tevents->end(); ++i) {
+                  MidiEvent* ev  = (MidiEvent*)i->second;
+                  if (ev->isNote()) {
+                        int pitch = drumInmap[ev->pitch()];
+                        ev->setPitch(pitch);
+                        }
+                  }
+            }
+
 // DEBUG: any note offs left?
 
       for (iEvent i = tevents->begin(); i != tevents->end(); ++i) {
             MidiEvent* ev  = (MidiEvent*)i->second;
             if (ev->isNoteOff()) {
-                  printf("+extra note-off! %d channel %d pitch %d velo %d\n",
-                           i->first, ev->channel(), ev->pitch(), ev->velo());
+                  printf("+extra note-off! %d pitch %d velo %d\n",
+                           i->first, ev->pitch(), ev->velo());
 //                  ev->dump();
                   }
             }
@@ -1492,7 +1500,15 @@ void MidiFile::processTrack2(MidiTrack* track)
             int etick = part->posTick() + part->lenTick();
             iEvent r1 = tevents->lower_bound(stick);
             iEvent r2 = tevents->lower_bound(etick);
-            part->events()->insert(r1, r2);
+            int startTick = part->posTick();
+
+            EventList* el = part->events();
+            for (iEvent i = r1; i != r2; ++i) {
+                  MidiEvent* ev = (MidiEvent*)(i->second);
+                  int ntick = ev->posTick() - startTick;
+                  ev->setPosTick(ntick);
+                  el->add(ev, ntick);
+                  }
             tevents->erase(r1, r2);
             }
 
@@ -1559,6 +1575,9 @@ bool MidiFile::read()
                   return true;
             }
 
+      for (int i = 0; i < 16; ++i)
+            midiPorts[0].resetIstate(i, false);
+
       TrackList* tl = song->tracks();
       for (iTrack t = tl->begin(); t != tl->end(); ++t) {
             MidiTrack* track = dynamic_cast<MidiTrack*>(*t);
@@ -1602,7 +1621,6 @@ bool MidiFile::read()
                         }
                   }
             }
-
       return false;
       }
 
