@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: midictrl.cpp,v 1.3 2002/02/08 09:56:29 muse Exp $
+//  $Id: midictrl.cpp,v 1.1.1.1 2003/10/29 10:05:09 wschweer Exp $
 //
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -122,7 +122,6 @@ MidiController* readMidiController(Xml& xml)
       {
       MidiController* ctrl = new MidiController();
       ctrl->read(xml);
-      midiControllerList.push_back(ctrl);
       return ctrl;
       }
 
@@ -189,7 +188,8 @@ void MidiController::read(Xml& xml)
 //---------------------------------------------------------
 
 static QString ctrlTypes[] = {
-      "Control7", "Control14", "RPN", "NRPN", "XG-SysEx", "SysEx"
+      QString("Control7"), QString("Control14"), QString("RPN"),
+      QString("NRPN"), QString("XG-SysEx"), QString("SysEx")
       };
 
 //---------------------------------------------------------
@@ -239,20 +239,17 @@ void MidiControllerEditDialog::addControllerToView(MidiController* mctrl)
 //   MidiControllerEditDialog
 //---------------------------------------------------------
 
-MidiControllerEditDialog::MidiControllerEditDialog(QWidget* parent = 0, const char* name = 0, bool modal = false, WFlags fl = 0)
+MidiControllerEditDialog::MidiControllerEditDialog(QWidget* parent, const char* name, bool modal, WFlags fl)
    : MidiControllerEditDialogBase(parent, name, modal, fl)
       {
       viewController->setAllColumnsShowFocus(true);
       viewController->setColumnAlignment(COL_HNUM, AlignCenter);
       viewController->setColumnAlignment(COL_LNUM, AlignCenter);
-      viewController->setColumnAlignment(COL_MIN, AlignCenter);
-      viewController->setColumnAlignment(COL_MAX, AlignCenter);
+      viewController->setColumnAlignment(COL_MIN,  AlignCenter);
+      viewController->setColumnAlignment(COL_MAX,  AlignCenter);
       viewController->setColumnWidthMode(COL_NAME, QListView::Maximum);
 
-      for (iMidiController i = midiControllerList.begin();
-         i != midiControllerList.end(); ++i) {
-            addControllerToView(*i);
-            }
+      reject();   // populate list
       viewController->setCurrentItem(viewController->firstChild());
 
       connect(buttonNew,    SIGNAL(clicked()), SLOT(ctrlNew()));
@@ -267,10 +264,25 @@ MidiControllerEditDialog::MidiControllerEditDialog(QWidget* parent = 0, const ch
          SLOT(controllerChanged(QListViewItem*)));
 
       connect(saveAsButton, SIGNAL(clicked()), SLOT(saveAs()));
-      connect(loadButton, SIGNAL(clicked()), SLOT(load()));
+      connect(replaceButton, SIGNAL(clicked()), SLOT(replace()));
+      connect(mergeButton, SIGNAL(clicked()), SLOT(merge()));
       controllerChanged(viewController->currentItem());
       }
 
+//---------------------------------------------------------
+//   reject
+//---------------------------------------------------------
+
+void MidiControllerEditDialog::reject()
+      {
+      viewController->clear();
+      for (iMidiController i = midiControllerList.begin();
+         i != midiControllerList.end(); ++i) {
+            addControllerToView(*i);
+            }
+      MidiControllerEditDialogBase::reject();
+      }
+
 //---------------------------------------------------------
 //   ctrlNew
 //---------------------------------------------------------
@@ -278,7 +290,7 @@ MidiControllerEditDialog::MidiControllerEditDialog(QWidget* parent = 0, const ch
 void MidiControllerEditDialog::ctrlNew()
       {
       QListViewItem* item = new QListViewItem(viewController,
-         ctrlName[1], int2ctrlType(0), "1", "---", "0", "127");
+         QString(ctrlName[1]), int2ctrlType(0), QString("1"), QString("---"), QString("0"), QString("127"));
       viewController->setCurrentItem(item);
       controllerChanged(item);
       }
@@ -319,8 +331,7 @@ void MidiControllerEditDialog::accept()
             midiControllerList.push_back(c);
             item = item->nextSibling();
             }
-      if (close(true))
-            midiControllerEditDialog = 0;
+      MidiControllerEditDialogBase::accept();
       }
 
 //---------------------------------------------------------
@@ -348,8 +359,8 @@ void MidiControllerEditDialog::typeChanged(const QString& s)
       switch(ctrlType2Int(s)) {
             case 0:     // Control7
             case 1:     // Control14
-                  item->setText(COL_MIN, "0");
-                  item->setText(COL_MAX, "127");
+                  item->setText(COL_MIN, QString("0"));
+                  item->setText(COL_MAX, QString("127"));
                   break;
             case 2:     // RPN
             case 3:     // NRPN
@@ -375,13 +386,13 @@ void MidiControllerEditDialog::valueHChanged(int val)
       s.setNum(val);
       item->setText(COL_HNUM, s);
       if (item->text(COL_TYPE) == "Control7") {
-            item->setText(COL_NAME, ctrlName[val]);
-            entryName->setText(ctrlName[val]);
+            item->setText(COL_NAME, QString(ctrlName[val]));
+            entryName->setText(QString(ctrlName[val]));
             }
       if (item->text(COL_TYPE) == "Control14") {
             if (val < 32) {
-                  item->setText(COL_NAME, ctrl14Name[val]);
-                  entryName->setText(ctrl14Name[val]);
+                  item->setText(COL_NAME, QString(ctrl14Name[val]));
+                  entryName->setText(QString(ctrl14Name[val]));
                   spinboxLCtrlNo->setValue(val + 32);
                   QString lnum;
                   lnum.setNum(val + 32);
@@ -411,7 +422,7 @@ void MidiControllerEditDialog::valueLChanged(int val)
 void MidiControllerEditDialog::controllerChanged(QListViewItem* item)
       {
       if (item == 0) {
-            entryName->setText("");
+            entryName->setText(QString(""));
             comboType->setCurrentItem(0);
             spinboxHCtrlNo->setValue(1);
             spinboxLCtrlNo->setValue(33);
@@ -429,7 +440,7 @@ void MidiControllerEditDialog::controllerChanged(QListViewItem* item)
                   spinboxLCtrlNo->setEnabled(false);
                   spinboxMin->setValue(item->text(COL_MIN).toInt());
                   spinboxMax->setValue(item->text(COL_MAX).toInt());
-                  item->setText(COL_LNUM, "---");
+                  item->setText(COL_LNUM, QString("---"));
                   break;
             case 1:     // Control14
             case 2:     // RPN
@@ -492,10 +503,12 @@ void configMidiController()
 
 void MidiControllerEditDialog::saveAs()
       {
-      MFile file("controller", ".ctrl");
-      FILE* f = file.open("w", ctrl_file_pattern, this, false, true);
-      if (f == 0)
+      MFile file(QString("controller/"), QString(".ctrl"));
+      FILE* f = file.open("w", ctrl_file_pattern, this, false, true,
+         tr("MusE: save midi controller list"));
+      if (f == 0) {
             return;
+            }
       Xml xml(f);
       xml.header();
       xml.tag(0, "muse version=\"1.0\"");
@@ -517,16 +530,29 @@ void MidiControllerEditDialog::saveAs()
       xml.tag(1, "/muse");
       }
 
+void MidiControllerEditDialog::merge()
+      {
+      mergeReplace(false);
+      }
+
+void MidiControllerEditDialog::replace()
+      {
+      mergeReplace(true);
+      }
+
 //---------------------------------------------------------
-//   load
+//   mergeReplace
 //---------------------------------------------------------
 
-void MidiControllerEditDialog::load()
+void MidiControllerEditDialog::mergeReplace(bool replace)
       {
-      MFile file("controller", ".ctrl");
-      FILE* f = file.open("r", ctrl_file_pattern, this, false, true);
+      MFile file(QString("controller/"), QString(".ctrl"));
+      FILE* f = file.open("r", ctrl_file_pattern, this, false, true,
+         tr("MusE: load midi controller list"));
       if (f == 0)
             return;
+      if (replace)
+            viewController->clear();
       Xml xml(f);
       int mode = 0;
       for (;;) {
@@ -561,8 +587,7 @@ void MidiControllerEditDialog::load()
 //   addMidiController
 //---------------------------------------------------------
 
-void addMidiController(MidiController::ControllerType type,
-   int hval, int lval)
+void addMidiController(MidiController::ControllerType type, int hval, int lval)
       {
       hval &= 0x7f;
       lval &= 0x7f;
