@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: dcanvas.cpp,v 1.2 2002/02/27 15:53:45 muse Exp $
+//    $Id: dcanvas.cpp,v 1.3 2003/11/18 18:43:05 lunar_shuttle Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -19,10 +19,10 @@
 #include "midieditor.h"
 #include "drummap.h"
 #include "event.h"
+#include "mpevent.h"
 #include "xml.h"
 #include "globals.h"
 #include "midiport.h"
-#include "seq.h"
 #include "midithread.h"
 
 #define CARET   10
@@ -35,9 +35,10 @@
 DEvent::DEvent(MidiEvent* e, Part* p)
   : CItem(e, p)
       {
-      int instr = drumOutmap[e->pitch()];
+      int instr = e->pitch();
       int y  = instr * TH + TH/2;
-      setPos(QPoint(e->posTick(), y));
+      int tick = e->posTick() + p->posTick();
+      setPos(QPoint(tick, y));
       setBBox(QRect(-CARET2, -CARET2, CARET, CARET));
       }
 
@@ -56,7 +57,7 @@ void DrumCanvas::addItem(Part* part, Event* event)
 //---------------------------------------------------------
 
 DrumCanvas::DrumCanvas(MidiEditor* pr, QWidget* parent, int sx,
-   int sy, const char* name = 0)
+   int sy, const char* name)
    : EventCanvas(pr, parent, sx, sy, name)
       {
       setVirt(false);
@@ -67,22 +68,23 @@ DrumCanvas::DrumCanvas(MidiEditor* pr, QWidget* parent, int sx,
 //   moveItem
 //---------------------------------------------------------
 
-bool DrumCanvas::moveItem(CItem* item, const QPoint& pos, bool copy)
+bool DrumCanvas::moveItem(CItem* item, const QPoint& pos, DragType dtype)
       {
       DEvent* nevent   = (DEvent*) item;
+      MidiPart* part   = (MidiPart*)nevent->part();
       MidiEvent* event = (MidiEvent*)nevent->event();
       int ntick        = editor->rasterVal(pos.x());
-      int npitch       = drumMap[y2pitch(pos.y())].enote;
+      int npitch       = y2pitch(pos.y());
       MidiEvent* newEvent = new MidiEvent(*event);
 
       newEvent->setPitch(npitch);
-      newEvent->setPosTick(ntick);
+      newEvent->setPosTick(ntick - part->posTick());
 
-      if (copy) {
-            midiThread->msgAddEvent(newEvent, nevent->part(), false);
+      if (dtype == MOVE_COPY || dtype == MOVE_CLONE) {
+            midiThread->msgAddEvent(newEvent, part, false);
             }
       else {
-            midiThread->msgChangeEvent(event, newEvent, nevent->part(), false);
+            midiThread->msgChangeEvent(event, newEvent, part, false);
             }
       return true;
       }
@@ -101,9 +103,10 @@ CItem* DrumCanvas::newItem(const QPoint& p, int state)
             velo = drumMap[instr].lv2;
       else if (state == (ControlButton | ShiftButton))
             velo = drumMap[instr].lv1;
-      int x = editor->rasterVal(p.x());
-      MidiEvent* e = new MidiEvent(track()->outPort(), track()->outChannel(),
-         x, MidiEvent::Note, drumMap[instr].anote, velo, 0,
+      int tick = editor->rasterVal(p.x());
+      tick     -= curPart->posTick();
+      MidiEvent* e = new MidiEvent(
+         tick, MidiEvent::Note, instr, velo, 0,
          drumMap[instr].len);
       return new DEvent(e, curPart);
       }
@@ -126,11 +129,11 @@ void DrumCanvas::newItem(CItem* item, bool noSnap)
       {
       DEvent* nevent = (DEvent*) item;
       MidiEvent* event = (MidiEvent*)nevent->event();
-      if (noSnap)
-            event->setPosTick(item->x());
-      else
-            event->setPosTick(editor->rasterVal(item->x()));
-      int npitch = drumMap[y2pitch(item->y())].enote;
+      int x = item->x();
+      if (!noSnap)
+            x = editor->rasterVal(x);
+      event->setPosTick(x - nevent->part()->posTick());
+      int npitch = event->pitch();
       event->setPitch(npitch);
 
       //
@@ -170,6 +173,8 @@ bool DrumCanvas::deleteItem(CItem* item)
 void DrumCanvas::drawItem(QPainter&p, const CItem*item, const QRect&) const
       {
       p.setPen(black);
+      DEvent* e   = (DEvent*) item;
+      MidiEvent* me = (MidiEvent*)e->event();
 
       int x = mapx(item->pos().x());
       int y = mapy(item->pos().y());
@@ -180,11 +185,8 @@ void DrumCanvas::drawItem(QPainter&p, const CItem*item, const QRect&) const
       pa.setPoint(2, x + CARET2, y);
       pa.setPoint(3, x,          y + CARET2);
 
-      DEvent* e   = (DEvent*) item;
-      MidiEvent* me = (MidiEvent*)e->event();
-      int pitch   = me->pitch();
       int velo    = me->velo();
-      DrumMap* dm = &drumMap[pitch];
+      DrumMap* dm = &drumMap[y2pitch(y)];
 
       QColor color;
       if (velo < dm->lv1)
@@ -310,8 +312,9 @@ void DrumCanvas::cmd(int cmd)
             case CMD_SELECT_ILOOP:     // select inside loop
                   for (iCItem k = items.begin(); k != items.end(); ++k) {
                         DEvent* nevent =(DEvent*)(k->second);
+                        Part* part = nevent->part();
                         Event* event = nevent->event();
-                        int tick  = event->posTick();
+                        int tick  = event->posTick() + part->posTick();
                         if (tick < song->lpos() || tick >= song->rpos())
                               selectItem(k->second, false);
                         else
@@ -321,8 +324,9 @@ void DrumCanvas::cmd(int cmd)
             case CMD_SELECT_OLOOP:     // select outside loop
                   for (iCItem k = items.begin(); k != items.end(); ++k) {
                         DEvent* nevent =(DEvent*)(k->second);
+                        Part* part = nevent->part();
                         MidiEvent* event = (MidiEvent*)nevent->event();
-                        int tick  = event->posTick();
+                        int tick  = event->posTick() + part->posTick();
                         if (tick < song->lpos() || tick >= song->rpos())
                               selectItem(k->second, true);
                         else
@@ -345,6 +349,21 @@ void DrumCanvas::cmd(int cmd)
             case CMD_LOAD:
                   printf("DrumCanvas:: cmd not implemented %d\n", cmd);
                   break;
+            case CMD_FIXED_LEN: //Set notes to the length specified in the drummap
+                  if (selectionSize()) {
+                        song->startUndo();
+                        for (iCItem k = items.begin(); k != items.end(); ++k) {
+                              if (k->second->isSelected()) {
+                                    DEvent* devent =(DEvent*)(k->second);
+                                    MidiEvent* event = (MidiEvent*)devent->event();
+                                    MidiEvent* newEvent = new MidiEvent(*event);
+                                    newEvent->setLenTick(drumMap[event->pitch()].len);
+                                    midiThread->msgChangeEvent(event, newEvent, devent->part() , false);
+                              }
+                        }
+                        song->endUndo(SC_EVENT_MODIFIED);
+                  }
+                  break;
             }
       redraw();
       }
@@ -403,7 +422,7 @@ QTextDrag* DrumCanvas::getTextDrag(QWidget* parent)
       char* fbuf  = (char*)mmap(0, n+1, PROT_READ|PROT_WRITE,
          MAP_PRIVATE, fileno(tmp), 0);
       fbuf[n] = 0;
-      QTextDrag* drag = new QTextDrag(fbuf, parent);
+      QTextDrag* drag = new QTextDrag(QString(fbuf), parent);
       drag->setSubtype("eventlist");
       munmap(fbuf, n);
       fclose(tmp);
@@ -419,7 +438,7 @@ void DrumCanvas::copy()
       {
       QTextDrag* drag = getTextDrag(0);
       if (drag)
-            QApplication::clipboard()->setData(drag);
+            QApplication::clipboard()->setData(drag, QClipboard::Clipboard);
       }
 
 //---------------------------------------------------------
@@ -445,8 +464,6 @@ int DrumCanvas::pasteAt(const QString& pt, int pos)
                               MidiEvent* e = new MidiEvent();
                               e->read(xml);
                               e->setPosTick(e->posTick() + pos);
-                              e->setPort(track()->outPort());
-                              e->setChannel(track()->outChannel());
                               midiThread->msgAddEvent(e, curPart, false);
                               }
                         else
@@ -484,7 +501,7 @@ void DrumCanvas::startDrag(CItem* /* item*/, bool copymode)
       {
       QTextDrag* drag = getTextDrag(this);
       if (drag) {
-//            QApplication::clipboard()->setData(drag);
+//            QApplication::clipboard()->setData(drag, QClipboard::Clipboard);
 
             if (copymode)
                   drag->dragCopy();
@@ -544,13 +561,15 @@ void DrumCanvas::viewDropEvent(QDropEvent* event)
 //   keyPressed
 //---------------------------------------------------------
 
-void DrumCanvas::keyPressed(int pitch, bool)
+void DrumCanvas::keyPressed(int enote, bool)
       {
-      int port    = track()->outPort();
-      int channel = track()->outChannel();
+      int instr   = drumInmap[enote];
+      int anote   = drumMap[instr].anote;
+      int port    = drumMap[instr].port;
+      int channel = drumMap[instr].channel;
 
       // play note:
-      MidiEvent e(port, channel, 0, MidiEvent::Note, pitch, 127, 0, 0);
+      MidiPlayEvent e(port, channel, 0x90, anote, 127); //No mapping of drumnotes on other side
       midiThread->playMidiEvent(&e);
       }
 
@@ -558,17 +577,47 @@ void DrumCanvas::keyPressed(int pitch, bool)
 //   keyReleased
 //---------------------------------------------------------
 
-void DrumCanvas::keyReleased(int pitch, bool)
+void DrumCanvas::keyReleased(int enote, bool)
       {
-      int port    = track()->outPort();
-      int channel = track()->outChannel();
+      int instr   = drumInmap[enote];
+      int anote   = drumMap[instr].anote;
+      int port    = drumMap[instr].port;
+      int channel = drumMap[instr].channel;
 
       // release note:
-      MidiEvent e(port, channel, 0, MidiEvent::Note, pitch, 0, 0, 0);
+      MidiPlayEvent e(port, channel, 0x90, anote, 0);
       midiThread->playMidiEvent(&e);
       }
 
 //---------------------------------------------------------
+//   mapChanged
+//---------------------------------------------------------
+
+void DrumCanvas::mapChanged(int spitch, int dpitch)
+      {
+      MidiTrack* curTrack = track();
+
+      PartList* parts=curTrack->parts();
+      for (iPart part = parts->begin(); part != parts->end(); ++part) {
+            EventList* events = part->second->events();
+            for (iEvent i = events->begin(); i != events->end(); ++i) {
+                  MidiEvent* event = (MidiEvent*) i->second;
+                  int pitch = event->pitch();
+                  if (pitch == spitch) {
+                        MidiEvent* newEvent = new MidiEvent(*event);
+                        newEvent->setPitch(dpitch);
+                        midiThread->msgChangeEvent(event, newEvent, part->second, false);
+                        }
+                  else if (pitch == dpitch) {
+                        MidiEvent* newEvent = new MidiEvent(*event);
+                        newEvent->setPitch(spitch);
+                        midiThread->msgChangeEvent(event, newEvent, part->second, false);
+                        }
+                  }
+            }
+      song->update(SC_DRUMMAP);
+      }
+//---------------------------------------------------------
 //   resizeEvent
 //---------------------------------------------------------
 
