@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: dlist.cpp,v 1.1 2002/01/30 13:25:31 muse Exp $
+//    $Id: dlist.cpp,v 1.4 2003/11/19 21:28:40 lunar_shuttle Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -153,12 +153,17 @@ void DList::draw(QPainter& p, const QRect& rect)
 //   devicesPopupMenu
 //---------------------------------------------------------
 
-void DList::devicesPopupMenu(DrumMap* t, int x, int y)
+void DList::devicesPopupMenu(DrumMap* t, int x, int y, bool changeAll)
       {
       QPopupMenu* p = midiPortsPopup(this);
       int n = p->exec(mapToGlobal(QPoint(x, y)), 0);
-      if (n != -1)
-            t->port = n;
+      if (n != -1) {
+            if (!changeAll)
+                  t->port = n;
+            else
+            for (int i=0; i<DRUM_MAPSIZE; i++)
+                  drumMap[i].port = n;
+            }
       delete p;
       }
 
@@ -170,10 +175,30 @@ void DList::viewMousePressEvent(QMouseEvent* ev)
       {
       int x = ev->x();
       int y = ev->y();
+      int instr = y/TH;
       int button = ev->button();
       bool shift = ev->state() & ShiftButton;
       unsigned pitch = y / TH;
       DrumMap* dm = &drumMap[pitch];
+      if (currentlySelected != 0) {
+            if (dm != currentlySelected) {
+                  currentlySelected->selected = false;
+                  dm->selected = true;
+                  currentlySelected = dm;
+                  emit selectionChanged(currentlySelected);
+                  emit keyFilterChanged(instr);
+                  song->update(SC_DRUMMAP);
+                  repaint();
+            }
+      }
+      else {
+            currentlySelected = dm;
+            dm->selected = true;
+            emit selectionChanged(currentlySelected);
+            emit keyFilterChanged(instr);
+            song->update(SC_DRUMMAP);
+            repaint();
+      }
 
       startY = y;
       sPitch = pitch;
@@ -188,6 +213,14 @@ void DList::viewMousePressEvent(QMouseEvent* ev)
       else if (button == QMouseEvent::MidButton)
             incVal = -1;
 
+      // Check if we're already editing anything and have pressed the mouse elsewhere
+      // In that case, treat it as if a return was pressed
+      if (button == QMouseEvent::LeftButton) {
+                  if ((editEntry && editEntry != dm  || col != selectedColumn) && editEntry != 0) {
+                        returnPressed();
+                        }
+                  }
+
       switch (col) {
             case COL_NONE:
                   break;
@@ -196,8 +229,10 @@ void DList::viewMousePressEvent(QMouseEvent* ev)
                         dm->mute = !dm->mute;
                   break;
             case COL_PORT:
-                  if (button == QMouseEvent::RightButton)
-                        devicesPopupMenu(dm, mapx(x), mapy(pitch * TH));
+                  if (button == QMouseEvent::RightButton) {
+                        bool changeAll = ev->state() & ControlButton;
+                        devicesPopupMenu(dm, mapx(x), mapy(pitch * TH), changeAll);
+                        }
                   break;
             case COL_QNT:
                   dm->quant += incVal;
@@ -209,8 +244,17 @@ void DList::viewMousePressEvent(QMouseEvent* ev)
                         val = 0;
                   else if (val > 127)
                         val = 127;
+                  // Check if there is any other drumMap with the same inmap value
+                  // If so, switch the inmap between the instruments
+                  for (int i=0; i<DRUM_MAPSIZE; i++) {
+                        if (drumMap[i].enote == val && &drumMap[i] != dm) {
+                              drumInmap[dm->enote] = i;
+                              drumMap[i].enote = dm->enote;
+                              break;
+                        }
+                  }
                   dm->enote = val;
-                  drumInmap[val] = pitch;
+                  drumInmap[val] = pitch; // The inmap for the particular note now points to the instrument we're currently at
                   break;
             case COL_LEN:
                   val = dm->len + incVal;
@@ -225,6 +269,7 @@ void DList::viewMousePressEvent(QMouseEvent* ev)
                   else if (val > 127)
                         val = 127;
                   dm->anote = val;
+                  emit keyPressed(dm->enote, shift); //Send input-key
                   break;
             case COL_CHANNEL:
                   val = dm->channel + incVal;
@@ -232,7 +277,12 @@ void DList::viewMousePressEvent(QMouseEvent* ev)
                         val = 0;
                   else if (val > 127)
                         val = 127;
-                  dm->channel = val;
+                  if (ev->state() & ControlButton) {
+                        for (int i=0; i<DRUM_MAPSIZE; i++)
+                              drumMap[i].channel = val;
+                  }
+                  else
+                        dm->channel = val;
                   break;
             case COL_LV1:
                   val = dm->lv1 + incVal;
@@ -267,11 +317,7 @@ void DList::viewMousePressEvent(QMouseEvent* ev)
                   dm->lv4 = val;
                   break;
             case COL_NAME:
-                  if (button == QMouseEvent::LeftButton) {
-                        if (editEntry && editEntry != dm)
-                              returnPressed();
-                        }
-                  emit keyPressed(dm->anote, shift);
+                  emit keyPressed(dm->enote, shift); //Send input key
                   break;
 #if 0
             case COL_CHANNEL:
@@ -305,13 +351,13 @@ void DList::viewMouseDoubleClickEvent(QMouseEvent* ev)
       {
       int x = ev->x();
       int y = ev->y();
-//      int button = ev->button();
       unsigned pitch = y / TH;
       DrumMap* dm = &drumMap[pitch];
 
       int section = header->sectionAt(x);
 
-      if (section == COL_NAME) {
+      if (section == COL_NAME || section == COL_LEN || section == COL_LV1 ||
+            section == COL_LV2 || section == COL_LV3 || section == COL_LV4) {
             editEntry = dm;
             if (editor == 0) {
                   editor = new QLineEdit(this);
@@ -323,9 +369,39 @@ void DList::viewMouseDoubleClickEvent(QMouseEvent* ev)
             int colw = rmapx(header->sectionSize(section));
             int coly = mapy(pitch * TH);
             int colh = rmapy(TH);
-            editor->setText(dm->name);
+            selectedColumn = section; //Store selected column to have an idea of which one was selected when return is pressed
+            switch (section) {
+                  case COL_NAME:
+                  editor->setText(dm->name);
+                  break;
+
+                  case COL_LEN: {
+                  editor->setText(QString::number(dm->len));
+                  break;
+                  }
+
+                  case COL_LV1:
+                  editor->setText(QString::number(dm->lv1));
+                  break;
+
+                  case COL_LV2:
+                  editor->setText(QString::number(dm->lv2));
+                  break;
+
+                  case COL_LV3:
+                  editor->setText(QString::number(dm->lv3));
+                  break;
+
+                  case COL_LV4:
+                  editor->setText(QString::number(dm->lv4));
+                  break;
+            }
+
             editor->end(false);
             editor->setGeometry(colx, coly, colw, colh);
+            //In all cases but the column name, select all text:
+            if (section != COL_NAME)
+                  editor->selectAll();
             editor->show();
             }
       else
@@ -373,10 +449,51 @@ void DList::sizeChange(int, int, int)
 
 void DList::returnPressed()
       {
+      int val = -1;
+      if (selectedColumn != COL_NAME) {
+            val = atoi(editor->text().ascii());
+            if (selectedColumn != COL_LEN) {
+                  if (val > 127) //Check bounds for lv1-lv4 values
+                  val = 127;
+                  if (val < 0)
+                  val = 0;
+                  }
+            }
+
+      switch(selectedColumn) {
+            case COL_NAME:
+                  editEntry->name = editor->text();
+                  break;
+
+            case COL_LEN:
+                  editEntry->len = atoi(editor->text().ascii());
+                  break;
+
+            case COL_LV1:
+                  editEntry->lv1 = val;
+                  break;
+
+            case COL_LV2:
+                  editEntry->lv2 = val;
+                  break;
+
+            case COL_LV3:
+                  editEntry->lv3 = val;
+                  break;
+
+            case COL_LV4:
+                  editEntry->lv4 = val;
+                  break;
+
+            default:
+                  printf("Return pressed in unknown column\n");
+                  break;
+            }
+      selectedColumn = -1;
       editor->hide();
-      editEntry->name = editor->text();
       editEntry = 0;
       setFocus();
+      redraw();
       }
 
 //---------------------------------------------------------
@@ -396,6 +513,17 @@ void DList::tracklistChanged()
       {
       }
 
+//---------------------------------------------------------
+//   songChanged
+//---------------------------------------------------------
+
+void DList::songChanged(int flags)
+      {
+      if (flags & SC_DRUMMAP)
+            redraw();
+      }
+
+
 //---------------------------------------------------------
 //   DList
 //---------------------------------------------------------
@@ -414,7 +542,17 @@ DList::DList(QHeader* h, QWidget* parent, int ymag)
       drag = NORMAL;
       editor = 0;
       editEntry = 0;
+      currentlySelected = 0;
+      selectedColumn = -1;
       }
+//---------------------------------------------------------
+//   ~DList
+//---------------------------------------------------------
+DList::~DList()
+{
+      if (currentlySelected != 0)
+            currentlySelected->selected = false; //Reset the global
+}
 
 //---------------------------------------------------------
 //   viewMouseMoveEvent
@@ -455,11 +593,13 @@ void DList::viewMouseReleaseEvent(QMouseEvent* ev)
             drumMap[sPitch] = drumMap[dPitch];
             drumMap[dPitch] = dm;
             setCursor(QCursor(arrowCursor));
+
             drumInmap[drumMap[sPitch].enote]  = sPitch;
             drumOutmap[drumMap[sPitch].anote] = sPitch;
             drumInmap[drumMap[dPitch].enote]  = dPitch;
             drumOutmap[drumMap[dPitch].anote] = dPitch;
-            emit mapChanged();
+            currentlySelected = &drumMap[dPitch];
+            emit mapChanged(sPitch, dPitch); //Pitch change throughout the whole track done in canvas
             }
       drag = NORMAL;
       redraw();
@@ -475,10 +615,20 @@ void DList::viewMouseReleaseEvent(QMouseEvent* ev)
 
       switch (col) {
             case COL_NAME:
-                  emit keyReleased(dm->anote, shift);
+                  emit keyReleased(dm->enote, shift);
+                  break;
+            case COL_ANOTE:
+                  emit keyReleased(dm->enote, shift);
                   break;
             default:
                   break;
             }
       }
 
+int DList::getSelectedInstrument()
+      {
+            if (currentlySelected == 0)
+                  return -1;
+
+            return drumInmap[currentlySelected->enote];
+      }
