@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: drumedit.cpp,v 1.1 2002/01/30 14:10:08 muse Exp $
+//    $Id: drumedit.cpp,v 1.3 2003/11/19 21:28:40 lunar_shuttle Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -51,9 +51,14 @@ static const char* map_file_pattern[] = {
       };
 int DrumEdit::_quantInit = 96;
 int DrumEdit::_rasterInit = 96;
+int DrumEdit::_widthInit = 600;
+int DrumEdit::_heightInit = 400;
+int DrumEdit::_dlistWidthInit = 50; 
+int DrumEdit::_dcanvasWidthInit = 300; 
 
 static const int xscale = -10;
 static const int yscale = 1;
+static const int drumeditTools = PointerTool | PencilTool | RubberTool;
 
 //---------------------------------------------------------
 //   DWhatsThis::text
@@ -63,26 +68,26 @@ QString DWhatsThis::text(const QPoint& pos)
       {
       int section = header->sectionAt(pos.x());
       if (section == -1)
-            return 0;
+            return QString::null;
       switch(section) {
-            case 0: return  header->tr("mute instrument"); break;
-            case 1: return  header->tr("sound name"); break;
-            case 2: return  header->tr("quantisation"
+            case 0: return  QHeader::tr("mute instrument"); break;
+            case 1: return  QHeader::tr("sound name"); break;
+            case 2: return  QHeader::tr("quantisation"
                   "\ncurrently not used"); break;
-            case 3: return  header->tr("this input note triggers the sound"); break;
-            case 4: return  header->tr("note length"); break;
-            case 5: return  header->tr("this note is send for the sound"); break;
-            case 6: return  header->tr("output channel"
+            case 3: return  QHeader::tr("this input note triggers the sound"); break;
+            case 4: return  QHeader::tr("note length"); break;
+            case 5: return  QHeader::tr("this note is send for the sound"); break;
+            case 6: return  QHeader::tr("output channel"
                   "\ncurrently not used"); break;
-            case 7: return  header->tr("output port"
+            case 7: return  QHeader::tr("output port"
                   "\ncurrently not used"); break;
-            case 8: return  header->tr("velocity level 1"); break;
-            case 9: return  header->tr("velocity level 2"); break;
-            case 10: return header->tr("velocity level 3"); break;
-            case 11: return header->tr("velocity level 4"); break;
+            case 8: return  QHeader::tr("velocity level 1"); break;
+            case 9: return  QHeader::tr("velocity level 2"); break;
+            case 10: return QHeader::tr("velocity level 3"); break;
+            case 11: return QHeader::tr("velocity level 4"); break;
             default: break;
             }
-      return 0;
+      return QString::null;
       }
 
 //---------------------------------------------------------
@@ -99,18 +104,18 @@ void DHeaderTip::maybeTip(const QPoint &pos)
          w->height());
       QString p;
       switch(section) {
-            case 0:  p = w->tr("mute instrument"); break;
-            case 1:  p = w->tr("sound name"); break;
-            case 2:  p = w->tr("quantisation"); break;
-            case 3:  p = w->tr("this input note triggers the sound"); break;
-            case 4:  p = w->tr("note length"); break;
-            case 5:  p = w->tr("this note is send for the sound"); break;
-            case 6:  p = w->tr("output channel"); break;
-            case 7:  p = w->tr("output port"); break;
-            case 8:  p = w->tr("velocity level 1"); break;
-            case 9:  p = w->tr("velocity level 2"); break;
-            case 10: p = w->tr("velocity level 3"); break;
-            case 11: p = w->tr("velocity level 4"); break;
+            case 0:  p = QHeader::tr("mute instrument"); break;
+            case 1:  p = QHeader::tr("sound name"); break;
+            case 2:  p = QHeader::tr("quantisation"); break;
+            case 3:  p = QHeader::tr("this input note triggers the sound"); break;
+            case 4:  p = QHeader::tr("note length"); break;
+            case 5:  p = QHeader::tr("this note is send for the sound"); break;
+            case 6:  p = QHeader::tr("output channel"); break;
+            case 7:  p = QHeader::tr("output port"); break;
+            case 8:  p = QHeader::tr("velocity level 1"); break;
+            case 9:  p = QHeader::tr("velocity level 2"); break;
+            case 10: p = QHeader::tr("velocity level 3"); break;
+            case 11: p = QHeader::tr("velocity level 4"); break;
             default: return;
             }
       tip(r, p);
@@ -122,6 +127,12 @@ void DHeaderTip::maybeTip(const QPoint &pos)
 
 void DrumEdit::closeEvent(QCloseEvent* e)
       {
+      //Store values of the horizontal splitter 
+      QValueList<int> sizes = split2->sizes(); 
+      QValueList<int>::Iterator it = sizes.begin(); 
+      _dlistWidthInit = *it; //There are only 2 values stored in the sizelist, size of dlist widget and dcanvas widget 
+      it++; 
+      _dcanvasWidthInit = *it; 
       emit deleted((int)this);
       e->accept();
       }
@@ -130,14 +141,15 @@ void DrumEdit::closeEvent(QCloseEvent* e)
 //   DrumEdit
 //---------------------------------------------------------
 
-DrumEdit::DrumEdit(PartList* pl, QWidget* parent = 0, const char* name = 0)
+DrumEdit::DrumEdit(PartList* pl, QWidget* parent, const char* name)
    : MidiEditor(_quantInit, _rasterInit, pl, parent, name)
       {
+      resize(_widthInit, _heightInit);
       selEvent = 0;
       selPart  = 0;
       //---------Pulldown Menu----------------------------
       QPopupMenu* menuFile = new QPopupMenu(this);
-      menuBar()->insertItem("&File", menuFile);
+      menuBar()->insertItem(tr("&File"), menuFile);
 
       menuFile->insertItem(*openIcon, tr("Load Map"), DrumCanvas::CMD_LOAD);
       menuFile->setAccel(CTRL+Key_O, DrumCanvas::CMD_LOAD);
@@ -146,6 +158,10 @@ DrumEdit::DrumEdit(PartList* pl, QWidget* parent = 0, const char* name = 0)
 
       menuEdit = new QPopupMenu(this);
       menuBar()->insertItem(tr("&Edit"), menuEdit);
+      undoRedo->addTo(menuEdit);
+      QAccel* qa = new QAccel(this); 
+      qa->connectItem(qa->insertItem(CTRL+Key_Z), song, SLOT(undo()));
+      menuEdit->insertSeparator();
       menuEdit->insertItem(tr("Cut"),   DrumCanvas::CMD_CUT);
       menuEdit->setAccel(CTRL+Key_X,    DrumCanvas::CMD_CUT);
       menuEdit->insertItem(tr("Copy"),  DrumCanvas::CMD_COPY);
@@ -154,7 +170,10 @@ DrumEdit::DrumEdit(PartList* pl, QWidget* parent = 0, const char* name = 0)
       menuEdit->setAccel(CTRL+Key_V,    DrumCanvas::CMD_PASTE);
       menuEdit->insertSeparator();
       menuEdit->insertItem(tr("Delete Events"), DrumCanvas::CMD_DEL);
+      menuEdit->setAccel(Key_Delete, DrumCanvas::CMD_DEL);
       menuEdit->insertSeparator();
+      menuEdit->insertItem(tr("Set fixed length"), DrumCanvas::CMD_FIXED_LEN); 
+      menuEdit->setAccel(ALT+Key_L,     DrumCanvas::CMD_FIXED_LEN); 
 
       QPopupMenu* menuSelect = new QPopupMenu(this);
       menuSelect->insertItem(tr("Select All"),   DrumCanvas::CMD_SELECT_ALL);
@@ -174,14 +193,14 @@ DrumEdit::DrumEdit(PartList* pl, QWidget* parent = 0, const char* name = 0)
       //---------------------------------------------------
 
       tools = new QToolBar(this, "drum-tools");
-      new QToolButton(*openIcon, "Load Drummap",
-					    QString::null, this, SLOT(load()),
-					    tools, "load drummap from file");
+      new QToolButton(*openIcon, tr("Load Drummap"),
+                  QString::null, this, SLOT(load()), 
+                  tools, "load drummap from file"); 
 
-      new QToolButton(*saveIcon, "Store Drummap",
-					    QString::null,
-					    this, SLOT(save()),
-					    tools, "store drummap to file");
+      new QToolButton(*saveIcon, tr("Store Drummap"),
+                  QString::null, 
+                  this, SLOT(save()), 
+                  tools, "store drummap to file"); 
 
       QWhatsThis::whatsThisButton(tools);
 
@@ -199,10 +218,10 @@ DrumEdit::DrumEdit(PartList* pl, QWidget* parent = 0, const char* name = 0)
       midiin->setPixmap(*midiinIcon);
       midiin->setToggleButton(true);
 
-      EditToolBar* etb = new EditToolBar(this,
-         PointerTool | PencilTool | RubberTool);
+      tools2 = new EditToolBar(this, drumeditTools);
 
-      new TransportToolbar(this);
+      QToolBar* transport = new QToolBar(this);
+      transportAction->addTo(transport);
 
       // dontt show pitch value in toolbar
       toolbar = new Toolbar1(this, _rasterInit, _quantInit, false);
@@ -213,20 +232,23 @@ DrumEdit::DrumEdit(PartList* pl, QWidget* parent = 0, const char* name = 0)
       //---------------------------------------------------
 
       split1            = new Splitter(Vertical, mainw, "split1");
-      QPushButton* ctrl = new QPushButton("ctrl", mainw, "Ctrl");
+      QPushButton* ctrl = new QPushButton(tr("ctrl"), mainw, "Ctrl");
+      ctrl->setFont(font3);
       hscroll           = new ScrollScale(-25, -2, xscale, 20000, Horizontal, mainw);
       ctrl->setFixedSize(50, hscroll->sizeHint().height());
       QToolTip::add(ctrl, tr("Add Controller View"));
 
       QSizeGrip* corner = new QSizeGrip(mainw);
+      corner->setFixedHeight(hscroll->sizeHint().height());
 
       mainGrid->setRowStretch(0, 100);
       mainGrid->setColStretch(1, 100);
+
       mainGrid->addMultiCellWidget(split1, 0, 0, 0, 2);
       mainGrid->addWidget(ctrl,    1, 0);
       mainGrid->addWidget(hscroll, 1, 1);
       mainGrid->addWidget(corner,  1, 2, AlignBottom|AlignRight);
-      mainGrid->addRowSpacing(1, hscroll->sizeHint().height());
+//      mainGrid->addRowSpacing(1, hscroll->sizeHint().height());
 
       split2              = new Splitter(Horizontal, split1, "split2");
       QWidget* split1w1   = new QWidget(split2, "split1w1");
@@ -238,12 +260,16 @@ DrumEdit::DrumEdit(PartList* pl, QWidget* parent = 0, const char* name = 0)
       vscroll             = new ScrollScale(1, 5, yscale, DRUM_MAPSIZE*TH, Vertical, split1w2);
       int offset = -(division/4);
       canvas->setOrigin(offset, 0);
+      canvas->setCanvasTools(drumeditTools);
+      canvas->setFocus();
+      connect(canvas, SIGNAL(toolChanged(int)), tools2, SLOT(set(int)));
+
       time->setOrigin(offset, 0);
       vscroll->showMag(false);
 
       QValueList<int> mops;
-      mops.append(50);
-      mops.append(300);
+      mops.append(_dlistWidthInit); 
+      mops.append(_dcanvasWidthInit); 
       split2->setSizes(mops);
 
       gridS2->setRowStretch(1, 100);
@@ -276,7 +302,8 @@ DrumEdit::DrumEdit(PartList* pl, QWidget* parent = 0, const char* name = 0)
       dlist = new DList(header, split1w1, yscale);
       connect(dlist, SIGNAL(keyPressed(int, bool)), canvas, SLOT(keyPressed(int, bool)));
       connect(dlist, SIGNAL(keyReleased(int, bool)), canvas, SLOT(keyReleased(int, bool)));
-      connect(dlist, SIGNAL(mapChanged()), canvas, SLOT(mapChanged()));
+      connect(dlist, SIGNAL(mapChanged(int, int)), canvas, SLOT(mapChanged(int, int)));
+      connect(dlist, SIGNAL(selectionChanged(DrumMap*)), SLOT(dlistSelectionChanged(DrumMap*)));
 
       gridS1->setRowStretch(1, 100);
       gridS1->setColStretch(0, 100);
@@ -285,6 +312,7 @@ DrumEdit::DrumEdit(PartList* pl, QWidget* parent = 0, const char* name = 0)
 
       connect(canvas, SIGNAL(newWidth(int)), SLOT(newCanvasWidth(int)));
       connect(song, SIGNAL(songChanged(int)), SLOT(songChanged(int)));
+      connect(song, SIGNAL(songChanged(int)),      dlist, SLOT(songChanged(int)));
       connect(vscroll, SIGNAL(scrollChanged(int)), canvas, SLOT(setYPos(int)));
       connect(hscroll, SIGNAL(scrollChanged(int)), canvas, SLOT(setXPos(int)));
       connect(hscroll, SIGNAL(scaleChanged(int)),  canvas, SLOT(setXMag(int)));
@@ -295,8 +323,7 @@ DrumEdit::DrumEdit(PartList* pl, QWidget* parent = 0, const char* name = 0)
       connect(hscroll, SIGNAL(scrollChanged(int)),   time,   SLOT(setXPos(int)));
       connect(hscroll, SIGNAL(scaleChanged(int)), time,   SLOT(setXMag(int)));
 
-      connect(etb, SIGNAL(toolChanged(int)), canvas, SLOT(setTool(int)));
-//      connect(etb, SIGNAL(toolChanged(int)), ctrlCanvas, SLOT(setTool(int)));
+      connect(tools2, SIGNAL(toolChanged(int)), canvas, SLOT(setTool(int)));
 
       connect(canvas, SIGNAL(selectionChanged(int, Event*, Part*)), this,
          SLOT(setSelection(int, Event*, Part*)));
@@ -375,8 +402,7 @@ void DrumEdit::setSelection(int tick, Event* e, Part* p)
                selEvent->lenTick(),
                selEvent->pitch(),
                selEvent->velo(),
-               selEvent->veloOff(),
-               selEvent->channel()+1);
+               selEvent->veloOff());
             }
       selectionChanged();
       }
@@ -387,14 +413,19 @@ void DrumEdit::setSelection(int tick, Event* e, Part* p)
 
 void DrumEdit::soloChanged(bool flag)
       {
-      song->setSolo(flag ? canvas->track() : 0);
+      canvas->track()->node()->setSolo(flag);
       }
 
-void DrumEdit::soloChanged(Track* t)
+void DrumEdit::soloChanged(SNode* t)
       {
-      toolbar->setSolo(t == canvas->track());
+      toolbar->setSolo(t == canvas->track()->node());
       }
 
+
+void DrumEdit::dlistSelectionChanged(DrumMap* dm)
+      {
+      QString name = dm->name;
+      }
 //---------------------------------------------------------
 //   setRaster
 //---------------------------------------------------------
@@ -429,6 +460,7 @@ void DrumEdit::noteinfoChanged(NoteInfo::ValType type, int val)
       MidiEvent* event = new MidiEvent(*selEvent);
       switch(type) {
             case NoteInfo::VAL_TIME:
+                  event->setPosTick(val - selPart->posTick());
                   break;
             case NoteInfo::VAL_LEN:
                   event->setLenTick(val);
@@ -439,9 +471,6 @@ void DrumEdit::noteinfoChanged(NoteInfo::ValType type, int val)
             case NoteInfo::VAL_VELOFF:
                   event->setVeloOff(val);
                   break;
-            case NoteInfo::VAL_CHANNEL:
-//                  event->setChannel(val-1);
-                  break;
             case NoteInfo::VAL_PITCH:
                   event->setPitch(val);
                   break;
@@ -501,10 +530,6 @@ void DrumEdit::readStatus(Xml& xml)
                               canvas->setMidiin(val);
                               midiin->setOn(val);
                               }
-                        else if (tag == "ctrledit") {
-                              CtrlEdit* ctrl = addCtrl();
-                              ctrl->readStatus(xml);
-                              }
                         else if (tag == split1->name())
                               split1->readStatus(xml);
                         else if (tag == split2->name())
@@ -515,6 +540,10 @@ void DrumEdit::readStatus(Xml& xml)
                               header->readStatus(xml);
                         else if (tag == "xmag")
                               hscroll->setMag(xml.parseInt());
+                        else if (tag == "ctrledit") {
+                              CtrlEdit* ctrl = addCtrl();
+                              ctrl->readStatus(xml);
+                              }
                         else if (tag == "xpos")
                               hscroll->setPos(xml.parseInt());
                         else if (tag == "ymag")
@@ -557,6 +586,14 @@ void DrumEdit::readConfiguration(Xml& xml)
                               _quantInit = xml.parseInt();
                         else if (tag == "raster")
                               _rasterInit = xml.parseInt();
+                        else if (tag == "width")
+                              _widthInit = xml.parseInt();
+                        else if (tag == "height")
+                              _heightInit = xml.parseInt();
+                        else if (tag == "dcanvaswidth") 
+                              _dcanvasWidthInit = xml.parseInt(); 
+                        else if (tag == "dlistwidth") 
+                              _dlistWidthInit = xml.parseInt(); 
                         else
                               xml.unknown("DrumEdit");
                         break;
@@ -579,6 +616,10 @@ void DrumEdit::writeConfiguration(int level, Xml& xml)
       xml.tag(level++, "drumedit");
       xml.intTag(level, "quant", _quantInit);
       xml.intTag(level, "raster", _rasterInit);
+      xml.intTag(level, "width", _widthInit);
+      xml.intTag(level, "height", _heightInit);
+      xml.intTag(level, "dlistwidth", _dlistWidthInit); 
+      xml.intTag(level, "dcanvaswidth", _dcanvasWidthInit);
       xml.tag(level, "/drumedit");
       }
 
@@ -588,12 +629,12 @@ void DrumEdit::writeConfiguration(int level, Xml& xml)
 
 void DrumEdit::load()
       {
-      QString fn = getOpenFileName("drummaps", map_file_pattern,
-         this, "Muse: Load Drum Map");
+      QString fn = getOpenFileName(tr("drummaps"), map_file_pattern,
+         this, tr("Muse: Load Drum Map"));
       if (fn.isEmpty())
             return;
       bool popenFlag;
-      FILE* f = fileOpen(this, fn, ".map", "r", popenFlag, true);
+      FILE* f = fileOpen(this, fn, QString(".map"), "r", popenFlag, true);
       if (f == 0)
             return;
 
@@ -640,12 +681,12 @@ ende:
 
 void DrumEdit::save()
       {
-      QString fn = getSaveFileName("drummaps", map_file_pattern,
-        this, "MusE: Store Drum Map");
+      QString fn = getSaveFileName(QString("drummaps"), map_file_pattern,
+        this, tr("MusE: Store Drum Map"));
       if (fn.isEmpty())
             return;
       bool popenFlag;
-      FILE* f = fileOpen(this, fn, ".map", "w", popenFlag, false, true);
+      FILE* f = fileOpen(this, fn, QString(".map"), "w", popenFlag, false, true);
       if (f == 0)
             return;
       Xml xml(f);
@@ -686,7 +727,7 @@ void DrumEdit::cmd(int cmd)
 
 void DrumEdit::clipboardChanged()
       {
-      QMimeSource* ms = QApplication::clipboard()->data();
+      QMimeSource* ms = QApplication::clipboard()->data(QClipboard::Clipboard);
       if (ms && ms->format(0)) {
             bool flag = strcmp(ms->format(0), "text/eventlist;charset=UTF-8") == 0;
             menuEdit->setItemEnabled(DrumCanvas::CMD_PASTE, flag);
@@ -717,6 +758,12 @@ CtrlEdit* DrumEdit::addCtrl()
       connect(ctrlEdit, SIGNAL(timeChanged(int)),   SLOT(setTime(int)));
       connect(ctrlEdit, SIGNAL(destroyedCtrl(CtrlEdit*)), SLOT(removeCtrl(CtrlEdit*)));
       connect(ctrlEdit, SIGNAL(yposChanged(int)), toolbar, SLOT(setInt(int)));
+      connect(tools2,   SIGNAL(toolChanged(int)),   ctrlEdit, SLOT(setTool(int)));
+      connect(dlist, SIGNAL(keyFilterChanged(const char )), ctrlEdit, SLOT(setKeyfilter(const char )));
+
+      int curSelectedInstrument = dlist->getSelectedInstrument();
+      if (curSelectedInstrument != -1)
+            ctrlEdit->setKeyfilter((char)curSelectedInstrument);
 
       ctrlEdit->setXPos(hscroll->pos());
       ctrlEdit->setXMag(hscroll->getScaleValue());
@@ -755,3 +802,17 @@ void DrumEdit::newCanvasWidth(int w)
             }
       }
 
+//---------------------------------------------------------
+//   resizeEvent
+//---------------------------------------------------------
+
+void DrumEdit::resizeEvent(QResizeEvent* ev)
+      {
+      QWidget::resizeEvent(ev);
+      _widthInit = ev->size().width();
+      _heightInit = ev->size().height();
+ 
+      //TODO: Make the dlist not expand/shrink, but the canvas instead 
+      }
+
+
