@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: pianoroll.cpp,v 1.1 2002/01/30 14:10:08 muse Exp $
+//    $Id: pianoroll.cpp,v 1.2 2003/11/07 23:38:51 lunar_shuttle Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -42,6 +42,8 @@
 
 int PianoRoll::_quantInit = 96;
 int PianoRoll::_rasterInit = 96;
+int PianoRoll::_widthInit = 600;
+int PianoRoll::_heightInit = 400;
 int PianoRoll::_quantStrengthInit = 80;      // 1 - 100%
 int PianoRoll::_quantLimitInit = 50;         // tick value
 bool PianoRoll::_quantLenInit = false;
@@ -51,14 +53,16 @@ int PianoRoll::colorModeInit = 0;
 static const int xscale = -10;
 static const int yscale = 1;
 static const int pianoWidth = 40;
+static int pianorollTools = PointerTool | PencilTool | RubberTool | DrawTool;
 
 //---------------------------------------------------------
 //   PianoRoll
 //---------------------------------------------------------
 
-PianoRoll::PianoRoll(PartList* pl, QWidget* parent = 0, const char* name = 0)
+PianoRoll::PianoRoll(PartList* pl, QWidget* parent, const char* name)
    : MidiEditor(_quantInit, _rasterInit, pl, parent, name)
       {
+      resize(_widthInit, _heightInit);
       selEvent       = 0;
       selPart        = 0;
       quantConfig    = 0;
@@ -72,6 +76,13 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent = 0, const char* name = 0)
       //---------Men----------------------------------
       menuEdit = new QPopupMenu(this);
       menuBar()->insertItem(tr("&Edit"), menuEdit);
+      undoRedo->addTo(menuEdit);
+      QAccel* ud = new QAccel(this);
+      ud->connectItem(ud->insertItem(CTRL+Key_Z), song, SLOT(undo()));
+      QAccel* rd = new QAccel(this);
+      rd->connectItem(rd->insertItem(CTRL+Key_Y), song, SLOT(redo()));
+
+      menuEdit->insertSeparator();
       menuEdit->insertItem(tr("Cut"),   PianoCanvas::CMD_CUT);
       menuEdit->setAccel(CTRL+Key_X,    PianoCanvas::CMD_CUT);
       menuEdit->insertItem(tr("Copy"),  PianoCanvas::CMD_COPY);
@@ -80,12 +91,16 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent = 0, const char* name = 0)
       menuEdit->setAccel(CTRL+Key_V,    PianoCanvas::CMD_PASTE);
       menuEdit->insertSeparator();
       menuEdit->insertItem(tr("Delete Events"), PianoCanvas::CMD_DEL);
+      menuEdit->setAccel(Key_Delete, PianoCanvas::CMD_DEL);
       menuEdit->insertSeparator();
 
       QPopupMenu* menuSelect = new QPopupMenu(this);
       menuSelect->insertItem(tr("Select All"),   PianoCanvas::CMD_SELECT_ALL);
+      menuSelect->setAccel(CTRL+Key_A, PianoCanvas::CMD_SELECT_ALL);
       menuSelect->insertItem(tr("Select None"),  PianoCanvas::CMD_SELECT_NONE);
+      menuSelect->setAccel(CTRL+SHIFT+Key_A, PianoCanvas::CMD_SELECT_NONE);
       menuSelect->insertItem(tr("Invert"),       PianoCanvas::CMD_SELECT_INVERT);
+      menuSelect->setAccel(CTRL+Key_I, PianoCanvas::CMD_SELECT_INVERT);
       menuSelect->insertSeparator();
       menuSelect->insertItem(tr("Inside Loop"),  PianoCanvas::CMD_SELECT_ILOOP);
       menuSelect->insertItem(tr("Outside Loop"), PianoCanvas::CMD_SELECT_OLOOP);
@@ -161,11 +176,12 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent = 0, const char* name = 0)
       speaker->setPixmap(*speakerIcon);
       speaker->setToggleButton(true);
 
-      tools2 = new EditToolBar(this, PointerTool | PencilTool | RubberTool | DrawTool);
+      tools2 = new EditToolBar(this, pianorollTools);
 
       //-------------------------------------------------------------
       //    Transport Bar
-      new TransportToolbar(this);
+      QToolBar* transport = new QToolBar(this);
+      transportAction->addTo(transport);
 
       toolbar = new Toolbar1(this, _rasterInit, _quantInit);
       info    = new NoteInfo(this);
@@ -175,7 +191,7 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent = 0, const char* name = 0)
       //---------------------------------------------------
 
       splitter = new Splitter(Vertical, mainw, "splitter");
-      QPushButton* ctrl = new QPushButton("ctrl", mainw, "Ctrl");
+      QPushButton* ctrl = new QPushButton(tr("ctrl"), mainw, "Ctrl");
       QToolTip::add(ctrl, tr("Add Controller View"));
       hscroll = new ScrollScale(-25, -2, xscale, 20000, Horizontal, mainw);
       ctrl->setFixedSize(pianoWidth, hscroll->sizeHint().height());
@@ -199,6 +215,9 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent = 0, const char* name = 0)
 
       int offset = -(division/4);
       canvas->setOrigin(offset, 0);
+      canvas->setCanvasTools(pianorollTools);
+      canvas->setFocus();
+      connect(canvas, SIGNAL(toolChanged(int)), tools2, SLOT(set(int)));
       time->setOrigin(offset, 0);
 
       gridS1->setRowStretch(2, 100);
@@ -243,8 +262,7 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent = 0, const char* name = 0)
       vscroll->setPos(KH * 30);
       setSelection(0, 0, 0);
 
-      connect(song, SIGNAL(songChanged(int)), SLOT(songChanged(int)));
-      connect(song, SIGNAL(soloChanged(SoundSource*)), SLOT(soloChanged(SoundSource*)));
+      connect(song, SIGNAL(songChanged(int)), SLOT(songChanged1(int)));
 
       setCaption(canvas->getCaption());
       int s, e;
@@ -271,6 +289,17 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent = 0, const char* name = 0)
       selectionChanged(); // enable/disable "Copy" & "Paste"
       }
 
+//---------------------------------------------------------
+//   songChanged1
+//---------------------------------------------------------
+
+void PianoRoll::songChanged1(int bits)
+      {
+      if (bits & SC_SOLO)
+            toolbar->setSolo(canvas->track()->solo());
+      songChanged(bits);
+      }
+
 //---------------------------------------------------------
 //   follow
 //---------------------------------------------------------
@@ -331,8 +360,7 @@ void PianoRoll::setSelection(int tick, Event* e, Part* p)
                selEvent->lenTick(),
                selEvent->pitch(),
                selEvent->velo(),
-               selEvent->veloOff(),
-               selEvent->channel()+1);
+               selEvent->veloOff());
             }
       else {
             info->setEnabled(false);
@@ -353,6 +381,7 @@ void PianoRoll::noteinfoChanged(NoteInfo::ValType type, int val)
       MidiEvent* event = new MidiEvent(*selEvent);
       switch(type) {
             case NoteInfo::VAL_TIME:
+                  event->setPosTick(val - selPart->posTick());
                   break;
             case NoteInfo::VAL_LEN:
                   event->setLenTick(val);
@@ -363,9 +392,6 @@ void PianoRoll::noteinfoChanged(NoteInfo::ValType type, int val)
             case NoteInfo::VAL_VELOFF:
                   event->setVeloOff(val);
                   break;
-            case NoteInfo::VAL_CHANNEL:
-//                  event->setChannel(val-1);
-                  break;
             case NoteInfo::VAL_PITCH:
                   event->setPitch(val);
                   break;
@@ -448,6 +474,10 @@ void PianoRoll::readConfiguration(Xml& xml)
                               _toInit = xml.parseInt();
                         else if (tag == "colormode")
                               colorModeInit = xml.parseInt();
+                        else if (tag == "width")
+                              _widthInit = xml.parseInt();
+                        else if (tag == "height")
+                              _heightInit = xml.parseInt();
                         else
                               xml.unknown("PianoRoll");
                         break;
@@ -473,18 +503,20 @@ void PianoRoll::writeConfiguration(int level, Xml& xml)
       xml.intTag(level, "quantLimit", _quantLimitInit);
       xml.intTag(level, "quantLen", _quantLenInit);
       xml.intTag(level, "to", _toInit);
+      xml.intTag(level, "width", _widthInit);
+      xml.intTag(level, "height", _heightInit);
       xml.intTag(level, "colormode", colorModeInit);
       xml.etag(level, "pianoroll");
       }
 
-void PianoRoll::soloChanged(SoundSource* s)
-      {
-      toolbar->setSolo(s == canvas->track());
-      }
+//---------------------------------------------------------
+//   soloChanged
+//    signal from solo button
+//---------------------------------------------------------
 
 void PianoRoll::soloChanged(bool flag)
       {
-      song->setSolo(flag ? canvas->track() : 0);
+      canvas->track()->node()->setSolo(flag);
       }
 
 void PianoRoll::setRaster(int val)
@@ -493,6 +525,7 @@ void PianoRoll::setRaster(int val)
       MidiEditor::setRaster(val);
       canvas->setFocus();     // give back focus after kb input
       }
+
 void PianoRoll::setQuant(int val)
       {
       _quantInit = val;
@@ -622,7 +655,6 @@ void PianoRoll::keyPressEvent(QKeyEvent* event)
             return;
             }
       bool shift = event->state() & ShiftButton;
-//      bool ctrl  = event->state() & ControlButton;
 
       int index = 0;
       int n = sizeof(rasterTable);
@@ -636,6 +668,9 @@ void PianoRoll::keyPressEvent(QKeyEvent* event)
 
       PianoCanvas* pc = (PianoCanvas*)canvas;
       switch(event->key()) {
+            case Key_Escape:
+                  close();
+                  return;
             case Key_Right:
                   pc->pianoCmd(shift ? CMD_INSERT : CMD_RIGHT);
                   return;
@@ -672,6 +707,43 @@ void PianoRoll::keyPressEvent(QKeyEvent* event)
             case Key_T:
                   val = rasterTable[index + ((off == 0) ? 9 : 0)];
                   break;
+            case Key_E:
+                  if (colorMode == 0)
+                        colorMode = 1;
+                  else if (colorMode == 1)
+                        colorMode = 2;
+                  else
+                        colorMode = 0;
+                  setEventColorMode(colorMode);
+                  return;
+            case Key_H: {
+                  //This is a quite dirty hack. Since it's impossible to set scale value directly without using the [1,1024] domain,
+                  //just increase/decrease until we reach next scale value. Would be better to put a function in
+                  //scrollscale which receives a value from [scaleMin,scaleMax] directly. Not efficient for all in the world! /M.L.
+                  int xmag = hscroll->mag();
+                  int xscale = hscroll->getScaleValue();
+                  if (xscale < hscroll->getScaleMax()) {
+                        int xscale_next = xscale + 1;
+                        while (hscroll->getScaleValue() != xscale_next) {
+                              xmag+=10;
+                              hscroll->setXmag(xmag);
+                        }
+                  }
+                  return;
+            }
+            case Key_G: {
+                  int xmag = hscroll->mag();
+                  int xscale = hscroll->getScaleValue();
+                  //I guess the rounding of the numbers makes it impossible to reach min /M.L.
+                  if (xscale > hscroll->getScaleMin() + 1  && xmag > 20) {
+                        int xscale_next = xscale - 1;
+                        while (hscroll->getScaleValue() != xscale_next) {
+                              xmag-=10;
+                              hscroll->setXmag(xmag);
+                        }
+                  }
+                  return;
+            }
             case Key_Period:
                   val = rasterTable[index + ((off == 18) ? 9 : 18)];
                   break;
@@ -742,11 +814,19 @@ void PianoRoll::setEventColorMode(int mode)
 
 void PianoRoll::clipboardChanged()
       {
-      QMimeSource* ms = QApplication::clipboard()->data();
-      if (ms && ms->format(0)) {
-            bool flag = strcmp(ms->format(0), "text/eventlist;charset=UTF-8") == 0;
-            menuEdit->setItemEnabled(PianoCanvas::CMD_PASTE, flag);
+      QMimeSource* ms = QApplication::clipboard()->data(QClipboard::Clipboard);
+      bool flag = false;
+      if (ms) {
+            for (int i = 0;; ++i) {
+                  if (ms->format(i) == 0)
+                        break;
+//                  printf("clipboard changed %s\n", ms->format(i));
+                  flag = strcmp(ms->format(i), "text/eventlist;charset=UTF-8") == 0;
+                  if (flag)
+                        break;
+                  }
             }
+      menuEdit->setItemEnabled(PianoCanvas::CMD_PASTE, flag);
       }
 
 //---------------------------------------------------------
@@ -771,3 +851,14 @@ void PianoRoll::setSpeaker(bool val)
       canvas->playEvents(_playEvents);
       }
 
+//---------------------------------------------------------
+//   resizeEvent
+//---------------------------------------------------------
+
+void PianoRoll::resizeEvent(QResizeEvent* ev)
+      {
+      QWidget::resizeEvent(ev);
+      _widthInit = ev->size().width();
+      _heightInit = ev->size().height();
+      }
+
