@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: prcanvas.cpp,v 1.2 2002/02/27 08:48:09 muse Exp $
+//    $Id: prcanvas.cpp,v 1.1.1.1 2003/10/29 10:06:10 wschweer Exp $
 //  (C) Copyright 1999,2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -20,6 +20,7 @@
 #include "prcanvas.h"
 #include "midiport.h"
 #include "event.h"
+#include "mpevent.h"
 #include "globals.h"
 
 #include "cmd.h"
@@ -34,8 +35,9 @@
 NEvent::NEvent(Event* e, Part* p, int y) : CItem(e, p)
       {
       y = y - KH/4;
-      setPos(QPoint(e->posTick(), y));
-      setBBox(QRect(e->posTick(), y, e->lenTick(), KH/2));
+      int tick = e->posTick() + p->posTick();
+      setPos(QPoint(tick, y));
+      setBBox(QRect(tick, y, e->lenTick(), KH/2));
       }
 
 //---------------------------------------------------------
@@ -167,7 +169,7 @@ void PianoCanvas::drawItem(QPainter& p, const CItem* item,
                               if (velo < 64)
                                     color.setRgb(velo*4, 0, 0xff);
                               else
-                                    color.setRgb(0xff, 0, (velo-64) * 4);
+                                    color.setRgb(0xff, 0, (127-velo) * 4);
                               }
                               break;
                         }
@@ -191,37 +193,35 @@ void PianoCanvas::viewMouseDoubleClickEvent(QMouseEvent* event)
 
 //---------------------------------------------------------
 //   moveItem
-//    wird nach dem Moven eines Objektes aufgerufen
+//    called after moving an object
 //---------------------------------------------------------
 
-bool PianoCanvas::moveItem(CItem* item, const QPoint& pos, bool copy)
+bool PianoCanvas::moveItem(CItem* item, const QPoint& pos, DragType dtype)
       {
-      NEvent* nevent   = (NEvent*) item;
-      MidiEvent* event = (MidiEvent*)nevent->event();
-      int ntick        = editor->rasterVal(pos.x());
-      int npitch       = y2pitch(pos.y());
+      NEvent* nevent      = (NEvent*) item;
+      MidiEvent* event    = (MidiEvent*)nevent->event();
+      int npitch          = y2pitch(pos.y());
       MidiEvent* newEvent = new MidiEvent(*event);
 
       if (event->pitch() != npitch && _playEvents) {
-            int port         = track()->outPort();
-            int channel      = track()->outChannel();
+            int port    = track()->outPort();
+            int channel = track()->outChannel();
             // release note:
-            MidiEvent ev1(port, channel, 0, MidiEvent::Note, event->pitch(), 0, 0, 0);
+            MidiPlayEvent ev1(port, channel, 0x90, event->pitch(), 0);
             midiThread->playMidiEvent(&ev1);
-            MidiEvent ev2(port, channel, 0, MidiEvent::Note, npitch, event->velo(), 0, 0);
+            MidiPlayEvent ev2(port, channel, 0x90, npitch, event->velo());
             midiThread->playMidiEvent(&ev2);
             }
-
+      Part* part = nevent->part();
       newEvent->setPitch(npitch);
+      int ntick = editor->rasterVal(pos.x()) - part->posTick();
       newEvent->setPosTick(ntick);
       newEvent->setLenTick(event->lenTick());
 
-      if (copy) {
-            midiThread->msgAddEvent(newEvent, nevent->part(), false);
-            }
-      else {
-            midiThread->msgChangeEvent(event, newEvent, nevent->part(), false);
-            }
+      if (dtype == MOVE_COPY || dtype == MOVE_CLONE)
+            midiThread->msgAddEvent(newEvent, part, false);
+      else
+            midiThread->msgChangeEvent(event, newEvent, part, false);
       return true;
       }
 
@@ -234,9 +234,9 @@ CItem* PianoCanvas::newItem(const QPoint& p, int)
       int pitch = y2pitch(p.y());
       int tick  = editor->rasterVal1(p.x());
       int len   = p.x() - tick;
-      MidiEvent* e = new MidiEvent(track()->outPort(),
-         track()->outChannel(), tick, MidiEvent::Note,
-        pitch, curVelo, 0, len);
+      tick     -= curPart->posTick();
+      MidiEvent* e = new MidiEvent(tick, MidiEvent::Note,
+         pitch, curVelo, 0, len);
       return new NEvent(e, curPart, pitch2y(pitch));
       }
 
@@ -244,16 +244,18 @@ void PianoCanvas::newItem(CItem* item, bool noSnap)
       {
       NEvent* nevent = (NEvent*) item;
       MidiEvent* event = (MidiEvent*)nevent->event();
-      if (noSnap) {
-            event->setPosTick(item->x()); //round down
-            event->setLenTick(item->width());
-            }
-      else {
-            event->setPosTick(editor->rasterVal1(item->x())); //round down
-            event->setLenTick(editor->quantVal(item->width()));
+      int x = item->x();
+      int w = item->width();
+
+      if (!noSnap) {
+            x = editor->rasterVal1(x); //round down
+            w = editor->quantVal(w);
             }
+      Part* part = nevent->part();
+      event->setPosTick(x - part->posTick());
+      event->setLenTick(w);
       event->setPitch(y2pitch(item->y()));
-      midiThread->msgAddEvent(event, nevent->part());
+      midiThread->msgAddEvent(event, part);
       }
 
 //---------------------------------------------------------
@@ -319,7 +321,7 @@ void PianoCanvas::pianoCmd(int cmd)
                   for (std::list<MidiEvent*>::iterator i = elist.begin(); i != elist.end(); ++i) {
                         MidiEvent* event = *i;
                         MidiEvent* newEvent = new MidiEvent(*event);
-                        newEvent->setPosTick(event->posTick() + editor->raster());
+                        newEvent->setPosTick(event->posTick() + editor->raster() - part->posTick());
                         midiThread->msgChangeEvent(event, newEvent, part, false);
                         }
                   song->endUndo(SC_EVENT_MODIFIED);
@@ -343,7 +345,7 @@ void PianoCanvas::pianoCmd(int cmd)
                   for (std::list<MidiEvent*>::iterator i = elist.begin(); i != elist.end(); ++i) {
                         MidiEvent* event = *i;
                         MidiEvent* newEvent = new MidiEvent(*event);
-                        newEvent->setPosTick(event->posTick() - editor->raster());
+                        newEvent->setPosTick(event->posTick() - editor->raster() - part->posTick());
                         midiThread->msgChangeEvent(event, newEvent, part, false);
                         }
                   song->endUndo(SC_EVENT_MODIFIED);
@@ -360,21 +362,22 @@ void PianoCanvas::pianoCmd(int cmd)
 
 void PianoCanvas::pianoPressed(int pitch, bool shift)
       {
-      int port    = track()->outPort();
-      int channel = track()->outChannel();
+      int port      = track()->outPort();
+      int channel   = track()->outChannel();
+      pitch        += track()->transposition;
 
       // play note:
-      MidiEvent e(port, channel, 0, MidiEvent::Note, pitch, 127, 0, 0);
+      MidiPlayEvent e(port, channel, 0x90, pitch, 127);
       midiThread->playMidiEvent(&e);
 
       if (_steprec && pos[0] >= start_tick && pos[0] < end_tick) {
             if (curPart == 0)
                   return;
             int len  = editor->quant();
-            int tick = pos[0]; //CDW
+            int tick = pos[0] - curPart->posTick(); //CDW
             if (shift)
                   tick -= editor->rasterStep(tick);
-            MidiEvent* e = new MidiEvent(port, channel,
+            MidiEvent* e = new MidiEvent(
                tick, MidiEvent::Note, pitch, 127, 0, len);
             midiThread->msgAddEvent(e, curPart);
             tick += editor->rasterStep(tick);
@@ -391,9 +394,10 @@ void PianoCanvas::pianoReleased(int pitch, bool)
       {
       int port    = track()->outPort();
       int channel = track()->outChannel();
+      pitch      += track()->transposition;
 
       // release key:
-      MidiEvent e(port, channel, 0, MidiEvent::Note, pitch, 0, 0, 0);
+      MidiPlayEvent e(port, channel, 0x90, pitch, 0);
       midiThread->playMidiEvent(&e);
       }
 
@@ -686,13 +690,14 @@ void PianoCanvas::quantize(int strength, int limit, bool quantLen)
       for (iCItem k = items.begin(); k != items.end(); ++k) {
             NEvent* nevent =(NEvent*)(k->second);
             MidiEvent* event = (MidiEvent*)nevent->event();
+            Part* part = nevent->part();
             if (event->type() != MidiEvent::Note)
                   continue;
 
             if ((cmdRange & CMD_RANGE_SELECTED) && !k->second->isSelected())
                   continue;
 
-            int tick  = event->posTick();
+            int tick = event->posTick() + part->posTick();
 
             if ((cmdRange & CMD_RANGE_LOOP)
                && ((tick < song->lpos() || tick >= song->rpos())))
@@ -712,11 +717,11 @@ void PianoCanvas::quantize(int strength, int limit, bool quantLen)
                   len += ((diff * strength) / 100);
 
             // something changed?
-            if ((event->posTick() != tick) || (event->lenTick() != len)) {
+            if (((event->posTick() + part->posTick()) != tick) || (event->lenTick() != len)) {
                   MidiEvent* newEvent = new MidiEvent(*event);
-                  newEvent->setPosTick(tick);
+                  newEvent->setPosTick(tick - part->posTick());
                   newEvent->setLenTick(len);
-                  midiThread->msgChangeEvent(event, newEvent, nevent->part(), false);
+                  midiThread->msgChangeEvent(event, newEvent, part, false);
                   }
             }
       song->endUndo(SC_EVENT_MODIFIED);
@@ -772,8 +777,8 @@ void PianoCanvas::midiNote(int pitch, int velo)
                         return;
                         }
                   }
-            MidiEvent* e = new MidiEvent(track()->outPort(),
-               track()->outChannel(), tick, MidiEvent::Note, pitch, velo, 0, len);
+            MidiEvent* e = new MidiEvent(
+               tick - curPart->posTick(), MidiEvent::Note, pitch, velo, 0, len);
             midiThread->msgAddEvent(e, curPart);
             tick += editor->rasterStep(tick);
             if (tick != song->cpos())
@@ -816,8 +821,10 @@ QTextDrag* PianoCanvas::getTextDrag(QWidget* parent)
       Xml xml(tmp);
 
       int level = 0;
+      xml.tag(level++, "eventlist");
       for (ciEvent e = el.begin(); e != el.end(); ++e)
             e->second->write(level, xml, -startTick);
+      xml.etag(--level, "eventlist");
 
       //---------------------------------------------------
       //    read tmp file into QTextDrag Object
@@ -835,7 +842,7 @@ QTextDrag* PianoCanvas::getTextDrag(QWidget* parent)
       char* fbuf  = (char*)mmap(0, n+1, PROT_READ|PROT_WRITE,
          MAP_PRIVATE, fileno(tmp), 0);
       fbuf[n] = 0;
-      QTextDrag* drag = new QTextDrag(fbuf, parent);
+      QTextDrag* drag = new QTextDrag(QString(fbuf), parent);
       drag->setSubtype("eventlist");
       munmap(fbuf, n);
       fclose(tmp);
@@ -851,38 +858,42 @@ void PianoCanvas::copy()
       {
       QTextDrag* drag = getTextDrag(0);
       if (drag)
-            QApplication::clipboard()->setData(drag);
+            QApplication::clipboard()->setData(drag, QClipboard::Clipboard);
       }
 
 //---------------------------------------------------------
 //   paste
 //---------------------------------------------------------
 
-int PianoCanvas::pasteAt(const QString& pt, int pos)
+void PianoCanvas::pasteAt(const QString& pt, int pos)
       {
       const char* p = pt.latin1();
       Xml xml(p);
-      song->startUndo();
       for (;;) {
             Xml::Token token = xml.parse();
             const QString& tag = xml.s1();
             switch (token) {
                   case Xml::Error:
                   case Xml::End:
-                        song->endUndo(SC_EVENT_INSERTED);
-                        return pos;
+                        return;
                   case Xml::TagStart:
-                        if (tag == "event") {
-                              MidiEvent* e = new MidiEvent();
-                              e->read(xml);
-                              e->setPosTick(e->posTick() + pos);
-                              e->setPort(track()->outPort());
-                              e->setChannel(track()->outChannel());
-                              midiThread->msgAddEvent(e, curPart, false);
+                        if (tag == "eventlist") {
+                              song->startUndo();
+                              EventList* el = new EventList();
+                              el->read(xml, "eventlist");
+                              for (iEvent i = el->begin(); i != el->end(); ++i) {
+                                    MidiEvent* e = (MidiEvent*)(i->second);
+                                    e->setPosTick(e->posTick() + pos - curPart->posTick());
+                                    midiThread->msgAddEvent(e, curPart, false);
+                                    }
+                              song->endUndo(SC_EVENT_INSERTED);
+                              delete el;
+                              return;
                               }
                         else
-                              xml.unknown("PianoCanvas::paste");
+                              xml.unknown("pasteAt");
                         break;
+                  case Xml::Attribut:
                   case Xml::TagEnd:
                   default:
                         break;
@@ -898,7 +909,7 @@ int PianoCanvas::pasteAt(const QString& pt, int pos)
 void PianoCanvas::paste()
       {
       QCString subtype("eventlist");
-      QMimeSource* ms = QApplication::clipboard()->data();
+      QMimeSource* ms = QApplication::clipboard()->data(QClipboard::Clipboard);
       QString pt;
       if (!QTextDrag::decode(ms, pt, subtype)) {
             printf("cannot paste: bad data type\n");
@@ -915,7 +926,7 @@ void PianoCanvas::startDrag(CItem* /* item*/, bool copymode)
       {
       QTextDrag* drag = getTextDrag(this);
       if (drag) {
-//            QApplication::clipboard()->setData(drag);
+//            QApplication::clipboard()->setData(drag, QClipboard::Clipboard);
 
             if (copymode)
                   drag->dragCopy();
@@ -963,7 +974,6 @@ void PianoCanvas::viewDropEvent(QDropEvent* event)
             return;
             }
       if (QTextDrag::decode(event, text)) {
-            printf("drop <%s>\n", text.ascii());
             int x = editor->rasterVal(event->pos().x());
             if (x < 0)
                   x = 0;
@@ -991,7 +1001,7 @@ void PianoCanvas::itemPressed(const CItem* item)
       int velo         = event->velo();
 
       // play note:
-      MidiEvent e(port, channel, 0, MidiEvent::Note, playedPitch, velo, 0, 0);
+      MidiPlayEvent e(port, channel, 0x90, playedPitch, velo);
       midiThread->playMidiEvent(&e);
       }
 
@@ -999,15 +1009,15 @@ void PianoCanvas::itemPressed(const CItem* item)
 //   itemReleased
 //---------------------------------------------------------
 
-void PianoCanvas::itemReleased(const CItem* item, const QPoint&)
+void PianoCanvas::itemReleased(const CItem*, const QPoint&)
       {
       if (!_playEvents)
             return;
-      int port         = track()->outPort();
-      int channel      = track()->outChannel();
+      int port    = track()->outPort();
+      int channel = track()->outChannel();
 
       // release note:
-      MidiEvent ev(port, channel, 0, MidiEvent::Note, playedPitch, 0, 0, 0);
+      MidiPlayEvent ev(port, channel, 0x90, playedPitch, 0);
       midiThread->playMidiEvent(&ev);
       playedPitch = -1;
       }
@@ -1026,10 +1036,10 @@ void PianoCanvas::itemMoved(const CItem* item, const QPoint& pos)
             MidiEvent* event = (MidiEvent*)nevent->event();
 
             // release note:
-            MidiEvent ev1(port, channel, 0, MidiEvent::Note, playedPitch, 0, 0, 0);
+            MidiPlayEvent ev1(port, channel, 0x90, playedPitch, 0);
             midiThread->playMidiEvent(&ev1);
             // play note:
-            MidiEvent e2(port, channel, 0, MidiEvent::Note, npitch, event->velo(), 0, 0);
+            MidiPlayEvent e2(port, channel, 0x90, npitch, event->velo());
             midiThread->playMidiEvent(&e2);
             playedPitch = npitch;
             }
