@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: miditransform.cpp,v 1.1 2002/01/30 14:10:07 muse Exp $
+//  $Id: miditransform.cpp,v 1.1.1.1 2003/10/29 10:05:04 wschweer Exp $
 //
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -137,8 +137,8 @@ static MidiTransformationList mtlist;
 //    buttonNew buttonDelete buttonApply buttonOk
 //---------------------------------------------------------
 
-MidiTransformerDialog::MidiTransformerDialog(QWidget* parent = 0,
-   const char* name = 0, bool modal = false, WFlags fl = 0)
+MidiTransformerDialog::MidiTransformerDialog(QWidget* parent,
+   const char* name, bool modal, WFlags fl)
    : MidiTransformDialogBase(parent, name, modal, fl)
       {
       data         = new MidiTransformPrivate;
@@ -196,9 +196,9 @@ MidiTransformerDialog::MidiTransformerDialog(QWidget* parent = 0,
                   data->cmt = *i;
             }
       if (data->cmt == 0) {
-            data->cmt = new MidiTransformation("New");
+            data->cmt = new MidiTransformation(tr("New"));
             mtlist.push_back(data->cmt);
-            presetList->insertItem("New");
+            presetList->insertItem(tr("New"));
             }
       data->cindex = 0;
       presetList->setCurrentItem(0);
@@ -294,7 +294,7 @@ void MidiTransformation::write(int level, Xml& xml)
 
 void readMidiTransform(Xml& xml)
       {
-      MidiTransformation trans("new");
+      MidiTransformation trans(QWidget::tr("new"));
 
       for (;;) {
             Xml::Token token = xml.parse();
@@ -388,7 +388,7 @@ void MidiTransformerDialog::accept()
       apply();
       reject();
       }
-
+#if 0
 //---------------------------------------------------------
 //   reject
 //---------------------------------------------------------
@@ -397,6 +397,7 @@ void MidiTransformerDialog::reject()
       {
       close(false);
       }
+#endif
 
 //---------------------------------------------------------
 //   transformEvent
@@ -1354,7 +1355,7 @@ void MidiTransformerDialog::selVal1aChanged(int val)
             selVal1a->setSuffix(" - " + pitch2string(val));
             }
       else
-            selVal1a->setSuffix("");
+            selVal1a->setSuffix(QString(""));
       }
 
 //---------------------------------------------------------
@@ -1369,7 +1370,7 @@ void MidiTransformerDialog::selVal1bChanged(int val)
             selVal1b->setSuffix(" - " + pitch2string(val));
             }
       else
-            selVal1b->setSuffix("");
+            selVal1b->setSuffix(QString(""));
       }
 
 //---------------------------------------------------------
