@@ -1,12 +1,12 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: audionodebutton.cpp,v 1.1 2002/01/30 12:08:38 muse Exp $
+//  $Id: audionodebutton.cpp,v 1.1.1.1 2003/10/29 10:05:23 wschweer Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
-#include "../audiothread.h"
+#include "../audio.h"
 #include "audionodebutton.h"
 
 //---------------------------------------------------------
@@ -29,4 +29,3 @@ void AudioNodeButton::toggled(bool val)
       emit audioNodeToggled(val, src);
       }
 
-
