@@ -1,13 +1,13 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: panknob.cpp,v 1.1 2002/01/30 12:08:38 muse Exp $
+//  $Id: panknob.cpp,v 1.1.1.1 2003/10/29 10:05:23 wschweer Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
-#include <math.h>
-#include "../audiothread.h"
+#include <cmath>
+#include "../audio.h"
 #include "panknob.h"
 #include "globals.h"
 
@@ -28,7 +28,7 @@ PanKnob::PanKnob(QWidget* parent, AudioNode* s)
 
 void PanKnob::valueChanged(double val)
       {
-      audioThread->msgSetPan(src, val);
+      audio->msgSetPan(src, val);
       }
 
 
