@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: rack.h,v 1.2 2001/11/20 15:19:33 muse Exp $
+//  $Id: rack.h,v 1.1.1.1 2003/10/29 10:05:23 wschweer Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -24,7 +24,9 @@ class EffectRack : public QListBox {
       virtual QSize minimumSizeHint() const;
 
    private slots:
-      void buttonPressed(int, QListBoxItem*, const QPoint&);
+      void menuRequested(QListBoxItem*, const QPoint&);
+      void doubleClicked(QListBoxItem*);
+      void songChanged(int);
 
    public:
       EffectRack(QWidget*, AudioNode*, int);
