@@ -1,17 +1,20 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: routecombo.cpp,v 1.1 2002/01/30 12:08:38 muse Exp $
+//  $Id: routecombo.cpp,v 1.1.1.1 2003/10/29 10:05:23 wschweer Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
-#include "../audiothread.h"
+#include "node.h"
 #include "routecombo.h"
 #include <stdio.h>
-#include "audioport.h"
-#include "../driver/audiodev.h"
 #include "song.h"
+#include "audio.h"
+#include "driver/audiodev.h"
+#include "globals.h"
+
+#include <qlistbox.h>
 
 //---------------------------------------------------------
 //   RouteComboBox
@@ -20,7 +23,11 @@
 RouteComboBox::RouteComboBox(QWidget* parent, AudioNode* s)
    : QComboBox(false, parent, "route")
       {
+      setFont(font1);
       src = s;
+      QListBox* lb = new QListBox(this);
+      setListBox(lb);
+      lb->setFixedWidth(90);
       connect(this, SIGNAL(activated(const QString&)), SLOT(setRoute(const QString&)));
       }
 
@@ -30,24 +37,13 @@ RouteComboBox::RouteComboBox(QWidget* parent, AudioNode* s)
 
 void RouteComboBox::setRoute(const QString& r)
       {
-      AudioPort* port  = &audioPort;
-      AudioDevice* dev = port->device();
-      if (dev && (port->rwFlags() & 1)) {
-            if (dev->name() == r) {
-                  audioThread->msgAddRoute(src, port);
-                  return;
-                  }
+      AudioNode* dst = name2Node(r);
+      if (src == &audioOutput) {
+            if (audioDevice)
+                  audioDevice->setOutputPort(r);
             }
-      if (r == "Master")
-            audioThread->msgAddRoute(src, song->master());
-      else if (r == "Group A")
-            audioThread->msgAddRoute(src, song->group(0));
-      else if (r == "Group B")
-            audioThread->msgAddRoute(src, song->group(1));
-      else if (r == "Group C")
-            audioThread->msgAddRoute(src, song->group(2));
-      else if (r == "Group D")
-            audioThread->msgAddRoute(src, song->group(3));
+      else
+            audio->msgSetRoute(src, dst);
+      song->update(SC_ROUTE);
       }
 
-
