@@ -1,13 +1,13 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: volslider.cpp,v 1.1 2002/01/30 12:08:38 muse Exp $
+//  $Id: volslider.cpp,v 1.1.1.1 2003/10/29 10:05:23 wschweer Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
-#include <math.h>
-#include "../audiothread.h"
+#include <cmath>
+#include "../audio.h"
 #include "volslider.h"
 #include "globals.h"
 
@@ -19,6 +19,7 @@ VolSlider::VolSlider(QWidget* parent, AudioNode* s)
    : Slider(parent, "vol", Slider::Vertical, Slider::None, Slider::BgTrough | Slider::BgSlot)
       {
       src = s;
+      setRange(minSlider-0.1, 10.0);
       connect(this, SIGNAL(valueChanged(double)), SLOT(valueChanged(double)));
       }
 
@@ -28,9 +29,14 @@ VolSlider::VolSlider(QWidget* parent, AudioNode* s)
 
 void VolSlider::valueChanged(double val)
       {
-      double vol = pow(10.0, val/20.0);
-      audioThread->msgSetVolume(src, vol);
+      double vol;
+      if (val <= minSlider) {
+            vol = 0.0;
+            val -= 1.0; // display special value "off"
+            }
+      else
+            vol = pow(10.0, val/20.0);
+      audio->msgSetVolume(src, vol);
       emit volChanged(val);
       }
 
-
