@@ -1,14 +1,15 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: midiitransform.cpp,v 1.1 2002/01/30 14:10:08 muse Exp $
+//  $Id: midiitransform.cpp,v 1.1.1.1 2003/10/29 10:06:15 wschweer Exp $
 //
-//  (C) Copyright 2001 Werner Schweer (ws@seh.de)
+//  (C) Copyright 2001-2003 Werner Schweer (ws@seh.de)
 //=========================================================
 
 #include <stdio.h>
 #include <list>
 #include <qpushbutton.h>
+#include <qbuttongroup.h>
 #include <qlineedit.h>
 #include <qcombobox.h>
 #include <qlistbox.h>
@@ -19,6 +20,7 @@
 
 #include "spinboxFP.h"
 #include "event.h"
+#include "mpevent.h"
 #include "midiitransform.h"
 #include "track.h"
 #include "song.h"
@@ -27,12 +29,6 @@
 #include "comboQuant.h"
 #include "pitchedit.h"
 
-class MidiTransformPrivate {
-   public:
-      MidiTransformation* cmt;
-      int cindex;       // current index in preset list
-      };
-
 static int eventTypeTable[] = {
       1, 6, 4, 7, 8, 10, 11
       };
@@ -45,8 +41,8 @@ struct TDict {
       };
 
 static const TDict oplist[] = {
-      TDict(Transform, "Transform"),
-      TDict(Delete, "Filter")
+      TDict(Transform, QString("Transform")),
+      TDict(Delete, QString("Filter"))
       };
 
 //---------------------------------------------------------
@@ -59,7 +55,7 @@ class MidiInputTransformation {
       QString comment;
 
       ValOp selEventOp;
-      MidiEvent::EventType selType;
+      int selType;
 
       ValOp selVal1;
       int selVal1a, selVal1b;
@@ -71,7 +67,7 @@ class MidiInputTransformation {
       int selChannela, selChannelb;
 
       TransformOperator procEvent;
-      MidiEvent::EventType eventType;
+      int eventType;
       TransformOperator procVal1;
       int procVal1a, procVal1b;
       TransformOperator procVal2;
@@ -87,7 +83,7 @@ class MidiInputTransformation {
       MidiInputTransformation(const QString& s) {
             name         = s;
             selEventOp   = All;
-            selType      = MidiEvent::Note;
+            selType      = 0x90;
             selVal1      = Ignore;
             selVal1a     = 0;
             selVal1b     = 0;
@@ -95,7 +91,7 @@ class MidiInputTransformation {
             selVal2a     = 0;
             selVal2b     = 0;
             procEvent    = Keep;
-            eventType    = MidiEvent::Note;
+            eventType    = 0x90;
             procVal1     = Keep;
             procVal1a    = 0;
             procVal1b    = 0;
@@ -116,19 +112,12 @@ class MidiInputTransformation {
             procChannelb = 0;
             }
       void write(int level, Xml& xml) const;
-      int apply(MidiEvent* ev) const;
+      int apply(MidiPlayEvent* ev) const;
       };
 
-class MidiInputTransformPrivate {
-   public:
-      MidiInputTransformation* cmt;
-      int cindex;                   // current index in preset list
-      int cmodul;                   // current index in modules list
-      };
-
-typedef std::list<MidiInputTransformation* > MidiInputTransformationList;
-typedef std::list<MidiInputTransformation* >::iterator iMidiInputTransformation;
-typedef std::list<MidiInputTransformation* >::const_iterator ciMidiInputTransformation;
+typedef std::list<MidiInputTransformation*> MidiInputTransformationList;
+typedef std::list<MidiInputTransformation*>::iterator iMidiInputTransformation;
+typedef std::list<MidiInputTransformation*>::const_iterator ciMidiInputTransformation;
 
 // this is the list of defined transformations:
 static MidiInputTransformationList mtlist;
@@ -149,10 +138,10 @@ static ITransModul modules[MIDI_INPUT_TRANSFORMATIONS];
 //    (filter)
 //---------------------------------------------------------
 
-bool applyMidiInputTransformation(MidiEvent* event)
+bool applyMidiInputTransformation(MidiPlayEvent* event)
       {
       for (int i = 0; i < 4; ++i) {
-            if (modules[i].valid) {
+            if (modules[i].valid && modules[i].transform) {
                   int rv = modules[i].transform->apply(event);
                   if (rv == 1)
                         printf("drop input event\n");
@@ -208,14 +197,14 @@ static bool filterValOp(ValOp op, int val, int val1, int val2)
 //            2 - event changed
 //---------------------------------------------------------
 
-int MidiInputTransformation::apply(MidiEvent* event) const
+int MidiInputTransformation::apply(MidiPlayEvent* event) const
       {
       switch (selEventOp) {
             case Equal:
                   switch(event->type()) {
-                        case MidiEvent::Note:
-                        case MidiEvent::NoteOff:
-                              if (selType != MidiEvent::Note)
+                        case 0x90:
+                        case 0x80:
+                              if (selType != 0x90)
                                     return 0;
                               break;
                         default:
@@ -226,9 +215,9 @@ int MidiInputTransformation::apply(MidiEvent* event) const
                   break;
             case Unequal:
                   switch(event->type()) {
-                        case MidiEvent::Note:
-                        case MidiEvent::NoteOff:
-                              if (selType == MidiEvent::Note)
+                        case 0x90:
+                        case 0x80:
+                              if (selType == 0x90)
                                     return 0;
                               break;
                         default:
@@ -486,6 +475,7 @@ int MidiInputTransformation::apply(MidiEvent* event) const
 //    funcOp       funcQuantVal
 //    buttonNew    buttonDelete
 //
+//    modulGroup
 //    modul1select  modul1enable
 //    modul2select  modul2enable
 //    modul3select  modul3enable
@@ -498,13 +488,13 @@ int MidiInputTransformation::apply(MidiEvent* event) const
 //    procChannelOp procChannelVala procChannelValb
 //---------------------------------------------------------
 
-MidiInputTransformDialog::MidiInputTransformDialog(QWidget* parent = 0,
-   const char* name = 0, bool modal = false, WFlags fl = 0)
+MidiInputTransformDialog::MidiInputTransformDialog(QWidget* parent,
+   const char* name, bool modal, WFlags fl)
    : MidiInputTransformDialogBase(parent, name, modal, fl)
       {
-      data         = new MidiInputTransformPrivate;
-      data->cmt    = 0;
-      data->cindex = 0;
+      cindex = 0;
+      cmodul = 0;
+      cmt    = 0;
 
       for (unsigned i = 0; i < sizeof(oplist)/sizeof(*oplist); ++i)
             funcOp->insertItem(oplist[i].text, i);
@@ -535,14 +525,11 @@ MidiInputTransformDialog::MidiInputTransformDialog(QWidget* parent = 0,
       connect(procVal2a, SIGNAL(valueChanged(int)), SLOT(procVal2aChanged(int)));
       connect(procVal2b, SIGNAL(valueChanged(int)), SLOT(procVal2bChanged(int)));
 
-      connect(modul1select, SIGNAL(toggled(bool)), SLOT(modul1selectChanged(bool)));
       connect(modul1enable, SIGNAL(toggled(bool)), SLOT(modul1enableChanged(bool)));
-      connect(modul2select, SIGNAL(toggled(bool)), SLOT(modul2selectChanged(bool)));
       connect(modul2enable, SIGNAL(toggled(bool)), SLOT(modul2enableChanged(bool)));
-      connect(modul3select, SIGNAL(toggled(bool)), SLOT(modul3selectChanged(bool)));
       connect(modul3enable, SIGNAL(toggled(bool)), SLOT(modul3enableChanged(bool)));
-      connect(modul4select, SIGNAL(toggled(bool)), SLOT(modul4selectChanged(bool)));
       connect(modul4enable, SIGNAL(toggled(bool)), SLOT(modul4enableChanged(bool)));
+      connect(modulGroup,   SIGNAL(clicked(int)),  SLOT(changeModul(int)));
 
       connect(selPortOp,   SIGNAL(activated(int)), SLOT(selPortOpSel(int)));
       connect(selPortVala, SIGNAL(valueChanged(int)), SLOT(selPortValaChanged(int)));
@@ -566,20 +553,19 @@ MidiInputTransformDialog::MidiInputTransformDialog(QWidget* parent = 0,
 
       for (iMidiInputTransformation i = mtlist.begin(); i != mtlist.end(); ++i) {
             presetList->insertItem((*i)->name);
-            if (data->cmt == 0)
-                  data->cmt = *i;
+            if (cmt == 0)
+                  cmt = *i;
             }
-      if (data->cmt == 0) {
+      if (cmt == 0) {
             // create default "New" preset
-            data->cmt = new MidiInputTransformation("New");
-            mtlist.push_back(data->cmt);
-            presetList->insertItem("New");
+            cmt = new MidiInputTransformation(tr("New"));
+            mtlist.push_back(cmt);
+            presetList->insertItem(tr("New"));
             }
-      data->cindex = 0;
       presetList->setCurrentItem(0);
-      data->cmodul = 0;
       modul1select->setOn(true);
-      modul1selectChanged(true);
+      changeModul(0);
+
       modul1enable->setChecked(modules[0].valid);
       modul2enable->setChecked(modules[1].valid);
       modul3enable->setChecked(modules[2].valid);
@@ -596,15 +582,6 @@ void MidiInputTransformDialog::closeEvent(QCloseEvent* ev)
       QWidget::closeEvent(ev);
       }
 
-//---------------------------------------------------------
-//   ~MidiInputTransformDialog
-//---------------------------------------------------------
-
-MidiInputTransformDialog::~MidiInputTransformDialog()
-      {
-      delete data;
-      }
-
 //---------------------------------------------------------
 //   writeMidiTransforms
 //---------------------------------------------------------
@@ -696,7 +673,7 @@ void MidiInputTransformation::write(int level, Xml& xml) const
 
 void readMidiInputTransform(Xml& xml)
       {
-      MidiInputTransformation trans("new");
+      MidiInputTransformation trans(QString("new"));
       int apply = -1;
 
       for (;;) {
@@ -718,7 +695,7 @@ void readMidiInputTransform(Xml& xml)
                         else if (tag == "procEventOp")
                               trans.procEvent = TransformOperator(xml.parseInt());
                         else if (tag == "eventType")
-                              trans.eventType = MidiEvent::EventType(xml.parseInt());
+                              trans.eventType = xml.parseInt();
                         else if (tag == "procVal1Op")
                               trans.procVal1 = TransformOperator(xml.parseInt());
                         else if (tag == "procVal1a")
@@ -734,7 +711,7 @@ void readMidiInputTransform(Xml& xml)
                         else if (tag == "selEventOp")
                               trans.selEventOp = ValOp(xml.parseInt());
                         else if (tag == "selEventType")
-                              trans.selType = MidiEvent::EventType(xml.parseInt());
+                              trans.selType = xml.parseInt();
                         else if (tag == "selVal1Op")
                               trans.selVal1 = ValOp(xml.parseInt());
                         else if (tag == "selVal1a")
@@ -854,9 +831,9 @@ void MidiInputTransformDialog::setValOp(QWidget* a, QWidget* b, ValOp op)
 void MidiInputTransformDialog::selEventOpSel(int val)
       {
       selType->setEnabled(val != All);
-      data->cmt->selEventOp = ValOp(val);
-      selVal1aChanged(data->cmt->selVal1a);
-      selVal1bChanged(data->cmt->selVal1b);
+      cmt->selEventOp = ValOp(val);
+      selVal1aChanged(cmt->selVal1a);
+      selVal1bChanged(cmt->selVal1b);
       }
 
 //---------------------------------------------------------
@@ -865,9 +842,9 @@ void MidiInputTransformDialog::selEventOpSel(int val)
 
 void MidiInputTransformDialog::selTypeSel(int val)
       {
-      data->cmt->selType = MidiEvent::EventType(eventTypeTable[val]);
-      selVal1aChanged(data->cmt->selVal1a);
-      selVal1bChanged(data->cmt->selVal1b);
+      cmt->selType = eventTypeTable[val];
+      selVal1aChanged(cmt->selVal1a);
+      selVal1bChanged(cmt->selVal1b);
       }
 
 //---------------------------------------------------------
@@ -877,7 +854,7 @@ void MidiInputTransformDialog::selTypeSel(int val)
 void MidiInputTransformDialog::selVal1OpSel(int val)
       {
       setValOp(selVal1a, selVal1b, ValOp(val));
-      data->cmt->selVal1 = ValOp(val);
+      cmt->selVal1 = ValOp(val);
       }
 
 //---------------------------------------------------------
@@ -887,7 +864,7 @@ void MidiInputTransformDialog::selVal1OpSel(int val)
 void MidiInputTransformDialog::selVal2OpSel(int val)
       {
       setValOp(selVal2a, selVal2b, ValOp(val));
-      data->cmt->selVal2 = ValOp(val);
+      cmt->selVal2 = ValOp(val);
       }
 
 //---------------------------------------------------------
@@ -898,7 +875,7 @@ void MidiInputTransformDialog::procEventOpSel(int val)
       {
       TransformOperator op = val == 0 ? Keep : Fix;
       procType->setEnabled(op == Fix);
-      data->cmt->procEvent = op;
+      cmt->procEvent = op;
       }
 
 //---------------------------------------------------------
@@ -907,7 +884,7 @@ void MidiInputTransformDialog::procEventOpSel(int val)
 
 void MidiInputTransformDialog::procEventTypeSel(int val)
       {
-      data->cmt->eventType = MidiEvent::EventType(eventTypeTable[val]);
+      cmt->eventType = eventTypeTable[val];
       }
 
 //---------------------------------------------------------
@@ -916,7 +893,7 @@ void MidiInputTransformDialog::procEventTypeSel(int val)
 
 void MidiInputTransformDialog::procVal1OpSel(int val)
       {
-      data->cmt->procVal1 = TransformOperator(val);
+      cmt->procVal1 = TransformOperator(val);
       switch(TransformOperator(val)) {
             case Keep:
             case Invert:
@@ -955,7 +932,7 @@ void MidiInputTransformDialog::procVal1OpSel(int val)
 void MidiInputTransformDialog::procVal2OpSel(int val)
       {
       TransformOperator op = TransformOperator(procVal2Map[val]);
-      data->cmt->procVal2 = op;
+      cmt->procVal2 = op;
 
       switch (op) {
             case Keep:
@@ -1013,13 +990,13 @@ void MidiInputTransformDialog::funcOpSel(int val)
       procChannelVala->setEnabled(isFuncOp);
       procChannelValb->setEnabled(isFuncOp);
       if (isFuncOp) {
-            procEventOpSel(data->cmt->procEvent);
-            procVal1OpSel(data->cmt->procVal1);
-            procVal2OpSel(data->cmt->procVal2);
-            procPortOpSel(data->cmt->procPort);
-            procChannelOpSel(data->cmt->procChannel);
+            procEventOpSel(cmt->procEvent);
+            procVal1OpSel(cmt->procVal1);
+            procVal2OpSel(cmt->procVal2);
+            procPortOpSel(cmt->procPort);
+            procChannelOpSel(cmt->procChannel);
             }
-      data->cmt->funcOp = op;
+      cmt->funcOp = op;
       }
 
 //---------------------------------------------------------
@@ -1051,11 +1028,11 @@ void MidiInputTransformDialog::presetNew()
 
 void MidiInputTransformDialog::presetDelete()
       {
-      if (data->cindex != -1) {
+      if (cindex != -1) {
             iMidiInputTransformation mt = mtlist.begin();
-            for (int i = 0; i < data->cindex; ++i, ++mt) {
+            for (int i = 0; i < cindex; ++i, ++mt) {
                   mtlist.erase(mt);
-                  presetList->removeItem(data->cindex);
+                  presetList->removeItem(cindex);
                   break;
                   }
             }
@@ -1067,12 +1044,12 @@ void MidiInputTransformDialog::presetDelete()
 
 void MidiInputTransformDialog::nameChanged(const QString& s)
       {
-      data->cmt->name = s;
-      QListBoxItem* item = presetList->item(data->cindex);
+      cmt->name = s;
+      QListBoxItem* item = presetList->item(cindex);
       if (s != item->text()) {
             disconnect(presetList,   SIGNAL(highlighted(QListBoxItem*)),
                this, SLOT(presetChanged(QListBoxItem*)));
-            presetList->changeItem(s, data->cindex);
+            presetList->changeItem(s, cindex);
             connect(presetList,   SIGNAL(highlighted(QListBoxItem*)),
                SLOT(presetChanged(QListBoxItem*)));
             }
@@ -1084,7 +1061,7 @@ void MidiInputTransformDialog::nameChanged(const QString& s)
 
 void MidiInputTransformDialog::commentChanged()
       {
-      data->cmt->comment = commentEntry->text();
+      cmt->comment = commentEntry->text();
       }
 
 //---------------------------------------------------------
@@ -1093,13 +1070,13 @@ void MidiInputTransformDialog::commentChanged()
 
 void MidiInputTransformDialog::selVal1aChanged(int val)
       {
-      data->cmt->selVal1a = val;
-      if ((data->cmt->selEventOp != All)
-         && (data->cmt->selType == MidiEvent::Note)) {
+      cmt->selVal1a = val;
+      if ((cmt->selEventOp != All)
+         && (cmt->selType == 0x90)) {
             selVal1a->setSuffix(" - " + pitch2string(val));
             }
       else
-            selVal1a->setSuffix("");
+            selVal1a->setSuffix(QString(""));
       }
 
 //---------------------------------------------------------
@@ -1108,13 +1085,13 @@ void MidiInputTransformDialog::selVal1aChanged(int val)
 
 void MidiInputTransformDialog::selVal1bChanged(int val)
       {
-      data->cmt->selVal1b = val;
-      if ((data->cmt->selEventOp != All)
-         && (data->cmt->selType == MidiEvent::Note)) {
+      cmt->selVal1b = val;
+      if ((cmt->selEventOp != All)
+         && (cmt->selType == 0x90)) {
             selVal1b->setSuffix(" - " + pitch2string(val));
             }
       else
-            selVal1b->setSuffix("");
+            selVal1b->setSuffix(QString(""));
       }
 
 //---------------------------------------------------------
@@ -1123,7 +1100,7 @@ void MidiInputTransformDialog::selVal1bChanged(int val)
 
 void MidiInputTransformDialog::selVal2aChanged(int val)
       {
-      data->cmt->selVal2a = val;
+      cmt->selVal2a = val;
       }
 
 //---------------------------------------------------------
@@ -1132,7 +1109,7 @@ void MidiInputTransformDialog::selVal2aChanged(int val)
 
 void MidiInputTransformDialog::selVal2bChanged(int val)
       {
-      data->cmt->selVal2b = val;
+      cmt->selVal2b = val;
       }
 
 //---------------------------------------------------------
@@ -1141,7 +1118,7 @@ void MidiInputTransformDialog::selVal2bChanged(int val)
 
 void MidiInputTransformDialog::procVal1aChanged(int val)
       {
-      data->cmt->procVal1a = val;
+      cmt->procVal1a = val;
       }
 
 //---------------------------------------------------------
@@ -1150,7 +1127,7 @@ void MidiInputTransformDialog::procVal1aChanged(int val)
 
 void MidiInputTransformDialog::procVal1bChanged(int val)
       {
-      data->cmt->procVal1b = val;
+      cmt->procVal1b = val;
       }
 
 //---------------------------------------------------------
@@ -1159,7 +1136,7 @@ void MidiInputTransformDialog::procVal1bChanged(int val)
 
 void MidiInputTransformDialog::procVal2aChanged(int val)
       {
-      data->cmt->procVal2a = val;
+      cmt->procVal2a = val;
       }
 
 //---------------------------------------------------------
@@ -1168,7 +1145,7 @@ void MidiInputTransformDialog::procVal2aChanged(int val)
 
 void MidiInputTransformDialog::procVal2bChanged(int val)
       {
-      data->cmt->procVal2b = val;
+      cmt->procVal2b = val;
       }
 
 //---------------------------------------------------------
@@ -1214,7 +1191,7 @@ void MidiInputTransformDialog::modul4enableChanged(bool val)
 void MidiInputTransformDialog::selPortOpSel(int val)
       {
       setValOp(selPortVala, selPortValb, ValOp(val));
-      data->cmt->selPort = ValOp(val);
+      cmt->selPort = ValOp(val);
       }
 
 //---------------------------------------------------------
@@ -1223,7 +1200,7 @@ void MidiInputTransformDialog::selPortOpSel(int val)
 
 void MidiInputTransformDialog::selPortValaChanged(int val)
       {
-      data->cmt->selPorta = val;
+      cmt->selPorta = val;
       }
 
 //---------------------------------------------------------
@@ -1232,7 +1209,7 @@ void MidiInputTransformDialog::selPortValaChanged(int val)
 
 void MidiInputTransformDialog::selPortValbChanged(int val)
       {
-      data->cmt->selPortb = val;
+      cmt->selPortb = val;
       }
 
 //---------------------------------------------------------
@@ -1242,7 +1219,7 @@ void MidiInputTransformDialog::selPortValbChanged(int val)
 void MidiInputTransformDialog::selChannelOpSel(int val)
       {
       setValOp(selChannelVala, selChannelValb, ValOp(val));
-      data->cmt->selChannel = ValOp(val);
+      cmt->selChannel = ValOp(val);
       }
 
 //---------------------------------------------------------
@@ -1251,7 +1228,7 @@ void MidiInputTransformDialog::selChannelOpSel(int val)
 
 void MidiInputTransformDialog::selChannelValaChanged(int val)
       {
-      data->cmt->selChannela = val;
+      cmt->selChannela = val;
       }
 
 //---------------------------------------------------------
@@ -1260,7 +1237,7 @@ void MidiInputTransformDialog::selChannelValaChanged(int val)
 
 void MidiInputTransformDialog::selChannelValbChanged(int val)
       {
-      data->cmt->selChannelb = val;
+      cmt->selChannelb = val;
       }
 
 //---------------------------------------------------------
@@ -1269,7 +1246,7 @@ void MidiInputTransformDialog::selChannelValbChanged(int val)
 
 void MidiInputTransformDialog::procPortOpSel(int val)
       {
-      data->cmt->procPort = TransformOperator(val);
+      cmt->procPort = TransformOperator(val);
       switch(TransformOperator(val)) {
             case Keep:
             case Invert:
@@ -1307,7 +1284,7 @@ void MidiInputTransformDialog::procPortOpSel(int val)
 
 void MidiInputTransformDialog::procPortValaChanged(int val)
       {
-      data->cmt->procPorta = val;
+      cmt->procPorta = val;
       }
 
 //---------------------------------------------------------
@@ -1316,7 +1293,7 @@ void MidiInputTransformDialog::procPortValaChanged(int val)
 
 void MidiInputTransformDialog::procPortValbChanged(int val)
       {
-      data->cmt->procPortb = val;
+      cmt->procPortb = val;
       }
 
 //---------------------------------------------------------
@@ -1325,7 +1302,7 @@ void MidiInputTransformDialog::procPortValbChanged(int val)
 
 void MidiInputTransformDialog::procChannelOpSel(int val)
       {
-      data->cmt->procChannel = TransformOperator(val);
+      cmt->procChannel = TransformOperator(val);
       switch(TransformOperator(val)) {
             case Keep:
             case Invert:
@@ -1363,7 +1340,7 @@ void MidiInputTransformDialog::procChannelOpSel(int val)
 
 void MidiInputTransformDialog::procChannelValaChanged(int val)
       {
-      data->cmt->procChannela = val;
+      cmt->procChannela = val;
       }
 
 //---------------------------------------------------------
@@ -1372,7 +1349,7 @@ void MidiInputTransformDialog::procChannelValaChanged(int val)
 
 void MidiInputTransformDialog::procChannelValbChanged(int val)
       {
-      data->cmt->procChannelb = val;
+      cmt->procChannelb = val;
       }
 
 //---------------------------------------------------------
@@ -1381,27 +1358,23 @@ void MidiInputTransformDialog::procChannelValbChanged(int val)
 
 void MidiInputTransformDialog::changeModul(int k)
       {
-      modul1select->setOn(k == 0);
-      modul2select->setOn(k == 1);
-      modul3select->setOn(k == 2);
-      modul4select->setOn(k == 3);
+printf("change modul %d\n", k);
 
-      data->cmodul = k;
+      cmodul = k;       // current modul
 
       if (modules[k].transform == 0) {
             printf("transform %d ist null\n", k);
-            modules[k].transform = data->cmt;
+            modules[k].transform = cmt;
             }
       else {
             //---------------------------------------------
             //  search transformation in list
             //---------------------------------------------
+
             int idx = 0;
             iMidiInputTransformation i;
             for (i = mtlist.begin(); i != mtlist.end(); ++i, ++idx) {
                   if (*i == modules[k].transform) {
-//                        printf("change to transform %d <%s>\n",
-//                           k, data->cmt->name.latin1());
                         presetList->setCurrentItem(idx);
                         break;
                         }
@@ -1411,67 +1384,27 @@ void MidiInputTransformDialog::changeModul(int k)
             }
       }
 
-//---------------------------------------------------------
-//   modul1selectChanged
-//---------------------------------------------------------
-
-void MidiInputTransformDialog::modul1selectChanged(bool flag)
-      {
-      if (flag)
-            changeModul(0);
-      }
-
-//---------------------------------------------------------
-//   modul2selectChanged
-//---------------------------------------------------------
-
-void MidiInputTransformDialog::modul2selectChanged(bool flag)
-      {
-      if (flag)
-            changeModul(1);
-      }
-
-//---------------------------------------------------------
-//   modul3selectChanged
-//---------------------------------------------------------
-
-void MidiInputTransformDialog::modul3selectChanged(bool flag)
-      {
-      if (flag)
-            changeModul(2);
-      }
-
-//---------------------------------------------------------
-//   modul4selectChanged
-//---------------------------------------------------------
-
-void MidiInputTransformDialog::modul4selectChanged(bool flag)
-      {
-      if (flag)
-            changeModul(3);
-      }
-
 //---------------------------------------------------------
 //   presetChanged
 //---------------------------------------------------------
 
 void MidiInputTransformDialog::presetChanged(QListBoxItem* item)
       {
-      data->cindex = presetList->index(item);
-
-printf("data->cindex %d\n", data->cindex);
+      cindex = presetList->index(item);
 
       //---------------------------------------------------
       //   search transformation in list and set
-      //   data->cmt
+      //   cmt
       //---------------------------------------------------
 
       iMidiInputTransformation i;
       for (i = mtlist.begin(); i != mtlist.end(); ++i) {
             if (item->text() == (*i)->name) {
-                  data->cmt = *i;
-                  if (data->cmodul != -1)
-                        modules[data->cmodul].transform = *i;
+                  printf("found %s\n", (*i)->name.latin1());
+                  cmt = *i;
+                  if (cmodul != -1) {
+                        modules[cmodul].transform = *i;
+                        }
                   break;
                   }
             }
@@ -1479,36 +1412,35 @@ printf("data->cindex %d\n", data->cindex);
             printf("MidiInputTransformDialog::presetChanged: not found\n");
             return;
             }
-//      printf("set preset <%s>\n", data->cmt->name.latin1());
-      nameEntry->setText(data->cmt->name);
-      commentEntry->setText(data->cmt->comment);
+      nameEntry->setText(cmt->name);
+      commentEntry->setText(cmt->comment);
 
-      selEventOp->setCurrentItem(data->cmt->selEventOp);
-      selEventOpSel(data->cmt->selEventOp);
+      selEventOp->setCurrentItem(cmt->selEventOp);
+      selEventOpSel(cmt->selEventOp);
 
       for (unsigned i = 0; i < sizeof(eventTypeTable)/sizeof(*eventTypeTable); ++i) {
-            if (eventTypeTable[i] == data->cmt->selType) {
+            if (eventTypeTable[i] == cmt->selType) {
                   selType->setCurrentItem(i);
                   break;
                   }
             }
 
-      selVal1Op->setCurrentItem(data->cmt->selVal1);
-      selVal1OpSel(data->cmt->selVal1);
+      selVal1Op->setCurrentItem(cmt->selVal1);
+      selVal1OpSel(cmt->selVal1);
 
-      selVal2Op->setCurrentItem(data->cmt->selVal2);
-      selVal2OpSel(data->cmt->selVal2);
+      selVal2Op->setCurrentItem(cmt->selVal2);
+      selVal2OpSel(cmt->selVal2);
 
-      selPortOp->setCurrentItem(data->cmt->selPort);
-      selPortOpSel(data->cmt->selPort);
+      selPortOp->setCurrentItem(cmt->selPort);
+      selPortOpSel(cmt->selPort);
 
-      selChannelOp->setCurrentItem(data->cmt->selChannel);
-      selChannelOpSel(data->cmt->selChannel);
+      selChannelOp->setCurrentItem(cmt->selChannel);
+      selChannelOpSel(cmt->selChannel);
 
       {
       unsigned i;
       for (i = 0; i < sizeof(oplist)/sizeof(*oplist); ++i) {
-            if (oplist[i].id == data->cmt->funcOp) {
+            if (oplist[i].id == cmt->funcOp) {
                   funcOp->setCurrentItem(i);
                   break;
                   }
@@ -1518,44 +1450,43 @@ printf("data->cindex %d\n", data->cindex);
       funcOpSel(i);
       }
 
-      procEventOp->setCurrentItem(data->cmt->procEvent);
-      procEventOpSel(data->cmt->procEvent);
+      procEventOp->setCurrentItem(cmt->procEvent);
+      procEventOpSel(cmt->procEvent);
 
-      procVal1Op->setCurrentItem(data->cmt->procVal1);
-      procVal1OpSel(data->cmt->procVal1);
+      procVal1Op->setCurrentItem(cmt->procVal1);
+      procVal1OpSel(cmt->procVal1);
 
       for (unsigned i = 0; i < sizeof(procVal2Map)/sizeof(*procVal2Map); ++i) {
-            if (procVal2Map[i] == data->cmt->procVal2) {
+            if (procVal2Map[i] == cmt->procVal2) {
                   procVal2Op->setCurrentItem(i);
                   break;
                   }
             }
 
-      selVal1a->setValue(data->cmt->selVal1a);
-      selVal1b->setValue(data->cmt->selVal1b);
-      selVal1aChanged(data->cmt->selVal1a);
-      selVal1bChanged(data->cmt->selVal1b);
+      selVal1a->setValue(cmt->selVal1a);
+      selVal1b->setValue(cmt->selVal1b);
+      selVal1aChanged(cmt->selVal1a);
+      selVal1bChanged(cmt->selVal1b);
 
-      selVal2a->setValue(data->cmt->selVal2a);
-      selVal2b->setValue(data->cmt->selVal2b);
+      selVal2a->setValue(cmt->selVal2a);
+      selVal2b->setValue(cmt->selVal2b);
 
-      selPortVala->setValue(data->cmt->selPorta);
-      selPortValb->setValue(data->cmt->selPortb);
+      selPortVala->setValue(cmt->selPorta);
+      selPortValb->setValue(cmt->selPortb);
 
-      selChannelVala->setValue(data->cmt->selChannela);
-      selChannelValb->setValue(data->cmt->selChannelb);
+      selChannelVala->setValue(cmt->selChannela);
+      selChannelValb->setValue(cmt->selChannelb);
 
-      procVal1a->setValue(data->cmt->procVal1a);
-      procVal1b->setValue(data->cmt->procVal1b);
+      procVal1a->setValue(cmt->procVal1a);
+      procVal1b->setValue(cmt->procVal1b);
 
-      procVal2a->setValue(data->cmt->procVal2a);
-      procVal2b->setValue(data->cmt->procVal2b);
+      procVal2a->setValue(cmt->procVal2a);
+      procVal2b->setValue(cmt->procVal2b);
 
-      procPortVala->setValue(data->cmt->procPorta);
-      procPortValb->setValue(data->cmt->procPortb);
+      procPortVala->setValue(cmt->procPorta);
+      procPortValb->setValue(cmt->procPortb);
 
-      procChannelVala->setValue(data->cmt->procChannela);
-      procChannelValb->setValue(data->cmt->procChannelb);
+      procChannelVala->setValue(cmt->procChannela);
+      procChannelValb->setValue(cmt->procChannelb);
       }
 
-
