@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: mitplugin.h,v 1.3 2002/01/03 11:30:04 muse Exp $
+//  $Id: mitplugin.h,v 1.1.1.1 2003/10/29 10:06:16 wschweer Exp $
 //
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -11,7 +11,7 @@
 
 #include <list>
 
-class MidiEvent;
+class MidiPlayEvent;
 class Xml;
 
 //---------------------------------------------------------
@@ -21,7 +21,7 @@ class Xml;
 
 class MITPlugin {
    public:
-      virtual void process(MidiEvent* event) = 0;
+      virtual void process(MidiPlayEvent* event) = 0;
       virtual void readStatus(Xml&) {}
       virtual void writeStatus(int, Xml&) const {}
       };
@@ -30,7 +30,7 @@ typedef std::list<MITPlugin*> MITPluginList;
 typedef MITPluginList::iterator iMITPlugin;
 
 extern MITPluginList mitPlugins;
-extern void processMidiInputTransformPlugins(MidiEvent*);
+extern void processMidiInputTransformPlugins(MidiPlayEvent*);
 extern void writeStatusMidiInputTransformPlugins(int, Xml&);
 extern void readStatusMidiInputTransformPlugin(Xml&);
 
