@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: pos.cpp,v 1.1 2002/01/30 12:08:38 muse Exp $
+//  $Id: pos.cpp,v 1.1.1.1 2003/10/29 10:05:17 wschweer Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -29,10 +29,6 @@ Pos::Pos()
       sn         = -1;
       }
 
-//---------------------------------------------------------
-//   Pos
-//---------------------------------------------------------
-
 Pos::Pos(const Pos& p)
       {
       _type      = p._type;
@@ -42,6 +38,68 @@ Pos::Pos(const Pos& p)
       sn         = -1;
       }
 
+Pos::Pos(int t, bool ticks)
+      {
+      if (ticks) {
+            _type      = TICKS;
+            _posTick   = t;
+            _posSample = 0;
+            }
+      else {
+            _type      = SAMPLES;
+            _posSample = t;
+            _posTick   = 0;
+            }
+      _posTime   = 0.0;
+      sn         = -1;
+      }
+
+Pos::Pos(const QString& s)
+      {
+      int m, b, t;
+      sscanf(s.latin1(), "%04d.%02d.%03d", &m, &b, &t);
+      int tick = sigmap.bar2tick(m, b, t);
+      _type      = TICKS;
+      _posTick   = tick;
+      _posTime   = 0.0;
+      _posSample = 0;
+      sn         = -1;
+      }
+
+//---------------------------------------------------------
+//   setType
+//---------------------------------------------------------
+
+void Pos::setType(TType t)
+      {
+      if (t == _type)
+            return;
+
+      switch(_type) {
+            case TIME:
+                  if (t == TICKS)
+                        _posTick = tempomap.time2tick(_posTime, _posTick, &sn);
+                  else
+                        _posSample = int(_posTime * sampleRate);
+                  break;
+
+            case TICKS:
+                  _posTime = tempomap.tick2time(_posTick, _posTime, &sn);
+                  _posSample = int(_posTime * sampleRate);
+                  break;
+
+            case SAMPLES:
+                  if (t == TICKS)
+                        _posTick = tempomap.time2tick(double(_posSample)/double(sampleRate),
+                           _posTick, &sn);
+                  else
+                        _posTime = double(_posSample)/double(sampleRate);
+                  break;
+            }
+      _type = t;
+      sn    = -1;
+      }
+
 //---------------------------------------------------------
 //   operator+=
 //---------------------------------------------------------
@@ -615,27 +673,6 @@ Pos::Pos(int min, int sec, int frame, int subframe)
       }
 
 
-Pos::Pos(int t)
-      {
-      _type      = TICKS;
-      _posTick   = t;
-      _posTime   = 0.0;
-      _posSample = 0;
-      sn         = -1;
-      }
-
-Pos::Pos(const QString& s)
-      {
-      int m, b, t;
-      sscanf(s.latin1(), "%04d.%02d.%03d", &m, &b, &t);
-      int tick = sigmap.bar2tick(m, b, t);
-      _type      = TICKS;
-      _posTick   = tick;
-      _posTime   = 0.0;
-      _posSample = 0;
-      sn         = -1;
-      }
-
 void Pos::mbt(int* bar, int* beat, int* tick) const
       {
       sigmap.tickValues(posTick(), bar, beat, tick);
