@@ -1,13 +1,15 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: items.cpp,v 1.1 2002/01/30 12:08:38 muse Exp $
+//    $Id: items.cpp,v 1.1.1.1 2003/10/29 10:05:26 wschweer Exp $
 //  (C) Copyright 1999,2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
 #include <stdio.h>
+#include <assert.h>
 #include <qpainter.h>
 
+#include "globals.h"
 #include "items.h"
 #include "layout.h"
 #include "symbols.h"
@@ -309,21 +311,21 @@ void Text::draw(QPainter& p) const
 //---------------------------------------------------------
 
 QuantItem::QuantItem(int t, MidiEvent* e, MidiPart* p, int n, int r)
-  : Text(t, QFont("Carter", 12, QFont::Bold), Qt::blue)
+  : Text(t, QFont(QString("Carter"), 12, QFont::Bold), Qt::blue)
       {
       _part = p;
       _event = e;
       nq = n;
       rq = r;
       switch (nq) {
-            case 1536:  setText("Q1"); break;
-            case  768:  setText("Q2"); break;
-            case  384:  setText("Q4"); break;
-            case  192:  setText("Q8"); break;
-            case   96:  setText("Q16"); break;
-            case   48:  setText("Q32"); break;
-            case   24:  setText("Q64"); break;
-            default:  setText("Q"); break;
+            case 1536:  setText(QString("Q1")); break;
+            case  768:  setText(QString("Q2")); break;
+            case  384:  setText(QString("Q4")); break;
+            case  192:  setText(QString("Q8")); break;
+            case   96:  setText(QString("Q16")); break;
+            case   48:  setText(QString("Q32")); break;
+            case   24:  setText(QString("Q64")); break;
+            default:    setText(QString("Q")); break;
             }
       setAlign(QPainter::AlignHCenter|QPainter::AlignBottom);
       }
@@ -429,7 +431,7 @@ void LyricsItem::draw(QPainter& p) const
                   if (editState) {
                         if (s.isEmpty()) {
                               r = p.boundingRect(rp.x(), rp.y(), 0, 0,
-                                 align | QPainter::DontClip, " ");
+                                 align | QPainter::DontClip, QString(" "));
                               }
                         else {
                               r = p.boundingRect(rp.x(), rp.y(), 0, 0,
@@ -605,8 +607,6 @@ void Symbol::move(MidiEditor*, SystemLayout* system, bool copyflag)
       newEvent->setB(ny);
       newEvent->setC(0);
       MidiTrack* track = sr->stave->track;
-      newEvent->setPort(track->outPort());
-      newEvent->setChannel(track->outChannel());
       MidiPart* part  = (MidiPart*)(track->findPart(ntick));
 
       if (!copyflag)
