@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: layout.cpp,v 1.1 2002/01/30 12:08:38 muse Exp $
+//    $Id: layout.cpp,v 1.1.1.1 2003/10/29 10:05:28 wschweer Exp $
 //  (C) Copyright 1999,2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -9,6 +9,7 @@
 #include "song.h"
 #include "globals.h"
 #include "symbols.h"
+#include <assert.h>
 #include <values.h>
 #include <algorithm>
 #include "event.h"
@@ -123,7 +124,7 @@ Stave::Stave(MidiTrack* t, SplitMode m, int l)
 //  Liste der mglichen enharmonischen Verwechslungen
 //    fr jeden Ton
 
-static char table0[12][5] = {
+static signed char table0[12][5] = {
       //  -   #  b   ## bb
       { 0, -1, 0, -1, 1  },   // c
       { 0,  0, 1, -1, 1  },   // #c
@@ -144,7 +145,7 @@ static char table0[12][5] = {
 //    abhngig von der Tonart (feste Versetzungszeichen)
 //
 
-static char table1[15][12] = {
+static signed char table1[15][12] = {
       { 0, 1, 2, 2, 3, 3, 4, 4, 5, 6, 6, 7 },  // ces
       { 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 6, 7 },  // ges
       { 0, 1, 1, 2, 3, 3, 4, 4, 5, 5, 6, 7 },  // des
@@ -547,7 +548,7 @@ void SystemLayout::move(ScoreItem*, const QPoint& p)
 //   xy2tick
 //---------------------------------------------------------
 
-int SystemLayout::xy2tick(const QPoint& p, int* ry=0, StaveRow** sr=0) const
+int SystemLayout::xy2tick(const QPoint& p, int* ry, StaveRow** sr) const
       {
       int x = p.x();
       int y = p.y();
@@ -827,11 +828,10 @@ void SystemLayout::layout1()
       for (ciTrack t = _tracks.begin(); t != _tracks.end(); ++t) {
             MidiTrack* track =  (MidiTrack*)(*t);
             PartList* pl = track->parts();
+//printf("track %p parts %d\n", track, track->parts()->size());
             if (!pl->empty()) {
                   int ft = pl->begin()->second->posTick();
-                  iPart ip = pl->end();
-                  --ip;
-                  int lt = ip->second->posTick() + ip->second->lenTick();
+                  int lt = pl->rbegin()->second->endTick();
                   if (ft < firstTick)
                         firstTick = ft;
                   if (lt > lastTick)
@@ -845,6 +845,8 @@ void SystemLayout::layout1()
                   staves.push_back(Stave(track, NOSPLIT, lines));
                   }
             }
+//printf("layout1 %d %d-%d\n", _tracks.size(), firstTick, lastTick);
+
       if (firstTick == MAXINT) {
             printf("SystemLayout::layout1: no parts found\n");
             abort();
@@ -1365,9 +1367,10 @@ void SystemLayout::updateTracks(std::list<int>& tl)
       _tracks.clear();
       TrackList* st = song->tracks();
       for (iTrack t = st->begin(); t != st->end(); ++t) {
+            Track* track = *t;
             for (std::list<int>::const_iterator i = tl.begin(); i != tl.end(); ++i) {
-                  if (*i == (*t)->sn())
-                        _tracks.push_back(*t);
+                  if (*i == track->sn())
+                        _tracks.push_back(track);
                   }
             }
       }
