@@ -1,16 +1,17 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: ncanvas.cpp,v 1.1 2002/01/30 12:08:38 muse Exp $
+//    $Id: ncanvas.cpp,v 1.1.1.1 2003/10/29 10:05:30 wschweer Exp $
 //  (C) Copyright 1999,2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
 #include <stdio.h>
 #include <qpainter.h>
-#include <math.h>
+#include <cmath>
 #include <qinputdialog.h>
 #include <qtimer.h>
 #include <qcursor.h>
+#include <assert.h>
 
 #include "layout.h"
 
@@ -424,7 +425,6 @@ void ScoreCanvas::viewMousePressEvent(QMouseEvent* ev)
                         int rq = sr->stave->rq;
                         if (showQuantDialog(&ntick, &nq, &rq)) {
                               MidiEvent* e = new MidiEvent(
-                                 track->outPort(), track->outChannel(),
                                  ntick, MidiEvent::Quantize, nq, rq, 0,
                                  0);
                               switch(sr->stave->mode) {
@@ -465,7 +465,6 @@ void ScoreCanvas::viewMousePressEvent(QMouseEvent* ev)
                         int pitch = system->y2pitch(ev->y());
                         tick      = (tick / editor->raster()) * editor->raster();
                         MidiEvent* e = new MidiEvent(
-                           track->outPort(), track->outChannel(),
                            tick, MidiEvent::Note, pitch, curVelo, 0,
                            editor->quant());
                         curItem = new NoteItem(tick, e, part);
@@ -499,7 +498,6 @@ void ScoreCanvas::viewMousePressEvent(QMouseEvent* ev)
                         int xoff = int(mm2dot(leftMargin)) + system->bracketWidth;
                         QPoint p(start.x() - xoff, ny);
                         MidiEvent* e = new MidiEvent(
-                           track->outPort(), track->outChannel(),
                            tick, MidiEvent::Symbol,
                            paletteItem, ny, 0, 0);
                         curItem = new Symbol(p, tick, e, part);
@@ -687,6 +685,7 @@ void ScoreCanvas::endMoveItems(const QPoint&, bool copyflag)
             selectItem(i->second);
             i->second->move(editor, system, copyflag);
             }
+
       song->endUndo(copyflag ? SC_EVENT_INSERTED : SC_EVENT_REMOVED|SC_EVENT_INSERTED);
       moving.clear();
       updateSelection();
