@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: score.cpp,v 1.1 2002/01/30 12:08:39 muse Exp $
+//    $Id: score.cpp,v 1.1.1.1 2003/10/29 10:05:26 wschweer Exp $
 //  (C) Copyright 1999,2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -44,6 +44,8 @@
 
 int Score::_quantInit = 96;
 int Score::_rasterInit = 96;
+int Score::_widthInit = 600;
+int Score::_heightInit = 400;
 
 //---------------------------------------------------------
 //   Score Editor
@@ -52,6 +54,7 @@ int Score::_rasterInit = 96;
 Score::Score(PartList* pl)
    : MidiEditor(_quantInit, _rasterInit, pl)
       {
+      resize(_widthInit, _heightInit);
       paletteBg  = new QButtonGroup(this, "paletteBg");
       paletteBg->hide();
       paletteBg->setExclusive(true);
@@ -98,7 +101,8 @@ Score::Score(PartList* pl)
 
       //-------------------------------------------------------------
       //    Transport Bar
-      new TransportToolbar(this);
+      QToolBar* transport = new QToolBar(this);
+      transportAction->addTo(transport);
 
       //---------ToolBar 1----------------------------------
       toolbar = new Toolbar1(this, _rasterInit, _quantInit);
@@ -108,7 +112,7 @@ Score::Score(PartList* pl)
       //    Palettes
       //---------------------------------------------------
 
-      QToolBar* toolbar3  = new QToolBar("Palettes", this, Top, true, "toolbar3");
+      QToolBar* toolbar3  = new QToolBar(tr("Palettes"), this, this, true, "toolbar3");
 //      QHBox* hbox4 = new QHBox(toolbar3, "hbox4");
       QToolButton* lyricsB = new QToolButton(toolbar3, "lyricsB");
       lyricsB->setText(tr("Lyrics"));
@@ -130,7 +134,7 @@ Score::Score(PartList* pl)
       // Toolbar 2
       //---------------------------------------------------
 
-      QToolBar* toolbar2  = new QToolBar("Score", this, Top, true, "toolbar2");
+      QToolBar* toolbar2  = new QToolBar(tr("Score"), this, this, true, "toolbar2");
 
       QActionGroup* action2 = new QActionGroup(toolbar2, "action2", true);
       for (int k = 0; k < 7; ++k) {
@@ -296,7 +300,7 @@ void Score::setSelection(int tick, MidiEvent* e, MidiPart* p)
             info->setValue(NoteInfo::VAL_PITCH, e->pitch());
             info->setValue(NoteInfo::VAL_VELON, e->velo());
             info->setValue(NoteInfo::VAL_VELOFF, e->veloOff());
-            info->setValue(NoteInfo::VAL_CHANNEL, e->channel() + 1);
+//            info->setValue(NoteInfo::VAL_CHANNEL, p->track()->outChannel() + 1);
             }
       else {
             // set illegal values
@@ -305,7 +309,7 @@ void Score::setSelection(int tick, MidiEvent* e, MidiPart* p)
             info->setValue(NoteInfo::VAL_PITCH, -5);
             info->setValue(NoteInfo::VAL_VELON, -5);
             info->setValue(NoteInfo::VAL_VELOFF, -5);
-            info->setValue(NoteInfo::VAL_CHANNEL, -5);
+//            info->setValue(NoteInfo::VAL_CHANNEL, -5);
             }
       }
 
@@ -516,9 +520,9 @@ void Score::noteinfoChanged(NoteInfo::ValType type, int val)
             case NoteInfo::VAL_VELOFF:
                   event->setVeloOff(val);
                   break;
-            case NoteInfo::VAL_CHANNEL:
+//            case NoteInfo::VAL_CHANNEL:
 //                  event->setChannel(val-1);
-                  break;
+//                  break;
             case NoteInfo::VAL_PITCH:
                   event->setPitch(val);
                   break;
@@ -538,7 +542,8 @@ void Score::soloChanged(Track*)
 
 void Score::configBackground()
       {
-      scoreBg = getImageFileName("wallpapers", image_file_pattern, this);
+      scoreBg = getImageFileName(QString("wallpapers"), image_file_pattern, this,
+        tr("MusE: config wallpaper"));
       if (scoreBg.isEmpty())
             return;
       canvas->setBg(QPixmap(scoreBg));
