@@ -1,46 +1,49 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: seq.cpp,v 1.4 2002/02/27 15:53:45 muse Exp $
+//  $Id: seq.cpp,v 1.1.1.1 2003/10/29 10:05:04 wschweer Exp $
 //
-//  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
+//  (C) Copyright 1999-2002 Werner Schweer (ws@seh.de)
 //=========================================================
 
+#include "config.h"
+
 #include <stdio.h>
 #include <errno.h>
-
 #include <sys/time.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/mman.h>
 #include <assert.h>
+#include <fcntl.h>
 
+#include <qstring.h>
 #include <qtimer.h>
 
 #include "seq.h"
-#include <qstring.h>
-#include <fcntl.h>
 #include "xml.h"
 #include "song.h"
 #include "midi.h"
-#include "device.h"
 #include "globals.h"
 #include "plugins/plugin.h"
 #include "midictrl.h"
 #include "midiport.h"
-#include "audioport.h"
 #include "driver/mididev.h"
 #include "driver/audiodev.h"
 #include "event.h"
 #include "minstrument.h"
-#include "audiomix.h"
-#include "sf/sndfile.h"
 #include "driver/alsamidi.h"
 #include "synth.h"
-#include "audiothread.h"
+#include "audio.h"
 #include "audioprefetch.h"
 #include "midithread.h"
+#include "node.h"
+#include "driver/jackaudio.h"
+#include "wave.h"
+#include <sndfile.h>
+
+Sequencer* seq;
 
 //---------------------------------------------------------
 //   Sequencer
@@ -48,43 +51,133 @@
 
 Sequencer::Sequencer()
       {
-      if (realTimePriority < sched_get_priority_min(SCHED_RR))
-            realTimePriority = sched_get_priority_min(SCHED_RR);
-      else if (realTimePriority > sched_get_priority_max(SCHED_RR))
-            realTimePriority = sched_get_priority_max(SCHED_RR);
+      if (realTimePriority < sched_get_priority_min(SCHED_FIFO))
+            realTimePriority = sched_get_priority_min(SCHED_FIFO);
+      else if (realTimePriority > sched_get_priority_max(SCHED_FIFO))
+            realTimePriority = sched_get_priority_max(SCHED_FIFO);
 
-      midiThread = new MidiThread(realTimeScheduling,
-         realTimePriority, lockMemory);
-      audioThread = new AudioThread(realTimeScheduling,
-         realTimePriority/3, lockMemory);
-      audioPrefetch = new AudioPrefetch(realTimeScheduling,
-         realTimePriority/4, lockMemory);
+      midiThread = new MidiThread(realTimeScheduling ? realTimePriority : 0, "Midi");
+      audio = new Audio();
+      audioPrefetch = new AudioPrefetch(realTimeScheduling ? realTimePriority/4 : 0,
+         "AudioDisc");
+      }
+
+int watchAudio, watchAudioPrefetch, watchMidi, watchALSA;
+
+//---------------------------------------------------------
+//   watchdog thread
+//---------------------------------------------------------
+
+static void* watchdog(void*)
+      {
+      int policy;
+      if ((policy = sched_getscheduler (0)) < 0) {
+            printf("Cannot get current client scheduler: %s\n", strerror(errno));
+            }
+      if (policy != SCHED_FIFO)
+            printf("watchdog _NOT_ running SCHED_FIFO\n");
+      else if (debugMsg)
+            printf("watchdog set to SCHED_FIFO priority %d\n",
+               sched_get_priority_max(SCHED_FIFO));
+
+      int fatal = 0;
+      for (;;) {
+            watchAudio = 0;
+            watchMidi = 0;
+
+            sleep(3);
+
+            bool timeout = false;
+            if (watchMidi == 0)
+                  timeout = true;
+            if (!noAudio) {
+                  if (!noAudio) {
+                        if (watchAudio == 0)
+                              timeout = true;
+                        if (watchAudio > 500000)
+                              timeout = true;
+                        }
+                  }
+            if (timeout)
+                  ++fatal;
+            else
+                  fatal = 0;
+            if (fatal >= 3) {
+                  printf("WatchDog: fatal error, realtime task timeout\n");
+                  printf("   (%d,%d-%d) - stopping all services\n",
+                     watchMidi, watchAudio, fatal);
+                  break;
+                  }
+//            printf("wd %d %d %d\n", watchMidi, watchAudio, fatal);
+            }
+      midiThread->stop(true);
+      if (!noAudio) {
+            audioPrefetch->stop(true);
+            audio->stop(true);
+            }
+      exit(-1);
       }
 
 //---------------------------------------------------------
 //   start
 //---------------------------------------------------------
 
-void Sequencer::startThread()
+void Sequencer::start()
       {
+      if (realTimeScheduling) {
+            doSetuid();
+            struct sched_param rt_param;
+            memset(&rt_param, 0, sizeof(rt_param));
+            rt_param.sched_priority = sched_get_priority_max(SCHED_FIFO);
+
+            pthread_attr_t* attributes = (pthread_attr_t*) malloc(sizeof(pthread_attr_t));
+            pthread_attr_init(attributes);
+
+            if (pthread_attr_setschedpolicy(attributes, SCHED_FIFO)) {
+                  printf("cannot set FIFO scheduling class for RT thread\n");
+                  }
+            if (pthread_attr_setschedparam (attributes, &rt_param)) {
+                  printf("Cannot set scheduling priority for RT thread (%s)\n", strerror(errno));
+                  }
+            if (pthread_attr_setscope (attributes, PTHREAD_SCOPE_SYSTEM)) {
+                  printf("Cannot set scheduling scope for RT thread\n");
+                  }
+            if (pthread_create(&watchdogThread, attributes, ::watchdog, this)) {
+                  perror("creating watchdog thread failed:");
+                  }
+            pthread_attr_destroy(attributes);
+            undoSetuid();
+            }
       if (!noAudio) {
-            audioThread->start();
+            audio->start();
             audioPrefetch->start();
             }
       midiThread->start();
       }
 
+//---------------------------------------------------------
+//   isRunning
+//---------------------------------------------------------
+
+bool Sequencer::isRunning()
+      {
+      return midiThread->isRunning();
+      }
+
 //---------------------------------------------------------
 //   stop
 //---------------------------------------------------------
 
-void Sequencer::stopThread()
+void Sequencer::stop()
       {
-      midiThread->stop();
+      song->setStopPlay(false);
+      midiThread->stop(true);
       if (!noAudio) {
-            audioPrefetch->stop();
-            audioThread->stop();
+            audioPrefetch->stop(true);
+            audio->stop(true);
             }
+      if (realTimeScheduling)
+            pthread_cancel(watchdogThread);
       }
 
 //---------------------------------------------------------
@@ -94,7 +187,7 @@ void Sequencer::stopThread()
 Sequencer::~Sequencer()
       {
       delete audioPrefetch;
-      delete audioThread;
+      delete audio;
       delete midiThread;
       }
 
@@ -131,18 +224,6 @@ void Sequencer::writeConfiguration(int level, Xml& xml)
       xml.intTag(level, "rcGotoLeft", rcGotoLeftMarkNote);
       xml.intTag(level, "rcPlay",     rcPlayNote);
 
-      SndFile* master = audioThread->masterMix();
-      if (master) {
-            SndFile* master = audioThread->masterMix();
-            xml.tag(level++, "mixdown");
-            xml.intTag(level, "record",     audioThread->mixdownRecord());
-            xml.strTag(level, "path",       master->path().latin1());
-            xml.intTag(level, "channels",   master->channels());
-            xml.intTag(level, "format",     master->format());
-            xml.intTag(level, "samplebits", master->sampleBits());
-            xml.etag(level--, "mixdown");
-            }
-
       for (int i = 0; i < MIDI_PORTS; ++i) {
             MidiPort* mport = &midiPorts[i];
             MidiDevice* dev = mport->device();
@@ -154,28 +235,21 @@ void Sequencer::writeConfiguration(int level, Xml& xml)
                   xml.etag(level--, "midiport");
                   }
             }
-      AudioPort* port  = &audioPort;
-      AudioDevice* dev = port->device();
-      if (dev) {
-            xml.tag(level++, "audioport idx=\"%d\"", 0);
-            xml.strTag(level, "name",   dev->name().latin1());
-            xml.intTag(level, "record", port->rwFlags() & 0x2 ? 1 : 0);
-            xml.intTag(level, "play",   port->rwFlags() & 0x1);
-            QString s;
-            port->writeConfiguration(level, xml);
-            xml.tag(level--, "/audioport");
-            }
 
-      for (int i = 0; i < AUDIO_GROUPS; ++i) {
+      for (int i = 0; i < mixerGroups; ++i) {
             xml.tag(level++, "audiogroup idx=\"%d\"", i);
-            song->group(i)->writeConfiguration(level, xml);
+            audioGroups[i].writeConfiguration(level, xml);
             xml.tag(level--, "/audiogroup");
             }
 
       xml.tag(level++, "audiomaster");
-      song->master()->writeConfiguration(level, xml);
+      audioOutput.writeConfiguration(level, xml);
       xml.tag(level--, "/audiomaster");
 
+      xml.tag(level++, "audioinput");
+      audioInput.writeConfiguration(level, xml);
+      xml.tag(level--, "/audioinput");
+
       xml.tag(level, "/sequencer");
       }
 
@@ -257,6 +331,8 @@ void Sequencer::readConfiguration(Xml& xml)
                               readConfigAudioGroup(xml);
                         else if (tag == "audiomaster")
                               readConfigAudioMaster(xml);
+                        else if (tag == "audioinput")
+                              readConfigAudioInput(xml);
                         else if (tag == "rcStop")
                               rcStopNote = xml.parseInt();
                         else if (tag == "rcEnable")
@@ -267,8 +343,6 @@ void Sequencer::readConfiguration(Xml& xml)
                               rcGotoLeftMarkNote = xml.parseInt();
                         else if (tag == "rcPlay")
                               rcPlayNote = xml.parseInt();
-                        else if (tag == "mixdown")
-                              readMixdown(xml);
                         else
                               xml.unknown("Seq");
                         break;
@@ -290,8 +364,8 @@ void Sequencer::readMixdown(Xml& xml)
       {
       QString path;
       int channels   = 2;
-      int format     = SF_FORMAT_WAV | SF_FORMAT_PCM;
-      int samplebits = 16;
+      int format     = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
+      bool record    = false;
 
       for (;;) {
             Xml::Token token = xml.parse();
@@ -307,20 +381,19 @@ void Sequencer::readMixdown(Xml& xml)
                         else if (tag == "format")
                               format = xml.parseInt();
                         else if (tag == "samplebits")
-                              samplebits = xml.parseInt();
-                        else if (tag == "record") {
-                              bool record = xml.parseInt();
-                              if (record)
-                                    audioThread->msgOpenMixdownRecord();
-                              }
+                              ;
+                        else if (tag == "record")
+                              record = xml.parseInt();
                         else
                               xml.unknown("mixdown");
                         break;
                   case Xml::TagEnd:
                         if (tag == "mixdown") {
-                              SndFile* sf = new SndFile(path);
-                              sf->setFormat(format, channels, sampleRate, samplebits);
-                              audioThread->msgSetMixdownFile(sf);
+                              SndFile* sf = getWave(path, true);
+                              sf->setFormat(format, channels, sampleRate);
+//TODO                              audio->msgSetRecFile(&audioOutput, sf);
+//                              if (record)
+//                                    audio->msgSetRecord(&audioOutput, true);
                               return;
                               }
                   default:
@@ -394,10 +467,9 @@ void Sequencer::readConfigMidiPort(Xml& xml)
 
 void Sequencer::readConfigAudioPort(Xml& xml)
       {
-      int idx;
       QString device;
-      AudioDevice* dev;
-      AudioPort* port = 0;
+      AudioDevice* dev = 0;
+      AudioNode* node = &audioOutput;
       int rwFlags = 0;
 
       for (;;) {
@@ -414,29 +486,35 @@ void Sequencer::readConfigAudioPort(Xml& xml)
                         else if (tag == "play")
                               rwFlags |= xml.parseInt() ? 1 : 0;
                         else if (tag == "audionode")
-                              port->readConfiguration(xml);
+                              node->readConfiguration(xml);
                         else
                               xml.unknown("AudioDevice");
                         break;
                   case Xml::Attribut:
-                        if (tag == "idx") {
-                              idx = xml.s2().toInt();
-                              idx = 0;
-                              port = &audioPort;
-                              }
                         break;
                   case Xml::TagEnd:
                         if (tag == "audioport") {
-                              if (!device.isEmpty()) {
+/*                              if (useAlsaAudio && !device.isEmpty()) {
                                     dev = audioDevices.find(device);
                                     if (dev) {
                                           port->setrwFlags(rwFlags);
-                                          audioThread->msgSetAudioDevice(port, dev);
+                                          audio->msgSetAudioDevice(port, dev);
                                           }
-                                    audioThread->msgAddRoute(port, port->route());
+                                    audio->msgAddRoute(port, port->route());
                                     }
-                              return;
-                              }
+                              else*/ if (useJackAudio) {
+#if HAVE_JACK
+                                    dev = jackAudioDevice;
+                                    if (dev) {
+//                                        node->setrwFlags(rwFlags);
+//                                        jackAudioDevice->stop();
+//                                        audio->msgSetAudioDevice(port, dev);
+//                                        jackAudioDevice->start();
+                                          }
+#endif
+                                    }
+                        return;
+                        }
                   default:
                         break;
                   }
@@ -449,7 +527,7 @@ void Sequencer::readConfigAudioPort(Xml& xml)
 
 void Sequencer::readConfigAudioGroup(Xml& xml)
       {
-      AudioMixer* mixer = 0;
+      AudioNode* mixer = 0;
       for (;;) {
             Xml::Token token = xml.parse();
             if (token == Xml::Error || token == Xml::End)
@@ -465,17 +543,17 @@ void Sequencer::readConfigAudioGroup(Xml& xml)
                   case Xml::Attribut:
                         if (tag == "idx") {
                               int idx = xml.s2().toInt();
-                              if (idx > AUDIO_GROUPS) {
+                              if (idx > mixerGroups) {
                                     fprintf(stderr, "bad audio group %d (>%d)\n",
                                        idx, AUDIO_GROUPS);
                                     idx = 0;
                                     }
-                              mixer = song->group(idx);
+                              mixer = &audioGroups[idx];
                               }
                         break;
                   case Xml::TagEnd:
                         if (tag == "audiogroup") {
-                              audioThread->msgAddRoute(mixer, mixer->route());
+//TODO                              audio->msgAddRoute(mixer, mixer->route());
                               return;
                               }
                   default:
@@ -490,7 +568,7 @@ void Sequencer::readConfigAudioGroup(Xml& xml)
 
 void Sequencer::readConfigAudioMaster(Xml& xml)
       {
-      AudioMixer* mixer = song->master();
+      AudioNode* mixer = &audioOutput;
       for (;;) {
             Xml::Token token = xml.parse();
             if (token == Xml::Error || token == Xml::End)
@@ -504,11 +582,36 @@ void Sequencer::readConfigAudioMaster(Xml& xml)
                               xml.unknown("AudioMaster");
                         break;
                   case Xml::TagEnd:
-                        if (tag == "audiomaster") {
-//                              song->msgAddRoute(*mixer, mixer->route());
-//                              mixer.setRoute(audioPort); // override settings
+                        if (tag == "audiomaster")
+                              return;
+                  default:
+                        break;
+                  }
+            }
+      }
+
+//---------------------------------------------------------
+//   readConfigAudioInput
+//---------------------------------------------------------
+
+void Sequencer::readConfigAudioInput(Xml& xml)
+      {
+      AudioNode* mixer = &audioInput;
+      for (;;) {
+            Xml::Token token = xml.parse();
+            if (token == Xml::Error || token == Xml::End)
+                  break;
+            QString tag = xml.s1();
+            switch (token) {
+                  case Xml::TagStart:
+                        if (tag == "audionode")
+                              mixer->readConfiguration(xml);
+                        else
+                              xml.unknown("AudioInput");
+                        break;
+                  case Xml::TagEnd:
+                        if (tag == "audioinput")
                               return;
-                              }
                   default:
                         break;
                   }
