@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: seq.h,v 1.3 2002/02/13 11:42:56 muse Exp $
+//  $Id: seq.h,v 1.1.1.1 2003/10/29 10:05:00 wschweer Exp $
 //
 //  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -15,7 +15,6 @@
 #include <unistd.h>
 #include "mtc.h"
 #include "value.h"
-#include "audiomix.h"
 #include "tempo.h"
 
 class Xml;
@@ -40,19 +39,24 @@ class Sequencer : public QObject {
       void readConfigAudioPort(Xml& xml);
       void readConfigAudioGroup(Xml& xml);
       void readConfigAudioMaster(Xml& xml);
+      void readConfigAudioInput(Xml& xml);
       void readConfigMidiPort(Xml& xml);
       void readMixdown(Xml& xml);
       bool notify();
+      pthread_t watchdogThread;
 
    public:
       Sequencer();
       ~Sequencer();
 
-      void startThread();
-      void stopThread();
+      void start();
+      void stop();
+      bool isRunning();
 
       void writeConfiguration(int, Xml&);
       void readConfiguration(Xml&);
       };
+extern Sequencer* seq;
+extern Sequencer* seq;
 #endif
 
