@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: song.cpp,v 1.3 2002/02/27 15:53:45 muse Exp $
+//  $Id: song.cpp,v 1.1.1.1 2003/10/29 10:05:00 wschweer Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -11,45 +11,37 @@
 #include "undo.h"
 #include "seq.h"
 #include "key.h"
-#include "device.h"
 #include "globals.h"
 #include "event.h"
 #include "drummap.h"
 #include "marker/marker.h"
-#include "audioport.h"
-#include "memory.h"
 #include "synth.h"
-#include "audiothread.h"
+#include "audio.h"
 #include "midithread.h"
+#include "mididev.h"
 #include "amixer.h"
 
 #include <unistd.h>
 #include <stdio.h>
 #include <errno.h>
 #include <qapplication.h>
+#include <qmessagebox.h>
+
+Song* song;
 
 //---------------------------------------------------------
 //   Song
 //---------------------------------------------------------
 
-Song::Song(const char* name = 0)
+Song::Song(const char* name)
    :QObject(0, name)
       {
       undoList     = new UndoList;
       redoList     = new UndoList;
       _markerList  = new MarkerList;
       _karaoke     = new EventList;
-      _mixerGroups = AUDIO_GROUPS;
-
-      // set master defaults
-      _master.setRoute(&audioPort);
-      _master.setVolume(1.0);
-      audioPort.connect(&_master);
-      for (int i = 0; i < _mixerGroups; ++i) {
-            _group[i].setRoute(&_master);
-            _master.connect(&_group[i]);
-            }
-      clear();
+      _globalPitchShift = 0;
+      clear(false);
       }
 
 //---------------------------------------------------------
@@ -66,7 +58,7 @@ Song::~Song()
 
 //---------------------------------------------------------
 //   setTempo
-//    called from transport window
+//    public slot
 //---------------------------------------------------------
 
 void Song::setTempo(int newTempo)
@@ -90,24 +82,31 @@ void Song::setSig(int z, int n)
 //    Track Manipulationen
 //---------------------------------------------------------
 
-Track* Song::addTrack(Track::TrackType type = Track::MIDI, Track* t = 0)
+Track* Song::addTrack(Track::TrackType type, Track* t)
       {
       Track* track = 0;
       switch(type) {
             case Track::MIDI:
+                  track = new MidiTrack();
+                  track->setType(Track::MIDI);
+                  break;
             case Track::DRUM:
                   track = new MidiTrack();
+                  track->setType(Track::DRUM);
                   break;
             case Track::WAVE:
                   track = new WaveTrack();
                   if (t) {
                         WaveTrack* st = (WaveTrack*) t;
-                        track->setOutPort(st->outPort());
+                        WaveTrack* dt = (WaveTrack*) track;
+                        dt->setPorts(st->ports());
+                        //TODO
+//                        track->setOutPort(st->outPort());
 //                        track->setInPort(st->inPort());
-                        track->setOutChannel(st->Track::outChannel());
-                        ((WaveTrack*)track)->setRoute(st->route());
-                        ((WaveTrack*)track)->setPan(st->pan());
-                        ((WaveTrack*)track)->setVolume(st->volume());
+//                        track->setOutChannel(st->Track::outChannel());
+//                        ((WaveTrack*)track)->setRoute(st->route());
+//                        ((WaveTrack*)track)->setPan(st->pan());
+//                        ((WaveTrack*)track)->setVolume(st->volume());
                         }
                   break;
             }
@@ -207,8 +206,12 @@ void Song::deleteEvent(MidiEvent* event, MidiPart* part)
 
 void Song::cmdAddRecordedEvents(MidiTrack* mt, EventList* events, int startTick)
       {
-      if (events->empty())
+// printf("cmdAddRecordedEvents\n");
+      if (events->empty()) {
+            if (debugMsg)
+                  printf("no events recorded\n");
             return;
+            }
 
       iEvent s;
       iEvent e;
@@ -227,13 +230,21 @@ void Song::cmdAddRecordedEvents(MidiTrack* mt, EventList* events, int startTick)
             endTick = rpos();
             }
       else {
+            // search for last noteOff:
+            endTick = 0;
+            for (iEvent i = events->begin(); i != events->end(); ++i) {
+                  Event* ev = i->second;
+                  int l     = ev->endTick();
+                  if (l > endTick)
+                        endTick = l;
+                  }
             e = events->end();
-            iEvent i = e;
-            --i;
-            endTick = i->first;
             }
-      if (startTick > endTick)
+      if (startTick > endTick) {
+            if (debugMsg)
+                  printf("no events in record area\n");
             return;
+            }
 
       //---------------------------------------------------
       //    if startTick points into a part,
@@ -253,28 +264,33 @@ void Song::cmdAddRecordedEvents(MidiTrack* mt, EventList* events, int startTick)
                   break;
             }
       if (ip == pl->end()) {
-            part = new MidiPart(mt);
+            if (debugMsg)
+                  printf("create new part for recorded events\n");
+            // create new part
+            part      = new MidiPart(mt);
             startTick = roundDownBar(startTick);
             endTick   = roundUpBar(endTick);
             part->setPosTick(startTick);
-            part->setLenTick(endTick-startTick);
+            part->setLenTick(endTick - startTick);
             part->setName(mt->name());
             // copy events
             EventList* de = part->events();
             for (iEvent i = s; i != e; ++i) {
-                  MidiEvent* event = new MidiEvent(*(MidiEvent*)(i->second));
-                  event->setPosTick(i->first);
-                  event->setPort(mt->outPort());
-                  event->setChannel(mt->outChannel());
+                  const MidiEvent* old = (MidiEvent*)(i->second);
+                  MidiEvent* event = new MidiEvent(*old);
+                  event->move(-startTick);
                   de->add(event);
                   }
+            startUndo();
             midiThread->msgAddPart(part);
+            endUndo(SC_PART_INSERTED);
             return;
             }
 
       startUndo();
       int flags = SC_EVENT_INSERTED;
-      if (endTick > part->posTick() + part->lenTick()) {
+      int partTick = part->posTick();
+      if (endTick > partTick + part->lenTick()) {
             Part* newPart = part->clone();
             if (_recMode == REC_REPLACE) {
                   // TODO: delete Events
@@ -282,6 +298,8 @@ void Song::cmdAddRecordedEvents(MidiTrack* mt, EventList* events, int startTick)
             endTick = 0;
             for (iEvent i = s; i != e; ++i) {
                   MidiEvent* event = (MidiEvent*)i->second;
+                  int tick = event->posTick() - partTick;
+                  event->setPosTick(tick);
                   midiThread->msgAddEvent(event, (MidiPart*)newPart, false);
                   if (endTick < event->posTick() + event->lenTick())
                         endTick = event->posTick() + event->lenTick();
@@ -296,6 +314,8 @@ void Song::cmdAddRecordedEvents(MidiTrack* mt, EventList* events, int startTick)
                   }
             for (iEvent i = s; i != e; ++i) {
                   MidiEvent* event = (MidiEvent*)i->second;
+                  int tick = event->posTick() - partTick;
+                  event->setPosTick(tick);
                   midiThread->msgAddEvent(event, part, false);
                   }
             }
@@ -330,6 +350,7 @@ void Song::setLoop(bool f)
       {
       if (loopFlag != f) {
             loopFlag = f;
+            loopAction->setOn(loopFlag);
             emit loopChanged(loopFlag);
             }
       }
@@ -342,9 +363,29 @@ void Song::setLoop(bool f)
 void Song::setRecord(bool f)
       {
       if (recordFlag != f) {
+            if (f) {
+#if 0
+                  // check for midi devices suitable for recording
+                  bool portFound = false;
+                  for (int i = 0; i < MIDI_PORTS; ++i) {
+                        MidiDevice* dev = midiPorts[i].device();
+                        if (dev && (dev->rwFlags() & 0x2))
+                              portFound = true;
+                        }
+                  if (!portFound) {
+                        QMessageBox::critical(qApp->mainWidget(), "MusE: Record",
+                           "There are no midi devices configured for recording");
+                        f = false;
+                        }
+#endif
+                  }
+            else {
+                  bounceTrack = 0;
+                  }
             if (playFlag && f)
                   f = false;
             recordFlag = f;
+            recordAction->setOn(recordFlag);
             emit recordChanged(recordFlag);
             }
       }
@@ -358,6 +399,7 @@ void Song::setPunchin(bool f)
       {
       if (punchinFlag != f) {
             punchinFlag = f;
+            punchinAction->setOn(punchinFlag);
             emit punchinChanged(punchinFlag);
             }
       }
@@ -371,6 +413,7 @@ void Song::setPunchout(bool f)
       {
       if (punchoutFlag != f) {
             punchoutFlag = f;
+            punchoutAction->setOn(punchoutFlag);
             emit punchoutChanged(punchoutFlag);
             }
       }
@@ -417,11 +460,30 @@ void Song::setMasterFlag(bool val)
 
 void Song::setPlay(bool f)
       {
-      midiThread->msgPlay(f);
+      setStopPlay(f ? true : playFlag);
+      }
+
+void Song::setStop(bool f)
+      {
+      setStopPlay(f ? false : playFlag);
+      }
+
+void Song::setStopPlay(bool f)
+      {
+      playAction->blockSignals(true);
+      stopAction->blockSignals(true);
+
       if (playFlag != f) {
             playFlag = f;
-            emit playChanged(playFlag);
+            midiThread->msgPlay(playFlag);
+            emit playChanged(playFlag);   // signal transport window
             }
+
+      playAction->setOn(playFlag);
+      stopAction->setOn(!playFlag);
+
+      stopAction->blockSignals(false);
+      playAction->blockSignals(false);
       }
 
 //---------------------------------------------------------
@@ -438,67 +500,12 @@ void Song::swapTracks(int i1, int i2)
       *it2 = track;
       }
 
-//---------------------------------------------------------
-//   clear
-//    signal - emit signals for changes if true
-//---------------------------------------------------------
-
-void Song::clear(bool signal = false)
-      {
-      if (seq) {
-            midiThread->msgPlay(false);
-            while (playFlag) {
-                  qApp->processEvents();
-                  }
-            }
-      _name          = "";
-      _komponist1    = "";
-      _komponist2    = "";
-      _tracks.clear();
-      tempomap.clear();
-      sigmap.clear();
-      undoList->clear();
-      redoList->clear();
-      _markerList->clear();
-      pos[0]         = 0;
-      pos[1]         = 0;
-      pos[2]         = 0;
-      _showPageNo    = true;
-      _showMeasureNo = true;
-      _showTrackname = true;
-      _masterFlag    = true;
-      loopFlag       = false;
-      loopFlag       = false;
-      punchinFlag    = false;
-      punchoutFlag   = false;
-      playFlag       = false;
-      recordFlag     = false;
-      soloFlag       = false;
-      // seq
-      _mtype         = MT_UNKNOWN;
-      _karaokeFlag   = false;
-      _karaoke->clear();
-      _recMode       = REC_OVERDUP;
-      _cycleMode     = CYCLE_NORMAL;
-      _click         = false;
-      _quantize      = false;
-      _len           = 0;           // song len in ticks
-      _follow        = JUMP;
-      // _tempo         = 500000;      // default tempo 120
-      dirty          = false;
-      initDrumMap();
-      if (signal) {
-            emit loopChanged(false);
-            recordChanged(false);
-            }
-      }
-
 //---------------------------------------------------------
 //   setPos
 //---------------------------------------------------------
 
-void Song::setPos(int idx, int val, bool sig = true,
-   bool isSeek = true, bool adjustScrollbar = false)
+void Song::setPos(int idx, int val, bool sig,
+   bool isSeek, bool adjustScrollbar)
       {
 //      printf("setPos(%d) %d sig=%d,seek=%d,scroll=%d\n",
 //         idx, val, sig, isSeek, adjustScrollbar);
@@ -600,9 +607,16 @@ void Song::rewindStart()
 //   update
 //---------------------------------------------------------
 
-void Song::update(int flags = -1)
+void Song::update(int flags)
       {
+      static int level = 0;
+      if (level) {
+            printf("Song::update %08x, level %d\n", flags, level);
+            return;
+            }
+      ++level;
       emit songChanged(flags);
+      --level;
       }
 
 //---------------------------------------------------------
@@ -630,7 +644,7 @@ void Song::setChannelMute(int channel, bool val)
             if (track->outChannel() == channel)
                   track->setMute(val);
             }
-      emit muteChanged(0);
+      emit songChanged(SC_MUTE);
       }
 
 //---------------------------------------------------------
@@ -652,10 +666,6 @@ void Song::initLen()
                   }
             }
       _len = roundUpBar(_len);
-      //debug
-      //int bar, beat, tick;
-      //tickValues(_len, &bar, &beat, &tick);
-      //printf("song: initLen() %d %d\n", _len, bar);
       }
 
 //---------------------------------------------------------
@@ -803,45 +813,6 @@ void Song::setMType(MType t)
       _mtype = t;
       }
 
-//---------------------------------------------------------
-//   nextEvents
-//    collects all events between stick - etick
-//---------------------------------------------------------
-
-void Song::nextEvents(int stick, int etick, EventList* list)
-      {
-      for (iTrack t = _tracks.begin(); t != _tracks.end(); ++t) {
-            if ((*t)->type() != Track::MIDI && (*t)->type() != Track::DRUM)
-                  continue;
-            MidiTrack* track = (MidiTrack*)(*t);
-            PartList* pl = track->parts();
-            for (iPart p = pl->begin(); p != pl->end(); ++p) {
-                  MidiPart* part    = (MidiPart*)(p->second);
-                  EventList* events = part->events();
-
-                  int delay   = track->delay;
-                  iEvent ie   = events->lower_bound(stick - delay);
-                  iEvent iend = events->lower_bound(etick - delay);
-
-                  for (; ie != iend; ++ie) {
-                        MidiEvent* ev = (MidiEvent*)ie->second;
-                        //
-                        //  dont play any meta events
-                        //
-                        if (ev->type() == MidiEvent::Meta)
-                              continue;
-                        if (track->type() == Track::DRUM) {
-                              int instr = drumOutmap[ev->pitch()];
-                              if (ev->isNote() && drumMap[instr].mute)
-                                    continue;
-                              }
-                        ev->setTrk(track); // DEBUG
-                        list->add(ev, ev->posTick() + delay);
-                        }
-                  }
-            }
-      }
-
 //---------------------------------------------------------
 //   beat
 //---------------------------------------------------------
@@ -908,36 +879,22 @@ Marker* Song::setMarkerLock(Marker* m, bool f)
       return m;
       }
 
-bool Song::mute(const SoundSource* s) const
-      {
-      return soloFlag ? s->soloMute() : s->mute();
-      }
-
-void Song::setMute(SoundSource* s, bool flag)
-      {
-      if (soloFlag)
-            s->setSolo(flag ? s : 0);
-      else
-            s->setMute(flag);
-      emit muteChanged(s);
-      }
-
 //---------------------------------------------------------
-//   setSolo
-//    reset solo if s==0
+//   setRecordFlag
 //---------------------------------------------------------
 
-void Song::setSolo(SoundSource* s)
+void Song::setRecordFlag(Track* track, bool val)
       {
-      soloFlag = (s != 0);
-      for (iTrack i = tracks()->begin(); i != tracks()->end(); ++i) {
-            SoundSource* ss = dynamic_cast<SoundSource*>(*i);
-            ss->setSolo(s ? ss != s : false);
+      if (track->type() == Track::WAVE) {
+            WaveTrack* audioTrack = (WaveTrack*)track;
+            audioTrack->setRecordFlag1(val);
+            audio->msgSetRecord(audioTrack, val);
             }
-      audioPort.setSolo(s ? &audioPort != s : false);
-      for (int i = 0; i < _mixerGroups; ++i)
-            _group[i].setSolo(s ? &_group[i] != s : false);
-      emit soloChanged(s);
+      else {
+            track->setRecordFlag1(val);
+            track->setRecordFlag2(val);
+            }
+      emit recordFlagChanged(track);
       }
 
 //---------------------------------------------------------
@@ -946,20 +903,9 @@ void Song::setSolo(SoundSource* s)
 
 void Song::rescanAlsaPorts()
       {
-      printf("rescan alsa ports\n");
       emit midiPortsChanged();
       }
 
-//---------------------------------------------------------
-//   endUndo
-//---------------------------------------------------------
-
-void Song::endUndo(int flags)
-      {
-      updateFlags = flags;
-      endMsgCmd();
-      }
-
 //---------------------------------------------------------
 //   endMsgCmd
 //---------------------------------------------------------
@@ -984,35 +930,107 @@ void Song::updateAudioMixer()
             audioMixer->updateMixer();
       }
 
-void Song::undo()  {
+//---------------------------------------------------------
+//   undo
+//---------------------------------------------------------
+
+void Song::undo()
+      {
       updateFlags = 0;
       midiThread->undo();
       redoAction->setEnabled(true);
       undoAction->setEnabled(!undoList->empty());
+      if (updateFlags & SC_AUDIO_MIXER) {
+            updateFlags &= ~SC_AUDIO_MIXER;
+            updateAudioMixer();
+            }
       emit songChanged(updateFlags);
       }
 
-void Song::redo()  {
+//---------------------------------------------------------
+//   redo
+//---------------------------------------------------------
+
+void Song::redo()
+      {
       updateFlags = 0;
       midiThread->redo();
       undoAction->setEnabled(true);
       redoAction->setEnabled(!redoList->empty());
+      if (updateFlags & SC_AUDIO_MIXER) {
+            updateFlags &= ~SC_AUDIO_MIXER;
+            updateAudioMixer();
+            }
       emit songChanged(updateFlags);
       }
 
 //---------------------------------------------------------
-//   processMsg
+//   processAudioMsg
 //    executed in midiThread context
 //---------------------------------------------------------
 
-void Song::processMsg(const MidiMsg* msg)
+void Song::processAudioMsg(const AudioMsg* msg)
+      {
+      switch(msg->id) {
+            case AUDIO_ADD_PART:
+                  addPart(msg->spart);
+                  undoOp(UndoOp::AddPart, msg->spart);
+                  updateFlags = SC_PART_INSERTED;
+                  break;
+            case AUDIO_REMOVE_PART:
+                  {
+                  Part* part = msg->spart;
+                  undoOp(UndoOp::DeletePart, part);
+                  part->track()->parts()->remove(part);
+                  part->events()->incARef(-1);
+                  updateFlags = SC_PART_REMOVED;
+                  }
+                  break;
+            case AUDIO_CHANGE_PART:
+                  changePart(msg->spart, msg->dpart);
+                  undoOp(UndoOp::ModifyPart, msg->spart, msg->dpart);
+                  msg->spart->events()->incARef(-1);
+                  updateFlags = SC_PART_MODIFIED;
+                  break;
+            case AUDIO_REMOVE_TRACK:
+                  {
+                  WaveTrack* t = msg->track;
+                  int idx = _tracks.index(t);
+                  undoOp(UndoOp::DeleteTrack, idx, t);
+                  _tracks.erase(t);
+                  }
+                  break;
+            case AUDIO_UNDO:
+                  doUndo();
+                  break;
+            case AUDIO_REDO:
+                  doRedo();
+                  break;
+            }
+      }
+
+//---------------------------------------------------------
+//   processMidiMsg
+//    executed in midiThread context
+//---------------------------------------------------------
+
+void Song::processMidiMsg(const MidiMsg* msg)
       {
       switch(msg->id) {
             case SEQM_ADD_TRACK:
                   addTrack((Track*)(msg->p1));
                   break;
-            case SEQM_REMOVE_TRACKS:
-                  removeMarkedTracks();
+            case SEQM_REMOVE_TRACK:
+                  {
+                  Track* t = (Track*)(msg->p1);
+                  if (t->type() == Track::WAVE)
+                        audio->msgRemoveTrack((WaveTrack*)t);
+                  else {
+                        int idx = _tracks.index(t);
+                        undoOp(UndoOp::DeleteTrack, idx, t);
+                        _tracks.erase(t);
+                        }
+                  }
                   break;
             case SEQM_CHANGE_TRACK:
                   changeTrack((Track*)msg->p1, (Track*)msg->p2);
@@ -1036,13 +1054,18 @@ void Song::processMsg(const MidiMsg* msg)
                   updateFlags = SC_PART_INSERTED;
                   break;
             case SEQM_REMOVE_PART:
-                  undoOp(UndoOp::DeletePart, (Part*)msg->p1);
-                  removePart((Part*)msg->p1);
+                  {
+                  Part* part = (Part*)msg->p1;
+                  undoOp(UndoOp::DeletePart, part);
+                  part->track()->parts()->remove(part);
+                  part->events()->incARef(-1);
                   updateFlags = SC_PART_REMOVED;
+                  }
                   break;
             case SEQM_CHANGE_PART:
                   changePart((Part*)msg->p1, (Part*)msg->p2);
                   undoOp(UndoOp::ModifyPart, (Part*)msg->p1, (Part*)msg->p2);
+                  ((Part*)msg->p1)->events()->incARef(-1);
                   updateFlags = SC_PART_MODIFIED;
                   break;
             case SEQM_ADD_EVENT:
@@ -1078,6 +1101,10 @@ void Song::processMsg(const MidiMsg* msg)
                   updateFlags = SC_TEMPO;
                   break;
 
+            case SEQM_SET_GLOBAL_TEMPO:
+                  tempomap.setGlobalTempo(msg->a);
+                  break;
+
             case SEQM_REMOVE_TEMPO:
                   undoOp(UndoOp::DeleteTempo, msg->a, msg->b);
                   tempomap.delTempo(msg->a);
@@ -1091,10 +1118,10 @@ void Song::processMsg(const MidiMsg* msg)
                   break;
 
             case SEQM_UNDO:
-                  doUndo();
+                  audio->msgUndo();
                   break;
             case SEQM_REDO:
-                  doRedo();
+                  audio->msgRedo();
                   break;
             default:
 printf("unknown seq message %d\n", msg->id);
@@ -1102,4 +1129,92 @@ printf("unknown seq message %d\n", msg->id);
             }
       }
 
+//---------------------------------------------------------
+//   panic
+//---------------------------------------------------------
+
+void Song::panic()
+      {
+      midiThread->msgPanic();
+      }
+
+//---------------------------------------------------------
+//   masterVolChanged
+//---------------------------------------------------------
+
+void Song::masterVolChanged(int a)
+      {
+      midiThread->masterVolChanged(a);
+      }
+
+//---------------------------------------------------------
+//   ctrlChanged
+//---------------------------------------------------------
+
+void Song::ctrlChanged(int a, int b, int c, int d)
+      {
+      midiThread->ctrlChanged(a, b, c, d);
+      }
+
+//---------------------------------------------------------
+//   appearanceChanged
+//---------------------------------------------------------
+
+void Song::appearanceChanged()
+      {
+      update();
+      }
+
+//---------------------------------------------------------
+//   clear
+//    signal - emit signals for changes if true
+//    called from constructor as clear(false) and
+//    from MusE::clearSong() as clear(true)
+//---------------------------------------------------------
+
+void Song::clear(bool signal)
+      {
+      bounceTrack    = 0;
+      _name          = "";
+      _komponist1    = "";
+      _komponist2    = "";
+      _tracks.clearDelete();
+      tempomap.clear();
+      sigmap.clear();
+      undoList->clear();
+      redoList->clear();
+      _markerList->clear();
+      pos[0]         = 0;
+      pos[1]         = 0;
+      pos[2]         = 0;
+      _showPageNo    = true;
+      _showMeasureNo = true;
+      _showTrackname = true;
+      _masterFlag    = true;
+      loopFlag       = false;
+      loopFlag       = false;
+      punchinFlag    = false;
+      punchoutFlag   = false;
+      playFlag       = false;
+      recordFlag     = false;
+      soloFlag       = false;
+      // seq
+      _mtype         = MT_UNKNOWN;
+      _karaokeFlag   = false;
+      _karaoke->clear();
+      _recMode       = REC_OVERDUP;
+      _cycleMode     = CYCLE_NORMAL;
+      _click         = false;
+      _quantize      = false;
+      _len           = 0;           // song len in ticks
+      _follow        = JUMP;
+      // _tempo         = 500000;      // default tempo 120
+      dirty          = false;
+      initDrumMap();
+      if (signal) {
+            emit loopChanged(false);
+            recordChanged(false);
+            }
+      }
+
 
