@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: songfile.cpp,v 1.1 2002/01/30 14:10:07 muse Exp $
+//  $Id: songfile.cpp,v 1.2 2003/10/29 22:16:38 spamatica Exp $
 //
 //  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -23,13 +23,22 @@
 #include "event.h"
 #include "marker/marker.h"
 #include "midiport.h"
-#include "sf/sndfile.h"
-#include "audiothread.h"
+#include "audio.h"
 #include "mitplugin.h"
+#include "wave.h"
+#include "midictrl.h"
+#include "amixer.h"
 
-// midictrl.h:
-extern void writeMidiController(int, Xml&);
-extern void readMidiController(Xml&);
+struct ClonePart {
+      const EventList* el;
+      int id;
+      ClonePart(const EventList* e, int i) : el(e), id(i) {}
+      };
+
+typedef std::list<ClonePart> CloneList;
+typedef CloneList::iterator iClone;
+
+static CloneList cloneList;
 
 //---------------------------------------------------------
 //   NKey::write
@@ -101,20 +110,45 @@ void Scale::read(Xml& xml)
 //   Part::write
 //---------------------------------------------------------
 
-void Part::write(int level, Xml& xml, int offset=0) const
+void Part::write(int level, Xml& xml) const
       {
-      xml.tag(level++, "part");
+      const EventList* el = cevents();
+      int id              = -1;
+      bool dumpEvents     = true;
+
+      if (el->arefCount() > 1) {
+            for (iClone i = cloneList.begin(); i != cloneList.end(); ++i) {
+                  if (i->el == el) {
+                        id = i->id;
+                        dumpEvents = false;
+                        break;
+                        }
+                  }
+            if (id == -1) {
+                  id = cloneList.size();
+                  ClonePart cp(el, id);
+                  cloneList.push_back(cp);
+                  }
+            }
+
+      if (id != -1)
+            xml.tag(level++, "part cloneId=\"%d\"", id);
+      else
+            xml.tag(level++, "part");
       xml.strTag(level, "name", _name);
 
       PosLen poslen(*this);
-      poslen.setPosTick(posTick() + offset);
+      int tickpos = posTick();
+      poslen.setPosTick(tickpos);
       poslen.write(level, xml, "poslen");
       xml.intTag(level, "selected", _selected);
       xml.intTag(level, "color", _colorIndex);
-      xml.intTag(level, "mute", _mute);
-      const EventList* el = cevents();
-      for (ciEvent e = el->begin(); e != el->end(); ++e)
-            e->second->write(level, xml, offset);
+      if (_mute)
+            xml.intTag(level, "mute", _mute);
+      if (dumpEvents) {
+            for (ciEvent e = el->begin(); e != el->end(); ++e)
+                  e->second->write(level, xml, tickpos);
+            }
       xml.etag(level, "part");
       }
 
@@ -122,8 +156,11 @@ void Part::write(int level, Xml& xml, int offset=0) const
 //   Part::read
 //---------------------------------------------------------
 
-void Part::read(Xml& xml, int offset=0)
+void Part::read(Xml& xml)
       {
+      int id = -1;
+      bool containsEvents = false;
+
       for (;;) {
             Xml::Token token = xml.parse();
             const QString& tag = xml.s1();
@@ -134,8 +171,9 @@ void Part::read(Xml& xml, int offset=0)
                   case Xml::TagStart:
                         if (tag == "name")
                               _name = xml.parse1();
-                        else if (tag == "poslen")
+                        else if (tag == "poslen") {
                               PosLen::read(xml, "poslen");
+                              }
                         else if (tag == "pos") {
                               Pos pos;
                               pos.read(xml, "pos");  // obsolete
@@ -153,19 +191,52 @@ void Part::read(Xml& xml, int offset=0)
                         else if (tag == "mute")
                               _mute = xml.parseInt();
                         else if (tag == "event") {
+                              containsEvents = true;
                               Event* e = newEvent();
-                              e->setPort(track()->outPort());
-                              e->setChannel(track()->outChannel());
                               e->read(xml);
-                              e->move(offset);
-                              _events->add(e);
+                              // tickpos is relative to start of part
+                              e->move(-posTick());
+                              int tick = e->posTick();
+                              if ((tick < 0) || (tick >= lenTick())) {
+                                    printf("ReadEvent: warning: event not in part: %d - %d -%d, discarded\n",
+                                       0, tick, lenTick());
+                                    }
+                              else
+                                    _events->add(e);
                               }
                         else
                               xml.unknown("Part");
                         break;
+                  case Xml::Attribut:
+                        if (tag == "cloneId")
+                              id = xml.s2().toInt();
+                        break;
                   case Xml::TagEnd:
-                        if (tag == "part")
+                        if (tag == "part") {
+                              if (id != -1) {
+                                    // clone part
+                                    if (containsEvents) {
+                                          // add to cloneList:
+                                          ClonePart cp(_events, id);
+                                          cloneList.push_back(cp);
+                                          }
+                                    else {
+                                          // replace event list with clone event
+                                          // list
+                                          for (iClone i = cloneList.begin();
+                                             i != cloneList.end(); ++i) {
+                                                if (i->id == id) {
+                                                      delete _events;
+                                                      _events = (EventList*)(i->el);
+                                                      _events->incRef(1);
+                                                      _events->incARef(1);
+                                                      break;
+                                                      }
+                                                }
+                                          }
+                                    }
                               return;
+                              }
                   default:
                         break;
                   }
@@ -179,6 +250,7 @@ void Part::read(Xml& xml, int offset=0)
 void Track::write(int level, Xml& xml) const
       {
       xml.tag(level++, "track type=\"%d\"", _type);
+      xml.intTag(level, "height", height());
       xml.intTag(level, "channel", outChannel());
       xml.intTag(level, "device", outPort());
       xml.intTag(level, "inportMap", inPortMask());
@@ -188,7 +260,6 @@ void Track::write(int level, Xml& xml) const
             xml.strTag(level, "comment", _comment);
       xml.intTag(level, "locked", _locked);
       xml.intTag(level, "selected", _selected);
-      xml.intTag(level, "record", _recordFlag);
       const PartList* pl = cparts();
       for (ciPart p = pl->begin(); p != pl->end(); ++p)
             p->second->write(level, xml);
@@ -215,15 +286,13 @@ void Track::read(Xml& xml)
                               _comment = xml.parse1();
                         else if (tag == "selected")
                               _selected = xml.parseInt();
+                        else if (tag == "height")
+                              _height = xml.parseInt();
                         else if (tag == "locked")
                               _locked = xml.parseInt();
                         else if (tag == "part") {
                               Part* p = newPart();
                               p->read(xml);
-//DEBUG
-//PosLen *lp = p;
-//printf("part %p:%p  %d len %d %d\n",
-//     p, lp, p->posTick(), p->lenTick(), p->lenSample());
                               parts()->add(p);
                               }
                         else if (tag == "wavepart") {
@@ -243,8 +312,6 @@ void Track::read(Xml& xml)
                               setInPortMask(xml.parseInt());
                         else if (tag == "inchannelMap")
                               setInChannelMask(xml.parseInt());
-                        else if (tag == "record")
-                              setRecordFlag(xml.parseInt());
                         else
                               xml.unknown("Track");
                         break;
@@ -290,8 +357,9 @@ void MidiTrack::write(int level, Xml& xml) const
       xml.intTag(level, "len", len);
       xml.intTag(level, "compression", compression);
       xml.intTag(level, "mute", mute());
-      xml.intTag(level, "solomute", soloMute());
+      xml.intTag(level, "solo", solo());
       xml.intTag(level, "midiThru", midiThruFlag());
+      xml.intTag(level, "record", recordFlag());
 
       xml.tag(level, "/miditrack");
       }
@@ -376,12 +444,12 @@ void MidiTrack::read(Xml& xml)
                               compression = xml.parseInt();
                         else if (tag == "staff")
                               readStaff(xml);
-                        else if (tag == "mute")
-                              setMute(xml.parseInt());
-                        else if (tag == "solomute")
-                              setSolo(xml.parseInt());
+                        else if (SNode::readProperty(xml, tag))
+                              ;
                         else if (tag == "midiThru")
-                              setMidiThruFlag(xml.parseInt());
+                              _midiThruFlag = xml.parseInt();
+                        else if (tag == "record")
+                              _recordFlag = xml.parseInt();
                         else
                               xml.unknown("MidiTrack");
                         break;
@@ -397,74 +465,18 @@ void MidiTrack::read(Xml& xml)
             }
       }
 
+//---------------------------------------------------------
+//   write
+//---------------------------------------------------------
+
 void WaveTrack::write(int level, Xml& xml) const
       {
       xml.tag(level++, "wavetrack");
       Track::write(level, xml);
       AudioNode::writeConfiguration(level, xml);
-      if (_recFile) {
-            xml.tag(level++, "recfile");
-            xml.strTag(level, "path",       _recFile->path());
-            xml.intTag(level, "channels",   _recFile->channels());
-            xml.intTag(level, "format",     _recFile->format());
-            xml.intTag(level, "samplebits", _recFile->sampleBits());
-            xml.etag(level--, "recfile");
-            }
       xml.etag(level, "wavetrack");
       }
 
-//---------------------------------------------------------
-//   readRecfile
-//---------------------------------------------------------
-
-void WaveTrack::readRecfile(Xml& xml)
-      {
-      QString path;
-      int channels   = 2;
-      int format     = SF_FORMAT_WAV | SF_FORMAT_PCM;
-      int samplebits = 16;
-
-      for (;;) {
-            Xml::Token token = xml.parse();
-            if (token == Xml::Error || token == Xml::End)
-                  break;
-            const QString& tag = xml.s1();
-            switch (token) {
-                  case Xml::TagStart:
-                        if (tag == "path")
-                              path = xml.parse1();
-                        else if (tag == "channels")
-                              channels = xml.parseInt();
-                        else if (tag == "format")
-                              format = xml.parseInt();
-                        else if (tag == "samplebits")
-                              samplebits = xml.parseInt();
-                        else
-                              xml.unknown("recfile");
-                        break;
-                  case Xml::TagEnd:
-                        if (tag == "recfile") {
-                              SndFile* sf = new SndFile(path);
-                              sf->setFormat(format, channels, sampleRate, samplebits);
-                              if (sf->openWrite()) {
-                                    printf("open wave file failed\n");
-                                    delete sf;
-                                    }
-                              else {
-                                    if (_recFile)
-                                          delete _recFile;
-                                    _recFile = sf;
-                                    }
-                              return;
-                              }
-                  default:
-                        break;
-                  }
-            }
-
-
-      }
-
 //---------------------------------------------------------
 //   read
 //---------------------------------------------------------
@@ -479,13 +491,13 @@ void WaveTrack::read(Xml& xml)
                   case Xml::End:
                         return;
                   case Xml::TagStart:
-                        if (tag == "track")
+                        if (tag == "track") {
+                              WaveTrack::firstWaveTrack = false;
                               Track::read(xml);
+                              }
                         else if (tag == "audionode") {
                               AudioNode::readConfiguration(xml);
                               }
-                        else if (tag == "recfile")
-                              readRecfile(xml);
                         else
                               xml.unknown("WaveTrack");
                         break;
@@ -563,16 +575,20 @@ void Song::write(int level, Xml& xml) const
       xml.intTag(level, "quantize", _quantize);
       xml.intTag(level, "len", _len);
       xml.intTag(level, "follow", _follow);
+      if (_globalPitchShift)
+            xml.intTag(level, "globalPitchShift", _globalPitchShift);
 
       if (!_copyright.isEmpty())
             xml.strTag(level, "copyright", _copyright);
 
+      cloneList.clear();
       for (ciTrack i = _tracks.begin(); i != _tracks.end(); ++i)
             (*i)->write(level, xml);
       tempomap.write(level, xml);
       sigmap.write(level, xml);
       _markerList->write(level, xml);
 
+      xml.strTag(level, "AudioInRoute", node2Name(audioInput.route()));
       xml.tag(level, "/song");
       }
 
@@ -618,6 +634,7 @@ Font Song::readFont(Xml& xml, const char* name)
 
 void Song::read(Xml& xml)
       {
+      cloneList.clear();
       for (;;) {
             Xml::Token token;
             token = xml.parse();
@@ -695,7 +712,7 @@ void Song::read(Xml& xml)
                         else if (tag == "follow")
                               _follow  = FollowMode(xml.parseInt());
                         else if (tag == "copyright")
-                              _copyright  = xml.parse("copyright");
+                              _copyright  = xml.parse(QString("copyright"));
                         else if (tag == "tempolist") {
                               tempomap.read(xml);
                               }
@@ -710,14 +727,17 @@ void Song::read(Xml& xml)
                               WaveTrack* track = new WaveTrack();
                               track->read(xml);
                               _tracks.add(track);
-
-                              // track is not yet connected:
-                              AudioNode* route = track->route();
-                              track->setRoute(0); // to avoid disconect errors
-                              audioThread->msgAddRoute(track, route);
                               }
                         else if (tag == "marker")
                               readMarker(xml);
+                        else if (tag == "AudioInRoute") {
+                              QString s = xml.parse1();
+                              AudioNode* n = name2Node(s);
+                              if (n)
+                                    audioInput.connectOut(n);
+                              }
+                        else if (tag == "globalPitchShift")
+                              _globalPitchShift = xml.parseInt();
                         else
                               xml.unknown("Song");
                         break;
@@ -822,8 +842,8 @@ void MusE::readToplevels(Xml& xml)
                               toplevels.back().cobject()->readStatus(xml);
                               }
                         else if (tag == "audiomixer") {
-                              TopWin* tw = startAudioMixer();
-                              tw->readStatus(xml);
+                              startAudioMixer();
+                              toplevels.back().cobject()->readStatus(xml);
                               }
                         else if (tag == "marker") {
                               startMarkerView();
@@ -1071,7 +1091,6 @@ void MusE::read(Xml& xml)
       {
       bool skipmode = true;
       int pos;
-
       for (;;) {
             Xml::Token token = xml.parse();
             const QString& tag = xml.s1();
@@ -1109,9 +1128,14 @@ void MusE::read(Xml& xml)
                         else if (tag == "clip") {
                               Clip* clip = new Clip();
                               clip->read(xml);
+                              // if file not found, remove clip
+//                              if (clip->file() == 0)
+//DEBUG                                    delete clip;
+                              }
+                        else if (tag == "mctrl") {
+                              MidiController* ctrl = readMidiController(xml);
+                              midiControllerList.push_back(ctrl);
                               }
-                        else if (tag == "mctrl")
-                              readMidiController(xml);
                         else if (tag == "mplugin")
                               readStatusMidiInputTransformPlugin(xml);
                         else
@@ -1120,11 +1144,12 @@ void MusE::read(Xml& xml)
                   case Xml::Attribut:
                         break;
                   case Xml::TagEnd:
-                        if (!skipmode && tag == "muse")
+                        if (!skipmode && tag == "muse") {
+                              initDrumMap();
                               return;
+                              }
                   default:
                         break;
                   }
             }
-      initDrumMap();
       }
