@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: organ.cpp,v 1.5 2002/02/11 10:10:57 muse Exp $
+//  $Id: organ.cpp,v 1.1.1.1 2003/10/29 10:06:07 wschweer Exp $
 //
 //  Parts of this file taken from:
 //      Organ - Additive Organ Synthesizer Voice
@@ -10,7 +10,7 @@
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
 
-#include <math.h>
+#include <cmath>
 #include <stdio.h>
 #include <sys/time.h>
 #include <pthread.h>
@@ -21,7 +21,7 @@
 #include "organ.h"
 
 #define RESOLUTION   (16384)
-#define VOICES          16    // max polyphonie
+#define VOICES          128    // max polyphonie
 
 enum EnvelopeState {
       ATTACK,
@@ -173,7 +173,6 @@ class Organ : public Mess {
       void noteoff(int channel, int pitch);
       void setController(int channel, int ctrl, int val);
       void sysex(const unsigned char* data, int len);
-      void parameterRequest(int pn);
 
    public:
       Organ(const char* classname);
@@ -286,18 +285,6 @@ Organ::~Organ()
             }
       }
 
-//---------------------------------------------------------
-//   curTime
-//---------------------------------------------------------
-
-static double curTime()
-      {
-      struct timeval t;
-      struct timezone tz;
-      gettimeofday(&t, &tz);
-      return (double)((double)t.tv_sec + (t.tv_usec / 1000000.0));
-      }
-
 //---------------------------------------------------------
 //   table_pos
 //---------------------------------------------------------
@@ -335,21 +322,19 @@ bool Organ::init()
       setController(0, HARM3, maxdata);
       setController(0, HARM4, maxdata);
       setController(0, HARM5, maxdata);
+      setController(0, ATTACK_LO,  maxdata/100);
+      setController(0, DECAY_LO, 0);
+      setController(0, SUSTAIN_LO, maxdata);
+      setController(0, RELEASE_LO, maxdata/2);
+      setController(0, ATTACK_HI,  maxdata/100);
+      setController(0, DECAY_HI, 0);
+      setController(0, SUSTAIN_HI, maxdata);
+      setController(0, RELEASE_HI, maxdata/2);
 
       setController(0, BRASS, 1);
       setController(0, FLUTE, 1);
       setController(0, REED,  1);
 
-      setController(0, ATTACK_LO,  3);
-      setController(0, ATTACK_HI,  3);
-      setController(0, RELEASE_LO, 3);
-      setController(0, RELEASE_HI, 3);
-
-      setController(0, DECAY_LO, 0);
-      setController(0, DECAY_HI, 0);
-      setController(0, SUSTAIN_HI, maxdata);
-      setController(0, SUSTAIN_LO, maxdata);
-
       for (int i = 0; i < VOICES; ++i)
             voices[i].isOn = false;
       return false;
@@ -386,8 +371,9 @@ void Organ::processEvent(MEvent* ev)
                         case CTRL_LDATA:  dataLo = ev->dataB(); break;
                         case CTRL_HDATA:  dataHi = ev->dataB(); break;
                         }
-                  if (ev->dataA() == CTRL_HNRPN)
+                  if (ev->dataA() == CTRL_HDATA) {
                         setController(ev->chan(), ctrlLo+ctrlHi*128, dataLo+dataHi*128);
+                        }
                   break;
 
             case SND_SEQ_EVENT_SYSEX:
@@ -604,23 +590,6 @@ void Organ::setController(int /*channel*/, int ctrl, int data)
       controller[ctrl] = data;
       }
 
-//---------------------------------------------------------
-//   parameterRequest
-//---------------------------------------------------------
-
-void Organ::parameterRequest(int ctrl)
-      {
-      if (ctrl >= NUM_CONTROLLER) {
-            fprintf(stderr, "Organ: illegal controller %d request\n", ctrl);
-            return;
-            }
-      unsigned char data[] = { 0x7c, 0x1, 0x2, 0x0, 0x0, 0x0 };
-      data[3] = ctrl;
-      data[4] = controller[ctrl] & 0x7f;
-      data[5] = (controller[ctrl] >> 7) & 0x7f;
-      sendSysex(data, sizeof(data));
-      }
-
 //---------------------------------------------------------
 //   sysex
 //---------------------------------------------------------
@@ -640,7 +609,7 @@ void Organ::sysex(const unsigned char* data, int len)
                         }
                   if (data[2] == 1) {     // organ
                         if (data[3] == 1) {  // get parameter
-                              parameterRequest(data[4]);
+                              // parameterRequest(data[4]);
                               return;
                               }
                         }
@@ -677,10 +646,10 @@ static Mess* inst(const char* name)
       }
 
 //---------------------------------------------------------
-//   __init
+//   initMess
 //---------------------------------------------------------
 
-void __init()
+void initMess()
       {
       Mess::initMess(
           1222,
