@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: organgui.cpp,v 1.1 2002/01/30 12:08:39 muse Exp $
+//  $Id: organgui.cpp,v 1.1.1.1 2003/10/29 10:06:06 wschweer Exp $
 //
 //    This is a simple GUI implemented with QT for
 //    organ software synthesizer.
@@ -141,20 +141,31 @@ void Preset::writeConfiguration(Xml& xml, int level)
 
 //---------------------------------------------------------
 //   initParameter
-//    request current parameter settings from synti
+//    set initial parameters as in organ.cpp
 //---------------------------------------------------------
 
 void OrganGui::initParameter()
       {
-      for (int i = 0; i < NUM_CONTROLLER; ++i) {
-            putchar(0xf0);
-            putchar(0x7c);    // mess
-            putchar(0x1);     // organ
-            putchar(0x1);     // parameter request
-            putchar(i);
-            putchar(0xf7);
-            }
-      fflush(stdout);
+      int maxdata = 128*128-1;
+
+      setParam(HARM0, maxdata);
+      setParam(HARM1, maxdata);
+      setParam(HARM2, maxdata);
+      setParam(HARM3, maxdata);
+      setParam(HARM4, maxdata);
+      setParam(HARM5, maxdata);
+      setParam(ATTACK_LO,  maxdata/100);
+      setParam(DECAY_LO, 0);
+      setParam(SUSTAIN_LO, maxdata);
+      setParam(RELEASE_LO, maxdata/2);
+      setParam(ATTACK_HI,  maxdata/100);
+      setParam(DECAY_HI, 0);
+      setParam(SUSTAIN_HI, maxdata);
+      setParam(RELEASE_HI, maxdata/2);
+
+      setParam(BRASS, 1);
+      setParam(FLUTE, 1);
+      setParam(REED,  1);
       }
 
 //---------------------------------------------------------
@@ -314,25 +325,6 @@ void OrganGui::setPreset(Preset* preset)
                   }
             preset->ctrl[i] = val;
             }
-
-      //
-      // send sysex to synti
-      //
-#if 0
-      putchar(0xf0);
-      putchar(0x7c);    // mess
-      putchar(0x1);     // organ
-      putchar(0x3);     // setPreset
-      const char* name = preset->name.latin1();
-      while (*name)
-            putchar(*name++ & 0x7f);
-      putchar(0);
-      for (int i = 0; i < NUM_CONTROLLER; ++i) {
-            putchar(i);
-            putchar(preset->ctrl[i]);
-            }
-      putchar(0xf7);
-#endif
       }
 
 //---------------------------------------------------------
@@ -452,12 +444,17 @@ void OrganGui::eventReceived(int a, int b, int c)
                         case 0x06:  dataHi = c; break;
                         case 0x026: dataLo = c; break;
                         }
+                  // controller events are always transmitted
+                  // ctrlLo - ctrlHi - dataLo - dataHi
                   if (b == 0x06)
                         setParam(ctrlLo + ctrlHi*128, dataLo + dataHi*128);
                   break;
-
+            case 0x80:
+            case 0x90:
+                  break;
             default:
-                  fprintf(stderr, "OrganGUi: event received\n");
+                  fprintf(stderr, "OrganGUi: unknown event typ 0x%02x received\n",
+                     a);
                   break;
             }
       }
@@ -468,12 +465,13 @@ void OrganGui::eventReceived(int a, int b, int c)
 
 void OrganGui::loadPresetsPressed()
       {
-      QString s  = "presets/synthi/organ";
-      QString fn = getOpenFileName(s, presetFileTypes, this, "MusE: Load Organ Presets");
+      QString s("presets/synthi/organ/");
+      QString fn = getOpenFileName(s, presetFileTypes, this,
+         tr("MusE: Load Organ Presets"));
       if (fn.isEmpty())
             return;
       bool popenFlag;
-      FILE* f = fileOpen(this, fn, ".pre", "r", popenFlag, true);
+      FILE* f = fileOpen(this, fn, QString(".pre"), "r", popenFlag, true);
       if (f == 0)
             return;
       presets.clear();
@@ -526,12 +524,13 @@ ende:
 
 void OrganGui::savePresetsPressed()
       {
-      QString s  = "presets/synthi/organ";
-      QString fn = getSaveFileName(s, presetFileTypes, this, "MusE: Save Organ Presets");
+      QString s("presets/synthi/organ/");
+      QString fn = getSaveFileName(s, presetFileTypes, this,
+         tr("MusE: Save Organ Presets"));
       if (fn.isEmpty())
             return;
       bool popenFlag;
-      FILE* f = fileOpen(this, fn, ".pre", "w", popenFlag, false, true);
+      FILE* f = fileOpen(this, fn, QString(".pre"), "w", popenFlag, false, true);
       if (f == 0)
             return;
       Xml xml(f);
@@ -582,6 +581,8 @@ int main(int argc, char* argv[])
                   }
             }
       instanceName = argv[1];
+      if (argc >= 3)
+            museProject  = argv[2];
 
       for (int i = 0; i < _NSIG; ++i) {
             if (i != SIGABRT)
