@@ -1,12 +1,14 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: synth.cpp,v 1.8 2002/02/27 08:48:09 muse Exp $
+//    $Id: synth.cpp,v 1.1.1.1 2003/10/29 10:06:00 wschweer Exp $
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
 #include <qdir.h>
 #include <dlfcn.h>
+#include <qpopupmenu.h>
+
 #include "app.h"
 #include "synth.h"
 #include "xml.h"
@@ -15,8 +17,9 @@
 #include "mess.h"
 #include "ladspa.h"
 #include "song.h"
-#include "audiothread.h"
+#include "audio.h"
 #include "event.h"
+#include "mpevent.h"
 #include "midithread.h"
 #include <sys/mman.h>
 
@@ -30,6 +33,8 @@ int nsynthis = 0;
 Synth** synthis;              // static array of available synthis
 SynthIList synthiInstances;  // this list is owned by seq thread
 
+extern void connectNodes(AudioNode*, AudioNode*);
+
 //---------------------------------------------------------
 //   SynthI
 //---------------------------------------------------------
@@ -37,21 +42,23 @@ SynthIList synthiInstances;  // this list is owned by seq thread
 SynthI::SynthI(Synth* s)
    : synthesizer(s)
       {
-      _mdev    = 0;
-      _mess    = (Mess*)synthesizer->instantiate();
+      _mdev       = 0;
+      _mess       = (Mess*)synthesizer->instantiate();
       _mess->euid = euid;
       _mess->ruid = ruid;
+      if (! realTimeScheduling)
+	      _mess->realTimePriority = 0;
       synthesizer->activate(_mess);
-      _name    = _mess->name();
+      setIName(QString(_mess->name()));
       _gui     = false;
       _readFd  = -1;
       _writeFd = -1;
       setVolume(1.0);
       setPan(0.0);
-      setRoute(song->master());
-      AudioNode::setChannels(s->channels());
+      AudioNode::setPorts(s->channels());
+      connectOut(&audioOutput);       // default: connect to master
       printf("create Synth instance <%s> channels %d %d\n",
-         _name.latin1(), s->channels(), channels());
+         iname().latin1(), s->channels(), ports());
       }
 
 SynthI::~SynthI()
@@ -68,14 +75,11 @@ SynthI::~SynthI()
 
 void SynthI::sysexReceived(const unsigned char* p, int n)
       {
-// printf("FROM GUI: sysex %p\n", _mdev);
       if (!_mdev)
             return;
-      MidiEvent* event = new MidiEvent(_mdev->port()|0x100, 0, 0,
-         MidiEvent::Sysex, n, p);
-      midiThread->eventReceived(event);
-      _mdev->putEvent(event);     // echo to softSynth
-      delete event;
+      MidiPlayEvent event(_mdev->port()|0x100, p, n);
+      midiThread->eventReceived(&event);
+      _mdev->putEvent(&event);     // echo to softSynth
       }
 
 //---------------------------------------------------------
@@ -86,15 +90,13 @@ void SynthI::sysexReceived(const unsigned char* p, int n)
 
 void SynthI::eventReceived(int a, int b, int c)
       {
-//      printf("FROM GUI mdev %p\n", _mdev);
       if (_mdev == 0)
             return;
-      midiThread->eventReceived(_mdev->port()|0x100, a, b, c);
+      midiThread->eventReceived(_mdev->port()|0x80, a, b, c);
       switch(a & 0xf0) {
             case 0xb0:
                   {
-                  MidiEvent event(0, 0, 0, MidiEvent::Ctrl7,
-                    b, c, 0, 0);
+                  MidiPlayEvent event(0, 0, 0xb0, b, c);
                   _mdev->putEvent(&event);   // echo to softSynth
                   }
                   break;
@@ -106,7 +108,7 @@ void SynthI::eventReceived(int a, int b, int c)
 
 //---------------------------------------------------------
 //   showGui
-//    execution context: midiThread
+//    execution context: midithread
 //---------------------------------------------------------
 
 void SynthI::showGui(bool f)
@@ -156,6 +158,7 @@ void SynthI::showGui(bool f)
                         return;
                         }
                   }
+
             int rfiledes[2];
             int wfiledes[2];
             int rv = pipe(rfiledes);
@@ -168,7 +171,6 @@ void SynthI::showGui(bool f)
                   printf("pipe failed\n");
                   return;
                   }
-
             _guiPid = fork();
             if (_guiPid == 0) {       // Child side
                   dup2(rfiledes[1], 1);    // connect stdout to pipe
@@ -177,11 +179,16 @@ void SynthI::showGui(bool f)
                   dup2(wfiledes[0], 0);    // connect stdin to pipe
                   close(wfiledes[0]);
                   close(wfiledes[1]);
-                  const char* const new_argv[3] = {
+
+                  for (int i = 3; i < 50; ++i)
+                        close(i);
+                  const char* const new_argv[4] = {
                         classname,
                         instname,
+                        museProject.latin1(),
                         0
                         };
+
                   // Restore the signals
                   sigaction (SIGINT, &intr, 0);
                   sigaction (SIGQUIT, &quit, 0);
@@ -194,7 +201,8 @@ void SynthI::showGui(bool f)
                   //    c) gui does not run uid root
 
                   struct sched_param sp;
-                  sp.sched_priority = 0;
+                  memset(&sp, 0, sizeof(sp));
+                  // sp.sched_priority = 0;
 	            sched_setscheduler(0, SCHED_OTHER, &sp);
                   munlockall();
                   setuid(getuid());
@@ -206,6 +214,7 @@ void SynthI::showGui(bool f)
                   ;
             _readFd  = rfiledes[0];
             _writeFd = wfiledes[1];
+
             if ((sigaction (SIGINT, &intr, 0)
                | sigaction (SIGQUIT, &quit, 0)
                | sigprocmask (SIG_SETMASK, &omask, 0)) != 0) {
@@ -230,15 +239,16 @@ bool SynthI::hasGui() const
 
 //---------------------------------------------------------
 //   getData
+//    - connect LADSPA ports
+//    - run LADSPA ports
 //---------------------------------------------------------
 
-void SynthI::getData(float* buffer, int /*idx*/)
+bool SynthI::getData(int ports, unsigned long nframes, float** buffer)
       {
-      for (int i = 0; i < channels(); ++i) {
-            synthesizer->connect((LADSPA_Handle*)_mess, i,
-               buffer + segmentSize * i);
-            }
-      synthesizer->run((LADSPA_Handle*)_mess, segmentSize);
+      for (int i = 0; i < ports; ++i)
+            synthesizer->connect((LADSPA_Handle*)_mess, i, buffer[i]);
+      synthesizer->run((LADSPA_Handle*)_mess, nframes);
+      return true;
       }
 
 //---------------------------------------------------------
@@ -248,11 +258,10 @@ void SynthI::getData(float* buffer, int /*idx*/)
 
 static bool addSynthi(QFileInfo* fi, int idx)
       {
-      void* handle = dlopen(fi->filePath().data(), RTLD_NOW);
-//      void* handle = dlopen(fi->filePath().data(), RTLD_LAZY);
+      void* handle = dlopen(fi->filePath().ascii(), RTLD_NOW);
       if (handle == 0) {
             fprintf(stderr, "addSynthi: dlopen(%s) failed: %s\n",
-               fi->filePath().data(), dlerror());
+               fi->filePath().ascii(), dlerror());
             return false;
             }
       LADSPA_Descriptor_Function ladspa = (LADSPA_Descriptor_Function)dlsym(handle, "ladspa_descriptor");
@@ -264,7 +273,7 @@ static bool addSynthi(QFileInfo* fi, int idx)
                      "Unable to find ladspa_descriptor() function in plugin "
                      "library file \"%s\": %s.\n"
                      "Are you sure this is a LADSPA plugin file?\n",
-                     fi->filePath().data(), txt);
+                     fi->filePath().ascii(), txt);
                   return false;
                   }
             }
@@ -295,12 +304,13 @@ static bool addSynthi(QFileInfo* fi, int idx)
       LADSPA_Properties properties = descr->Properties;
       if (LADSPA_IS_INPLACE_BROKEN(properties)) {
 	      fprintf(stderr,
-               "Plugin \"%s\" is not capable of in-place processing and "
+               "Synti \"%s\" is not capable of in-place processing and "
                "therefore cannot be used by this program.\n",
                descr->Name);
             return false;
             }
-      printf("found soft synth <%s> <%s>\n", descr->Label, descr->Name);
+      if (debugMsg)
+            printf("found soft synth <%s> <%s>\n", descr->Label, descr->Name);
       synthis[idx] = new Synth(fi, ladspa, descr);
       return true;
       }
@@ -316,12 +326,14 @@ void initMidiSynth()
 
       if (!alsaFound)
             return;
-      QDir pluginDir(s, "*.so", QDir::Files);
-printf("searching for software synthesizer in <%s>\n", s.latin1());
+      QDir pluginDir(s, QString("*.so"), QDir::Files);
+      if (debugMsg)
+            printf("searching for software synthesizer in <%s>\n", s.latin1());
       if (pluginDir.exists()) {
             const QFileInfoList* list = pluginDir.entryInfoList();
             nsynthis = list->count();
-printf("%d soft synth found\n", nsynthis);
+            if (debugMsg)
+                  printf("%d soft synth found\n", nsynthis);
             synthis  = new (Synth*)[nsynthis];
             QFileInfoListIterator it(*list);
             QFileInfo* fi;
@@ -437,8 +449,6 @@ void MusE::writeSoftSynth(int level, Xml& xml) const
                               continue;
                         }
 
-                  int a = ev.dataA();
-                  int b = ev.dataB();
                   unsigned char* p = ev.data();
                   xml.nput(level++, "<event type=\"%d\"", type);
                   if (p) {
@@ -452,11 +462,11 @@ void MusE::writeSoftSynth(int level, Xml& xml) const
                               }
                         xml.nput("\n");
                         xml.tag(level, "/event");
-                        delete p;
+                        delete[] p;
                         ev.setData(0);
                         }
                   else {
-                        xml.put(" a=\"%d\" b=\"%d\"/>", a, b);
+                        xml.put(" a=\"%d\" b=\"%d\"/>", ev.dataA(), ev.dataB());
                         }
                   --level;
                   }
@@ -531,18 +541,28 @@ void MusE::readSoftSynth(Xml& xml)
                               if (synth == 0)
                                     return;
                               // synth is not yet connected:
-                              audioThread->msgAddSynthI(synth);
+                              audio->msgAddSynthI(synth);
                               midiThread->msgAddSynthI(synth);
+                              midiThread->msgShowInstrumentGui(synth, guiVisible);
+                              sleep(2);  // TODO: wait for GUI
                               if (port != -1) {
                                     EventList* el = synth->midiState();
+                                    MPEventList mpl;
+                                    int tick = 0;
+                                    // we must use a tick to get the right
+                                    // event sorting order
                                     for (iEvent i = el->begin(); i != el->end(); ++i) {
                                           MidiEvent* ev = (MidiEvent*)(i->second);
-                                          ev->setPort(port);
-                                          ev->setChannel(0);
-                                          midiThread->playMidiEvent(ev);
+                                          mpl.add(tick, ev, port, 0);
+                                          // might add more than one raw event
+                                          tick += 16;
                                           }
+                                    for (iMPEvent i = mpl.begin(); i != mpl.end(); ++i) {
+                                          midiThread->playMidiEvent(&(i->second));
+                                          synth->writeToGui(&(i->second));
+                                          }
+                                    mpl.clear();
                                     }
-                              midiThread->msgShowInstrumentGui(synth, guiVisible);
                               return;
                               }
                   default:
@@ -572,7 +592,7 @@ void SynthI::populatePatchPopup(QPopupMenu* menu, int ch, MType)
       while (mp) {
             int id = ((mp->hbank & 0xff) << 16)
                       + ((mp->lbank & 0xff) << 8) + mp->prog;
-            menu->insertItem(mp->name, id);
+            menu->insertItem(QString(mp->name), id);
             mp = _mess->getNextPatch(ch, mp);
             }
       }
@@ -601,45 +621,62 @@ void Song::removeSoftSynth()
       for (ii = synthiInstances.begin(); ii != synthiInstances.end();) {
             iSynthI ni = ii;
             ++ni;
-            audioThread->msgRemoveSynthI(*ii);
+            audio->msgRemoveSynthI(*ii);
+            delete *ii;
             ii = ni;
             }
-      update();
       }
 
 //---------------------------------------------------------
 //   writeToGui
 //---------------------------------------------------------
 
-void SynthI::writeToGui(const MidiEvent* ev)
+void SynthI::writeToGui(const MidiPlayEvent* e)
       {
       if (!_gui)
             return;
-      switch (ev->type()) {
-            case MidiEvent::Sysex:
+
+      int chn = e->channel();
+      int a   = e->dataA();
+      int b   = e->dataB();
+      int typ = e->type();
+
+      char data[3];
+      data[0] = typ + chn;
+      data[1] = a;
+      data[2] = b;
+
+      switch(typ) {
+            case 0x90:
+            case 0x80:
+            case 0xb0:
+            case 0xa0:
+            case 0xe0:
                   {
-                  int n = ev->dataLen();
-                  char data[n + 2];
-                  data[0]   = 0xf0;
-                  data[n+1] = 0xf7;
-                  memcpy(data+1, ev->data(), n);
-                  int nn = write(_writeFd, data, n+2);
-                  if (nn != n+2)
-                        perror("write to gui failed");
-                  }
-                  break;
-            case MidiEvent::Ctrl7:
-                  {
-                  char data[3];
-                  data[0] = 0xb0 + ev->channel();
-                  data[1] = ev->dataA();
-                  data[2] = ev->dataB();
                   int nn = write(_writeFd, data, 3);
                   if (nn != 3)
                         perror("write to gui failed");
                   }
                   break;
+            case 0xc0:
+            case 0xd0:
+                  {
+                  int nn = write(_writeFd, data, 2);
+                  if (nn != 3)
+                        perror("write to gui failed");
+                  }
+                  break;
+            case 0xf0:
+                  {
+                  unsigned char c = 0xf0;
+                  write(_writeFd, &c, 1);
+                  write(_writeFd, e->data(), e->len());
+                  c = 0xf7;
+                  write(_writeFd, &c, 1);
+                  }
+                  break;
             default:
+                  printf("Synti: GUI write: illegal event type %d\n", typ);
                   break;
             }
       }
