@@ -714,10 +714,10 @@ static Mess* inst(const char* name)
       }
 
 //---------------------------------------------------------
-//   __init
+//   initMess
 //---------------------------------------------------------
 
-void __init()
+void initMess()
       {
       Mess::initMess(
           1224,
