@@ -34,4 +34,4 @@ enum {
 
 #define NUM_CONTROLLER		32
 
-#endif __VAM_H
+#endif /* __VAM_H */
