@@ -567,12 +567,13 @@ void VAMGui::eventReceived(int a, int b, int c)
 void VAMGui::loadPresetsPressed()
 {
 	QString iname;
-	QString s  = getenv("HOME");
-	QString fn = getOpenFileName(s, presetFileTypes, this, "MusE: Load VAM Presets");
+	QString s(getenv("HOME"));
+	QString fn = getOpenFileName(s, presetFileTypes, this,
+         tr("MusE: Load VAM Presets"));
 	if (fn.isEmpty())
 		return;
 	bool popenFlag;
-	FILE* f = fileOpen(this, fn, ".pre", "r", popenFlag, true);
+	FILE* f = fileOpen(this, fn, QString(".pre"), "r", popenFlag, true);
 	if (f == 0)
 		return;
 	presets.clear();
@@ -640,7 +641,7 @@ ende:
 void VAMGui::doSavePresets(const QString& fn, bool showWarning)
 {
 	bool popenFlag;
-	FILE* f = fileOpen(this, fn, ".pre", "w", popenFlag, false, showWarning);
+	FILE* f = fileOpen(this, fn, QString(".pre"), "w", popenFlag, false, showWarning);
 	if (f == 0)
 		return;
 	Xml xml(f);
@@ -665,8 +666,9 @@ void VAMGui::doSavePresets(const QString& fn, bool showWarning)
 
 void VAMGui::savePresetsPressed()
 {
-	QString s = getenv("MUSE");
-	QString fn = getSaveFileName(s, presetFileTypes, this, "MusE: Save VAM Presets");
+	QString s(getenv("MUSE"));
+	QString fn = getSaveFileName(s, presetFileTypes, this,
+         tr("MusE: Save VAM Presets"));
 	if (fn.isEmpty())
 		return;
 	doSavePresets (fn, true);
