@@ -1,13 +1,11 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: transport.cpp,v 1.1 2002/01/30 14:54:03 muse Exp $
+//  $Id: transport.cpp,v 1.1.1.1 2003/10/29 10:05:00 wschweer Exp $
 //
 //  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
-#include <stdio.h>
-#include <qpixmap.h>
 #include <qvbox.h>
 #include <qlabel.h>
 #include <qslider.h>
@@ -15,7 +13,6 @@
 #include <qframe.h>
 #include <qwhatsthis.h>
 #include <qtooltip.h>
-#include <qbuttongroup.h>
 #include <qlayout.h>
 #include <qtoolbutton.h>
 #include <qcombobox.h>
@@ -28,13 +25,14 @@
 #include "icons.h"
 #include "seq.h"
 #include "posedit.h"
+#include "sync.h"
 
-static const char* recordTransportText   = "Click this button to enable recording";
-static const char* stopTransportText     = "Click this button to stop playback";
-static const char* playTransportText     = "Click this button to start playback";
-static const char* startTransportText    = "Click this button to rewind to start position";
-static const char* frewindTransportText  = "Click this button to rewind";
-static const char* fforwardTransportText = "Click this button to forward current play position";
+static const char* recordTransportText   = QT_TR_NOOP("Click this button to enable recording");
+static const char* stopTransportText     = QT_TR_NOOP("Click this button to stop playback");
+static const char* playTransportText     = QT_TR_NOOP("Click this button to start playback");
+static const char* startTransportText    = QT_TR_NOOP("Click this button to rewind to start position");
+static const char* frewindTransportText  = QT_TR_NOOP("Click this button to rewind");
+static const char* fforwardTransportText = QT_TR_NOOP("Click this button to forward current play position");
 
 //---------------------------------------------------------
 //   toolButton
@@ -102,25 +100,24 @@ void Handle::mousePressEvent(QMouseEvent* ev)
 //    Widget fr Tempo + Signature
 //---------------------------------------------------------
 
-TempoSig::TempoSig(QWidget* parent, int w, int h)
-   : QFrame(parent, "TempoSig")
+TempoSig::TempoSig(QWidget* parent)
+   : QWidget(parent, "TempoSig")
       {
-      setFixedSize(w, h);
+      QBoxLayout* vb1 = new QVBoxLayout(this);
+      vb1->setAutoAdd(true);
+
       QFrame* f = new QFrame(this);
-      f->setFrameStyle(Panel | Sunken);
+      f->setFrameStyle(QFrame::Panel | QFrame::Sunken);
       f->setLineWidth(1);
 
-      l1         = new DoubleLabel(120.0, 20.0, 400.0, f);
-      l1->setSpecialText("extern");
-      l2         = new SigLabel(4, 4, f);
-      QLabel* l3 = new QLabel(tr("Tempo/Sig"), this);
-      int h1 = (h/8) * 3;
-      int h3 = h - 2* h1;
-      l1->setFixedSize(w-2, h1);
-      l2->setFixedSize(w-2, h1);
-      l3->setFixedSize(w-2, h3);
-      f->setFixedSize(w-2, h1 + h1);
+      QBoxLayout* vb2 = new QVBoxLayout(f);
+      vb2->setAutoAdd(true);
 
+      l1 = new DoubleLabel(120.0, 20.0, 400.0, f);
+      l1->setSpecialText(QString("extern"));
+      l2 = new SigLabel(4, 4, f);
+
+      QLabel* l3 = new QLabel(tr("Tempo/Sig"), this);
       l3->setFont(font2);
 
       l1->setBackgroundMode(PaletteLight);
@@ -129,10 +126,7 @@ TempoSig::TempoSig(QWidget* parent, int w, int h)
       l2->setAlignment(AlignCenter);
       l3->setAlignment(AlignCenter);
 
-      l1->move(1, 1);
-      l2->move(1, 1+h1);
-      l3->move(1, 1+2*h1-1);
-      connect(l1, SIGNAL(valueChanged(double)), SLOT(setTempo(double)));
+      connect(l1, SIGNAL(valueChanged(double,int)), SLOT(setTempo(double)));
       connect(l2, SIGNAL(valueChanged(int,int)), SIGNAL(sigChanged(int,int)));
       }
 
@@ -153,7 +147,9 @@ void TempoSig::setTempo(double t)
 void TempoSig::setTempo(int tempo)
       {
       double t = (1000000.0 * 60.0)/tempo;
+      l1->blockSignals(true);
       l1->setValue(t);
+      l1->blockSignals(false);
       }
 
 //---------------------------------------------------------
@@ -165,16 +161,26 @@ void TempoSig::setTimesig(int a, int b)
       l2->setValue(a, b);
       }
 
+//---------------------------------------------------------
+//   setRecord
+//---------------------------------------------------------
+
+void Transport::setRecord(bool flag)
+      {
+      buttons[5]->blockSignals(true);
+      buttons[5]->setOn(flag);
+      buttons[5]->blockSignals(false);
+      }
+
 //---------------------------------------------------------
 //   Transport
 //---------------------------------------------------------
 
-Transport::Transport(QWidget*, const char* name = 0)
+Transport::Transport(QWidget*, const char* name)
    : QWidget(0, name, WStyle_Customize | WType_TopLevel | WStyle_Tool
-  | WStyle_NoBorderEx | WStyle_StaysOnTop)
+  | WStyle_NoBorder | WStyle_StaysOnTop)
       {
       QHBoxLayout* hbox = new QHBoxLayout(this, 2, 5);
-      setMaximumHeight(80);
 
       hbox->setAutoAdd(true);
       lefthandle = new Handle(this, this);
@@ -185,19 +191,21 @@ Transport::Transport(QWidget*, const char* name = 0)
 
       QVBox* box1 = new QVBox(this);
       recMode     = new QComboBox(box1);
-      recMode->insertItem("Overdub", Song::REC_OVERDUP);
-      recMode->insertItem("Replace", Song::REC_REPLACE);
+      recMode->insertItem(tr("Overdub"), Song::REC_OVERDUP);
+      recMode->insertItem(tr("Replace"), Song::REC_REPLACE);
       recMode->setCurrentItem(song->recMode());
-      QLabel* l2 = new QLabel("Rec Mode", box1);
+      QLabel* l2 = new QLabel(tr("Rec Mode"), box1);
+      l2->setFont(font2);
       l2->setAlignment(AlignCenter);
       connect(recMode, SIGNAL(activated(int)), SLOT(setRecMode(int)));
 
       cycleMode = new QComboBox(box1);
-      cycleMode->insertItem("Normal",  Song::CYCLE_NORMAL);
-      cycleMode->insertItem("Mix",     Song::CYCLE_MIX);
-      cycleMode->insertItem("Replace", Song::CYCLE_REPLACE);
+      cycleMode->insertItem(tr("Normal"),  Song::CYCLE_NORMAL);
+      cycleMode->insertItem(tr("Mix"),     Song::CYCLE_MIX);
+      cycleMode->insertItem(tr("Replace"), Song::CYCLE_REPLACE);
       cycleMode->setCurrentItem(song->cycleMode());
-      QLabel* l3 = new QLabel("Cycle Rec", box1);
+      QLabel* l3 = new QLabel(tr("Cycle Rec"), box1);
+      l3->setFont(font2);
       l3->setAlignment(AlignCenter);
       connect(cycleMode, SIGNAL(activated(int)), SLOT(setCycleMode(int)));
 
@@ -206,12 +214,14 @@ Transport::Transport(QWidget*, const char* name = 0)
       //-----------------------------------------------------
 
       QVBox* button2 = new QVBox(this);
-      button2->setBackgroundMode(PaletteDark);
+//      button2->setBackgroundMode(PaletteDark);
       button2->setMargin(3);
 
-      QToolButton* b1 = newButton(button2, punchinIcon, "punchin", true);
-      QToolButton* b2 = newButton(button2, loopIcon, "loop", true);
-      QToolButton* b3 = newButton(button2, punchoutIcon, "punchout", true);
+      QToolButton* b1 = newButton(button2, punchinIcon, tr("punchin"), true);
+      QToolButton* b2 = newButton(button2, loopIcon, tr("loop"), true);
+      b2->setAccel(Key_Slash);
+
+      QToolButton* b3 = newButton(button2, punchoutIcon, tr("punchout"), true);
       QToolTip::add(b1, tr("Punch In"));
       QToolTip::add(b2, tr("Loop"));
       QToolTip::add(b3, tr("Punch Out"));
@@ -236,9 +246,11 @@ Transport::Transport(QWidget*, const char* name = 0)
       QVBox* marken = new QVBox(this);
       tl1 = new PosEdit(marken);
       QLabel* l5 = new QLabel(tr("Left Mark"), marken);
+      l5->setFont(font2);
       l5->setAlignment(AlignCenter);
       tl2 = new PosEdit(marken);
       QLabel* l6 = new QLabel(tr("Right Mark"), marken);
+      l6->setFont(font2);
       l6->setAlignment(AlignCenter);
 
       //-----------------------------------------------------
@@ -253,40 +265,35 @@ Transport::Transport(QWidget*, const char* name = 0)
       time2->setSmpte(true);
 
       slider = new QSlider(0, 200000, 1000, 0, Horizontal, box4);
-      slider->setFixedHeight(18);
 
       tb = new QHBox(box4);
 
-      buttons[0] = newButton(tb, startIcon, "rewind to start");
-      QWhatsThis::add(buttons[0], startTransportText);
+      buttons[0] = newButton(tb, startIcon, tr("rewind to start"));
+      QWhatsThis::add(buttons[0], tr(startTransportText));
 
-      buttons[1] = newButton(tb, frewindIcon, "rewind");
+      buttons[1] = newButton(tb, frewindIcon, tr("rewind"));
       buttons[1]->setAutoRepeat(true);
-      QWhatsThis::add(buttons[1], frewindTransportText);
+      QWhatsThis::add(buttons[1], tr(frewindTransportText));
 
-      buttons[2] = newButton(tb, fforwardIcon, "forward");
+      buttons[2] = newButton(tb, fforwardIcon, tr("forward"));
       buttons[2]->setAutoRepeat(true);
-      QWhatsThis::add(buttons[2], fforwardTransportText);
+      QWhatsThis::add(buttons[2], tr(fforwardTransportText));
 
-      buttons[3] = newButton(tb, stopIcon, "stop", true);
+      buttons[3] = newButton(tb, stopIcon, tr("stop"), true);
       buttons[3]->setOn(true);     // set STOP
-      QWhatsThis::add(buttons[3], stopTransportText);
+      QWhatsThis::add(buttons[3], tr(stopTransportText));
 
-      buttons[4] = newButton(tb, playIcon, "play", true);
-      QWhatsThis::add(buttons[4], playTransportText);
+      buttons[4] = newButton(tb, playIcon, tr("play"), true);
+      QWhatsThis::add(buttons[4], tr(playTransportText));
 
-      buttons[5] = newButton(tb, recordIcon, "record", true);
-      QWhatsThis::add(buttons[5], recordTransportText);
+      buttons[5] = newButton(tb, recordIcon, tr("record"), true);
+      QWhatsThis::add(buttons[5], tr(recordTransportText));
 
-      QButtonGroup* playBg = new QButtonGroup(this);
-      playBg->setExclusive(true);
-      playBg->hide();
-      playBg->insert(buttons[3], 0);
-      playBg->insert(buttons[4], 1);
+      connect(buttons[3], SIGNAL(toggled(bool)), SLOT(stopToggled(bool)));
+      connect(buttons[4], SIGNAL(toggled(bool)), SLOT(playToggled(bool)));
 
-      connect(buttons[4], SIGNAL(toggled(bool)), song, SLOT(setPlay(bool)));
       connect(buttons[5], SIGNAL(toggled(bool)), song, SLOT(setRecord(bool)));
-      connect(song, SIGNAL(recordChanged(bool)), buttons[5], SLOT(setOn(bool)));
+      connect(song, SIGNAL(recordChanged(bool)), SLOT(setRecord(bool)));
       connect(buttons[0], SIGNAL(clicked()), song, SLOT(rewindStart()));
       connect(buttons[1], SIGNAL(clicked()), song, SLOT(rewind()));
       connect(buttons[2], SIGNAL(clicked()), song, SLOT(forward()));
@@ -297,11 +304,13 @@ Transport::Transport(QWidget*, const char* name = 0)
 
       QVBox* button1 = new QVBox(this);
       button1->setMargin(3);
-      button1->setBackgroundMode(PaletteDark);
+//      button1->setBackgroundMode(PaletteDark);
 
-      quantizeButton = newButton(button1, "AC", "quantize during record", true);
-      clickButton    = newButton(button1, "Click", "metronom click on/off", true);
-      syncButton     = newButton(button1, "Sync", "external sync on/off", true);
+      quantizeButton = newButton(button1, tr("AC"), tr("quantize during record"), true);
+      clickButton    = newButton(button1, tr("Click"), tr("metronom click on/off"), true);
+      clickButton->setAccel(Key_C);
+
+      syncButton     = newButton(button1, tr("Sync"), tr("external sync on/off"), true);
 
       quantizeButton->setOn(song->quantize());
       clickButton->setOn(song->click());
@@ -320,8 +329,8 @@ Transport::Transport(QWidget*, const char* name = 0)
       //-----------------------------------------------------
 
       QVBox* box5  = new QVBox(this);
-      tempo        = new TempoSig(box5, 80, 46);
-      masterButton = newButton(box5, "Master", "use master track", true);
+      tempo        = new TempoSig(box5);
+      masterButton = newButton(box5, tr("Master"), tr("use master track"), true);
 
       connect(masterButton, SIGNAL(toggled(bool)), song, SLOT(setMasterFlag(bool)));
 
@@ -332,7 +341,7 @@ Transport::Transport(QWidget*, const char* name = 0)
       connect(time1, SIGNAL(valueChanged(const Pos&)), SLOT(cposChanged(const Pos&)));
       connect(time2, SIGNAL(valueChanged(const Pos&)), SLOT(cposChanged(const Pos&)));
 
-      connect(slider,SIGNAL(sliderMoved(int)),  SLOT(cposChanged(int)));
+      connect(slider,SIGNAL(valueChanged(int)),  SLOT(cposChanged(int)));
       connect(song, SIGNAL(posChanged(int, int, bool)), SLOT(setPos(int, int, bool)));
       connect(tempo, SIGNAL(tempoChanged(int)), song, SLOT(setTempo(int)));
       connect(tempo, SIGNAL(sigChanged(int, int)), song, SLOT(setSig(int, int)));
@@ -348,10 +357,10 @@ Transport::Transport(QWidget*, const char* name = 0)
 void Transport::setTempo(int t)
       {
       static int tempoVal = -1;
-      if (t != tempoVal)
+      if (t != tempoVal) {
             tempo->setTempo(t);
-      else
             tempoVal = t;
+            }
       }
 
 //---------------------------------------------------------
@@ -383,7 +392,9 @@ void Transport::setPos(int idx, int v, bool)
             case 0:
                   time1->setValue(v);
                   time2->setValue(v);
+                  slider->blockSignals(true);
                   slider->setValue(v);
+                  slider->blockSignals(false);
                   if (song->masterFlag())
                         setTempo(tempomap.tempo(v));
                   {
@@ -443,8 +454,12 @@ void Transport::rposChanged(const Pos& pos)
 
 void Transport::setPlay(bool f)
       {
+      buttons[3]->blockSignals(true);
+      buttons[4]->blockSignals(true);
       buttons[3]->setOn(!f);
       buttons[4]->setOn(f);
+      buttons[3]->blockSignals(false);
+      buttons[4]->blockSignals(false);
       }
 
 //---------------------------------------------------------
@@ -462,7 +477,9 @@ void Transport::setMasterFlag(bool f)
 
 void Transport::setClickFlag(bool f)
       {
+      clickButton->blockSignals(true);
       clickButton->setOn(f);
+      clickButton->blockSignals(false);
       }
 
 //---------------------------------------------------------
@@ -521,6 +538,8 @@ void Transport::songChanged(int flags)
             sigmap.timesig(cpos, z, n);
             setTimesig(z, n);
             }
+      if (flags & SC_MASTER)
+            masterButton->setOn(song->masterFlag());
      }
 
 //---------------------------------------------------------
@@ -544,5 +563,40 @@ void Transport::syncChanged(bool flag)
             }
       else
             tempo->setTempo(tempomap.tempo(song->cpos()));
+      playAction->setEnabled(!flag);
+      startAction->setEnabled(!flag);
+      stopAction->setEnabled(!flag);
+      rewindAction->setEnabled(!flag);
+      forwardAction->setEnabled(!flag);
+      }
+
+//---------------------------------------------------------
+//   stopToggled
+//---------------------------------------------------------
+
+void Transport::stopToggled(bool val)
+      {
+      if (val)
+            song->setStop(true);
+      else {
+            buttons[3]->blockSignals(true);
+            buttons[3]->setOn(true);
+            buttons[3]->blockSignals(false);
+            }
+      }
+
+//---------------------------------------------------------
+//   playToggled
+//---------------------------------------------------------
+
+void Transport::playToggled(bool val)
+      {
+      if (val)
+            song->setPlay(true);
+      else {
+            buttons[4]->blockSignals(true);
+            buttons[4]->setOn(true);
+            buttons[4]->blockSignals(false);
+            }
       }
 
