@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: value.cpp,v 1.1 2002/01/30 14:54:03 muse Exp $
+//  $Id: value.cpp,v 1.1.1.1 2003/10/29 10:05:11 wschweer Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -10,11 +10,11 @@
 #include "xml.h"
 
 
-IValue::IValue(QObject* parent=0, const char* name=0)
+IValue::IValue(QObject* parent, const char* name)
    : QObject(parent, name)
       {
       }
-BValue::BValue(QObject* parent=0, const char* name=0)
+BValue::BValue(QObject* parent, const char* name)
    : QObject(parent, name)
       {
       }
