@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: waveedit.h,v 1.2 2001/11/20 15:19:34 muse Exp $
+//    $Id: waveedit.h,v 1.1.1.1 2003/10/29 10:06:14 wschweer Exp $
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -17,6 +17,8 @@ class WaveView;
 class ScrollScale;
 class QSlider;
 class PosLabel;
+class QResizeEvent;
+class SNode;
 
 //---------------------------------------------------------
 //   WaveEdit
@@ -26,21 +28,24 @@ class WaveEdit : public MidiEditor {
       static QColor bg;
 
       WaveView* view;
-      PartList* parts;
       QSlider* ymag;
       QToolBar* tools;
       QToolBar* tb1;
       QToolButton* solo;
       PosLabel* pos1;
       PosLabel* pos2;
-      int partSn;
+
+      static int _widthInit, _heightInit;
 
       Q_OBJECT
       virtual void closeEvent(QCloseEvent*);
+      virtual void resizeEvent(QResizeEvent* ev);
 
    private slots:
       void cmd(int);
       void setTime(int t);
+      void soloChanged(SNode* s);
+      void soloChanged(bool flag);
 
    signals:
       void deleted(int);
