@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: canvas.cpp,v 1.1 2002/01/30 14:54:03 muse Exp $
+//    $Id: canvas.cpp,v 1.2 2003/10/29 14:19:26 lunar_shuttle Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -9,20 +9,22 @@
 #include "canvas.h"
 #include <qpainter.h>
 #include "song.h"
+#include "citem.h"
 #include <qpopupmenu.h>
 #include <qcursor.h>
 #include "icons.h"
 #include "../marker/marker.h"
-
+#include "event.h"
 #define ABS(x)  ((x) < 0) ? -(x) : (x)
 
 //---------------------------------------------------------
 //   Canvas
 //---------------------------------------------------------
 
-Canvas::Canvas(QWidget* parent, int sx, int sy, const char* name = 0)
+Canvas::Canvas(QWidget* parent, int sx, int sy, const char* name)
    : View(parent, sx, sy, name)
       {
+      canvasTools = 0;
       itemPopupMenu = 0;
       drag    = DRAG_OFF;
       _tool   = PointerTool;
@@ -56,7 +58,7 @@ void Canvas::setPos(int idx, int val, bool adjustScrollbar)
                         break;
                   case Song::JUMP:
                         if (npos >= width()) {
-                              int ppos =  val - rmapxDev(width()/4);
+                              int ppos =  val - rmapxDev(width()/8);
                               if (ppos < 0)
                                     ppos = 0;
                               emit followEvent(ppos);
@@ -73,16 +75,16 @@ void Canvas::setPos(int idx, int val, bool adjustScrollbar)
                               }
                         break;
                   case Song::CONTINUOUS:
-                        if (npos > (width()*5)/8) {
-                              int ppos =  pos[idx] - rmapxDev(width()*5/8);
+                        if (npos > (width()/2)) {
+                              int ppos =  pos[idx] - rmapxDev(width()/2);
                               if (ppos < 0)
                                     ppos = 0;
                               emit followEvent(ppos);
                               opos = mapx(pos[idx]);
                               npos = mapx(val);
                               }
-                        else if (npos < (width()*3)/8) {
-                              int ppos =  pos[idx] - rmapxDev(width()*3/8);
+                        else if (npos < (width()/2)) {
+                              int ppos =  pos[idx] - rmapxDev(width()/2);
                               if (ppos < 0)
                                     ppos = 0;
                               emit followEvent(ppos);
@@ -104,7 +106,7 @@ void Canvas::setPos(int idx, int val, bool adjustScrollbar)
             x = opos;
             }
       pos[idx] = val;
-      redraw(QRect(x, 0, w, height()));
+      redraw(QRect(x-1, 0, w+2, height()));
       }
 
 //---------------------------------------------------------
@@ -262,7 +264,7 @@ void Canvas::selectItem(CItem* e, bool flag)
 //    copy selection-List to moving-List
 //---------------------------------------------------------
 
-void Canvas::startMoving(const QPoint& pos, bool)
+void Canvas::startMoving(const QPoint& pos, DragType)
       {
       for (iCItem i = items.begin(); i != items.end(); ++i) {
             if (i->second->isSelected()) {
@@ -322,25 +324,27 @@ void Canvas::viewMousePressEvent(QMouseEvent* event)
       // special events if right button is clicked while operations
       // like moving or drawing lasso is performed.
       if (event->stateAfter() & RightButton) {
-	  switch (drag) {
-	        case DRAG_LASSO:
-	    		drag = DRAG_OFF;
-			redraw();
-			return;
-		case DRAG_MOVE:
-			drag = DRAG_OFF;
-			endMoveItems (start, false, 0);
-			return;
-		default:
-			break;
-	    }
+            switch (drag) { 
+                  case DRAG_LASSO: 
+                        drag = DRAG_OFF; 
+                        redraw(); 
+                        return; 
+                  case DRAG_MOVE: 
+                        drag = DRAG_OFF; 
+                        endMoveItems (start, MOVE_MOVE, 0); 
+                        return; 
+                  default: 
+                  break; 
+            } 
       }
-      
+
       // ignore event if (another) button is already active:
       if (keyState & (LeftButton|RightButton|MidButton)) {
             return;
             }
       bool shift      = keyState & ShiftButton;
+      bool alt        = keyState & AltButton;
+      bool ctrl       = keyState & ControlButton; 
       start           = event->pos();
 
       //---------------------------------------------------
@@ -382,8 +386,17 @@ void Canvas::viewMousePressEvent(QMouseEvent* event)
                         int n = itemPopupMenu->exec(QCursor::pos());
                         if (n != -1)
                               itemPopup(curItem, n, start);
+                        delete itemPopupMenu;
+                        }
+                  }
+            else {
+                  canvasPopupMenu = genCanvasPopup();
+                  if (canvasPopupMenu) {
+                        int n = canvasPopupMenu->exec(QCursor::pos(), 0);
+                        if (n != -1)
+                              canvasPopup(n);
+                        delete canvasPopupMenu;
                         }
-                  delete itemPopupMenu;
                   }
             }
       else if (event->button() == QMouseEvent::LeftButton) {
@@ -396,7 +409,23 @@ void Canvas::viewMousePressEvent(QMouseEvent* event)
                                     curPartId = curPart->sn();
                                     curPartChanged();
                                     }
-                              drag = shift ? DRAG_COPY_START : DRAG_MOVE_START;
+                              if (shift)
+                                    drag = DRAG_COPY_START;
+                              else if (alt) {
+                                    drag = DRAG_CLONE_START;
+// printf("CLONE start\n");
+                                    }
+                              else if (ctrl) { //Select all on the same pitch (e.g. same y-value) 
+                                          deselectAll(); 
+                                          for (iCItem i = items.begin(); i != items.end(); ++i) { 
+                                                if (i->second->y() == curItem->y() ) 
+                                                      selectItem(i->second, true); 
+                                          } 
+                                          updateSelection(); 
+                                          redraw(); 
+                                    } 
+                              else
+                                    drag = DRAG_MOVE_START;
                               }
                         else
                               drag = DRAG_LASSO_START;
@@ -463,27 +492,41 @@ void Canvas::viewMouseMoveEvent(QMouseEvent* event)
                   // proceed with DRAG_LASSO:
 
             case DRAG_LASSO:
-                  {
-                  QRect r(start.x(), start.y(), dist.x(), dist.y());
-//                  QRect rr = lasso | r;
-                  lasso = r;
+                  lasso = QRect(start.x(), start.y(), dist.x(), dist.y());
                   redraw();
-//                  redraw(map(rr));
-                  }
                   break;
 
             case DRAG_MOVE_START:
             case DRAG_COPY_START:
+            case DRAG_CLONE_START:
                   if (!moving)
                         break;
                   if (keyState & ControlButton) {
-                        if (ax > ay)
-                              drag = drag == DRAG_MOVE_START ? DRAGX_MOVE : DRAGX_COPY;
+                        if (ax > ay) {
+                              if (drag == DRAG_MOVE_START)
+                                    drag = DRAGX_MOVE;
+                              else if (drag == DRAG_COPY_START)
+                                    drag = DRAGX_COPY;
+                              else
+                                    drag = DRAGX_CLONE;
+                              }
+                        else {
+                              if (drag == DRAG_MOVE_START)
+                                    drag = DRAGY_MOVE;
+                              else if (drag == DRAG_COPY_START)
+                                    drag = DRAGY_COPY;
+                              else
+                                    drag = DRAGY_CLONE;
+                              }
+                        }
+                  else {
+                        if (drag == DRAG_MOVE_START)
+                              drag = DRAG_MOVE;
+                        else if (drag == DRAG_COPY_START)
+                              drag = DRAG_COPY;
                         else
-                              drag = drag == DRAG_MOVE_START ? DRAGY_MOVE : DRAGY_COPY;
+                              drag = DRAG_CLONE;
                         }
-                  else
-                        drag = drag == DRAG_MOVE_START ? DRAG_MOVE : DRAG_COPY;
                   setCursor();
                   if (!curItem->isSelected()) {
                         if (drag == DRAG_MOVE)
@@ -492,21 +535,31 @@ void Canvas::viewMouseMoveEvent(QMouseEvent* event)
                         updateSelection();
                         redraw();
                         }
-                  startMoving(pos, drag == DRAG_COPY);
+                  DragType dt;
+                  if (drag == DRAG_MOVE)
+                        dt = MOVE_MOVE;
+                  else if (drag == DRAG_COPY)
+                        dt = MOVE_COPY;
+                  else
+                        dt = MOVE_CLONE;
+                  startMoving(pos, dt);
                   break;
 
             case DRAG_MOVE:
             case DRAG_COPY:
+            case DRAG_CLONE:
                   moveItems(pos, 0);
                   break;
 
             case DRAGX_MOVE:
             case DRAGX_COPY:
+            case DRAGX_CLONE:
                   moveItems(pos, 1);
                   break;
 
             case DRAGY_MOVE:
             case DRAGY_COPY:
+            case DRAGY_CLONE:
                   moveItems(pos, 2);
                   break;
 
@@ -550,6 +603,7 @@ void Canvas::viewMouseReleaseEvent(QMouseEvent* event)
       switch (drag) {
             case DRAG_MOVE_START:
             case DRAG_COPY_START:
+            case DRAG_CLONE_START:
                   if (!shift)
                         deselectAll();
                   selectItem(curItem, !(shift && curItem->isSelected()));
@@ -558,22 +612,31 @@ void Canvas::viewMouseReleaseEvent(QMouseEvent* event)
                   itemReleased(curItem, curItem->pos());
                   break;
             case DRAG_COPY:
-                  endMoveItems(pos, true, 0);
+                  endMoveItems(pos, MOVE_COPY, 0);
                   break;
             case DRAGX_COPY:
-                  endMoveItems(pos, true, 1);
+                  endMoveItems(pos, MOVE_COPY, 1);
                   break;
             case DRAGY_COPY:
-                  endMoveItems(pos, true, 2);
+                  endMoveItems(pos, MOVE_COPY, 2);
                   break;
             case DRAG_MOVE:
-                  endMoveItems(pos, false, 0);
+                  endMoveItems(pos, MOVE_MOVE, 0);
                   break;
             case DRAGX_MOVE:
-                  endMoveItems(pos, false, 1);
+                  endMoveItems(pos, MOVE_MOVE, 1);
                   break;
             case DRAGY_MOVE:
-                  endMoveItems(pos, false, 2);
+                  endMoveItems(pos, MOVE_MOVE, 2);
+                  break;
+            case DRAG_CLONE:
+                  endMoveItems(pos, MOVE_CLONE, 0);
+                  break;
+            case DRAGX_CLONE:
+                  endMoveItems(pos, MOVE_CLONE, 1);
+                  break;
+            case DRAGY_CLONE:
+                  endMoveItems(pos, MOVE_CLONE, 2);
                   break;
             case DRAG_OFF:
                   break;
@@ -653,9 +716,9 @@ void Canvas::selectLasso(bool toggle)
 //          2     move only vertical
 //---------------------------------------------------------
 
-void Canvas::endMoveItems(const QPoint& pos, bool copyflag, int dir)
+void Canvas::endMoveItems(const QPoint& pos, DragType dragtype, int dir)
       {
-      startUndo(copyflag);
+      startUndo(dragtype);
 
       int dp = y2pitch(pos.y()) - y2pitch(start.y());
       int dx = pos.x() - start.x();
@@ -671,15 +734,15 @@ void Canvas::endMoveItems(const QPoint& pos, bool copyflag, int dir)
             int ny = pitch2y(y2pitch(y) + dp);
             QPoint newpos = raster(QPoint(nx, ny));
             selectItem(i->second, true);
-            if (moveItem(i->second, newpos, copyflag))
+            if (moveItem(i->second, newpos, dragtype))
                   i->second->move(newpos);
             if (moving.size() == 1) {
                   itemReleased(curItem, newpos);
                   }
-            if (copyflag)
+            if (dragtype == MOVE_COPY || dragtype == MOVE_CLONE)
                   selectItem(i->second, false);
             }
-      endUndo(copyflag);
+      endUndo(dragtype);
       moving.clear();
       updateSelection();
       redraw();
@@ -744,16 +807,19 @@ void Canvas::setCursor()
       switch (drag) {
             case DRAGX_MOVE:
             case DRAGX_COPY:
+            case DRAGX_CLONE:
                   QWidget::setCursor(QCursor(SizeHorCursor));
                   break;
 
             case DRAGY_MOVE:
             case DRAGY_COPY:
+            case DRAGY_CLONE:
                   QWidget::setCursor(QCursor(SizeVerCursor));
                   break;
 
             case DRAG_MOVE:
             case DRAG_COPY:
+            case DRAG_CLONE:
                   QWidget::setCursor(QCursor(SizeAllCursor));
                   break;
 
@@ -763,6 +829,7 @@ void Canvas::setCursor()
 
             case DRAG_DELETE:
             case DRAG_COPY_START:
+            case DRAG_CLONE_START:
             case DRAG_MOVE_START:
             case DRAG_NEW:
             case DRAG_LASSO_START:
@@ -781,8 +848,11 @@ void Canvas::setCursor()
                         case CutTool:
                               QWidget::setCursor(QCursor(*cutIcon, 4, 15));
                               break;
+                        case MuteTool:
+                              QWidget::setCursor(QCursor(*editmuteIcon, 4, 15));
+                              break;
                         default:
-                              QWidget::setCursor(QCursor(arrowCursor));
+                              QWidget::setCursor(QCursor(ArrowCursor));
                               break;
                         }
                   break;
@@ -795,7 +865,31 @@ void Canvas::setCursor()
 
 void Canvas::keyPress(QKeyEvent* event)
       {
-      event->ignore();
+      //Alt-P, shortcut for both DrumEditor and PianoRoll. Sets locators to include selected events 
+      if (event->key() == Key_P && event->state() & AltButton) {
+            int tick_max = 0;
+            int tick_min = INT_MAX;
+            bool found = false;
+
+            for (iCItem i= items.begin(); i != items.end(); i++) {
+                  if (!i->second->isSelected())
+                        continue;
+
+                  int tick = i->second->x();
+                  int len = i->second->event()->lenTick();
+                  found = true;
+                  if (tick + len > tick_max)
+                        tick_max = tick + len;
+                  if (tick < tick_min)
+                        tick_min = tick;
+            }
+            if (found) {
+                  song->setPos(1, tick_min);
+                  song->setPos(2, tick_max);
+            }
+      }
+      else
+            event->ignore();
       }
 
 //---------------------------------------------------------
@@ -821,3 +915,33 @@ int Canvas::selectionSize()
       return n;
       }
 
+//---------------------------------------------------------
+//   genCanvasPopup
+//---------------------------------------------------------
+
+QPopupMenu* Canvas::genCanvasPopup()
+      {
+      if (canvasTools == 0)
+            return 0;
+      QPopupMenu* canvasPopup = new QPopupMenu(this);
+
+      for (unsigned i = 0; i < 9; ++i) {
+            if ((canvasTools & (1 << i))==0)
+                  continue;
+            canvasPopup->insertItem(**toolList[i].icon, tr(toolList[i].tip), 1<<i);
+            }
+      canvasPopup->setActiveItem(0);
+      return canvasPopup;
+      }
+
+//---------------------------------------------------------
+//   canvasPopup
+//---------------------------------------------------------
+
+void Canvas::canvasPopup(int n)
+      {
+      setTool(n);
+      emit toolChanged(n);
+      }
+
+
