@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: dentry.h,v 1.2 2001/11/20 15:19:34 muse Exp $
+//    $Id: dentry.h,v 1.1.1.1 2003/10/29 10:06:29 wschweer Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -17,6 +17,12 @@ class QTimer;
 //---------------------------------------------------------
 
 class Dentry : public QLineEdit {
+      Q_OBJECT
+
+      Q_PROPERTY( int id READ id WRITE setId )
+      Q_PROPERTY( double value READ value WRITE setValue )
+      Q_PROPERTY( bool frame READ frame WRITE setFrame )
+
       int button;
       int starty;
       bool drawFrame;
@@ -24,13 +30,14 @@ class Dentry : public QLineEdit {
       double evx;
       int timecount;
 
-      Q_OBJECT
+      virtual void wheelEvent(QWheelEvent*);
       virtual void mousePressEvent(QMouseEvent*);
       virtual void mouseMoveEvent(QMouseEvent*);
       virtual void mouseDoubleClickEvent(QMouseEvent*);
       virtual void mouseReleaseEvent(QMouseEvent*);
 
    protected:
+      int _id;
       double val;
 
       virtual void incValue(double x) = 0;
@@ -45,14 +52,17 @@ class Dentry : public QLineEdit {
       void endEdit();
 
    signals:
-      void valueChanged(double);
+      void valueChanged(double, int);
 
    public slots:
       virtual void setValue(double);
 
    public:
-      Dentry(QWidget*);
+      Dentry(QWidget*, const char* name=0);
       double value() const { return val; }
+      bool frame() const { return drawFrame; }
       void setFrame(bool);
+      int id() const    { return _id; }
+      void setId(int i) { _id = i; }
       };
 #endif
