@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: gatetime.cpp,v 1.1 2002/01/30 14:54:03 muse Exp $
+//    $Id: gatetime.cpp,v 1.1.1.1 2003/10/29 10:06:24 wschweer Exp $
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -17,7 +17,7 @@
 //   GateTime
 //---------------------------------------------------------
 
-GateTime::GateTime(QWidget* parent, const char* name = 0)
+GateTime::GateTime(QWidget* parent, const char* name)
    : GateTimeBase(parent, name, true)
       {
       }
