@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: intlabel.h,v 1.2 2001/11/20 15:19:34 muse Exp $
+//    $Id: intlabel.h,v 1.1.1.1 2003/10/29 10:06:32 wschweer Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -35,7 +35,7 @@ class IntLabel : public Nentry {
 
    public:
       IntLabel(int, int, int, QWidget*, int _off = MAXINT,
-         const QString& = "", int lpos = 0);
+         const QString& = QString(""), int lpos = 0);
       void setOff(int v);
       void setSuffix(const QString& s) { suffix = s; }
       void setSpecialValueText(const QString& s);
