@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: lcombo.cpp,v 1.1 2002/01/30 14:54:03 muse Exp $
+//    $Id: lcombo.cpp,v 1.1.1.1 2003/10/29 10:06:26 wschweer Exp $
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -16,9 +16,9 @@
 //---------------------------------------------------------
 
 LabelCombo::LabelCombo(const QString& txt, QWidget* parent,
-   const char* name=0) : QWidget(parent, name)
+   const char* name) : QWidget(parent, name)
       {
-      setFixedHeight(20);
+//      setFixedHeight(20);
       QHBoxLayout* layout = new QHBoxLayout(this);
       QLabel* label = new QLabel(txt, this);
       box = new QComboBox(false, this);
@@ -32,7 +32,7 @@ LabelCombo::LabelCombo(const QString& txt, QWidget* parent,
       connect(box, SIGNAL(activated(int)), SIGNAL(activated(int)));
       }
 
-void LabelCombo::insertItem(const QString& txt, int index=-1)
+void LabelCombo::insertItem(const QString& txt, int index)
       {
       box->insertItem(txt, index);
       }
