@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: midisyncimpl.h,v 1.2 2001/11/20 15:19:34 muse Exp $
+//  $Id: midisyncimpl.h,v 1.1.1.1 2003/10/29 10:06:29 wschweer Exp $
 //
 //  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -19,8 +19,7 @@ class MidiSyncConfig : public MidiSyncConfigBase {
       Q_OBJECT
 
    private slots:
-      void extSyncChanged(bool);
-      void intSyncChanged(bool);
+      void syncChanged(int);
       void ok();
       void cancel();
       void apply();
