@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: posedit.cpp,v 1.1 2002/01/30 14:54:03 muse Exp $
+//    $Id: posedit.cpp,v 1.1.1.1 2003/10/29 10:06:32 wschweer Exp $
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -280,7 +280,7 @@ bool PosEditor::eventFilter(QObject *o, QEvent *e)
                   QString txt = ke->text();
                   if (!txt.isEmpty() && !sep.isEmpty() && txt[0] == sep[0]) {
                         // do the same thing as KEY_RIGHT when the user presses the separator key
-                        if (focusSec < sections.count()) {
+                        if (unsigned(focusSec) < sections.count()) {
                               if (cw->setFocusSection(focusSec+1))
                                     repaint(rect(), false);
                               }
@@ -336,14 +336,14 @@ bool PosEditor::setFocusSection(int idx)
 //   PosEdit
 //---------------------------------------------------------
 
-PosEdit::PosEdit(QWidget* parent, const char* name = 0)
+PosEdit::PosEdit(QWidget* parent, const char* name)
    : QWidget(parent, name)
       {
       init();
       updateButtons();
       }
 
-PosEdit::PosEdit(const Pos& time, QWidget* parent, const char* name = 0)
+PosEdit::PosEdit(const Pos& time, QWidget* parent, const char* name)
     : QWidget(parent, name)
       {
       init();
@@ -410,10 +410,10 @@ void PosEdit::setSections()
       ed->appendSection(QNumberSection(0,0));
       if (_smpte) {
             ed->appendSection(QNumberSection(0,0));
-            ed->setSeparator(":");
+            ed->setSeparator(QString(":"));
             }
       else {
-            ed->setSeparator(".");
+            ed->setSeparator(QString("."));
             }
       }
 
@@ -474,13 +474,6 @@ void PosEdit::setRange(const Pos& _min, const Pos& _max)
 
 void PosEdit::setValue(const Pos& time)
       {
-/*      if (!time.isValid()) {
-            m = 0;
-            b = 0;
-            t = 0;
-            return;
-            }
-*/
       if (time > maxValue() || time < minValue())
             return;
       if (_smpte)
@@ -488,7 +481,6 @@ void PosEdit::setValue(const Pos& time)
                &(sec[3].val));
       else
             time.mbt(&(sec[0].val), &(sec[1].val), &(sec[2].val));
-//      emit valueChanged(time);
       changed = false;
       ed->repaint(ed->rect(), false);
       }
@@ -565,7 +557,6 @@ void PosEdit::stepUp()
       ed->repaint(ed->rect(), false);
       }
 
-
 //---------------------------------------------------------
 //   stepDown
 //---------------------------------------------------------
