@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: scldraw.cpp,v 1.1 2002/01/30 14:54:03 muse Exp $
+//    $Id: scldraw.cpp,v 1.1.1.1 2003/10/29 10:06:23 wschweer Exp $
 //
 //    Copyright (C) 1997  Josef Wilgen
 //	This program is free software; you can redistribute it and/or modify
@@ -11,10 +11,10 @@
 //    (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
-#include <math.h>
+#include <cmath>
 #include "scldraw.h"
 #include <qpainter.h>
-#include <qfontmet.h>
+#include <qfontmetrics.h>
 #include <qrect.h>
 #include "mmath.h"
 
