@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: siglabel.h,v 1.2 2001/11/20 15:19:35 muse Exp $
+//    $Id: siglabel.h,v 1.1.1.1 2003/10/29 10:06:30 wschweer Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -10,9 +10,16 @@
 
 #include <qlabel.h>
 
+//---------------------------------------------------------
+//   SigLabel
+//    show/edit time signature
+//---------------------------------------------------------
+
 class SigLabel : public QLabel {
       Q_OBJECT
       virtual void mousePressEvent(QMouseEvent*);
+      virtual void wheelEvent(QWheelEvent*);
+      void incValue(bool zaehler, bool inc, int&, int&);
 
    protected:
       int z, n;
