@@ -1,54 +1,24 @@
-#include <math.h>
+#include <cmath>
 #include "mmath.h"
-#include <qdrawutl.h>
+#include <qdrawutil.h>
 #include <qpainter.h>
 #include "slider.h"
 
 //-------------------------------------------------------------
-//.H Slider | 3 | 02/04/98 | Qwt Widget Library | Qwt Programmer's Manual
-//.I sliders Sliders
-//.U  NAME
 //	Slider - The Slider Widget
 //
-//.U  SYNOPSIS
-//	#include <qwt_slider.h>
-//
-//.U DESCRIPTION
 //	Slider is a slider widget which operates on an interval
 //	of type double. Slider supports different layouts as
 //	well as a scale.
-//
-//
-//.U INHERITED CLASSES
-//	@^SliderBase@, @^QwtScaleIf@
-//
-//.U PUBLIC MEMBERS
-//.R
-//	Slider::Slider -- Constructor
-//	Slider::~Slider -- Destructor
-//	Slider::setThumbLength -- Set the length of the slider's thumb
-//	Slider::setThumbWidth -- Set the length of the slider's thumb
-//	Slider::setBorderWidth -- Set the border width
-//	Slider::setMargins -- Set the internal border distance
-//
-//.U SEE ALSO
-//	@^SliderBase@ and @^QwtScaleIf@ for the descriptions
-//	of the inherited members.
 //------------------------------------------------------------
 
 //------------------------------------------------------------
-//.C MEMBER FUNCTION DESCRIPTION
-//------------------------------------------------------------
-
-
-//------------------------------------------------------------
-//
 //.F	Slider::Slider
 //
 //		Constructor
 //
 //.u	Syntax:
-//.f	Slider::Slider(QWidget *parent, const char *name, OrientationX orient = Horizontal, ScalePos scalePos = None, int bgStyle = BgTrough)
+//.f	Slider::Slider(QWidget *parent, const char *name, Orientation orient = Horizontal, ScalePos scalePos = None, int bgStyle = BgTrough)
 //
 //.u	Parameters
 //.p
@@ -64,14 +34,13 @@
 //				slider button in a trough, Slider::BgSlot draws
 //				a slot underneath the button. An or-combination of both
 //				may also be used. The default is Slider::BgTrough.
-//
-//
 //------------------------------------------------------------
+
 Slider::Slider(QWidget *parent, const char *name,
-   OrientationX orient, ScalePos scalePos, int bgStyle)
+   Orientation orient, ScalePos scalePos, int bgStyle)
       : SliderBase(parent,name)
       {
-      ScaleDraw::OrientationX so;
+      _id = -1;
 
 
       if (bgStyle == BgSlot) {
@@ -86,46 +55,22 @@ Slider::Slider(QWidget *parent, const char *name,
             }
 
       d_borderWidth = 2;
-      d_scaleDist = 4;
-      d_orient = orient;
-      d_scaleStep = 0.0;
-      d_scalePos = scalePos;
-      d_xMargin = 0;
-      d_yMargin = 0;
-      d_bgStyle = bgStyle;
+      d_scaleDist   = 4;
+      d_scaleStep   = 0.0;
+      d_scalePos    = scalePos;
+      d_xMargin     = 0;
+      d_yMargin     = 0;
+      d_bgStyle     = bgStyle;
 
       if (bgStyle & BgTrough)
             d_bwTrough = d_borderWidth;
       else
             d_bwTrough = 0;
 
-    d_sliderRect.setRect(0,0,8,8);
-
-    switch(d_orient)
-    {
-    case Vertical:
-	if (d_scalePos == Right)
-	   so = ScaleDraw::Right;
-	else
-	   so = ScaleDraw::Left;
-	break;
-    default:
-	if (d_scalePos == Bottom)
-	   so = ScaleDraw::Bottom;
-	else
-	   so = ScaleDraw::Top;
-	break;
-    }
-
-      d_scale.setGeometry(0, 0, 40, so);
-	
-      if (d_orient == Vertical)
-            setMinimumSize(10,20);
-      else
-            setMinimumSize(20,10);
+      d_sliderRect.setRect(0, 0, 8, 8);
+      setOrientation(orient);
       }
 
-
 //------------------------------------------------------------
 //.F 	Slider::~Slider
 // 		Destructor
@@ -149,6 +94,7 @@ Slider::~Slider()
 //.p	int bd -- border width
 //
 //------------------------------------------------------------
+
 void Slider::setBorderWidth(int bd)
 {
     d_borderWidth = qwtMin(qwtMax(bd,0),10);
@@ -232,48 +178,37 @@ void Slider::fontChange(const QFont & /*oldFont*/)
 }
 
 //------------------------------------------------------------
-//.-
-//.F	Slider::drawSlider
-//		Draw the slider into the specified rectangle.	
-//
-//.u	Syntax:
-//.f	void Slider::drawSlider(QPaiter *p, const QRect &r)
-//
-//.u	Parameters:
-//.p	QPainter *p, const QRect &r
-//
+//    drawSlider
+//	   Draw the slider into the specified rectangle.	
 //------------------------------------------------------------
-void Slider::drawSlider (QPainter *p, const QRect &r)
-{
 
-    QColorGroup g = colorGroup();
-    QBrush brBack(g.background());
-    QBrush brMid;
-    QBrush brDark(g.dark());
+void Slider::drawSlider(QPainter *p, const QRect &r)
+      {
+      QColorGroup g = colorGroup();
+      QBrush brBack(g.background());
+      QBrush brMid;
+      QBrush brDark(g.dark());
 
-    QRect cr;
+      QRect cr;
 
-    int ipos,dist1;
-    double rpos;
-    int lineDist;
+      int ipos,dist1;
+      double rpos;
+      int lineDist;
 
-    if (d_bwTrough > 0)
-    {
-	qDrawShadePanel(p, r.x(), r.y(),
+      if (d_bwTrough > 0) {
+            qDrawShadePanel(p, r.x(), r.y(),
 			r.width(), r.height(),
 			g, TRUE, d_bwTrough,0);
-	cr.setRect(r.x() + d_bwTrough,
+      	cr.setRect(r.x() + d_bwTrough,
 		   r.y() + d_bwTrough,
 		   r.width() - 2*d_bwTrough,
 		   r.height() - 2*d_bwTrough);
-	brMid = QBrush(g.mid());
-
-    }
-    else
-    {
-	cr = r;
-	brMid = brBack;
-    }
+      	brMid = QBrush(g.mid());
+            }
+    else {
+            cr = r;
+            brMid = brBack;
+            }
 
     rpos = (value()  - minValue()) / (maxValue() - minValue());
 
@@ -664,37 +599,28 @@ void Slider::getScrollMode( const QPoint &p, int &scrollMode, int &direction )
 }
 
 //------------------------------------------------------------
-//.-
 //.F	Slider::paintEvent
 //	Qt paint event
 //
-//.u	Parameters
-//.p	QPaintEvent *e
-//
 //.u	Syntax
 //.f	void Slider::paintEvent(QPaintEvent *e)
-//
 //------------------------------------------------------------
-void Slider::paintEvent(QPaintEvent * /*e*/)
-{
-    QPainter p;
 
-    if(p.begin(this))
-    {
-	if (d_scalePos != None)
-	{
-	    p.fillRect(this->rect(), this->backgroundColor());
-	    d_scale.draw(&p);
-	}
-	
-	drawSlider(&p, d_sliderRect);
-    }
-    p.end();
+void Slider::paintEvent(QPaintEvent* /*e*/)
+      {
+      QPainter p;
 
-}
+      if (p.begin(this)) {
+            if (d_scalePos != None) {
+                  p.fillRect(this->rect(), this->backgroundColor());
+                  d_scale.draw(&p);
+                  }
+            drawSlider(&p, d_sliderRect);
+            }
+      p.end();
+      }
 
 //------------------------------------------------------------
-//.-
 //.F	Slider::resizeEvent
 //	Qt resize event
 //
@@ -703,8 +629,8 @@ void Slider::paintEvent(QPaintEvent * /*e*/)
 //
 //.u	Syntax
 //.f	void Slider::resizeEvent(QResizeEvent *e)
-//
 //------------------------------------------------------------
+
 void Slider::resizeEvent(QResizeEvent *e)
 {
 
@@ -795,18 +721,18 @@ void Slider::resizeEvent(QResizeEvent *e)
 //.f	void Slider::valueChange()
 //
 //------------------------------------------------------------
+
 void Slider::valueChange()
-{
-    QPainter p;
+      {
+      QPainter p;
 
-    p.begin(this);
-    drawSlider(&p,d_sliderRect);
-    p.end();
-
-    SliderBase::valueChange();
-
-}
+      p.begin(this);
+      drawSlider(&p, d_sliderRect);
+      p.end();
 
+      SliderBase::valueChange();
+      emit valueChanged(value(), _id);
+      }
 
 //------------------------------------------------------------
 //.-	
@@ -878,7 +804,7 @@ QSize Slider::sizeHint() const
                   case Vertical:
                         w = 2*d_xMargin + d_thumbWidth + 2*d_bwTrough + msWidth + d_scaleDist + 2;
                         break;
-                  default:
+                  case Horizontal:
                         h = 2*d_yMargin + d_thumbWidth + 2*d_bwTrough + msHeight + d_scaleDist;
                         break;
                   }
@@ -888,10 +814,67 @@ QSize Slider::sizeHint() const
                   case Vertical:
                         w = 16 + 2 * d_bwTrough;
                         break;
-                  default:
+                  case Horizontal:
                         h = 16 + 2 * d_bwTrough;
                         break;
                   }
             }
       return QSize(w, h);
       }
+
+//---------------------------------------------------------
+//   setOrientation
+//---------------------------------------------------------
+
+void Slider::setOrientation(Orientation o)
+      {
+      d_orient = o;
+      ScaleDraw::OrientationX so = ScaleDraw::Bottom;
+      switch(d_orient) {
+            case Vertical:
+                  if (d_scalePos == Right)
+                        so = ScaleDraw::Right;
+                  else
+                        so = ScaleDraw::Left;
+                  break;
+            case Horizontal:
+                  if (d_scalePos == Bottom)
+                        so = ScaleDraw::Bottom;
+                  else
+                        so = ScaleDraw::Top;
+                  break;
+            }
+
+      d_scale.setGeometry(0, 0, 40, so);
+      if (d_orient == Vertical)
+            setMinimumSize(10,20);
+      else
+            setMinimumSize(20,10);
+      QRect r = geometry();
+      setGeometry(r.x(), r.y(), r.height(), r.width());
+      update();
+      }
+
+Qt::Orientation Slider::orientation() const
+      {
+      return d_orient;
+      }
+
+double Slider::lineStep() const
+      {
+      return 1.0;
+      }
+
+double Slider::pageStep() const
+      {
+      return 1.0;
+      }
+
+void Slider::setLineStep(double)
+      {
+      }
+
+void Slider::setPageStep(double)
+      {
+      }
+
