@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: tb1.h,v 1.2 2001/11/20 15:19:35 muse Exp $
+//    $Id: tb1.h,v 1.1.1.1 2003/10/29 10:06:21 wschweer Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -9,7 +9,7 @@
 #define __TB1_H__
 
 #include <qwidget.h>
-#include "desk.h"
+#include <qtoolbar.h>
 
 class PosLabel;
 class PitchLabel;
@@ -21,7 +21,7 @@ class QToolButton;
 //   Toolbar1
 //---------------------------------------------------------
 
-class Toolbar1 : public ToolBar {
+class Toolbar1 : public QToolBar {
       QToolButton* solo;
       PosLabel* pos;
       PitchLabel* pitch;
@@ -48,7 +48,7 @@ class Toolbar1 : public ToolBar {
       void toChanged(int);
 
    public:
-      Toolbar1(MainWindow* parent, int r=96,
+      Toolbar1(QMainWindow* parent, int r=96,
          int q=96, bool showPitch=true);
       void setSolo(bool val);
       void setPitchMode(bool flag);
