@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: tempolabel.h,v 1.3 2002/01/03 10:51:54 muse Exp $
+//    $Id: tempolabel.h,v 1.1.1.1 2003/10/29 10:06:33 wschweer Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -32,10 +32,10 @@ class TempoLabel : public QLabel {
       };
 
 //---------------------------------------------------------
-//   TempoSpinbox
+//   TempoEdit
 //---------------------------------------------------------
 
-class TempoSpinBox : public QSpinBox {
+class TempoEdit : public QSpinBox {
       Q_OBJECT
 
    protected:
@@ -53,7 +53,7 @@ class TempoSpinBox : public QSpinBox {
       void valueChanged(double);
 
    public:
-      TempoSpinBox(QWidget*, const char* name = 0);
+      TempoEdit(QWidget*, const char* name = 0);
       };
 
 #endif
