@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: velocity.cpp,v 1.1 2002/01/30 14:54:04 muse Exp $
+//    $Id: velocity.cpp,v 1.1.1.1 2003/10/29 10:06:33 wschweer Exp $
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -17,7 +17,7 @@
 //   Velocity
 //---------------------------------------------------------
 
-Velocity::Velocity(QWidget* parent, const char* name = 0)
+Velocity::Velocity(QWidget* parent, const char* name)
    : VelocityBase(parent, name, true)
       {
       }
